@@ -12,7 +12,6 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::any::Any;
 use std::path::Path;
 use std::pin::Pin;
 use std::time::SystemTime;
@@ -30,8 +29,6 @@ use jj_lib::backend::BackendResult;
 use jj_lib::backend::ChangeId;
 use jj_lib::backend::Commit;
 use jj_lib::backend::CommitId;
-use jj_lib::backend::Conflict;
-use jj_lib::backend::ConflictId;
 use jj_lib::backend::CopyHistory;
 use jj_lib::backend::CopyId;
 use jj_lib::backend::CopyRecord;
@@ -117,10 +114,6 @@ impl JitBackend {
 
 #[async_trait]
 impl Backend for JitBackend {
-    fn as_any(&self) -> &dyn Any {
-        self
-    }
-
     fn name(&self) -> &str {
         "jit"
     }
@@ -193,14 +186,6 @@ impl Backend for JitBackend {
         self.inner.write_tree(path, contents).await
     }
 
-    fn read_conflict(&self, path: &RepoPath, id: &ConflictId) -> BackendResult<Conflict> {
-        self.inner.read_conflict(path, id)
-    }
-
-    fn write_conflict(&self, path: &RepoPath, contents: &Conflict) -> BackendResult<ConflictId> {
-        self.inner.write_conflict(path, contents)
-    }
-
     async fn read_commit(&self, id: &CommitId) -> BackendResult<Commit> {
         self.inner.read_commit(id).await
     }
