@@ -15,12 +15,12 @@
 use clap_complete::ArgValueCandidates;
 use itertools::Itertools as _;
 use jj_lib::op_store::RefTarget;
-use jj_lib::str_util::StringPattern;
 
-use super::find_local_bookmarks;
+use super::warn_unmatched_local_bookmarks;
 use crate::cli_util::CommandHelper;
 use crate::command_error::CommandError;
 use crate::complete;
+use crate::revset_util::parse_union_name_patterns;
 use crate::ui::Ui;
 
 /// Delete an existing bookmark and propagate the deletion to remotes on the
@@ -41,13 +41,12 @@ pub struct BookmarkDeleteArgs {
     /// select bookmarks by [wildcard pattern].
     ///
     /// [wildcard pattern]:
-    ///     https://jj-vcs.github.io/jj/latest/revsets/#string-patterns
+    ///     https://docs.jj-vcs.dev/latest/revsets/#string-patterns
     #[arg(
         required = true,
-        value_parser = StringPattern::parse,
         add = ArgValueCandidates::new(complete::local_bookmarks),
     )]
-    names: Vec<StringPattern>,
+    names: Vec<String>,
 }
 
 pub fn cmd_bookmark_delete(
@@ -57,7 +56,18 @@ pub fn cmd_bookmark_delete(
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
     let repo = workspace_command.repo().clone();
-    let matched_bookmarks = find_local_bookmarks(repo.view(), &args.names)?;
+    let name_expr = parse_union_name_patterns(ui, &args.names)?;
+    let name_matcher = name_expr.to_matcher();
+    let matched_bookmarks = repo
+        .view()
+        .local_bookmarks_matching(&name_matcher)
+        .collect_vec();
+    warn_unmatched_local_bookmarks(ui, repo.view(), &name_expr)?;
+    if matched_bookmarks.is_empty() {
+        writeln!(ui.status(), "No bookmarks to delete.")?;
+        return Ok(());
+    }
+
     let mut tx = workspace_command.start_transaction();
     for (name, _) in &matched_bookmarks {
         tx.repo_mut()
