@@ -24,18 +24,18 @@ use itertools::Itertools as _;
 use jj_lib::backend;
 use jj_lib::backend::CommitId;
 use jj_lib::config::ConfigValue;
-use jj_lib::ref_name::RefName;
 use jj_lib::repo::Repo as _;
 use jj_lib::revset::RevsetExpression;
 use jj_lib::str_util::StringExpression;
-use jj_lib::str_util::StringPattern;
 
+use super::warn_unmatched_local_or_remote_bookmarks;
 use crate::cli_util::CommandHelper;
 use crate::cli_util::RevisionArg;
 use crate::cli_util::default_ignored_remote_name;
 use crate::command_error::CommandError;
 use crate::commit_templater::CommitRef;
 use crate::complete;
+use crate::revset_util::parse_union_name_patterns;
 use crate::templater::TemplateRenderer;
 use crate::ui::Ui;
 
@@ -50,7 +50,7 @@ use crate::ui::Ui;
 /// See [`jj help -k bookmarks`] for more information.
 ///
 /// [`jj help -k bookmarks`]:
-///     https://jj-vcs.github.io/jj/latest/bookmarks
+///     https://docs.jj-vcs.dev/latest/bookmarks
 #[derive(clap::Args, Clone, Debug)]
 pub struct BookmarkListArgs {
     /// Show all tracking and non-tracking remote bookmarks including the ones
@@ -68,15 +68,14 @@ pub struct BookmarkListArgs {
     /// prefix to select remotes by [wildcard pattern].
     ///
     /// [wildcard pattern]:
-    ///     https://jj-vcs.github.io/jj/latest/revsets/#string-patterns
+    ///     https://docs.jj-vcs.dev/latest/revsets/#string-patterns
     #[arg(
         long = "remote",
         value_name = "REMOTE",
         conflicts_with_all = ["all_remotes"],
-        value_parser = StringPattern::parse,
         add = ArgValueCandidates::new(complete::git_remotes),
     )]
-    remotes: Option<Vec<StringPattern>>,
+    remotes: Option<Vec<String>>,
 
     /// Show remote tracked bookmarks only. Omits local Git-tracking bookmarks
     /// by default
@@ -93,9 +92,9 @@ pub struct BookmarkListArgs {
     /// select bookmarks by [wildcard pattern].
     ///
     /// [wildcard pattern]:
-    ///     https://jj-vcs.github.io/jj/latest/revsets/#string-patterns
-    #[arg(value_parser = StringPattern::parse, add = ArgValueCandidates::new(complete::bookmarks))]
-    names: Option<Vec<StringPattern>>,
+    ///     https://docs.jj-vcs.dev/latest/revsets/#string-patterns
+    #[arg(add = ArgValueCandidates::new(complete::bookmarks))]
+    names: Option<Vec<String>>,
 
     /// Show bookmarks whose local targets are in the given revisions
     ///
@@ -111,10 +110,10 @@ pub struct BookmarkListArgs {
     /// for more information.
     ///
     /// [`CommitRef` type]:
-    ///     https://jj-vcs.github.io/jj/latest/templates/#commitref-type
+    ///     https://docs.jj-vcs.dev/latest/templates/#commitref-type
     ///
     /// [`jj help -k templates`]:
-    ///     https://jj-vcs.github.io/jj/latest/templates/
+    ///     https://docs.jj-vcs.dev/latest/templates/
     #[arg(long, short = 'T', add = ArgValueCandidates::new(complete::template_aliases))]
     template: Option<String>,
 
@@ -139,38 +138,21 @@ pub fn cmd_bookmark_list(
     let view = repo.view();
 
     // Like cmd_git_push(), names and revisions are OR-ed.
-    let bookmark_names_to_list = if args.names.is_some() || args.revisions.is_some() {
-        let mut bookmark_names: HashSet<&RefName> = HashSet::new();
-        if let Some(patterns) = &args.names {
-            bookmark_names.extend(
-                view.bookmarks()
-                    .filter(|(name, _)| {
-                        patterns
-                            .iter()
-                            .any(|pattern| pattern.is_match(name.as_str()))
-                    })
-                    .map(|(name, _)| name),
-            );
-        }
-        if let Some(revisions) = &args.revisions {
-            // Match against local targets only, which is consistent with "jj git push".
-            let mut expression = workspace_command.parse_union_revsets(ui, revisions)?;
-            // Intersects with the set of local bookmark targets to minimize the lookup
-            // space.
-            expression.intersect_with(&RevsetExpression::bookmarks(StringExpression::all()));
-            let filtered_targets: HashSet<_> =
-                expression.evaluate_to_commit_ids()?.try_collect()?;
-            bookmark_names.extend(
-                view.local_bookmarks()
-                    .filter(|(_, target)| {
-                        target.added_ids().any(|id| filtered_targets.contains(id))
-                    })
-                    .map(|(name, _)| name),
-            );
-        }
-        Some(bookmark_names)
+    let name_expr = match (&args.names, &args.revisions) {
+        (Some(texts), _) => parse_union_name_patterns(ui, texts)?,
+        (None, Some(_)) => StringExpression::none(),
+        (None, None) => StringExpression::all(),
+    };
+    let name_matcher = name_expr.to_matcher();
+    let matched_local_targets: HashSet<_> = if let Some(revisions) = &args.revisions {
+        // Match against local targets only, which is consistent with "jj git push".
+        let mut expression = workspace_command.parse_union_revsets(ui, revisions)?;
+        // Intersects with the set of local bookmark targets to minimize the lookup
+        // space.
+        expression.intersect_with(&RevsetExpression::bookmarks(StringExpression::all()));
+        expression.evaluate_to_commit_ids()?.try_collect()?
     } else {
-        None
+        HashSet::new()
     };
 
     let template: TemplateRenderer<Rc<CommitRef>> = {
@@ -187,26 +169,31 @@ pub fn cmd_bookmark_list(
     };
 
     let ignored_tracked_remote = default_ignored_remote_name(repo.store());
+    let remote_expr = match &args.remotes {
+        Some(texts) => parse_union_name_patterns(ui, texts)?,
+        None => StringExpression::all(),
+    };
+    let remote_matcher = remote_expr.to_matcher();
     let mut bookmark_list_items: Vec<RefListItem> = Vec::new();
-    let bookmarks_to_list = view.bookmarks().filter(|(name, target)| {
-        bookmark_names_to_list
-            .as_ref()
-            .is_none_or(|bookmark_names| bookmark_names.contains(name))
-            && (!args.conflicted || target.local_target.has_conflict())
-    });
+    let bookmarks_to_list = view
+        .bookmarks()
+        .filter(|(name, target)| {
+            name_matcher.is_match(name.as_str())
+                || target
+                    .local_target
+                    .added_ids()
+                    .any(|id| matched_local_targets.contains(id))
+        })
+        .filter(|(_, target)| !args.conflicted || target.local_target.has_conflict());
+    let mut any_conflicts = false;
     for (name, bookmark_target) in bookmarks_to_list {
         let local_target = bookmark_target.local_target;
+        any_conflicts |= local_target.has_conflict();
         let remote_refs = bookmark_target.remote_refs;
         let (mut tracked_remote_refs, untracked_remote_refs) = remote_refs
             .iter()
             .copied()
-            .filter(|(remote_name, _)| {
-                args.remotes.as_ref().is_none_or(|patterns| {
-                    patterns
-                        .iter()
-                        .any(|pattern| pattern.is_match(remote_name.as_str()))
-                })
-            })
+            .filter(|(remote_name, _)| remote_matcher.is_match(remote_name.as_str()))
             .partition::<Vec<_>, _>(|&(_, remote_ref)| remote_ref.is_tracked());
 
         if args.tracked {
@@ -273,6 +260,15 @@ pub fn cmd_bookmark_list(
         .try_for_each(|commit_ref| template.format(commit_ref, formatter.as_mut()))?;
     drop(formatter);
 
+    warn_unmatched_local_or_remote_bookmarks(ui, view, &name_expr)?;
+
+    if any_conflicts {
+        writeln!(
+            ui.hint_default(),
+            "Some bookmarks have conflicts. Use `jj bookmark set <name> -r <rev>` to resolve."
+        )?;
+    }
+
     #[cfg(feature = "git")]
     if jj_lib::git::get_git_backend(repo.store()).is_ok() {
         // Print only one of these hints. It's not important to mention unexported
@@ -451,11 +447,11 @@ fn sort(
 #[cfg(test)]
 mod tests {
     use jj_lib::backend::ChangeId;
-    use jj_lib::backend::MergedTreeId;
     use jj_lib::backend::MillisSinceEpoch;
     use jj_lib::backend::Signature;
     use jj_lib::backend::Timestamp;
     use jj_lib::backend::TreeId;
+    use jj_lib::merge::Merge;
     use jj_lib::op_store::RefTarget;
 
     use super::*;
@@ -464,7 +460,7 @@ mod tests {
         Arc::new(backend::Commit {
             parents: vec![],
             predecessors: vec![],
-            root_tree: MergedTreeId::resolved(TreeId::new(vec![])),
+            root_tree: Merge::resolved(TreeId::new(vec![])),
             change_id: ChangeId::new(vec![]),
             description: String::new(),
             author,
@@ -882,4 +878,19 @@ mod tests {
         chore               Test User       test.user@g.com  0             eve             test.user@g.com  0
         ");
     }
+
+    // Bookmarks are already sorted by name
+    // Test when sorting by name is not the only/last criteria
+    #[test]
+    fn test_sort_by_name_and_committer_date() {
+        insta::assert_snapshot!(
+            prepare_data_sort_and_snapshot(&[SortKey::Name, SortKey::AuthorDate]), @r"
+        Name                AuthorName      AuthorEmail      AuthorDate    CommitterName   CommitterEmail   CommitterDate
+        bug-fix@origin      Test User       test.user@g.com  3             Test User       test.user@g.com  0
+        bug-fix@upstream    Test User       test.user@g.com  1             Test User       test.user@g.com  0
+        chore               Test User       test.user@g.com  2             Test User       test.user@g.com  0
+        feature             Test User       test.user@g.com  1             Test User       test.user@g.com  0
+        feature@origin      -               -                -             -               -                -
+        ");
+    }
 }
