@@ -41,7 +41,7 @@ pub struct BookmarkUntrackArgs {
     /// Examples: bookmark@remote, glob:main@*, glob:jjfan-*@upstream
     ///
     /// [wildcard pattern]:
-    ///     https://jj-vcs.github.io/jj/latest/revsets/#string-patterns
+    ///     https://docs.jj-vcs.dev/latest/revsets/#string-patterns
     #[arg(
         required = true,
         value_name = "BOOKMARK@REMOTE",
@@ -59,7 +59,7 @@ pub fn cmd_bookmark_untrack(
     let repo = workspace_command.repo().clone();
     let ignored_remote = default_ignored_remote_name(repo.store());
     let mut symbols = Vec::new();
-    for (symbol, remote_ref) in find_trackable_remote_bookmarks(repo.view(), &args.names)? {
+    for (symbol, remote_ref) in find_trackable_remote_bookmarks(ui, repo.view(), &args.names)? {
         if ignored_remote.is_some_and(|ignored| symbol.remote == ignored) {
             // This restriction can be lifted if we want to support untracked @git
             // bookmarks.
