@@ -17,6 +17,7 @@ use clap_complete::ArgValueCompleter;
 use indexmap::IndexSet;
 use itertools::Itertools as _;
 use jj_lib::copies::CopyRecords;
+use jj_lib::merge::Diff;
 use jj_lib::repo::Repo as _;
 use jj_lib::rewrite::merge_commit_trees;
 use pollster::FutureExt as _;
@@ -108,10 +109,10 @@ pub(crate) struct DiffArgs {
     /// more information.
     ///
     /// [`TreeDiffEntry` type]:
-    ///     https://jj-vcs.github.io/jj/latest/templates/#treediffentry-type
+    ///     https://docs.jj-vcs.dev/latest/templates/#treediffentry-type
     ///
     /// [`jj help -k templates`]:
-    ///     https://jj-vcs.github.io/jj/latest/templates/
+    ///     https://docs.jj-vcs.dev/latest/templates/
     #[arg(
         long,
         short = 'T',
@@ -144,8 +145,8 @@ pub(crate) fn cmd_diff(
         };
         let from = resolve_revision(&args.from)?;
         let to = resolve_revision(&args.to)?;
-        from_tree = from.tree()?;
-        to_tree = to.tree()?;
+        from_tree = from.tree();
+        to_tree = to.tree();
 
         let records = get_copy_records(repo.store(), from.id(), to.id(), &matcher)?;
         copy_records.add_records(records)?;
@@ -222,7 +223,7 @@ pub(crate) fn cmd_diff(
         .show_diff(
             ui,
             ui.stdout_formatter().as_mut(),
-            [&from_tree, &to_tree],
+            Diff::new(&from_tree, &to_tree),
             &matcher,
             &copy_records,
             ui.term_width(),
