@@ -17,6 +17,7 @@ use std::io::Write as _;
 use clap_complete::ArgValueCandidates;
 use clap_complete::ArgValueCompleter;
 use itertools::Itertools as _;
+use jj_lib::merge::Diff;
 use jj_lib::object_id::ObjectId as _;
 use jj_lib::rewrite::merge_commit_trees;
 use pollster::FutureExt as _;
@@ -37,7 +38,7 @@ use crate::ui::Ui;
 /// the "from" revision to the "to" revision.
 ///
 /// [diff editor]:
-///     https://jj-vcs.github.io/jj/latest/config/#editing-diffs
+///     https://docs.jj-vcs.dev/latest/config/#editing-diffs
 ///
 /// Edit the right side of the diff until it looks the way you want. Once you
 /// close the editor, the revision specified with `-r` or `--to` will be
@@ -147,14 +148,15 @@ don't make any changes, then the operation will be aborted.",
         )
     };
     let base_tree = merge_commit_trees(tx.repo(), base_commits.as_slice()).block_on()?;
-    let tree = target_commit.tree()?;
-    let tree_id = diff_editor.edit([&base_tree, &tree], &matcher, format_instructions)?;
-    if tree_id == *target_commit.tree_id() {
+    let tree = target_commit.tree();
+    let edited_tree =
+        diff_editor.edit(Diff::new(&base_tree, &tree), &matcher, format_instructions)?;
+    if edited_tree.tree_ids() == target_commit.tree_ids() {
         writeln!(ui.status(), "Nothing changed.")?;
     } else {
         tx.repo_mut()
             .rewrite_commit(&target_commit)
-            .set_tree_id(tree_id)
+            .set_tree(edited_tree)
             .write()?;
         // rebase_descendants early; otherwise `new_commit` would always have
         // a conflicted change id at this point.
