@@ -65,10 +65,10 @@ pub(crate) struct FileShowArgs {
     /// If not specified, this defaults to the `templates.file_show` setting.
     ///
     /// [`TreeEntry` type]:
-    ///     https://jj-vcs.github.io/jj/latest/templates/#treeentry-type
+    ///     https://docs.jj-vcs.dev/latest/templates/#treeentry-type
     ///
     /// [`jj help -k templates`]:
-    ///     https://jj-vcs.github.io/jj/latest/templates/
+    ///     https://docs.jj-vcs.dev/latest/templates/
     #[arg(long, short = 'T', add = ArgValueCandidates::new(complete::template_aliases))]
     template: Option<String>,
 
@@ -90,7 +90,7 @@ pub(crate) fn cmd_file_show(
 ) -> Result<(), CommandError> {
     let workspace_command = command.workspace_helper(ui)?;
     let commit = workspace_command.resolve_single_rev(ui, &args.revision)?;
-    let tree = commit.tree()?;
+    let tree = commit.tree();
     // TODO: No need to add special case for empty paths when switching to
     // parse_union_filesets(). paths = [] should be "none()" if supported.
     let fileset_expression = workspace_command.parse_file_patterns(ui, &args.paths)?;
@@ -143,7 +143,7 @@ fn get_single_path(expression: &FilesetExpression) -> Option<&RepoPath> {
             // Not using pattern.as_path() because files-in:<path> shouldn't
             // select the literal <path> itself.
             FilePattern::FilePath(path) | FilePattern::PrefixPath(path) => Some(path),
-            FilePattern::FileGlob { .. } => None,
+            FilePattern::FileGlob { .. } | FilePattern::PrefixGlob { .. } => None,
         },
         _ => None,
     }
