@@ -43,6 +43,14 @@ pub(crate) struct FileTrackArgs {
     /// Paths to track
     #[arg(required = true, value_name = "FILESETS", value_hint = clap::ValueHint::AnyPath)]
     paths: Vec<String>,
+
+    /// Track paths even if they're ignored or too large
+    ///
+    /// By default, `jj file track` will not track files that are ignored by
+    /// .gitignore or exceed the maximum file size. This flag overrides those
+    /// restrictions, explicitly tracking the specified paths.
+    #[arg(long)]
+    include_ignored: bool,
 }
 
 #[instrument(skip_all)]
@@ -55,11 +63,15 @@ pub(crate) fn cmd_file_track(
     let matcher = workspace_command
         .parse_file_patterns(ui, &args.paths)?
         .to_matcher();
-    let options = workspace_command.snapshot_options_with_start_tracking_matcher(&matcher)?;
+
+    let mut options = workspace_command.snapshot_options_with_start_tracking_matcher(&matcher)?;
+    if args.include_ignored {
+        options.force_tracking_matcher = &matcher;
+    }
 
     let mut tx = workspace_command.start_transaction().into_inner();
     let (mut locked_ws, _wc_commit) = workspace_command.start_working_copy_mutation()?;
-    let (_tree_id, track_stats) = locked_ws.locked_wc().snapshot(&options).block_on()?;
+    let (_tree, track_stats) = locked_ws.locked_wc().snapshot(&options).block_on()?;
     let num_rebased = tx.repo_mut().rebase_descendants()?;
     if num_rebased > 0 {
         writeln!(ui.status(), "Rebased {num_rebased} descendant commits")?;
@@ -116,6 +128,8 @@ pub fn print_track_snapshot_stats(
                 This will increase the maximum file size allowed for new files, in this repository only.
               - Run `jj --config snapshot.max-new-file-size={size} file track {large_files_list}`
                 This will increase the maximum file size allowed for new files, for this command only.
+              - Run `jj file track --include-ignored {large_files_list}`
+                This will track the files even though they exceed the size limit.
             "
         )?;
     }
