@@ -18,15 +18,18 @@ use std::io::Write as _;
 use std::num::NonZeroU32;
 use std::path::Path;
 
+use itertools::Itertools as _;
 use jj_lib::git;
 use jj_lib::git::FetchTagsOverride;
 use jj_lib::git::GitFetch;
+use jj_lib::git::GitSettings;
 use jj_lib::git::expand_fetch_refspecs;
+use jj_lib::ref_name::RefName;
 use jj_lib::ref_name::RefNameBuf;
 use jj_lib::ref_name::RemoteName;
 use jj_lib::ref_name::RemoteNameBuf;
 use jj_lib::repo::Repo as _;
-use jj_lib::str_util::StringPattern;
+use jj_lib::str_util::StringExpression;
 use jj_lib::workspace::Workspace;
 
 use super::write_repository_level_trunk_alias;
@@ -41,6 +44,7 @@ use crate::commands::git::maybe_add_gitignore;
 use crate::git_util::absolute_git_url;
 use crate::git_util::print_git_import_stats;
 use crate::git_util::with_remote_git_callbacks;
+use crate::revset_util::parse_union_name_patterns;
 use crate::ui::Ui;
 
 /// Create a new repo backed by a clone of a Git repo
@@ -72,7 +76,7 @@ pub struct GitCloneArgs {
     /// [git.colocate config] is set to `false`.
     ///
     /// [git.colocate config]:
-    ///     https://jj-vcs.github.io/jj/latest/config/#default-colocation
+    ///     https://docs.jj-vcs.dev/latest/config/#default-colocation
     #[arg(long)]
     colocate: bool,
     /// Disable colocation of the Jujutsu repo with the git repo
@@ -85,7 +89,7 @@ pub struct GitCloneArgs {
     /// workspaces.
     ///
     /// [colocation docs]:
-    ///     https://jj-vcs.github.io/jj/latest/git-compatibility/#colocated-jujutsugit-repos
+    ///     https://docs.jj-vcs.dev/latest/git-compatibility/#colocated-jujutsugit-repos
     #[arg(long, conflicts_with = "colocate")]
     no_colocate: bool,
     /// Create a shallow clone of the given depth
@@ -106,14 +110,10 @@ pub struct GitCloneArgs {
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// expand `*` as a glob, e.g. `--branch 'glob:push-*'`. Other wildcard
     /// characters such as `?` are *not* supported. Can be repeated to specify
-    /// multiple branches, in which case the first match of the first `--branch`
-    /// argument is used as the working-copy parent.
-    #[arg(
-        long, short,
-        alias = "bookmark",
-        value_parser = StringPattern::parse,
-    )]
-    branch: Option<Vec<StringPattern>>,
+    /// multiple branches, in which case the first exact branch name is used as
+    /// the working-copy parent.
+    #[arg(long, short, alias = "bookmark")]
+    branch: Option<Vec<String>>,
 }
 
 fn clone_destination_for_source(source: &str) -> Option<&str> {
@@ -160,11 +160,15 @@ pub fn cmd_git_clone(
     fs::create_dir_all(&wc_path)
         .map_err(|err| user_error_with_message(format!("Failed to create {wc_path_str}"), err))?;
 
-    let colocate = if command.settings().git_settings()?.colocate {
+    let colocate = if command.settings().get_bool("git.colocate")? {
         !args.no_colocate
     } else {
         args.colocate
     };
+    let bookmark_expr = match &args.branch {
+        Some(texts) => parse_union_name_patterns(ui, texts)?,
+        None => StringExpression::all(),
+    };
 
     // Canonicalize because fs::remove_dir_all() doesn't seem to like e.g.
     // `/some/path/.`
@@ -182,13 +186,15 @@ pub fn cmd_git_clone(
             // If not explicitly specified on the CLI, configure the remote for only fetching
             // included tags for future fetches.
             args.fetch_tags.unwrap_or(FetchTagsMode::Included),
-            args.branch.as_deref(),
+            &bookmark_expr,
         )?;
         let default_branch = fetch_new_remote(
             ui,
             &mut workspace_command,
             remote_name,
-            args.branch.as_deref(),
+            // If we add default fetch patterns to jj's config, these patterns
+            // will be loaded here?
+            &bookmark_expr,
             args.depth,
             args.fetch_tags,
         )?;
@@ -273,7 +279,7 @@ fn configure_remote(
     remote_name: &RemoteName,
     source: &str,
     fetch_tags: FetchTagsMode,
-    branch_patterns: Option<&[StringPattern]>,
+    bookmark_expr: &StringExpression,
 ) -> Result<WorkspaceCommandHelper, CommandError> {
     let mut tx = workspace_command.start_transaction();
     git::add_remote(
@@ -281,7 +287,7 @@ fn configure_remote(
         remote_name,
         source,
         fetch_tags.as_fetch_tags(),
-        branch_patterns,
+        bookmark_expr,
     )?;
     tx.finish(ui, format!("add git remote {}", remote_name.as_symbol()))?;
     // Reload workspace to apply new remote configuration to
@@ -301,7 +307,7 @@ fn fetch_new_remote(
     ui: &Ui,
     workspace_command: &mut WorkspaceCommandHelper,
     remote_name: &RemoteName,
-    target_branches: Option<&[StringPattern]>,
+    bookmark_expr: &StringExpression,
     depth: Option<NonZeroU32>,
     fetch_tags: Option<FetchTagsMode>,
 ) -> Result<(Option<RefNameBuf>, bool), CommandError> {
@@ -311,16 +317,13 @@ fn fetch_new_remote(
         workspace_command.workspace_root().display()
     )?;
     let settings = workspace_command.settings();
-    let git_settings = settings.git_settings()?;
+    let git_settings = GitSettings::from_settings(settings)?;
     let should_track_default = settings.get_bool("git.track-default-bookmark-on-clone")?;
     let mut tx = workspace_command.start_transaction();
     let (default_branch, import_stats) = {
         let mut git_fetch = GitFetch::new(tx.repo_mut(), &git_settings)?;
 
-        let fetch_refspecs = expand_fetch_refspecs(
-            remote_name,
-            target_branches.unwrap_or(&[StringPattern::all()]).to_vec(),
-        )?;
+        let fetch_refspecs = expand_fetch_refspecs(remote_name, bookmark_expr.clone())?;
 
         with_remote_git_callbacks(ui, |cb| {
             git_fetch.fetch(
@@ -351,40 +354,49 @@ fn fetch_new_remote(
         (default_branch, import_stats)
     };
 
-    let working_branch = match target_branches {
-        Some(target_branches) => {
-            // Ensure that all targets match at least one branch, and record the matching
-            // branch for the first target
-            let mut first_match = None;
-            for pattern in target_branches {
-                let mut bookmarks = tx.repo().view().remote_bookmarks(remote_name);
-                let Some((bookmark, _)) =
-                    bookmarks.find(|(bookmark, _)| pattern.is_match(bookmark.as_str()))
-                else {
-                    return Err(user_error(format!(
-                        "No such branch matching pattern: {pattern}",
-                    )));
-                };
-                if first_match.is_none() {
-                    first_match = Some(bookmark.to_owned());
-                }
-            }
-            first_match
+    // Warn unmatched exact patterns, and record the first matching branch as
+    // the working branch. If there are no matching exact patterns, use the
+    // default branch of the remote.
+    let mut missing_branches = vec![];
+    let mut working_branch = None;
+    let bookmark_matcher = bookmark_expr.to_matcher();
+    let exact_bookmarks = bookmark_expr
+        .exact_strings()
+        .filter(|name| bookmark_matcher.is_match(name)) // exclude negative patterns
+        .map(RefName::new);
+    for name in exact_bookmarks {
+        let symbol = name.to_remote_symbol(remote_name);
+        if tx.repo().view().get_remote_bookmark(symbol).is_absent() {
+            missing_branches.push(name);
+        } else if working_branch.is_none() {
+            working_branch = Some(name);
         }
-        None => default_branch.clone(),
-    };
+    }
+    if working_branch.is_none() {
+        working_branch = default_branch.as_deref().filter(|name| {
+            let symbol = name.to_remote_symbol(remote_name);
+            tx.repo().view().get_remote_bookmark(symbol).is_present()
+        });
+    }
+    if !missing_branches.is_empty() {
+        writeln!(
+            ui.warning_default(),
+            "No matching branches found on remote: {}",
+            missing_branches
+                .iter()
+                .map(|name| name.as_symbol())
+                .join(", ")
+        )?;
+    }
 
-    let working_is_default = working_branch == default_branch;
-    if let Some(name) = &working_branch
+    let working_is_default = working_branch == default_branch.as_deref();
+    if let Some(name) = working_branch
         && working_is_default
         && should_track_default
     {
         // For convenience, create local bookmark as Git would do.
         let remote_symbol = name.to_remote_symbol(remote_name);
-        let remote_ref = tx.repo().get_remote_bookmark(remote_symbol);
-        if remote_ref.is_present() {
-            tx.repo_mut().track_remote_bookmark(remote_symbol)?;
-        }
+        tx.repo_mut().track_remote_bookmark(remote_symbol)?;
     }
     print_git_import_stats(ui, tx.repo(), &import_stats, true)?;
     if git_settings.auto_local_bookmark && !should_track_default {
@@ -395,5 +407,5 @@ fn fetch_new_remote(
         )?;
     }
     tx.finish(ui, "fetch from git remote into empty repo")?;
-    Ok((working_branch, working_is_default))
+    Ok((working_branch.map(ToOwned::to_owned), working_is_default))
 }
