@@ -12,33 +12,34 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::collections::HashSet;
+use std::io;
 
 use clap_complete::ArgValueCandidates;
 use itertools::Itertools as _;
 use jj_lib::config::ConfigGetResultExt as _;
 use jj_lib::git;
 use jj_lib::git::GitFetch;
+use jj_lib::git::GitSettings;
 use jj_lib::git::IgnoredRefspec;
 use jj_lib::git::IgnoredRefspecs;
 use jj_lib::git::expand_default_fetch_refspecs;
 use jj_lib::git::expand_fetch_refspecs;
 use jj_lib::git::get_git_backend;
+use jj_lib::ref_name::RefName;
 use jj_lib::ref_name::RemoteName;
 use jj_lib::repo::Repo as _;
-use jj_lib::str_util::StringMatcher;
-use jj_lib::str_util::StringPattern;
+use jj_lib::str_util::StringExpression;
 
 use crate::cli_util::CommandHelper;
 use crate::cli_util::WorkspaceCommandHelper;
 use crate::cli_util::WorkspaceCommandTransaction;
 use crate::command_error::CommandError;
-use crate::command_error::config_error;
 use crate::command_error::user_error;
 use crate::commands::git::get_single_remote;
 use crate::complete;
 use crate::git_util::print_git_import_stats;
 use crate::git_util::with_remote_git_callbacks;
+use crate::revset_util::parse_union_name_patterns;
 use crate::ui::Ui;
 
 /// Fetch from a Git remote
@@ -56,10 +57,9 @@ pub struct GitFetchArgs {
     #[arg(
         long, short,
         alias = "bookmark",
-        value_parser = StringPattern::parse,
         add = ArgValueCandidates::new(complete::bookmarks),
     )]
-    branch: Vec<StringPattern>,
+    branch: Option<Vec<String>>,
     /// Fetch only tracked bookmarks
     ///
     /// This fetches only bookmarks that are already tracked from the specified
@@ -77,14 +77,13 @@ pub struct GitFetchArgs {
     /// patterns.
     ///
     /// [string pattern]:
-    ///     https://jj-vcs.github.io/jj/latest/revsets#string-patterns
+    ///     https://docs.jj-vcs.dev/latest/revsets#string-patterns
     #[arg(
         long = "remote",
         value_name = "REMOTE",
-        value_parser = StringPattern::parse,
         add = ArgValueCandidates::new(complete::git_remotes),
     )]
-    remotes: Vec<StringPattern>,
+    remotes: Option<Vec<String>>,
     /// Fetch from all remotes
     #[arg(long, conflicts_with = "remotes")]
     all_remotes: bool,
@@ -97,68 +96,72 @@ pub fn cmd_git_fetch(
     args: &GitFetchArgs,
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
-    let remote_patterns = if args.all_remotes {
-        vec![StringPattern::all()]
-    } else if args.remotes.is_empty() {
-        get_default_fetch_remotes(ui, &workspace_command)?
+    let remote_expr = if args.all_remotes {
+        StringExpression::all()
+    } else if let Some(remotes) = &args.remotes {
+        parse_union_name_patterns(ui, remotes)?
     } else {
-        args.remotes.clone()
+        get_default_fetch_remotes(ui, &workspace_command)?
     };
+    let remote_matcher = remote_expr.to_matcher();
 
     let all_remotes = git::get_all_remote_names(workspace_command.repo().store())?;
-
-    let mut matching_remotes = HashSet::new();
-    for pattern in remote_patterns {
-        let remotes = all_remotes
-            .iter()
-            .filter(|r| pattern.is_match(r.as_str()))
-            .collect_vec();
-        if remotes.is_empty() {
-            writeln!(ui.warning_default(), "No git remotes matching '{pattern}'")?;
-        } else {
-            matching_remotes.extend(remotes);
-        }
+    let matching_remotes: Vec<&RemoteName> = all_remotes
+        .iter()
+        .filter(|r| remote_matcher.is_match(r.as_str()))
+        .map(AsRef::as_ref)
+        .collect();
+    let mut unmatched_remotes = remote_expr
+        .exact_strings()
+        .map(RemoteName::new)
+        // do linear search. all_remotes should be small.
+        .filter(|&name| all_remotes.iter().all(|r| r != name))
+        .peekable();
+    if unmatched_remotes.peek().is_some() {
+        writeln!(
+            ui.warning_default(),
+            "No matching remotes for names: {}",
+            unmatched_remotes.map(|name| name.as_symbol()).join(", ")
+        )?;
     }
-
     if matching_remotes.is_empty() {
         return Err(user_error("No git remotes to fetch from"));
     }
 
-    let remotes = matching_remotes
-        .iter()
-        .map(|r| r.as_ref())
-        .sorted()
-        .collect_vec();
-
     let mut tx = workspace_command.start_transaction();
 
-    let mut expansions = Vec::with_capacity(remotes.len());
+    let common_bookmark_expr = match &args.branch {
+        Some(texts) => Some(parse_union_name_patterns(ui, texts)?),
+        None => None,
+    };
+    let mut expansions = Vec::with_capacity(matching_remotes.len());
     if args.tracked {
-        for remote in &remotes {
-            let tracked_branches = tx
-                .repo()
-                .view()
-                .local_remote_bookmarks(remote)
-                .filter(|(_, targets)| targets.remote_ref.is_tracked())
-                .map(|(name, _)| StringPattern::exact(name))
-                .collect_vec();
-            expansions.push((remote, expand_fetch_refspecs(remote, tracked_branches)?));
+        for remote in &matching_remotes {
+            let bookmark_expr = StringExpression::union_all(
+                tx.repo()
+                    .view()
+                    .local_remote_bookmarks(remote)
+                    .filter(|(_, targets)| targets.remote_ref.is_tracked())
+                    .map(|(name, _)| StringExpression::exact(name))
+                    .collect(),
+            );
+            expansions.push((remote, expand_fetch_refspecs(remote, bookmark_expr)?));
         }
-    } else if args.branch.is_empty() {
+    } else if let Some(bookmark_expr) = &common_bookmark_expr {
+        for remote in &matching_remotes {
+            let expanded = expand_fetch_refspecs(remote, bookmark_expr.clone())?;
+            expansions.push((remote, expanded));
+        }
+    } else {
         let git_repo = get_git_backend(tx.repo_mut().store())?.git_repo();
-        for remote in &remotes {
+        for remote in &matching_remotes {
             let (ignored, expanded) = expand_default_fetch_refspecs(remote, &git_repo)?;
             warn_ignored_refspecs(ui, remote, ignored)?;
             expansions.push((remote, expanded));
         }
-    } else {
-        for remote in &remotes {
-            let expanded = expand_fetch_refspecs(remote, args.branch.clone())?;
-            expansions.push((remote, expanded));
-        }
     };
 
-    let git_settings = tx.settings().git_settings()?;
+    let git_settings = GitSettings::from_settings(tx.settings())?;
     let mut git_fetch = GitFetch::new(tx.repo_mut(), &git_settings)?;
 
     for (remote, expanded) in expansions {
@@ -169,12 +172,14 @@ pub fn cmd_git_fetch(
 
     let import_stats = git_fetch.import_refs()?;
     print_git_import_stats(ui, tx.repo(), &import_stats, true)?;
-    warn_if_branches_not_found(ui, &tx, &args.branch, &remotes)?;
+    if let Some(bookmark_expr) = &common_bookmark_expr {
+        warn_if_branches_not_found(ui, &tx, bookmark_expr, &matching_remotes)?;
+    }
     tx.finish(
         ui,
         format!(
             "fetch from git remote(s) {}",
-            remotes.iter().map(|n| n.as_symbol()).join(",")
+            matching_remotes.iter().map(|n| n.as_symbol()).join(",")
         ),
     )?;
     Ok(())
@@ -185,16 +190,13 @@ const DEFAULT_REMOTE: &RemoteName = RemoteName::new("origin");
 fn get_default_fetch_remotes(
     ui: &Ui,
     workspace_command: &WorkspaceCommandHelper,
-) -> Result<Vec<StringPattern>, CommandError> {
+) -> Result<StringExpression, CommandError> {
     const KEY: &str = "git.fetch";
     let settings = workspace_command.settings();
     if let Ok(remotes) = settings.get::<Vec<String>>(KEY) {
-        remotes
-            .into_iter()
-            .map(|r| parse_remote_pattern(&r))
-            .try_collect()
+        parse_union_name_patterns(ui, &remotes)
     } else if let Some(remote) = settings.get_string(KEY).optional()? {
-        Ok(vec![parse_remote_pattern(&remote)?])
+        parse_union_name_patterns(ui, [&remote])
     } else if let Some(remote) = get_single_remote(workspace_command.repo().store())? {
         // if nothing was explicitly configured, try to guess
         if remote != DEFAULT_REMOTE {
@@ -204,53 +206,41 @@ fn get_default_fetch_remotes(
                 remote = remote.as_symbol()
             )?;
         }
-        Ok(vec![StringPattern::exact(remote)])
+        Ok(StringExpression::exact(remote))
     } else {
-        Ok(vec![StringPattern::exact(DEFAULT_REMOTE)])
+        Ok(StringExpression::exact(DEFAULT_REMOTE))
     }
 }
 
-fn parse_remote_pattern(remote: &str) -> Result<StringPattern, CommandError> {
-    StringPattern::parse(remote).map_err(config_error)
-}
-
 fn warn_if_branches_not_found(
     ui: &mut Ui,
     tx: &WorkspaceCommandTransaction,
-    branches: &[StringPattern],
+    bookmark_expr: &StringExpression,
     remotes: &[&RemoteName],
-) -> Result<(), CommandError> {
-    let mut missing_branches = vec![];
-    for branch in branches {
-        let branch_matcher = branch.to_matcher();
-        let matches = remotes.iter().any(|&remote| {
-            let remote_matcher = StringMatcher::exact(remote);
-            tx.repo()
-                .view()
-                .remote_bookmarks_matching(&branch_matcher, &remote_matcher)
-                .next()
-                .is_some()
-                || tx
-                    .base_repo()
-                    .view()
-                    .remote_bookmarks_matching(&branch_matcher, &remote_matcher)
-                    .next()
-                    .is_some()
-        });
-        if !matches {
-            missing_branches.push(branch);
-        }
+) -> io::Result<()> {
+    let bookmark_matcher = bookmark_expr.to_matcher();
+    let mut missing_branches = bookmark_expr
+        .exact_strings()
+        .filter(|name| bookmark_matcher.is_match(name)) // exclude negative patterns
+        .map(RefName::new)
+        .filter(|name| {
+            remotes.iter().all(|&remote| {
+                let symbol = name.to_remote_symbol(remote);
+                let view = tx.repo().view();
+                let base_view = tx.base_repo().view();
+                view.get_remote_bookmark(symbol).is_absent()
+                    && base_view.get_remote_bookmark(symbol).is_absent()
+            })
+        })
+        .peekable();
+    if missing_branches.peek().is_none() {
+        return Ok(());
     }
-
-    if !missing_branches.is_empty() {
-        writeln!(
-            ui.warning_default(),
-            "No branch matching {} found on any specified/configured remote",
-            missing_branches.iter().map(|b| format!("`{b}`")).join(", ")
-        )?;
-    }
-
-    Ok(())
+    writeln!(
+        ui.warning_default(),
+        "No matching branches found on any specified/configured remote: {}",
+        missing_branches.map(|name| name.as_symbol()).join(", ")
+    )
 }
 
 fn warn_ignored_refspecs(
