@@ -23,6 +23,7 @@ use itertools::Itertools as _;
 use jj_lib::file_util;
 use jj_lib::git;
 use jj_lib::git::GitRefKind;
+use jj_lib::git::GitSettings;
 use jj_lib::git::parse_git_ref;
 use jj_lib::repo::ReadonlyRepo;
 use jj_lib::repo::Repo as _;
@@ -70,7 +71,7 @@ pub struct GitInitArgs {
     /// This option is mutually exclusive with `--git-repo`.
     ///
     /// [git.colocate config]:
-    ///     https://jj-vcs.github.io/jj/latest/config/#default-colocation
+    ///     https://docs.jj-vcs.dev/latest/config/#default-colocation
     #[arg(long, conflicts_with = "git_repo")]
     colocate: bool,
 
@@ -84,7 +85,7 @@ pub struct GitInitArgs {
     /// workspaces.
     ///
     /// [colocation docs]:
-    ///     https://jj-vcs.github.io/jj/latest/git-compatibility/#colocated-jujutsugit-repos
+    ///     https://docs.jj-vcs.dev/latest/git-compatibility/#colocated-jujutsugit-repos
     #[arg(long, conflicts_with = "colocate")]
     no_colocate: bool,
 
@@ -118,7 +119,7 @@ pub fn cmd_git_init(
         .and_then(|_| dunce::canonicalize(wc_path))
         .map_err(|e| user_error_with_message("Failed to create workspace", e))?;
 
-    let colocate = if command.settings().git_settings()?.colocate {
+    let colocate = if command.settings().get_bool("git.colocate")? {
         !args.no_colocate
     } else {
         args.colocate
@@ -233,7 +234,7 @@ fn init_git_refs(
     string_args: &[String],
     colocated: bool,
 ) -> Result<Arc<ReadonlyRepo>, CommandError> {
-    let mut git_settings = repo.settings().git_settings()?;
+    let mut git_settings = GitSettings::from_settings(repo.settings())?;
     let mut tx = start_repo_transaction(&repo, string_args);
     // There should be no old refs to abandon, but enforce it.
     git_settings.abandon_unreachable_commits = false;
@@ -243,8 +244,9 @@ fn init_git_refs(
         return Ok(repo);
     }
     if colocated {
-        // If git.auto-local-bookmark = true, local bookmarks could be created for
-        // the imported remote branches.
+        // If git.auto-local-bookmark = true or
+        // remotes.<name>.auto-track-bookmarks is set, local bookmarks could be
+        // created for the imported remote branches.
         let stats = git::export_refs(tx.repo_mut())?;
         print_git_export_stats(ui, &stats)?;
     }
