@@ -31,6 +31,7 @@ use jj_lib::config::ConfigGetResultExt as _;
 use jj_lib::git;
 use jj_lib::git::GitBranchPushTargets;
 use jj_lib::git::GitPushStats;
+use jj_lib::git::GitSettings;
 use jj_lib::index::IndexResult;
 use jj_lib::op_store::RefTarget;
 use jj_lib::operation::Operation;
@@ -48,7 +49,6 @@ use jj_lib::revset::RevsetExpression;
 use jj_lib::settings::UserSettings;
 use jj_lib::signing::SignBehavior;
 use jj_lib::str_util::StringExpression;
-use jj_lib::str_util::StringPattern;
 use jj_lib::view::View;
 
 use crate::cli_util::CommandHelper;
@@ -69,6 +69,7 @@ use crate::formatter::Formatter;
 use crate::formatter::FormatterExt as _;
 use crate::git_util::with_remote_git_callbacks;
 use crate::revset_util::parse_bookmark_name;
+use crate::revset_util::parse_union_name_patterns;
 use crate::ui::Ui;
 
 /// Push to a Git remote
@@ -88,10 +89,10 @@ use crate::ui::Ui;
 /// conflicts].
 ///
 /// [safety checks]:
-///     https://jj-vcs.github.io/jj/latest/bookmarks/#pushing-bookmarks-safety-checks
+///     https://docs.jj-vcs.dev/latest/bookmarks/#pushing-bookmarks-safety-checks
 ///
 /// [bookmark conflicts]:
-///     https://jj-vcs.github.io/jj/latest/bookmarks/#conflicts
+///     https://docs.jj-vcs.dev/latest/bookmarks/#conflicts
 
 #[derive(clap::Args, Clone, Debug)]
 #[command(group(ArgGroup::new("specific").args(&["bookmark", "change", "revisions", "named"]).multiple(true)))]
@@ -110,14 +111,13 @@ pub struct GitPushArgs {
     /// select bookmarks by [wildcard pattern].
     ///
     /// [wildcard pattern]:
-    ///     https://jj-vcs.github.io/jj/latest/revsets#string-patterns
+    ///     https://docs.jj-vcs.dev/latest/revsets#string-patterns
     #[arg(
         long, short,
         alias = "branch",
-        value_parser = StringPattern::parse,
         add = ArgValueCandidates::new(complete::local_bookmarks),
     )]
-    bookmark: Vec<StringPattern>,
+    bookmark: Vec<String>,
     /// Push all bookmarks (including new bookmarks)
     #[arg(long)]
     all: bool,
@@ -127,7 +127,7 @@ pub struct GitPushArgs {
     /// from the [relevant remote].
     ///
     /// [relevant remote]:
-    ///     https://jj-vcs.github.io/jj/latest/bookmarks#remotes-and-tracked-bookmarks
+    ///     https://docs.jj-vcs.dev/latest/bookmarks#remotes-and-tracked-bookmarks
     #[arg(long)]
     tracked: bool,
     /// Push all deleted bookmarks
@@ -137,13 +137,11 @@ pub struct GitPushArgs {
     /// correspond to missing local bookmarks.
     #[arg(long, conflicts_with = "specific")]
     deleted: bool,
+    // TODO: Delete in jj 0.42.0+
     /// Allow pushing new bookmarks
     ///
     /// Newly-created remote bookmarks will be tracked automatically.
-    ///
-    /// This can also be turned on by the `git.push-new-bookmarks` setting. If
-    /// it's set to `true`, `--allow-new` is no-op.
-    #[arg(long, short = 'N', conflicts_with = "what")]
+    #[arg(long, short = 'N', hide = true, conflicts_with = "what")]
     allow_new: bool,
     /// Allow pushing commits with empty descriptions
     #[arg(long)]
@@ -185,7 +183,7 @@ pub struct GitPushArgs {
     /// Specify a new bookmark name and a revision to push under that name, e.g.
     /// '--named myfeature=@'
     ///
-    /// Does not require --allow-new.
+    /// Automatically tracks the bookmark if it is new.
     #[arg(
         long,
         value_name = "NAME=REVISION",
@@ -220,6 +218,13 @@ pub fn cmd_git_push(
     command: &CommandHelper,
     args: &GitPushArgs,
 ) -> Result<(), CommandError> {
+    if args.allow_new {
+        writeln!(
+            ui.warning_default(),
+            "--allow-new is deprecated, track bookmarks manually or configure \
+             remotes.<name>.auto-track-bookmarks instead."
+        )?;
+    }
     let mut workspace_command = command.workspace_helper(ui)?;
 
     let default_remote;
@@ -336,8 +341,9 @@ pub fn cmd_git_push(
         }
 
         let view = tx.repo().view();
+        // TODO: Delete in jj 0.42.0+
         let allow_new = args.allow_new || tx.settings().get("git.push-new-bookmarks")?;
-        let bookmarks_by_name = find_bookmarks_to_push(view, &args.bookmark, remote)?;
+        let bookmarks_by_name = find_bookmarks_to_push(ui, view, &args.bookmark, remote)?;
         for &(name, targets) in &bookmarks_by_name {
             if !seen_bookmarks.insert(name) {
                 continue;
@@ -445,7 +451,7 @@ pub fn cmd_git_push(
     let targets = GitBranchPushTargets {
         branch_updates: bookmark_updates,
     };
-    let git_settings = tx.settings().git_settings()?;
+    let git_settings = GitSettings::from_settings(tx.settings())?;
     let push_stats = with_remote_git_callbacks(ui, |cb| {
         git::push_branches(tx.repo_mut(), &git_settings, remote, &targets, cb)
     })?;
@@ -794,11 +800,9 @@ fn classify_bookmark_update(
         BookmarkPushAction::Update(_) if !targets.remote_ref.is_tracked() && !allow_new => {
             Err(RejectedBookmarkUpdateReason {
                 message: format!("Refusing to create new remote bookmark {remote_symbol}"),
-                hint: Some(
-                    "Use --allow-new to push new bookmark. Use --remote to specify the remote to \
-                     push to."
-                        .to_owned(),
-                ),
+                hint: Some(format!(
+                    "Run `jj bookmark track {remote_symbol}` and try again."
+                )),
             })
         }
         BookmarkPushAction::Update(update) if update.new_target.is_none() && !allow_delete => {
@@ -823,10 +827,7 @@ fn ensure_new_bookmark_name(repo: &dyn Repo, name: &RefName) -> Result<(), Comma
     if repo.view().get_local_bookmark(name).is_present() {
         return Err(user_error_with_hint(
             format!("Bookmark already exists: {symbol}"),
-            format!(
-                "Use 'jj bookmark move' to move it, and 'jj git push -b {symbol} [--allow-new]' \
-                 to push it"
-            ),
+            format!("Use 'jj bookmark move' to move it, and 'jj git push -b {symbol}' to push it"),
         ));
     }
     if has_tracked_remote_bookmarks(repo, name) {
@@ -935,35 +936,38 @@ fn create_change_bookmarks(
 }
 
 fn find_bookmarks_to_push<'a>(
+    ui: &Ui,
     view: &'a View,
-    bookmark_patterns: &[StringPattern],
+    bookmark_patterns: &[String],
     remote: &RemoteName,
 ) -> Result<Vec<(&'a RefName, LocalAndRemoteRef<'a>)>, CommandError> {
-    let mut matching_bookmarks = vec![];
-    let mut unmatched_patterns = vec![];
-    for pattern in bookmark_patterns {
-        let matcher = pattern.to_matcher();
-        let mut matches = view
-            .local_remote_bookmarks_matching(&matcher, remote)
-            .filter(|(_, targets)| {
-                // If the remote exists but is not tracked, the absent local shouldn't
-                // be considered a deleted bookmark.
-                targets.local_target.is_present() || targets.remote_ref.is_tracked()
-            })
-            .peekable();
-        if matches.peek().is_none() {
-            unmatched_patterns.push(pattern);
-        }
-        matching_bookmarks.extend(matches);
-    }
-    match &unmatched_patterns[..] {
-        [] => Ok(matching_bookmarks),
-        [pattern] if pattern.is_exact() => Err(user_error(format!("No such bookmark: {pattern}"))),
-        patterns => Err(user_error(format!(
-            "No matching bookmarks for patterns: {}",
-            patterns.iter().join(", ")
-        ))),
+    let bookmark_expr = parse_union_name_patterns(ui, bookmark_patterns)?;
+    let bookmark_matcher = bookmark_expr.to_matcher();
+    let matching_bookmarks = view
+        .local_remote_bookmarks_matching(&bookmark_matcher, remote)
+        .filter(|(_, targets)| {
+            // If the remote exists but is not tracked, the absent local shouldn't
+            // be considered a deleted bookmark.
+            targets.local_target.is_present() || targets.remote_ref.is_tracked()
+        })
+        .collect();
+    let mut unmatched_names = bookmark_expr
+        .exact_strings()
+        .map(RefName::new)
+        .filter(|&name| {
+            let symbol = name.to_remote_symbol(remote);
+            view.get_local_bookmark(name).is_absent()
+                && !view.get_remote_bookmark(symbol).is_tracked()
+        })
+        .peekable();
+    if unmatched_names.peek().is_some() {
+        writeln!(
+            ui.warning_default(),
+            "No matching bookmarks for names: {}",
+            unmatched_names.map(|name| name.as_symbol()).join(", ")
+        )?;
     }
+    Ok(matching_bookmarks)
 }
 
 fn find_bookmarks_targeted_by_revisions<'a>(
