@@ -12,6 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use std::cmp::min;
+
 use clap_complete::ArgValueCandidates;
 use clap_complete::ArgValueCompleter;
 use itertools::Itertools as _;
@@ -50,7 +52,7 @@ use crate::ui::Ui;
 /// all revisions. See [`jj help -k revsets`] for information about the syntax.
 ///
 /// [`jj help -k revsets`]:
-///     https://jj-vcs.github.io/jj/latest/revsets/
+///     https://docs.jj-vcs.dev/latest/revsets/
 ///
 /// Spans of revisions that are not included in the graph per `--revisions` are
 /// rendered as a synthetic node labeled "(elided revisions)".
@@ -60,10 +62,10 @@ use crate::ui::Ui;
 /// All of these symbols can be [customized].
 ///
 /// [Immutable revisions]:
-///     https://jj-vcs.github.io/jj/latest/config/#set-of-immutable-commits
+///     https://docs.jj-vcs.dev/latest/config/#set-of-immutable-commits
 ///
 /// [customized]:
-///     https://jj-vcs.github.io/jj/latest/config/#node-style
+///     https://docs.jj-vcs.dev/latest/config/#node-style
 #[derive(clap::Args, Clone, Debug)]
 pub(crate) struct LogArgs {
     /// Which revisions to show
@@ -107,15 +109,18 @@ pub(crate) struct LogArgs {
     /// If not specified, this defaults to the `templates.log` setting.
     ///
     /// [built-in keywords]:
-    ///     https://jj-vcs.github.io/jj/latest/templates/#commit-keywords
+    ///     https://docs.jj-vcs.dev/latest/templates/#commit-keywords
     ///
     /// [`jj help -k templates`]:
-    ///     https://jj-vcs.github.io/jj/latest/templates/
+    ///     https://docs.jj-vcs.dev/latest/templates/
     #[arg(long, short = 'T', add = ArgValueCandidates::new(complete::template_aliases))]
     template: Option<String>,
     /// Show patch
     #[arg(long, short = 'p')]
     patch: bool,
+    /// Print the number of commits instead of showing them
+    #[arg(long, conflicts_with_all = ["DiffFormatArgs", "no_graph", "patch", "reversed", "template"])]
+    count: bool,
     #[command(flatten)]
     diff_format: DiffFormatArgs,
 }
@@ -130,6 +135,7 @@ pub(crate) fn cmd_log(
     let settings = workspace_command.settings();
 
     let fileset_expression = workspace_command.parse_file_patterns(ui, &args.paths)?;
+    let mut explicit_paths = fileset_expression.explicit_paths().collect_vec();
     let revset_expression = {
         // only use default revset if neither revset nor path are specified
         let mut expression = if args.revisions.is_empty() && args.paths.is_empty() {
@@ -149,12 +155,32 @@ pub(crate) fn cmd_log(
         }
         expression
     };
+
+    let revset = revset_expression.evaluate()?;
+
+    if args.count {
+        let (lower, upper) = revset.count_estimate()?;
+        let limit = args.limit.unwrap_or(usize::MAX);
+        let count = if limit <= lower {
+            limit
+        } else if upper == Some(lower) {
+            min(lower, limit)
+        } else {
+            revset
+                .iter()
+                .take(limit)
+                .process_results(|iter| iter.count())?
+        };
+        let mut formatter = ui.stdout_formatter();
+        writeln!(formatter, "{count}")?;
+        return Ok(());
+    }
+
     let prio_revset = settings.get_string("revsets.log-graph-prioritize")?;
     let prio_revset = workspace_command.parse_revset(ui, &RevisionArg::from(prio_revset))?;
 
     let repo = workspace_command.repo();
     let matcher = fileset_expression.to_matcher();
-    let revset = revset_expression.evaluate()?;
 
     let store = repo.store();
     let diff_renderer = workspace_command.diff_renderer_for_log(&args.diff_format, args.patch)?;
@@ -263,13 +289,19 @@ pub(crate) fn cmd_log(
                         .block_on()?;
                 }
 
-                let node_symbol = format_template(ui, &Some(commit), &node_template);
+                let commit = Some(commit);
+                let node_symbol = format_template(ui, &commit, &node_template);
                 graph.add_node(
                     &key,
                     &graphlog_edges,
                     &node_symbol,
                     &String::from_utf8_lossy(&buffer),
                 )?;
+
+                let tree = commit.map(|c| c.tree()).unwrap();
+                // TODO: propagate errors
+                explicit_paths.retain(|&path| tree.path_value(path).unwrap().is_absent());
+
                 for elided_target in elided_targets {
                     let elided_key = (elided_target, true);
                     let real_key = (elided_key.0.clone(), false);
@@ -309,8 +341,23 @@ pub(crate) fn cmd_log(
                         .show_patch(ui, formatter, &commit, matcher.as_ref(), width)
                         .block_on()?;
                 }
+
+                let tree = commit.tree();
+                // TODO: propagate errors
+                explicit_paths.retain(|&path| tree.path_value(path).unwrap().is_absent());
             }
         }
+
+        if !explicit_paths.is_empty() {
+            let ui_paths = explicit_paths
+                .iter()
+                .map(|&path| workspace_command.format_file_path(path))
+                .join(", ");
+            writeln!(
+                ui.warning_default(),
+                "No matching entries for paths: {ui_paths}"
+            )?;
+        }
     }
 
     // Check to see if the user might have specified a path when they intended
