@@ -14,9 +14,9 @@
 
 use crate::cli_util::CommandHelper;
 use crate::command_error::CommandError;
-use crate::movement_util::move_to_commit;
 use crate::movement_util::Direction;
 use crate::movement_util::MovementArgs;
+use crate::movement_util::move_to_commit;
 use crate::ui::Ui;
 /// Change the working copy revision relative to the parent revision
 ///
@@ -46,8 +46,6 @@ use crate::ui::Ui;
 /// |        |
 /// A        A
 /// ```
-/// If the working copy revision already has visible children, then `--edit` is
-/// implied
 #[derive(clap::Args, Clone, Debug)]
 #[command(verbatim_doc_comment)]
 pub(crate) struct PrevArgs {
@@ -73,7 +71,7 @@ pub(crate) struct PrevArgs {
 
 impl From<&PrevArgs> for MovementArgs {
     fn from(val: &PrevArgs) -> Self {
-        MovementArgs {
+        Self {
             offset: val.offset,
             edit: val.edit,
             no_edit: val.no_edit,
