@@ -21,6 +21,7 @@ use tracing::instrument;
 use crate::cli_util::CommandHelper;
 use crate::cli_util::RevisionArg;
 use crate::cli_util::print_conflicted_paths;
+use crate::cli_util::print_unmatched_explicit_paths;
 use crate::command_error::CommandError;
 use crate::command_error::cli_error;
 use crate::complete;
@@ -87,15 +88,17 @@ pub(crate) fn cmd_resolve(
     args: &ResolveArgs,
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
-    let matcher = workspace_command
-        .parse_file_patterns(ui, &args.paths)?
-        .to_matcher();
+    let fileset_expression = workspace_command.parse_file_patterns(ui, &args.paths)?;
+    let matcher = fileset_expression.to_matcher();
     let commit = workspace_command.resolve_single_rev(ui, &args.revision)?;
-    let tree = commit.tree()?;
+    let tree = commit.tree();
     let conflicts = tree
         .conflicts()
         .filter(|path| matcher.matches(&path.0))
         .collect_vec();
+
+    print_unmatched_explicit_paths(ui, &workspace_command, &fileset_expression, [&tree])?;
+
     if conflicts.is_empty() {
         return Err(cli_error(if args.paths.is_empty() {
             "No conflicts found at this revision"
@@ -118,12 +121,11 @@ pub(crate) fn cmd_resolve(
     workspace_command.check_rewritable([commit.id()])?;
     let merge_editor = workspace_command.merge_editor(ui, args.tool.as_deref())?;
     let mut tx = workspace_command.start_transaction();
-    let (new_tree_id, partial_resolution_error) =
-        merge_editor.edit_files(ui, &tree, &repo_paths)?;
+    let (new_tree, partial_resolution_error) = merge_editor.edit_files(ui, &tree, &repo_paths)?;
     let new_commit = tx
         .repo_mut()
         .rewrite_commit(&commit)
-        .set_tree_id(new_tree_id)
+        .set_tree(new_tree)
         .write()?;
     tx.finish(
         ui,
@@ -137,7 +139,7 @@ pub(crate) fn cmd_resolve(
         && let Some(mut formatter) = ui.status_formatter()
         && new_commit.has_conflict()
     {
-        let new_tree = new_commit.tree()?;
+        let new_tree = new_commit.tree();
         let new_conflicts = new_tree.conflicts().collect_vec();
         writeln!(
             formatter.labeled("warning").with_heading("Warning: "),
