@@ -14,6 +14,7 @@
 
 use itertools::Itertools as _;
 use jj_lib::copies::CopyRecords;
+use jj_lib::merge::Diff;
 use jj_lib::merged_tree::MergedTree;
 use jj_lib::repo::Repo as _;
 use jj_lib::repo_path::RepoPath;
@@ -26,6 +27,7 @@ use tracing::instrument;
 use crate::cli_util::CommandHelper;
 use crate::cli_util::print_conflicted_paths;
 use crate::cli_util::print_snapshot_stats;
+use crate::cli_util::print_unmatched_explicit_paths;
 use crate::command_error::CommandError;
 use crate::diff_util::DiffFormat;
 use crate::diff_util::get_copy_records;
@@ -44,7 +46,7 @@ use crate::ui::Ui;
 /// * [Conflicted bookmarks]
 ///
 /// [Conflicted bookmarks]:
-///     https://jj-vcs.github.io/jj/latest/bookmarks/#conflicts
+///     https://docs.jj-vcs.dev/latest/bookmarks/#conflicts
 #[derive(clap::Args, Clone, Debug)]
 pub(crate) struct StatusArgs {
     /// Restrict the status display to these paths
@@ -69,18 +71,19 @@ pub(crate) fn cmd_status(
         .get_wc_commit_id()
         .map(|id| repo.store().get_commit(id))
         .transpose()?;
-    let matcher = workspace_command
-        .parse_file_patterns(ui, &args.paths)?
-        .to_matcher();
+    let fileset_expression = workspace_command.parse_file_patterns(ui, &args.paths)?;
+    let matcher = fileset_expression.to_matcher();
     ui.request_pager();
     let mut formatter = ui.stdout_formatter();
     let formatter = formatter.as_mut();
 
     if let Some(wc_commit) = &maybe_wc_commit {
         let parent_tree = wc_commit.parent_tree(repo.as_ref())?;
-        let tree = wc_commit.tree()?;
+        let tree = wc_commit.tree();
 
-        let wc_has_changes = tree.id() != parent_tree.id();
+        print_unmatched_explicit_paths(ui, &workspace_command, &fileset_expression, [&tree])?;
+
+        let wc_has_changes = tree.tree_ids() != parent_tree.tree_ids();
         let wc_has_untracked = !snapshot_stats.untracked_paths.is_empty();
         if !wc_has_changes && !wc_has_untracked {
             writeln!(formatter, "The working copy has no changes.")?;
@@ -98,7 +101,7 @@ pub(crate) fn cmd_status(
                     .show_diff(
                         ui,
                         formatter,
-                        [&parent_tree, &tree],
+                        Diff::new(&parent_tree, &tree),
                         &matcher,
                         &copy_records,
                         width,
@@ -110,7 +113,7 @@ pub(crate) fn cmd_status(
                 writeln!(formatter, "Untracked paths:")?;
                 visit_collapsed_untracked_files(
                     snapshot_stats.untracked_paths.keys(),
-                    tree,
+                    tree.clone(),
                     |path, is_dir| {
                         let ui_path = workspace_command.path_converter().format_file_path(path);
                         writeln!(
@@ -144,7 +147,7 @@ pub(crate) fn cmd_status(
         if wc_commit.has_conflict() {
             // TODO: Conflicts should also be filtered by the `matcher`. See the related
             // TODO on `MergedTree::conflicts()`.
-            let conflicts = wc_commit.tree()?.conflicts().collect_vec();
+            let conflicts = wc_commit.tree().conflicts().collect_vec();
             writeln!(
                 formatter.labeled("warning").with_heading("Warning: "),
                 "There are unresolved conflicts at these paths:"
@@ -234,7 +237,8 @@ async fn visit_collapsed_untracked_files(
     tree: MergedTree,
     mut on_path: impl FnMut(&RepoPath, bool) -> Result<(), CommandError>,
 ) -> Result<(), CommandError> {
-    let mut stack = vec![tree];
+    let trees = tree.trees()?;
+    let mut stack = vec![trees];
 
     // TODO: This loop can be improved with BTreeMap cursors once that's stable,
     // would remove the need for the whole `skip_prefixed_by` thing and turn it
