@@ -30,7 +30,7 @@ Apply it by running one of these:
 - Fish: `jj util completion fish | source`
 - Nushell:
      ```nu
-     jj util completion nushell | save "completions-jj.nu"
+     jj util completion nushell | save -f "completions-jj.nu"
      use "completions-jj.nu" *  # Or `source "completions-jj.nu"`
      ```
 - Zsh:
@@ -43,7 +43,7 @@ Apply it by running one of these:
 See the docs on [command-line completion] for more details.
 
 [command-line completion]:
-    https://jj-vcs.github.io/jj/latest/install-and-setup/#command-line-completion
+    https://docs.jj-vcs.dev/latest/install-and-setup/#command-line-completion
 "#]
 #[derive(clap::Args, Clone, Debug)]
 #[command(verbatim_doc_comment)]
