@@ -12,19 +12,19 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use jj_lib::op_store::WorkspaceId;
+use jj_lib::ref_name::WorkspaceNameBuf;
 use tracing::instrument;
 
 use crate::cli_util::CommandHelper;
-use crate::command_error::user_error;
 use crate::command_error::CommandError;
+use crate::command_error::user_error;
 use crate::ui::Ui;
 
 /// Renames the current workspace
 #[derive(clap::Args, Clone, Debug)]
 pub struct WorkspaceRenameArgs {
     /// The name of the workspace to update to.
-    new_workspace_name: String,
+    new_workspace_name: WorkspaceNameBuf,
 }
 
 #[instrument(skip_all)]
@@ -33,15 +33,15 @@ pub fn cmd_workspace_rename(
     command: &CommandHelper,
     args: &WorkspaceRenameArgs,
 ) -> Result<(), CommandError> {
-    if args.new_workspace_name.is_empty() {
+    if args.new_workspace_name.as_str().is_empty() {
         return Err(user_error("New workspace name cannot be empty"));
     }
 
     let mut workspace_command = command.workspace_helper(ui)?;
 
-    let old_workspace_id = workspace_command.working_copy().workspace_id().clone();
-    let new_workspace_id = WorkspaceId::new(args.new_workspace_name.clone());
-    if new_workspace_id == old_workspace_id {
+    let old_name = workspace_command.working_copy().workspace_name().to_owned();
+    let new_name = &*args.new_workspace_name;
+    if new_name == old_name {
         writeln!(ui.status(), "Nothing changed.")?;
         return Ok(());
     }
@@ -49,28 +49,26 @@ pub fn cmd_workspace_rename(
     if workspace_command
         .repo()
         .view()
-        .get_wc_commit_id(&old_workspace_id)
+        .get_wc_commit_id(&old_name)
         .is_none()
     {
         return Err(user_error(format!(
             "The current workspace '{}' is not tracked in the repo.",
-            old_workspace_id.as_str()
+            old_name.as_symbol()
         )));
     }
 
     let mut tx = workspace_command.start_transaction().into_inner();
     let (mut locked_ws, _wc_commit) = workspace_command.start_working_copy_mutation()?;
 
-    locked_ws
-        .locked_wc()
-        .rename_workspace(new_workspace_id.clone());
+    locked_ws.locked_wc().rename_workspace(new_name.to_owned());
 
     tx.repo_mut()
-        .rename_workspace(&old_workspace_id, new_workspace_id)?;
+        .rename_workspace(&old_name, new_name.to_owned())?;
     let repo = tx.commit(format!(
-        "Renamed workspace '{}' to '{}'",
-        old_workspace_id.as_str(),
-        args.new_workspace_name
+        "Renamed workspace '{old}' to '{new}'",
+        old = old_name.as_symbol(),
+        new = new_name.as_symbol()
     ))?;
     locked_ws.finish(repo.op_id().clone())?;
 
