@@ -22,6 +22,8 @@ use clap_complete::CompletionCandidate;
 use indoc::indoc;
 use itertools::Itertools as _;
 use jj_lib::config::ConfigNamePathBuf;
+use jj_lib::file_util::normalize_path;
+use jj_lib::file_util::slash_path;
 use jj_lib::settings::UserSettings;
 use jj_lib::workspace::DefaultWorkspaceLoaderFactory;
 use jj_lib::workspace::WorkspaceLoaderFactory as _;
@@ -509,22 +511,34 @@ pub fn operations() -> Vec<CompletionCandidate> {
 }
 
 pub fn workspaces() -> Vec<CompletionCandidate> {
+    let template = indoc! {r#"
+        name ++ "\t" ++ if(
+            target.description(),
+            target.description().first_line(),
+            "(no description set)"
+        ) ++ "\n"
+    "#};
     with_jj(|jj, _| {
         let output = jj
             .build()
             .arg("workspace")
             .arg("list")
-            .arg("-T")
-            .arg(r#"name ++ "\t" ++ if(target.description(), target.description().first_line(), "(no description set)") ++ "\n""#)
+            .arg("--template")
+            .arg(template)
             .output()
             .map_err(user_error)?;
         let stdout = String::from_utf8_lossy(&output.stdout);
 
         Ok(stdout
             .lines()
-            .map(|line| {
-                let (name, desc) = line.split_once(": ").unwrap_or((line, ""));
-                CompletionCandidate::new(name).help(Some(desc.to_string().into()))
+            .filter_map(|line| {
+                let res = line.split_once("\t").map(|(name, desc)| {
+                    CompletionCandidate::new(name).help(Some(desc.to_string().into()))
+                });
+                if res.is_none() {
+                    eprintln!("Error parsing line {line}");
+                }
+                res
             })
             .collect())
     })
@@ -739,10 +753,44 @@ pub fn branch_name_equals_any_revision(current: &std::ffi::OsStr) -> Vec<Complet
         .collect()
 }
 
-fn dir_prefix_from<'a>(path: &'a str, current: &str) -> Option<&'a str> {
-    path[current.len()..]
-        .split_once(std::path::MAIN_SEPARATOR)
-        .map(|(next, _)| path.split_at(current.len() + next.len() + 1).0)
+fn path_completion_candidate_from(
+    current_prefix: &str,
+    normalized_prefix_path: &Path,
+    path: &Path,
+    mode: Option<clap::builder::StyledStr>,
+) -> Option<CompletionCandidate> {
+    let normalized_prefix = match normalized_prefix_path.to_str()? {
+        "." => "", // `.` cannot be normalized further, but doesn't prefix `path`.
+        normalized_prefix => normalized_prefix,
+    };
+
+    let path = slash_path(path);
+    let mut remainder = path.to_str()?.strip_prefix(normalized_prefix)?;
+
+    // Trailing slash might have been normalized away in which case we need to strip
+    // the leading slash in the remainder away, or else the slash would appear
+    // twice.
+    if current_prefix.ends_with(std::path::is_separator) {
+        remainder = remainder.strip_prefix('/').unwrap_or(remainder);
+    }
+
+    match remainder.split_inclusive('/').at_most_one() {
+        // Completed component is the final component in `path`, so we're completing the file to
+        // which `mode` refers.
+        Ok(file_completion) => Some(
+            CompletionCandidate::new(format!(
+                "{current_prefix}{}",
+                file_completion.unwrap_or_default()
+            ))
+            .help(mode),
+        ),
+
+        // Omit `mode` when completing only up to the next directory.
+        Err(mut components) => Some(CompletionCandidate::new(format!(
+            "{current_prefix}{}",
+            components.next().unwrap()
+        ))),
+    }
 }
 
 fn current_prefix_to_fileset(current: &str) -> String {
@@ -756,6 +804,10 @@ fn all_files_from_rev(rev: String, current: &std::ffi::OsStr) -> Vec<CompletionC
     let Some(current) = current.to_str() else {
         return Vec::new();
     };
+
+    let normalized_prefix = normalize_path(Path::new(current));
+    let normalized_prefix = slash_path(&normalized_prefix);
+
     with_jj(|jj, _| {
         let mut child = jj
             .build()
@@ -763,6 +815,8 @@ fn all_files_from_rev(rev: String, current: &std::ffi::OsStr) -> Vec<CompletionC
             .arg("list")
             .arg("--revision")
             .arg(rev)
+            .arg("--template")
+            .arg(r#"path.display() ++ "\n""#)
             .arg(current_prefix_to_fileset(current))
             .stdout(std::process::Stdio::piped())
             .stderr(std::process::Stdio::null())
@@ -774,11 +828,8 @@ fn all_files_from_rev(rev: String, current: &std::ffi::OsStr) -> Vec<CompletionC
             .lines()
             .take(1_000)
             .map_while(Result::ok)
-            .map(|path| {
-                if let Some(dir_path) = dir_prefix_from(&path, current) {
-                    return CompletionCandidate::new(dir_path);
-                }
-                CompletionCandidate::new(path)
+            .filter_map(|path| {
+                path_completion_candidate_from(current, &normalized_prefix, Path::new(&path), None)
             })
             .dedup() // directories may occur multiple times
             .collect())
@@ -794,6 +845,9 @@ fn modified_files_from_rev_with_jj_cmd(
         return Ok(Vec::new());
     };
 
+    let normalized_prefix = normalize_path(Path::new(current));
+    let normalized_prefix = slash_path(&normalized_prefix);
+
     // In case of a rename, one entry of `diff` results in two suggestions.
     let template = indoc! {r#"
         concat(
@@ -811,36 +865,27 @@ fn modified_files_from_rev_with_jj_cmd(
     let output = cmd.output().map_err(user_error)?;
     let stdout = String::from_utf8_lossy(&output.stdout);
 
-    let mut candidates = Vec::new();
     let mut include_renames = false;
-
-    for (mode, path) in stdout.lines().filter_map(|line| line.split_once(' ')) {
-        fn path_to_candidate(current: &str, mode: &str, path: &str) -> CompletionCandidate {
-            if let Some(dir_path) = dir_prefix_from(path, current) {
-                return CompletionCandidate::new(dir_path);
-            }
-
-            let help = match mode {
+    let mut candidates: Vec<_> = stdout
+        .lines()
+        .filter_map(|line| line.split_once(' '))
+        .filter_map(|(mode, path)| {
+            let mode = match mode {
                 "modified" => "Modified".into(),
                 "removed" => "Deleted".into(),
                 "added" => "Added".into(),
                 "renamed" => "Renamed".into(),
+                "renamed.source" => {
+                    include_renames = true;
+                    "Renamed".into()
+                }
                 "copied" => "Copied".into(),
-                _ => format!("unknown mode: '{mode}'"),
+                _ => format!("unknown mode: '{mode}'").into(),
             };
-            CompletionCandidate::new(path).help(Some(help.into()))
-        }
+            path_completion_candidate_from(current, &normalized_prefix, Path::new(path), Some(mode))
+        })
+        .collect();
 
-        if mode == "renamed.source" {
-            if !path.starts_with(current) {
-                continue;
-            }
-            candidates.push(path_to_candidate(current, "renamed", path));
-            include_renames |= true;
-        } else {
-            candidates.push(path_to_candidate(current, mode, path));
-        }
-    }
     if include_renames {
         candidates.sort_unstable_by(|a, b| Path::new(a.get_value()).cmp(Path::new(b.get_value())));
     }
@@ -860,6 +905,10 @@ fn conflicted_files_from_rev(rev: &str, current: &std::ffi::OsStr) -> Vec<Comple
     let Some(current) = current.to_str() else {
         return Vec::new();
     };
+
+    let normalized_prefix = normalize_path(Path::new(current));
+    let normalized_prefix = slash_path(&normalized_prefix);
+
     with_jj(|jj, _| {
         let output = jj
             .build()
@@ -874,16 +923,13 @@ fn conflicted_files_from_rev(rev: &str, current: &std::ffi::OsStr) -> Vec<Comple
 
         Ok(stdout
             .lines()
-            .map(|line| {
+            .filter_map(|line| {
                 let path = line
                     .split_whitespace()
                     .next()
                     .expect("resolve --list should contain whitespace after path");
 
-                if let Some(dir_path) = dir_prefix_from(path, current) {
-                    return CompletionCandidate::new(dir_path);
-                }
-                CompletionCandidate::new(path)
+                path_completion_candidate_from(current, &normalized_prefix, Path::new(path), None)
             })
             .dedup() // directories may occur multiple times
             .collect())
@@ -1013,7 +1059,7 @@ fn get_jj_command() -> Result<(JjBuilder, UserSettings), CommandError> {
         .and_then(dunce::canonicalize)
         .map_err(user_error)?;
     // No config migration for completion. Simply ignore deprecated variables.
-    let mut config_env = ConfigEnv::from_environment(&ui);
+    let mut config_env = ConfigEnv::from_environment();
     let maybe_cwd_workspace_loader = DefaultWorkspaceLoaderFactory.create(find_workspace_dir(&cwd));
     let _ = config_env.reload_user_config(&mut raw_config);
     if let Ok(loader) = &maybe_cwd_workspace_loader {
