@@ -12,13 +12,14 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::borrow::BorrowMut;
 use std::collections::HashMap;
 use std::fmt;
 use std::io;
 use std::io::Error;
 use std::io::Write;
 use std::mem;
+use std::ops::Deref;
+use std::ops::DerefMut;
 use std::ops::Range;
 use std::sync::Arc;
 
@@ -41,111 +42,121 @@ pub trait Formatter: Write {
     /// already formatted, such as in the graphical log.
     fn raw(&mut self) -> io::Result<Box<dyn Write + '_>>;
 
-    fn push_label(&mut self, label: &str) -> io::Result<()>;
+    fn push_label(&mut self, label: &str);
 
-    fn pop_label(&mut self) -> io::Result<()>;
+    fn pop_label(&mut self);
 }
 
-impl dyn Formatter + '_ {
-    pub fn labeled<S: AsRef<str>>(&mut self, label: S) -> LabeledWriter<&mut Self, S> {
-        LabeledWriter {
-            formatter: self,
-            label,
-        }
+impl<T: Formatter + ?Sized> Formatter for &mut T {
+    fn raw(&mut self) -> io::Result<Box<dyn Write + '_>> {
+        <T as Formatter>::raw(self)
     }
 
-    pub fn with_label<E: From<io::Error>>(
-        &mut self,
-        label: &str,
-        write_inner: impl FnOnce(&mut dyn Formatter) -> Result<(), E>,
-    ) -> Result<(), E> {
-        self.push_label(label)?;
-        // Call `pop_label()` whether or not `write_inner()` fails, but don't let
-        // its error replace the one from `write_inner()`.
-        write_inner(self).and(self.pop_label().map_err(Into::into))
+    fn push_label(&mut self, label: &str) {
+        <T as Formatter>::push_label(self, label);
     }
 
-    pub async fn with_label_async<E: From<io::Error>>(
-        &mut self,
-        label: &str,
-        write_inner: impl AsyncFnOnce(&mut dyn Formatter) -> Result<(), E>,
-    ) -> Result<(), E> {
-        self.push_label(label)?;
-        // Call `pop_label()` whether or not `write_inner()` fails, but don't let
-        // its error replace the one from `write_inner()`.
-        let result = write_inner(self).await;
-        result.and(self.pop_label().map_err(Into::into))
+    fn pop_label(&mut self) {
+        <T as Formatter>::pop_label(self);
     }
 }
 
-/// `Formatter` wrapper to write a labeled message with `write!()` or
-/// `writeln!()`.
-pub struct LabeledWriter<T, S> {
+impl<T: Formatter + ?Sized> Formatter for Box<T> {
+    fn raw(&mut self) -> io::Result<Box<dyn Write + '_>> {
+        <T as Formatter>::raw(self)
+    }
+
+    fn push_label(&mut self, label: &str) {
+        <T as Formatter>::push_label(self, label);
+    }
+
+    fn pop_label(&mut self) {
+        <T as Formatter>::pop_label(self);
+    }
+}
+
+/// [`Formatter`] adapters.
+pub trait FormatterExt: Formatter {
+    fn labeled(&mut self, label: &str) -> LabeledScope<&mut Self> {
+        LabeledScope::new(self, label)
+    }
+
+    fn into_labeled(self, label: &str) -> LabeledScope<Self>
+    where
+        Self: Sized,
+    {
+        LabeledScope::new(self, label)
+    }
+}
+
+impl<T: Formatter + ?Sized> FormatterExt for T {}
+
+/// [`Formatter`] wrapper to apply a label within a lexical scope.
+#[must_use]
+pub struct LabeledScope<T: Formatter> {
     formatter: T,
-    label: S,
 }
 
-impl<T, S> LabeledWriter<T, S> {
-    pub fn new(formatter: T, label: S) -> Self {
-        Self { formatter, label }
+impl<T: Formatter> LabeledScope<T> {
+    pub fn new(mut formatter: T, label: &str) -> Self {
+        formatter.push_label(label);
+        Self { formatter }
     }
 
+    // TODO: move to FormatterExt?
     /// Turns into writer that prints labeled message with the `heading`.
-    pub fn with_heading<H>(self, heading: H) -> HeadingLabeledWriter<T, S, H> {
+    pub fn with_heading<H>(self, heading: H) -> HeadingLabeledWriter<T, H> {
         HeadingLabeledWriter::new(self, heading)
     }
 }
 
-impl<'a, T, S> LabeledWriter<T, S>
-where
-    T: BorrowMut<dyn Formatter + 'a>,
-    S: AsRef<str>,
-{
-    pub fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> io::Result<()> {
-        self.with_labeled(|formatter| formatter.write_fmt(args))
-    }
-
-    fn with_labeled(
-        &mut self,
-        write_inner: impl FnOnce(&mut dyn Formatter) -> io::Result<()>,
-    ) -> io::Result<()> {
-        self.formatter
-            .borrow_mut()
-            .with_label(self.label.as_ref(), write_inner)
+impl<T: Formatter> Drop for LabeledScope<T> {
+    fn drop(&mut self) {
+        self.formatter.pop_label();
     }
 }
 
-/// Like `LabeledWriter`, but also prints the `heading` once.
+impl<T: Formatter> Deref for LabeledScope<T> {
+    type Target = T;
+
+    fn deref(&self) -> &Self::Target {
+        &self.formatter
+    }
+}
+
+impl<T: Formatter> DerefMut for LabeledScope<T> {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        &mut self.formatter
+    }
+}
+
+// There's no `impl Formatter for LabeledScope<T>` so nested .labeled() calls
+// wouldn't construct `LabeledScope<LabeledScope<T>>`.
+
+/// [`Formatter`] wrapper that prints the `heading` once.
 ///
 /// The `heading` will be printed within the first `write!()` or `writeln!()`
 /// invocation, which is handy because `io::Error` can be handled there.
-pub struct HeadingLabeledWriter<T, S, H> {
-    writer: LabeledWriter<T, S>,
+pub struct HeadingLabeledWriter<T: Formatter, H> {
+    formatter: LabeledScope<T>,
     heading: Option<H>,
 }
 
-impl<T, S, H> HeadingLabeledWriter<T, S, H> {
-    pub fn new(writer: LabeledWriter<T, S>, heading: H) -> Self {
+impl<T: Formatter, H> HeadingLabeledWriter<T, H> {
+    pub fn new(formatter: LabeledScope<T>, heading: H) -> Self {
         Self {
-            writer,
+            formatter,
             heading: Some(heading),
         }
     }
 }
 
-impl<'a, T, S, H> HeadingLabeledWriter<T, S, H>
-where
-    T: BorrowMut<dyn Formatter + 'a>,
-    S: AsRef<str>,
-    H: fmt::Display,
-{
+impl<T: Formatter, H: fmt::Display> HeadingLabeledWriter<T, H> {
     pub fn write_fmt(&mut self, args: fmt::Arguments<'_>) -> io::Result<()> {
-        self.writer.with_labeled(|formatter| {
-            if let Some(heading) = self.heading.take() {
-                write!(formatter.labeled("heading"), "{heading}")?;
-            }
-            formatter.write_fmt(args)
-        })
+        if let Some(heading) = self.heading.take() {
+            write!(self.formatter.labeled("heading"), "{heading}")?;
+        }
+        self.formatter.write_fmt(args)
     }
 }
 
@@ -224,13 +235,9 @@ impl<W: Write> Formatter for PlainTextFormatter<W> {
         Ok(Box::new(self.output.by_ref()))
     }
 
-    fn push_label(&mut self, _label: &str) -> io::Result<()> {
-        Ok(())
-    }
+    fn push_label(&mut self, _label: &str) {}
 
-    fn pop_label(&mut self) -> io::Result<()> {
-        Ok(())
-    }
+    fn pop_label(&mut self) {}
 }
 
 pub struct SanitizingFormatter<W> {
@@ -259,13 +266,9 @@ impl<W: Write> Formatter for SanitizingFormatter<W> {
         Ok(Box::new(self.output.by_ref()))
     }
 
-    fn push_label(&mut self, _label: &str) -> io::Result<()> {
-        Ok(())
-    }
+    fn push_label(&mut self, _label: &str) {}
 
-    fn pop_label(&mut self) -> io::Result<()> {
-        Ok(())
-    }
+    fn pop_label(&mut self) {}
 }
 
 #[derive(Clone, Debug, Default, PartialEq, Eq, serde::Deserialize)]
@@ -582,6 +585,7 @@ impl<W: Write> Write for ColorFormatter<W> {
     }
 
     fn flush(&mut self) -> Result<(), Error> {
+        self.write_new_style()?;
         self.output.flush()
     }
 }
@@ -592,24 +596,19 @@ impl<W: Write> Formatter for ColorFormatter<W> {
         Ok(Box::new(self.output.by_ref()))
     }
 
-    fn push_label(&mut self, label: &str) -> io::Result<()> {
+    fn push_label(&mut self, label: &str) {
         self.labels.push(label.to_owned());
-        Ok(())
     }
 
-    fn pop_label(&mut self) -> io::Result<()> {
+    fn pop_label(&mut self) {
         self.labels.pop();
-        if self.labels.is_empty() {
-            self.write_new_style()?;
-        }
-        Ok(())
     }
 }
 
 impl<W: Write> Drop for ColorFormatter<W> {
     fn drop(&mut self) {
-        // If a `ColorFormatter` was dropped without popping all labels first (perhaps
-        // because of an error), let's still try to reset any currently active style.
+        // If a `ColorFormatter` was dropped without flushing, let's try to
+        // reset any currently active style.
         self.labels.clear();
         self.write_new_style().ok();
     }
@@ -678,8 +677,8 @@ impl FormatRecorder {
         for (pos, op) in &self.ops {
             flush_data(formatter, *pos)?;
             match op {
-                FormatOp::PushLabel(label) => formatter.push_label(label)?,
-                FormatOp::PopLabel => formatter.pop_label()?,
+                FormatOp::PushLabel(label) => formatter.push_label(label),
+                FormatOp::PopLabel => formatter.pop_label(),
                 FormatOp::RawEscapeSequence(raw_escape_sequence) => {
                     formatter.raw()?.write_all(raw_escape_sequence)?;
                 }
@@ -718,14 +717,12 @@ impl Formatter for FormatRecorder {
         Ok(Box::new(RawEscapeSequenceRecorder(self)))
     }
 
-    fn push_label(&mut self, label: &str) -> io::Result<()> {
+    fn push_label(&mut self, label: &str) {
         self.push_op(FormatOp::PushLabel(label.to_owned()));
-        Ok(())
     }
 
-    fn pop_label(&mut self) -> io::Result<()> {
+    fn pop_label(&mut self) {
         self.push_op(FormatOp::PopLabel);
-        Ok(())
     }
 }
 
@@ -748,7 +745,6 @@ fn write_sanitized(output: &mut impl Write, buf: &[u8]) -> Result<(), Error> {
 #[cfg(test)]
 mod tests {
     use std::error::Error as _;
-    use std::str;
 
     use bstr::BString;
     use indexmap::IndexMap;
@@ -779,9 +775,9 @@ mod tests {
         // Test that PlainTextFormatter ignores labels.
         let mut output: Vec<u8> = vec![];
         let mut formatter = PlainTextFormatter::new(&mut output);
-        formatter.push_label("warning").unwrap();
+        formatter.push_label("warning");
         write!(formatter, "hello").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         insta::assert_snapshot!(to_snapshot_string(output), @"hello[EOF]");
     }
 
@@ -830,9 +826,9 @@ mod tests {
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
         for (label, color) in &colors {
-            formatter.push_label(label).unwrap();
+            formatter.push_label(label);
             write!(formatter, " {color} ").unwrap();
-            formatter.pop_label().unwrap();
+            formatter.pop_label();
             writeln!(formatter).unwrap();
         }
         drop(formatter);
@@ -889,13 +885,13 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
-        formatter.push_label("purple-bg").unwrap();
+        formatter.push_label("purple-bg");
         write!(formatter, " purple background ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         writeln!(formatter).unwrap();
-        formatter.push_label("gray").unwrap();
+        formatter.push_label("gray");
         write!(formatter, " gray ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         writeln!(formatter).unwrap();
         drop(formatter);
         insta::assert_snapshot!(to_snapshot_string(output), @r"
@@ -918,9 +914,9 @@ mod tests {
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
         for label in colors.keys() {
-            formatter.push_label(&label.replace(' ', "-")).unwrap();
+            formatter.push_label(&label.replace(' ', "-"));
             write!(formatter, " {label} ").unwrap();
-            formatter.pop_label().unwrap();
+            formatter.pop_label();
             writeln!(formatter).unwrap();
         }
         drop(formatter);
@@ -944,9 +940,9 @@ mod tests {
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
         write!(formatter, " before ").unwrap();
-        formatter.push_label("inside").unwrap();
+        formatter.push_label("inside");
         write!(formatter, " inside ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         write!(formatter, " after ").unwrap();
         drop(formatter);
         insta::assert_snapshot!(
@@ -970,39 +966,39 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
-        formatter.push_label("red_fg").unwrap();
+        formatter.push_label("red_fg");
         write!(formatter, " fg only ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         writeln!(formatter).unwrap();
-        formatter.push_label("blue_bg").unwrap();
+        formatter.push_label("blue_bg");
         write!(formatter, " bg only ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         writeln!(formatter).unwrap();
-        formatter.push_label("bold_font").unwrap();
+        formatter.push_label("bold_font");
         write!(formatter, " bold only ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         writeln!(formatter).unwrap();
-        formatter.push_label("italic_text").unwrap();
+        formatter.push_label("italic_text");
         write!(formatter, " italic only ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         writeln!(formatter).unwrap();
-        formatter.push_label("underlined_text").unwrap();
+        formatter.push_label("underlined_text");
         write!(formatter, " underlined only ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         writeln!(formatter).unwrap();
-        formatter.push_label("reversed_colors").unwrap();
+        formatter.push_label("reversed_colors");
         write!(formatter, " reverse only ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         writeln!(formatter).unwrap();
-        formatter.push_label("multiple").unwrap();
+        formatter.push_label("multiple");
         write!(formatter, " single rule ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         writeln!(formatter).unwrap();
-        formatter.push_label("red_fg").unwrap();
-        formatter.push_label("blue_bg").unwrap();
+        formatter.push_label("red_fg");
+        formatter.push_label("blue_bg");
         write!(formatter, " two rules ").unwrap();
-        formatter.pop_label().unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
+        formatter.pop_label();
         writeln!(formatter).unwrap();
         drop(formatter);
         insta::assert_snapshot!(to_snapshot_string(output), @r"
@@ -1029,19 +1025,48 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
-        formatter.push_label("not_bold").unwrap();
+        formatter.push_label("not_bold");
         write!(formatter, " not bold ").unwrap();
-        formatter.push_label("bold_font").unwrap();
+        formatter.push_label("bold_font");
         write!(formatter, " bold ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         write!(formatter, " not bold again ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         drop(formatter);
         insta::assert_snapshot!(
             to_snapshot_string(output),
             @"[3m[4m[38;5;1m[48;5;4m not bold [1m bold [0m[3m[4m[38;5;1m[48;5;4m not bold again [23m[24m[39m[49m[EOF]");
     }
 
+    #[test]
+    fn test_color_formatter_reset_on_flush() {
+        let config = config_from_string("colors.red = 'red'");
+        let mut output: Vec<u8> = vec![];
+        let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
+        formatter.push_label("red");
+        write!(formatter, "foo").unwrap();
+        formatter.pop_label();
+
+        // without flush()
+        insta::assert_snapshot!(
+            to_snapshot_string(formatter.output.clone()), @"[38;5;1mfoo[EOF]");
+
+        // flush() should emit the reset sequence.
+        formatter.flush().unwrap();
+        insta::assert_snapshot!(
+            to_snapshot_string(formatter.output.clone()), @"[38;5;1mfoo[39m[EOF]");
+
+        // New color sequence should be emitted as the state was reset.
+        formatter.push_label("red");
+        write!(formatter, "bar").unwrap();
+        formatter.pop_label();
+
+        // drop() should emit the reset sequence.
+        drop(formatter);
+        insta::assert_snapshot!(
+            to_snapshot_string(output), @"[38;5;1mfoo[39m[38;5;1mbar[39m[EOF]");
+    }
+
     #[test]
     fn test_color_formatter_no_space() {
         // Test that two different colors can touch.
@@ -1054,16 +1079,16 @@ mod tests {
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
         write!(formatter, "before").unwrap();
-        formatter.push_label("red").unwrap();
+        formatter.push_label("red");
         write!(formatter, "first").unwrap();
-        formatter.pop_label().unwrap();
-        formatter.push_label("green").unwrap();
+        formatter.pop_label();
+        formatter.push_label("green");
         write!(formatter, "second").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         write!(formatter, "after").unwrap();
         drop(formatter);
         insta::assert_snapshot!(
-            to_snapshot_string(output), @"before[38;5;1mfirst[39m[38;5;2msecond[39mafter[EOF]");
+            to_snapshot_string(output), @"before[38;5;1mfirst[38;5;2msecond[39mafter[EOF]");
     }
 
     #[test]
@@ -1076,9 +1101,9 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
-        formatter.push_label("red").unwrap();
+        formatter.push_label("red");
         write!(formatter, "\x1b[1mnot actually bold\x1b[0m").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         drop(formatter);
         insta::assert_snapshot!(
             to_snapshot_string(output), @"[38;5;1m␛[1mnot actually bold␛[0m[39m[EOF]");
@@ -1099,13 +1124,13 @@ mod tests {
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
         write!(formatter, " before outer ").unwrap();
-        formatter.push_label("outer").unwrap();
+        formatter.push_label("outer");
         write!(formatter, " before inner ").unwrap();
-        formatter.push_label("inner").unwrap();
+        formatter.push_label("inner");
         write!(formatter, " inside inner ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         write!(formatter, " after inner ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         write!(formatter, " after outer ").unwrap();
         drop(formatter);
         insta::assert_snapshot!(
@@ -1123,13 +1148,13 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
-        formatter.push_label("outer").unwrap();
+        formatter.push_label("outer");
         write!(formatter, " not colored ").unwrap();
-        formatter.push_label("inner").unwrap();
+        formatter.push_label("inner");
         write!(formatter, " colored ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         write!(formatter, " not colored ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         drop(formatter);
         insta::assert_snapshot!(
             to_snapshot_string(output),
@@ -1215,15 +1240,15 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
-        formatter.push_label("outer").unwrap();
+        formatter.push_label("outer");
         write!(formatter, "Blue on yellow, ").unwrap();
-        formatter.push_label("default_fg").unwrap();
+        formatter.push_label("default_fg");
         write!(formatter, " default fg, ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         write!(formatter, " and back.\nBlue on yellow, ").unwrap();
-        formatter.push_label("default_bg").unwrap();
+        formatter.push_label("default_bg");
         write!(formatter, " default bg, ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         write!(formatter, " and back.").unwrap();
         drop(formatter);
         insta::assert_snapshot!(to_snapshot_string(output), @r"
@@ -1243,11 +1268,11 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
-        formatter.push_label("outer1").unwrap();
-        formatter.push_label("inner2").unwrap();
+        formatter.push_label("outer1");
+        formatter.push_label("inner2");
         write!(formatter, " hello ").unwrap();
-        formatter.pop_label().unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
+        formatter.pop_label();
         drop(formatter);
         insta::assert_snapshot!(to_snapshot_string(output), @"[38;5;2m hello [39m[EOF]");
     }
@@ -1262,11 +1287,11 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
-        formatter.push_label("outer").unwrap();
-        formatter.push_label("inner").unwrap();
+        formatter.push_label("outer");
+        formatter.push_label("inner");
         write!(formatter, " hello ").unwrap();
-        formatter.pop_label().unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
+        formatter.pop_label();
         drop(formatter);
         insta::assert_snapshot!(to_snapshot_string(output), @" hello [EOF]");
     }
@@ -1284,17 +1309,17 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
-        formatter.push_label("a").unwrap();
+        formatter.push_label("a");
         write!(formatter, " a1 ").unwrap();
-        formatter.push_label("b").unwrap();
+        formatter.push_label("b");
         write!(formatter, " b1 ").unwrap();
-        formatter.push_label("c").unwrap();
+        formatter.push_label("c");
         write!(formatter, " c ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         write!(formatter, " b2 ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         write!(formatter, " a2 ").unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
         drop(formatter);
         insta::assert_snapshot!(
             to_snapshot_string(output),
@@ -1312,8 +1337,8 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
-        formatter.push_label("outer").unwrap();
-        formatter.push_label("inner").unwrap();
+        formatter.push_label("outer");
+        formatter.push_label("inner");
         write!(formatter, " inside ").unwrap();
         drop(formatter);
         insta::assert_snapshot!(to_snapshot_string(output), @"[38;5;2m inside [39m[EOF]");
@@ -1330,16 +1355,38 @@ mod tests {
         );
         let mut output: Vec<u8> = vec![];
         let mut formatter = ColorFormatter::for_config(&mut output, &config, true).unwrap();
-        formatter.push_label("outer").unwrap();
-        formatter.push_label("inner").unwrap();
+        formatter.push_label("outer");
+        formatter.push_label("inner");
         write!(formatter, " inside ").unwrap();
-        formatter.pop_label().unwrap();
-        formatter.pop_label().unwrap();
+        formatter.pop_label();
+        formatter.pop_label();
         drop(formatter);
         insta::assert_snapshot!(
             to_snapshot_string(output), @"[38;5;2m<<outer inner:: inside >>[39m[EOF]");
     }
 
+    #[test]
+    fn test_labeled_scope() {
+        let config = config_from_string(indoc! {"
+            [colors]
+            outer = 'blue'
+            inner = 'red'
+            'outer inner' = 'green'
+        "});
+        let mut output: Vec<u8> = vec![];
+        let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
+        writeln!(formatter.labeled("outer"), "outer").unwrap();
+        writeln!(formatter.labeled("outer").labeled("inner"), "outer-inner").unwrap();
+        writeln!(formatter.labeled("inner"), "inner").unwrap();
+        drop(formatter);
+        insta::assert_snapshot!(to_snapshot_string(output), @r"
+        [38;5;4mouter[39m
+        [38;5;2mouter-inner[39m
+        [38;5;1minner[39m
+        [EOF]
+        ");
+    }
+
     #[test]
     fn test_heading_labeled_writer() {
         let config = config_from_string(
@@ -1349,21 +1396,15 @@ mod tests {
         "#,
         );
         let mut output: Vec<u8> = vec![];
-        let mut formatter: Box<dyn Formatter> =
-            Box::new(ColorFormatter::for_config(&mut output, &config, false).unwrap());
-        formatter
-            .as_mut()
-            .labeled("inner")
-            .with_heading("Should be noop: ");
-        let mut writer = formatter
-            .as_mut()
-            .labeled("inner")
-            .with_heading("Heading: ");
+        let mut formatter = ColorFormatter::for_config(&mut output, &config, false).unwrap();
+        formatter.labeled("inner").with_heading("Should be noop: ");
+        let mut writer = formatter.labeled("inner").with_heading("Heading: ");
         write!(writer, "Message").unwrap();
         writeln!(writer, " continues").unwrap();
+        drop(writer);
         drop(formatter);
         insta::assert_snapshot!(to_snapshot_string(output), @r"
-        [38;5;1mHeading: [38;5;2mMessage[39m[38;5;2m continues[39m
+        [38;5;1mHeading: [38;5;2mMessage continues[39m
         [EOF]
         ");
     }
@@ -1371,16 +1412,13 @@ mod tests {
     #[test]
     fn test_heading_labeled_writer_empty_string() {
         let mut output: Vec<u8> = vec![];
-        let mut formatter: Box<dyn Formatter> = Box::new(PlainTextFormatter::new(&mut output));
-        let mut writer = formatter
-            .as_mut()
-            .labeled("inner")
-            .with_heading("Heading: ");
+        let mut formatter = PlainTextFormatter::new(&mut output);
+        let mut writer = formatter.labeled("inner").with_heading("Heading: ");
         // write_fmt() is called even if the format string is empty. I don't
         // know if that's guaranteed, but let's record the current behavior.
         write!(writer, "").unwrap();
         write!(writer, "").unwrap();
-        drop(formatter);
+        drop(writer);
         insta::assert_snapshot!(to_snapshot_string(output), @"Heading: [EOF]");
     }
 
@@ -1388,10 +1426,10 @@ mod tests {
     fn test_format_recorder() {
         let mut recorder = FormatRecorder::new();
         write!(recorder, " outer1 ").unwrap();
-        recorder.push_label("inner").unwrap();
+        recorder.push_label("inner");
         write!(recorder, " inner1 ").unwrap();
         write!(recorder, " inner2 ").unwrap();
-        recorder.pop_label().unwrap();
+        recorder.pop_label();
         write!(recorder, " outer2 ").unwrap();
 
         insta::assert_snapshot!(
@@ -1428,10 +1466,10 @@ mod tests {
         // Note: similar to test_format_recorder above
         let mut recorder = FormatRecorder::new();
         write!(recorder.raw().unwrap(), " outer1 ").unwrap();
-        recorder.push_label("inner").unwrap();
+        recorder.push_label("inner");
         write!(recorder.raw().unwrap(), " inner1 ").unwrap();
         write!(recorder.raw().unwrap(), " inner2 ").unwrap();
-        recorder.pop_label().unwrap();
+        recorder.pop_label();
         write!(recorder.raw().unwrap(), " outer2 ").unwrap();
 
         // Replayed raw escape sequences are labeled.
