@@ -21,7 +21,6 @@ use std::sync::Arc;
 use itertools::Itertools as _;
 use jj_lib::backend::BackendError;
 use jj_lib::backend::CopyId;
-use jj_lib::backend::MergedTreeId;
 use jj_lib::backend::TreeValue;
 use jj_lib::config::ConfigGetError;
 use jj_lib::config::ConfigGetResultExt as _;
@@ -31,6 +30,7 @@ use jj_lib::conflicts::MaterializedFileConflictValue;
 use jj_lib::conflicts::try_materialize_file_conflict_value;
 use jj_lib::gitignore::GitIgnoreFile;
 use jj_lib::matchers::Matcher;
+use jj_lib::merge::Diff;
 use jj_lib::merge::Merge;
 use jj_lib::merge::MergedTreeValue;
 use jj_lib::merged_tree::MergedTree;
@@ -295,10 +295,10 @@ impl DiffEditor {
     /// Starts a diff editor on the two directories.
     pub fn edit(
         &self,
-        trees: [&MergedTree; 2],
+        trees: Diff<&MergedTree>,
         matcher: &dyn Matcher,
         format_instructions: impl FnOnce() -> String,
-    ) -> Result<MergedTreeId, DiffEditError> {
+    ) -> Result<MergedTree, DiffEditError> {
         match &self.tool {
             DiffEditTool::Builtin => {
                 Ok(
@@ -430,7 +430,7 @@ impl MergeEditor {
         ui: &Ui,
         tree: &MergedTree,
         repo_paths: &[&RepoPath],
-    ) -> Result<(MergedTreeId, Option<MergeToolPartialResolutionError>), ConflictResolveError> {
+    ) -> Result<(MergedTree, Option<MergeToolPartialResolutionError>), ConflictResolveError> {
         let merge_tool_files: Vec<MergeToolFile> = repo_paths
             .iter()
             .map(|&repo_path| MergeToolFile::from_tree_and_path(tree, repo_path))
@@ -438,16 +438,16 @@ impl MergeEditor {
 
         match &self.tool {
             MergeTool::Builtin => {
-                let tree_id = edit_merge_builtin(tree, &merge_tool_files).map_err(Box::new)?;
-                Ok((tree_id, None))
+                let tree = edit_merge_builtin(tree, &merge_tool_files).map_err(Box::new)?;
+                Ok((tree, None))
             }
             MergeTool::Ours => {
-                let tree_id = pick_conflict_side(tree, &merge_tool_files, 0)?;
-                Ok((tree_id, None))
+                let tree = pick_conflict_side(tree, &merge_tool_files, 0)?;
+                Ok((tree, None))
             }
             MergeTool::Theirs => {
-                let tree_id = pick_conflict_side(tree, &merge_tool_files, 1)?;
-                Ok((tree_id, None))
+                let tree = pick_conflict_side(tree, &merge_tool_files, 1)?;
+                Ok((tree, None))
             }
             MergeTool::External(editor) => external::run_mergetool_external(
                 ui,
@@ -465,8 +465,8 @@ fn pick_conflict_side(
     tree: &MergedTree,
     merge_tool_files: &[MergeToolFile],
     add_index: usize,
-) -> Result<MergedTreeId, BackendError> {
-    let mut tree_builder = MergedTreeBuilder::new(tree.id());
+) -> Result<MergedTree, BackendError> {
+    let mut tree_builder = MergedTreeBuilder::new(tree.clone());
     for merge_tool_file in merge_tool_files {
         // We use file IDs here to match the logic for the other external merge tools.
         // This ensures that the behavior is consistent.
@@ -480,7 +480,7 @@ fn pick_conflict_side(
         }));
         tree_builder.set_or_remove(merge_tool_file.repo_path.clone(), new_tree_value);
     }
-    tree_builder.write_tree(tree.store())
+    tree_builder.write_tree()
 }
 
 #[cfg(test)]
