@@ -13,10 +13,12 @@
 // limitations under the License.
 
 use std::fs::OpenOptions;
+use std::io::Read as _;
 use std::io::Write as _;
 use std::path::PathBuf;
 use std::process::ExitCode;
 
+use bstr::ByteSlice as _;
 use clap::Parser;
 use itertools::Itertools as _;
 
@@ -63,6 +65,11 @@ struct Args {
     /// Duplicate stdout into this file.
     #[arg(long)]
     tee: Option<PathBuf>,
+
+    /// Read one byte at a time, and send one byte to the stdout before reading
+    /// the next byte.
+    #[arg(long, default_value_t = false)]
+    byte_mode: bool,
 }
 
 fn main() -> ExitCode {
@@ -77,35 +84,67 @@ fn main() -> ExitCode {
         assert!(!args.uppercase);
         assert!(!args.lowercase);
         assert!(args.append.is_none());
+        print!("{data}");
         data
+    } else if args.byte_mode {
+        assert!(!args.reverse);
+        assert!(args.append.is_none());
+
+        let mut stdout = vec![];
+        #[expect(clippy::unbuffered_bytes)]
+        for byte in std::io::stdin().bytes() {
+            let byte = byte.expect("Failed to read from stdin");
+            let output = if args.uppercase {
+                byte.to_ascii_uppercase()
+            } else if args.lowercase {
+                assert!(!args.uppercase);
+                byte.to_ascii_lowercase()
+            } else {
+                byte
+            };
+            stdout.push(output);
+            std::io::stdout()
+                .write_all(&[output])
+                .expect("Failed to write to stdout");
+        }
+        stdout
+            .to_str()
+            .expect("Output is not a valid UTF-8 string")
+            .to_owned()
     } else {
-        let mut stdout = std::io::stdin()
-            .lines()
+        let mut input = vec![];
+        std::io::stdin()
+            .read_to_end(&mut input)
+            .expect("Failed to read from stdin");
+        let mut stdout = input
+            .lines_with_terminator()
             .map(|line| {
-                format!("{}\n", {
-                    let line = if args.reverse {
-                        line.unwrap().chars().rev().collect()
-                    } else {
-                        line.unwrap()
-                    };
-                    if args.uppercase {
-                        assert!(!args.lowercase);
-                        line.to_uppercase()
-                    } else if args.lowercase {
-                        assert!(!args.uppercase);
-                        line.to_lowercase()
-                    } else {
-                        line
-                    }
-                })
+                let line = line
+                    .to_str()
+                    .expect("The input is not valid UTF-8 string")
+                    .to_owned();
+                let line = if args.reverse {
+                    line.chars().rev().collect()
+                } else {
+                    line
+                };
+                if args.uppercase {
+                    assert!(!args.lowercase);
+                    line.to_uppercase()
+                } else if args.lowercase {
+                    assert!(!args.uppercase);
+                    line.to_lowercase()
+                } else {
+                    line
+                }
             })
             .join("");
         if let Some(line) = args.append {
             stdout.push_str(&line);
         }
+        print!("{stdout}");
         stdout
     };
-    print!("{stdout}");
     if let Some(path) = args.tee {
         let mut file = OpenOptions::new()
             .create(true)
