@@ -126,7 +126,7 @@ fn test_absorb_simple() {
        index 0000000000..e69de29bb2
     [EOF]
     ");
-    insta::assert_snapshot!(get_evolog(&work_dir, "description(1)"), @r"
+    insta::assert_snapshot!(get_evolog(&work_dir, "subject(glob:1)"), @r"
     ○    kkmpptxz 5810eb0f 1
     ├─╮
     │ ○  yqosqzyt hidden 39b42898 (no description set)
@@ -139,7 +139,7 @@ fn test_absorb_simple() {
     ○  kkmpptxz hidden eb943711 (empty) 1
     [EOF]
     ");
-    insta::assert_snapshot!(get_evolog(&work_dir, "description(2)"), @r"
+    insta::assert_snapshot!(get_evolog(&work_dir, "subject(glob:2)"), @r"
     ○    zsuskuln dd109863 2
     ├─╮
     │ ○  vruxwmqv hidden 761492a8 (no description set)
@@ -176,12 +176,12 @@ fn test_absorb_replace_single_line_hunk() {
     insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Absorbed changes into 1 revisions:
-      qpvuntsm 5bdb5ca1 (conflict) 1
+      qpvuntsm 19034586 (conflict) 1
     Rebased 1 descendant commits.
-    Working copy  (@) now at: mzvwutvl 804fe9d9 (empty) (no description set)
-    Parent commit (@-)      : kkmpptxz 6068e8fc 2
+    Working copy  (@) now at: mzvwutvl f9c426f2 (empty) (no description set)
+    Parent commit (@-)      : kkmpptxz a5f84679 2
     New conflicts appeared in 1 commits:
-      qpvuntsm 5bdb5ca1 (conflict) 1
+      qpvuntsm 19034586 (conflict) 1
     Hint: To resolve the conflicts, start by creating a commit on top of
     the conflicted commit:
       jj new qpvuntsm
@@ -192,8 +192,8 @@ fn test_absorb_replace_single_line_hunk() {
     "###);
 
     insta::assert_snapshot!(get_diffs(&work_dir, "mutable()"), @r"
-    @  mzvwutvl 804fe9d9 (empty) (no description set)
-    ○  kkmpptxz 6068e8fc 2
+    @  mzvwutvl f9c426f2 (empty) (no description set)
+    ○  kkmpptxz a5f84679 2
     │  diff --git a/file1 b/file1
     │  index 0000000000..2f87e8e465 100644
     │  --- a/file1
@@ -209,7 +209,7 @@ fn test_absorb_replace_single_line_hunk() {
     │   1A
     │   2b
     │  ->>>>>>> Conflict 1 of 1 ends
-    ×  qpvuntsm 5bdb5ca1 (conflict) 1
+    ×  qpvuntsm 19034586 (conflict) 1
     │  diff --git a/file1 b/file1
     ~  new file mode 100644
        index 0000000000..0000000000
@@ -242,10 +242,10 @@ fn test_absorb_merge() {
     work_dir.run_jj(["new", "-m1"]).success();
     work_dir.write_file("file1", "1a\n1b\n0a\n");
 
-    work_dir.run_jj(["new", "-m2", "description(0)"]).success();
+    work_dir.run_jj(["new", "-m2", "subject(glob:0)"]).success();
     work_dir.write_file("file1", "0a\n2a\n2b\n");
 
-    let output = work_dir.run_jj(["new", "-m3", "description(1)", "description(2)"]);
+    let output = work_dir.run_jj(["new", "-m3", "subject(glob:1)", "subject(glob:2)"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Working copy  (@) now at: mzvwutvl 42875bf7 (empty) 3
@@ -338,10 +338,10 @@ fn test_absorb_discardable_merge_with_descendant() {
     work_dir.run_jj(["new", "-m1"]).success();
     work_dir.write_file("file1", "1a\n1b\n0a\n");
 
-    work_dir.run_jj(["new", "-m2", "description(0)"]).success();
+    work_dir.run_jj(["new", "-m2", "subject(glob:0)"]).success();
     work_dir.write_file("file1", "0a\n2a\n2b\n");
 
-    let output = work_dir.run_jj(["new", "description(1)", "description(2)"]);
+    let output = work_dir.run_jj(["new", "subject(glob:1)", "subject(glob:2)"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Working copy  (@) now at: mzvwutvl ad00b91a (empty) (no description set)
@@ -420,17 +420,17 @@ fn test_absorb_conflict() {
 
     work_dir.run_jj(["new", "root()"]).success();
     work_dir.write_file("file1", "2a\n2b\n");
-    let output = work_dir.run_jj(["rebase", "-r@", "-ddescription(1)"]);
+    let output = work_dir.run_jj(["rebase", "-r@", "-dsubject(glob:1)"]);
     insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Rebased 1 commits to destination
-    Working copy  (@) now at: kkmpptxz 66d44b8c (conflict) (no description set)
+    Working copy  (@) now at: kkmpptxz 01e6cd99 (conflict) (no description set)
     Parent commit (@-)      : qpvuntsm e35bcaff 1
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file1    2-sided conflict
     New conflicts appeared in 1 commits:
-      kkmpptxz 66d44b8c (conflict) (no description set)
+      kkmpptxz 01e6cd99 (conflict) (no description set)
     Hint: To resolve the conflicts, start by creating a commit on top of
     the conflicted commit:
       jj new kkmpptxz
@@ -550,14 +550,14 @@ fn test_absorb_deleted_file_with_multiple_hunks() {
     insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Absorbed changes into 2 revisions:
-      kkmpptxz 8407ab95 (conflict) 2
-      qpvuntsm f1473264 (conflict) 1
+      kkmpptxz 9210e16d (conflict) 2
+      qpvuntsm a52f61f7 (conflict) 1
     Rebased 1 descendant commits.
-    Working copy  (@) now at: zsuskuln b56f0c39 (no description set)
-    Parent commit (@-)      : kkmpptxz 8407ab95 (conflict) 2
+    Working copy  (@) now at: zsuskuln f8744d38 (no description set)
+    Parent commit (@-)      : kkmpptxz 9210e16d (conflict) 2
     New conflicts appeared in 2 commits:
-      kkmpptxz 8407ab95 (conflict) 2
-      qpvuntsm f1473264 (conflict) 1
+      kkmpptxz 9210e16d (conflict) 2
+      qpvuntsm a52f61f7 (conflict) 1
     Hint: To resolve the conflicts, start by creating a commit on top of
     the first conflicted commit:
       jj new qpvuntsm
@@ -570,7 +570,7 @@ fn test_absorb_deleted_file_with_multiple_hunks() {
     "###);
 
     insta::assert_snapshot!(get_diffs(&work_dir, "mutable()"), @r"
-    @  zsuskuln b56f0c39 (no description set)
+    @  zsuskuln f8744d38 (no description set)
     │  diff --git a/file2 b/file2
     │  deleted file mode 100644
     │  index 0000000000..0000000000
@@ -584,7 +584,7 @@ fn test_absorb_deleted_file_with_multiple_hunks() {
     │  -+++++++ Contents of side #2
     │  -1a
     │  ->>>>>>> Conflict 1 of 1 ends
-    ×  kkmpptxz 8407ab95 (conflict) 2
+    ×  kkmpptxz 9210e16d (conflict) 2
     │  diff --git a/file1 b/file1
     │  deleted file mode 100644
     │  index 0000000000..0000000000
@@ -611,7 +611,7 @@ fn test_absorb_deleted_file_with_multiple_hunks() {
     │  -1b
     │  +1a
     │   >>>>>>> Conflict 1 of 1 ends
-    ×  qpvuntsm f1473264 (conflict) 1
+    ×  qpvuntsm a52f61f7 (conflict) 1
     │  diff --git a/file1 b/file1
     ~  new file mode 100644
        index 0000000000..0000000000
@@ -810,9 +810,10 @@ fn test_absorb_paths() {
     work_dir.write_file("file1", "1A\n");
     work_dir.write_file("file2", "1A\n");
 
-    let output = work_dir.run_jj(["absorb", "unknown"]);
+    let output = work_dir.run_jj(["absorb", "nonexistent"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: No matching entries for paths: nonexistent
     Nothing changed.
     [EOF]
     ");
@@ -899,7 +900,7 @@ fn test_absorb_immutable() {
     Hint: Could not modify commit: qpvuntsm e35bcaff main | 1
     Hint: Immutable commits are used to protect shared history.
     Hint: For more information, see:
-          - https://jj-vcs.github.io/jj/latest/config/#set-of-immutable-commits
+          - https://docs.jj-vcs.dev/latest/config/#set-of-immutable-commits
           - `jj help -k config`, "Set of immutable commits"
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
