@@ -158,7 +158,7 @@ fn test_bookmark_bad_name() {
       | ^---
       |
       = expected <identifier>, <string_literal>, or <raw_string_literal>
-    Hint: See https://jj-vcs.github.io/jj/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
+    Hint: See https://docs.jj-vcs.dev/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
     [EOF]
     [exit status: 2]
     ");
@@ -175,7 +175,7 @@ fn test_bookmark_bad_name() {
       | ^^
       |
       = Expected non-empty string
-    Hint: See https://jj-vcs.github.io/jj/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
+    Hint: See https://docs.jj-vcs.dev/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
     [EOF]
     [exit status: 2]
     ");
@@ -192,7 +192,7 @@ fn test_bookmark_bad_name() {
       | ^---
       |
       = expected <identifier>, <string_literal>, or <raw_string_literal>
-    Hint: See https://jj-vcs.github.io/jj/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
+    Hint: See https://docs.jj-vcs.dev/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
     [EOF]
     [exit status: 2]
     ");
@@ -210,7 +210,7 @@ fn test_bookmark_bad_name() {
       | ^---
       |
       = expected <identifier>, <string_literal>, or <raw_string_literal>
-    Hint: See https://jj-vcs.github.io/jj/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
+    Hint: See https://docs.jj-vcs.dev/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
     [EOF]
     [exit status: 2]
     ");
@@ -258,9 +258,9 @@ fn test_bookmark_move() {
     let output = work_dir.run_jj(["bookmark", "move", "foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Error: No such bookmark: foo
+    Warning: No matching bookmarks for names: foo
+    No bookmarks to update.
     [EOF]
-    [exit status: 1]
     ");
 
     let output = work_dir.run_jj(["bookmark", "set", "foo"]);
@@ -436,12 +436,11 @@ fn test_bookmark_move_matching() {
     ");
 
     // No matching bookmarks within the source revisions
-    let output = work_dir.run_jj(["bookmark", "move", "--from=::@", "glob:a?"]);
+    let output = work_dir.run_jj(["bookmark", "move", "--from=::@", "glob:'a?'"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Error: No matching bookmarks for patterns: a?
+    No bookmarks to update.
     [EOF]
-    [exit status: 1]
     ");
 
     // Noop move
@@ -472,8 +471,27 @@ fn test_bookmark_move_matching() {
     ");
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
 
+    // Move multiple bookmarks by name
+    let output = work_dir.run_jj(["bookmark", "move", "b1", "c1"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Moved 2 bookmarks to vruxwmqv 0dd9a4b1 b1 c1 | (empty) head2
+    [EOF]
+    ");
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
+    @  b1 c1 0dd9a4b12283
+    ○   2cbf65662e56
+    ○   c2934cfbfb19
+    │ ○   9328ecc52471
+    │ ○  a1 a2 e8849ae12c70
+    ├─╯
+    ◆   000000000000
+    [EOF]
+    ");
+    work_dir.run_jj(["op", "restore", &setup_opid]).success();
+
     // Try to move multiple bookmarks, but one of them isn't fast-forward
-    let output = work_dir.run_jj(["bookmark", "move", "glob:?1"]);
+    let output = work_dir.run_jj(["bookmark", "move", "glob:'?1'"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Refusing to move bookmark backwards or sideways: a1
@@ -493,7 +511,7 @@ fn test_bookmark_move_matching() {
     ");
 
     // Select by revision and name
-    let output = work_dir.run_jj(["bookmark", "move", "--from=::a1+", "--to=a1+", "glob:?1"]);
+    let output = work_dir.run_jj(["bookmark", "move", "--from=::a1+", "--to=a1+", "glob:'?1'"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Moved 1 bookmarks to kkmpptxz 9328ecc5 a1 | (empty) head1
@@ -525,19 +543,19 @@ fn test_bookmark_move_conflicting() {
     work_dir.run_jj(["new", "root()", "-mB0"]).success();
     work_dir.run_jj(["new", "root()", "-mC0"]).success();
     work_dir
-        .run_jj(["new", "description(A0)", "-mA1"])
+        .run_jj(["new", "subject(glob:A0)", "-mA1"])
         .success();
 
     // Set up conflicting bookmark.
     work_dir
-        .run_jj(["bookmark", "create", "-rdescription(A0)", "foo"])
+        .run_jj(["bookmark", "create", "-rsubject(glob:A0)", "foo"])
         .success();
     work_dir
         .run_jj([
             "bookmark",
             "create",
             "--at-op=@-",
-            "-rdescription(B0)",
+            "-rsubject(glob:B0)",
             "foo",
         ])
         .success();
@@ -556,7 +574,7 @@ fn test_bookmark_move_conflicting() {
     ");
 
     // Can't move the bookmark to C0 since it's sibling.
-    let output = work_dir.run_jj(["bookmark", "set", "-rdescription(C0)", "foo"]);
+    let output = work_dir.run_jj(["bookmark", "set", "-rsubject(glob:C0)", "foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Refusing to move bookmark backwards or sideways: foo
@@ -567,7 +585,7 @@ fn test_bookmark_move_conflicting() {
 
     // Can move the bookmark to A1 since it's descendant of A0. It's not
     // descendant of B0, though.
-    let output = work_dir.run_jj(["bookmark", "set", "-rdescription(A1)", "foo"]);
+    let output = work_dir.run_jj(["bookmark", "set", "-rsubject(glob:A1)", "foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Moved 1 bookmarks to mzvwutvl 0f5f3e2c foo | (empty) A1
@@ -624,10 +642,13 @@ fn test_bookmark_rename() {
 
     work_dir.run_jj(["new"]).success();
     work_dir.run_jj(["describe", "-m=commit-2"]).success();
-    work_dir.run_jj(["bookmark", "create", "bremote"]).success();
     work_dir
-        .run_jj(["git", "push", "--allow-new", "-b=bremote"])
+        .run_jj(["bookmark", "create", "bremote", "buntracked"])
         .success();
+    work_dir
+        .run_jj(["git", "push", "--allow-new", "-b=bremote", "-b=buntracked"])
+        .success();
+
     let output = work_dir.run_jj(["bookmark", "rename", "bremote", "bremote2"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
@@ -635,6 +656,7 @@ fn test_bookmark_rename() {
     Hint: To rename the bookmark on the remote, you can `jj git push --bookmark bremote` first (to delete it on the remote), and then `jj git push --bookmark bremote2`. `jj git push --all --deleted` would also be sufficient.
     [EOF]
     ");
+    let op_id_after_rename = work_dir.current_operation_id();
     let output = work_dir.run_jj(["bookmark", "rename", "bremote2", "bremote"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
@@ -642,6 +664,38 @@ fn test_bookmark_rename() {
     Hint: Run `jj bookmark untrack 'glob:bremote@*'` to disassociate them.
     [EOF]
     ");
+    work_dir
+        .run_jj(["op", "restore", &op_id_after_rename])
+        .success();
+    let output = work_dir.run_jj(["git", "push", "--bookmark", "bremote2"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Changes to push to origin:
+      Add bookmark bremote2 to a9d7418c1c3f
+    [EOF]
+    ");
+    work_dir
+        .run_jj(["git", "push", "--named", "bremote-untracked=@"])
+        .success();
+    work_dir
+        .run_jj(["bookmark", "forget", "bremote-untracked"])
+        .success();
+    let output = work_dir.run_jj(["bookmark", "rename", "bremote2", "bremote-untracked"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Warning: The renamed bookmark already exists on the remote 'origin', tracking state was dropped.
+    Hint: To track the existing remote bookmark, run `jj bookmark track bremote-untracked@origin`
+    Warning: Tracked remote bookmarks for bookmark bremote2 were not renamed.
+    Hint: To rename the bookmark on the remote, you can `jj git push --bookmark bremote2` first (to delete it on the remote), and then `jj git push --bookmark bremote-untracked`. `jj git push --all --deleted` would also be sufficient.
+    [EOF]
+    ");
+
+    // rename an untracked bookmark
+    work_dir
+        .run_jj(["bookmark", "untrack", "buntracked@origin"])
+        .success();
+    let output = work_dir.run_jj(["bookmark", "rename", "buntracked", "buntracked2"]);
+    insta::assert_snapshot!(output, @"");
 }
 
 #[test]
@@ -677,14 +731,14 @@ fn test_bookmark_forget_glob() {
     ◆   000000000000
     [EOF]
     ");
-    let output = work_dir.run_jj(["bookmark", "forget", "glob:foo-[1-3]"]);
+    let output = work_dir.run_jj(["bookmark", "forget", "glob:'foo-[1-3]'"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Forgot 2 local bookmarks.
     [EOF]
     ");
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
-    let output = work_dir.run_jj(["bookmark", "forget", "glob:foo-[1-3]"]);
+    let output = work_dir.run_jj(["bookmark", "forget", "glob:'foo-[1-3]'"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Forgot 2 local bookmarks.
@@ -711,24 +765,29 @@ fn test_bookmark_forget_glob() {
     ");
 
     // Malformed glob
-    let output = work_dir.run_jj(["bookmark", "forget", "glob:foo-[1-3"]);
+    let output = work_dir.run_jj(["bookmark", "forget", "glob:'foo-[1-3'"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    error: invalid value 'glob:foo-[1-3' for '<NAMES>...': error parsing glob 'foo-[1-3': unclosed character class; missing ']'
-
-    For more information, try '--help'.
-    [EOF]
-    [exit status: 2]
-    ");
-
-    // We get an error if none of the globs match anything
-    let output = work_dir.run_jj(["bookmark", "forget", "glob:bar*", "glob:baz*", "glob:boom*"]);
-    insta::assert_snapshot!(output, @r"
-    ------- stderr -------
-    Error: No matching bookmarks for patterns: baz*, boom*
+    Error: Failed to parse name pattern: Invalid string pattern
+    Caused by:
+    1:  --> 1:1
+      |
+    1 | glob:'foo-[1-3'
+      | ^-------------^
+      |
+      = Invalid string pattern
+    2: error parsing glob 'foo-[1-3': unclosed character class; missing ']'
     [EOF]
     [exit status: 1]
     ");
+
+    // None of the globs match anything
+    let output = work_dir.run_jj(["bookmark", "forget", "glob:baz*", "glob:boom*"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    No bookmarks to forget.
+    [EOF]
+    ");
 }
 
 #[test]
@@ -770,14 +829,14 @@ fn test_bookmark_delete_glob() {
     ◆   000000000000
     [EOF]
     ");
-    let output = work_dir.run_jj(["bookmark", "delete", "glob:foo-[1-3]"]);
+    let output = work_dir.run_jj(["bookmark", "delete", "glob:'foo-[1-3]'"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Deleted 2 bookmarks.
     [EOF]
     ");
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
-    let output = work_dir.run_jj(["bookmark", "delete", "glob:foo-[1-3]"]);
+    let output = work_dir.run_jj(["bookmark", "delete", "glob:'foo-[1-3]'"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Deleted 2 bookmarks.
@@ -791,12 +850,11 @@ fn test_bookmark_delete_glob() {
 
     // We get an error if none of the globs match live bookmarks. Unlike `jj
     // bookmark forget`, it's not allowed to delete already deleted bookmarks.
-    let output = work_dir.run_jj(["bookmark", "delete", "glob:foo-[1-3]"]);
+    let output = work_dir.run_jj(["bookmark", "delete", "glob:'foo-[1-3]'"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Error: No matching bookmarks for patterns: foo-[1-3]
+    No bookmarks to delete.
     [EOF]
-    [exit status: 1]
     ");
 
     // Deleting a bookmark via both explicit name and glob pattern, or with
@@ -828,26 +886,38 @@ fn test_bookmark_delete_glob() {
     ");
 
     // Malformed glob
-    let output = work_dir.run_jj(["bookmark", "delete", "glob:foo-[1-3"]);
+    let output = work_dir.run_jj(["bookmark", "delete", "glob:'foo-[1-3'"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    error: invalid value 'glob:foo-[1-3' for '<NAMES>...': error parsing glob 'foo-[1-3': unclosed character class; missing ']'
-
-    For more information, try '--help'.
+    Error: Failed to parse name pattern: Invalid string pattern
+    Caused by:
+    1:  --> 1:1
+      |
+    1 | glob:'foo-[1-3'
+      | ^-------------^
+      |
+      = Invalid string pattern
+    2: error parsing glob 'foo-[1-3': unclosed character class; missing ']'
     [EOF]
-    [exit status: 2]
+    [exit status: 1]
     ");
 
     // Unknown pattern kind
     let output = work_dir.run_jj(["bookmark", "forget", "whatever:bookmark"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    error: invalid value 'whatever:bookmark' for '<NAMES>...': Invalid string pattern kind `whatever:`
-
-    For more information, try '--help'.
+    Error: Failed to parse name pattern: Invalid string pattern
+    Caused by:
+    1:  --> 1:1
+      |
+    1 | whatever:bookmark
+      | ^---------------^
+      |
+      = Invalid string pattern
+    2: Invalid string pattern kind `whatever:`
     Hint: Try prefixing with one of `exact:`, `glob:`, `regex:`, `substring:`, or one of these with `-i` suffix added (e.g. `glob-i:`) for case-insensitive matching
     [EOF]
-    [exit status: 2]
+    [exit status: 1]
     ");
 }
 
@@ -926,7 +996,7 @@ fn test_bookmark_forget_fetched_bookmark() {
 
     // Set up a git repo with a bookmark and a jj repo that has it as a remote.
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     let git_repo_path = test_env.env_root().join("git-repo");
@@ -1073,7 +1143,7 @@ fn test_bookmark_forget_deleted_or_nonexistent_bookmark() {
     // ======== Beginning of test setup ========
     // Set up a git repo with a bookmark and a jj repo that has it as a remote.
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     let git_repo_path = test_env.env_root().join("git-repo");
@@ -1107,18 +1177,22 @@ fn test_bookmark_forget_deleted_or_nonexistent_bookmark() {
     // ============ End of test setup ============
 
     // We can forget a deleted bookmark
-    work_dir
-        .run_jj(["bookmark", "forget", "--include-remotes", "feature1"])
-        .success();
+    let output = work_dir.run_jj(["bookmark", "forget", "--include-remotes", "feature1"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Forgot 1 local bookmarks.
+    Forgot 1 remote bookmarks.
+    [EOF]
+    ");
     insta::assert_snapshot!(get_bookmark_output(&work_dir), @"");
 
     // Can't forget a non-existent bookmark
     let output = work_dir.run_jj(["bookmark", "forget", "i_do_not_exist"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Error: No such bookmark: i_do_not_exist
+    Warning: No matching bookmarks for names: i_do_not_exist
+    No bookmarks to forget.
     [EOF]
-    [exit status: 1]
     ");
 }
 
@@ -1147,7 +1221,7 @@ fn test_bookmark_track_untrack() {
             "refs/heads/feature2",
         ],
     );
-    test_env.add_config("git.auto-local-bookmark = false");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = ''");
     let output = work_dir.run_jj(["git", "fetch"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
@@ -1183,6 +1257,15 @@ fn test_bookmark_track_untrack() {
     [EOF]
     ");
 
+    // Track non-existent remote bookmark
+    let output = work_dir.run_jj(["bookmark", "track", "feature3@origin"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Warning: No matching remote bookmarks for names: feature3@origin
+    Nothing changed.
+    [EOF]
+    ");
+
     // Track existing bookmark. Local bookmark should result in conflict.
     work_dir
         .run_jj(["bookmark", "create", "-r@", "feature2"])
@@ -1276,7 +1359,7 @@ fn test_bookmark_track_untrack() {
             "refs/heads/feature3",
         ],
     );
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let output = work_dir.run_jj(["git", "fetch"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
@@ -1314,30 +1397,84 @@ fn test_bookmark_track_conflict() {
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
 
+    // add three remotes
     let git_repo_path = test_env.env_root().join("git-repo");
     git::init_bare(git_repo_path);
     work_dir
         .run_jj(["git", "remote", "add", "origin", "../git-repo"])
         .success();
+    let git_repo_path = test_env.env_root().join("git-repo2");
+    git::init_bare(git_repo_path);
+    work_dir
+        .run_jj(["git", "remote", "add", "origin2", "../git-repo2"])
+        .success();
+    let git_repo_path = test_env.env_root().join("git-repo3");
+    git::init_bare(git_repo_path);
+    work_dir
+        .run_jj(["git", "remote", "add", "origin3", "../git-repo3"])
+        .success();
+
+    // create bookmark and push to origin
     work_dir.run_jj(["bookmark", "create", "main"]).success();
     work_dir.run_jj(["describe", "-m", "a"]).success();
     work_dir
         .run_jj(["git", "push", "--allow-new", "-b", "main"])
         .success();
-    work_dir
-        .run_jj(["bookmark", "untrack", "main@origin"])
-        .success();
+
+    // adjust main and push to origin2, again for origin3
     work_dir
         .run_jj(["describe", "-m", "b", "-r", "main", "--ignore-immutable"])
         .success();
+    work_dir
+        .run_jj(["git", "push", "-N", "-b", "main", "--remote", "origin2"])
+        .success();
+    work_dir
+        .run_jj(["describe", "-m", "c", "-r", "main", "--ignore-immutable"])
+        .success();
+    work_dir
+        .run_jj(["git", "push", "-N", "-b", "main", "--remote", "origin3"])
+        .success();
+
+    // stop and retrack origin; creates conflict
+    // origin2 and origin3 are not shown
+    work_dir
+        .run_jj(["bookmark", "untrack", "main@origin"])
+        .success();
     let output = work_dir.run_jj(["bookmark", "track", "main@origin"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Started tracking 1 remote bookmarks.
     main (conflicted):
-      + qpvuntsm?? 56b9f16b (empty) b
-      + qpvuntsm?? 7d5ca8e4 (empty) a
-      @origin (behind by 1 commits): qpvuntsm?? 7d5ca8e4 (empty) a
+      + qpvuntsm?? 467e027c (empty) c
+      + qpvuntsm?? 48ded843 (empty) a
+      @origin (behind by 1 commits): qpvuntsm?? 48ded843 (empty) a
+    [EOF]
+    ");
+
+    // origin2 differs but is not in conflict
+    insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
+    main (conflicted):
+      + qpvuntsm?? 467e027c (empty) c
+      + qpvuntsm?? 48ded843 (empty) a
+      @origin (behind by 1 commits): qpvuntsm?? 48ded843 (empty) a
+      @origin2 (ahead by 1 commits, behind by 2 commits): qpvuntsm hidden 579e0acd (empty) b
+      @origin3 (behind by 1 commits): qpvuntsm?? 467e027c (empty) c
+    [EOF]
+    ");
+
+    // retracking origin2 adds to the conflict
+    work_dir
+        .run_jj(["bookmark", "untrack", "main@origin2"])
+        .success();
+    let output = work_dir.run_jj(["bookmark", "track", "main@origin2"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Started tracking 1 remote bookmarks.
+    main (conflicted):
+      + qpvuntsm?? 467e027c (empty) c
+      + qpvuntsm?? 48ded843 (empty) a
+      + qpvuntsm?? 579e0acd (empty) b
+      @origin2 (behind by 2 commits): qpvuntsm?? 579e0acd (empty) b
     [EOF]
     ");
 }
@@ -1364,7 +1501,7 @@ fn test_bookmark_track_untrack_patterns() {
     );
 
     // Fetch new commit without auto tracking
-    test_env.add_config("git.auto-local-bookmark = false");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = ''");
     let output = work_dir.run_jj(["git", "fetch"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
@@ -1406,16 +1543,15 @@ fn test_bookmark_track_untrack_patterns() {
     // Track/untrack unknown bookmark
     insta::assert_snapshot!(work_dir.run_jj(["bookmark", "track", "glob:maine@*"]), @r"
     ------- stderr -------
-    Error: No matching remote bookmarks for patterns: maine@*
+    Nothing changed.
     [EOF]
-    [exit status: 1]
     ");
     insta::assert_snapshot!(
         work_dir.run_jj(["bookmark", "untrack", "maine@origin", "glob:maine@o*"]), @r"
     ------- stderr -------
-    Error: No matching remote bookmarks for patterns: maine@origin, maine@o*
+    Warning: No matching remote bookmarks for names: maine@origin
+    Nothing changed.
     [EOF]
-    [exit status: 1]
     ");
 
     // Track already tracked bookmark
@@ -1501,7 +1637,7 @@ fn test_bookmark_track_untrack_patterns() {
 #[test]
 fn test_bookmark_list() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
 
     // Initialize remote refs
     test_env.run_jj_in(".", ["git", "init", "remote"]).success();
@@ -1536,7 +1672,13 @@ fn test_bookmark_list() {
         .run_jj(["new", "root()", "-m", "local-only"])
         .success();
     local_dir
-        .run_jj(["bookmark", "create", "local-only", "absent-tracked"])
+        .run_jj([
+            "--config=remotes.origin.auto-track-bookmarks=''",
+            "bookmark",
+            "create",
+            "local-only",
+            "absent-tracked",
+        ])
         .success();
 
     // Mutate refs in local repository
@@ -1752,7 +1894,7 @@ fn test_bookmark_list() {
 #[test]
 fn test_bookmark_list_filtered() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
 
     // Initialize remote refs
@@ -1783,7 +1925,12 @@ fn test_bookmark_list_filtered() {
         .run_jj(["new", "root()", "-m", "local-keep"])
         .success();
     local_dir
-        .run_jj(["bookmark", "create", "local-keep"])
+        .run_jj([
+            "--config=remotes.origin.auto-track-bookmarks=''",
+            "bookmark",
+            "create",
+            "local-keep",
+        ])
         .success();
 
     // Mutate refs in local repository
@@ -1854,7 +2001,7 @@ fn test_bookmark_list_filtered() {
       @origin (ahead by 1 commits, behind by 1 commits): royxmykx hidden 331d500d (empty) remote-rewrite
     [EOF]
     ");
-    insta::assert_snapshot!(query(&["-rbookmarks(remote-rewrite)"]), @r"
+    insta::assert_snapshot!(query(&["-rbookmarks(glob:remote-rewrite)"]), @r"
     remote-rewrite: royxmykx e6970e0e (empty) rewritten
       @origin (ahead by 1 commits, behind by 1 commits): royxmykx hidden 331d500d (empty) remote-rewrite
     [EOF]
@@ -1868,7 +2015,7 @@ fn test_bookmark_list_filtered() {
       @origin (ahead by 1 commits, behind by 1 commits): royxmykx hidden 331d500d (empty) remote-rewrite
     [EOF]
     ");
-    insta::assert_snapshot!(query(&["--all-remotes", "-rbookmarks(remote-rewrite)"]), @r"
+    insta::assert_snapshot!(query(&["--all-remotes", "-rbookmarks(glob:remote-rewrite)"]), @r"
     remote-rewrite: royxmykx e6970e0e (empty) rewritten
       @git: royxmykx e6970e0e (empty) rewritten
       @origin (ahead by 1 commits, behind by 1 commits): royxmykx hidden 331d500d (empty) remote-rewrite
@@ -1888,7 +2035,7 @@ fn test_bookmark_list_filtered() {
     Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
-    insta::assert_snapshot!(query(&["--remote", "glob:gi?"]), @r"
+    insta::assert_snapshot!(query(&["--remote", "glob:'gi?'"]), @r"
     local-keep: kpqxywon 4b2bc95c (empty) local-keep
       @git: kpqxywon 4b2bc95c (empty) local-keep
     remote-keep: rlvkpnrz c2f2ee40 (empty) remote-keep
@@ -1923,7 +2070,7 @@ fn test_bookmark_list_filtered() {
     Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
-    insta::assert_snapshot!(query(&["-rbookmarks(remote-delete)"]), @"");
+    insta::assert_snapshot!(query(&["-rbookmarks(glob:remote-delete)"]), @"");
     insta::assert_snapshot!(query(&["-rremote-delete"]), @r"
     ------- stderr -------
     Error: Revision `remote-delete` doesn't exist
@@ -1933,7 +2080,7 @@ fn test_bookmark_list_filtered() {
     ");
 
     // Name patterns are OR-ed.
-    insta::assert_snapshot!(query(&["glob:*-keep", "remote-delete"]), @r"
+    insta::assert_snapshot!(query(&["glob:*-keep", "glob:remote-* & glob:*-delete"]), @r"
     local-keep: kpqxywon 4b2bc95c (empty) local-keep
     remote-delete (deleted)
       @origin: zsuskuln 0e6b7968 (empty) remote-delete
@@ -1944,14 +2091,22 @@ fn test_bookmark_list_filtered() {
     [EOF]
     ");
 
-    // Unmatched name pattern shouldn't be an error. A warning can be added later.
-    insta::assert_snapshot!(query(&["local-keep", "glob:push-*"]), @r"
+    // Unmatched exact name pattern should be warned. "remote-delete" exists in
+    // remote. "remote-rewrite" exists, but isn't included in the match.
+    insta::assert_snapshot!(
+        query(&["local-keep", "glob:push-*", "unknown | remote-delete ~ remote-rewrite"]), @r"
     local-keep: kpqxywon 4b2bc95c (empty) local-keep
+    remote-delete (deleted)
+      @origin: zsuskuln 0e6b7968 (empty) remote-delete
+    [EOF]
+    ------- stderr -------
+    Warning: No matching bookmarks for names: unknown
+    Hint: Bookmarks marked as deleted can be *deleted permanently* on the remote by running `jj git push --deleted`. Use `jj bookmark forget` if you don't want that.
     [EOF]
     ");
 
     // Name pattern and revset are OR-ed.
-    insta::assert_snapshot!(query(&["local-keep", "-rbookmarks(remote-rewrite)"]), @r"
+    insta::assert_snapshot!(query(&["local-keep", "-rbookmarks(glob:remote-rewrite)"]), @r"
     local-keep: kpqxywon 4b2bc95c (empty) local-keep
     remote-rewrite: royxmykx e6970e0e (empty) rewritten
       @origin (ahead by 1 commits, behind by 1 commits): royxmykx hidden 331d500d (empty) remote-rewrite
@@ -1961,7 +2116,7 @@ fn test_bookmark_list_filtered() {
     // … but still filtered by --remote
     insta::assert_snapshot!(query(&[
         "local-keep",
-        "-rbookmarks(remote-rewrite)",
+        "-rbookmarks(glob:remote-rewrite)",
         "--remote",
         "git",
     ]), @r"
@@ -1971,6 +2126,21 @@ fn test_bookmark_list_filtered() {
       @git: royxmykx e6970e0e (empty) rewritten
     [EOF]
     ");
+
+    // Syntax error in name pattern
+    insta::assert_snapshot!(query(&["foo &"]), @r"
+    ------- stderr -------
+    Error: Failed to parse name pattern: Syntax error
+    Caused by:  --> 1:6
+      |
+    1 | foo &
+      |      ^---
+      |
+      = expected `::`, `..`, `~`, or <primary>
+    Hint: See https://docs.jj-vcs.dev/latest/revsets/ or use `jj help -k revsets` for revsets syntax and how to quote symbols.
+    [EOF]
+    [exit status: 1]
+    ");
 }
 
 #[test]
@@ -2008,7 +2178,7 @@ fn test_bookmark_list_quoted_name() {
 #[test]
 fn test_bookmark_list_much_remote_divergence() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
 
     // Initialize remote refs
     test_env.run_jj_in(".", ["git", "init", "remote"]).success();
@@ -2042,7 +2212,12 @@ fn test_bookmark_list_much_remote_divergence() {
         local_dir.run_jj(["new", "-m", "local-only"]).success();
     }
     local_dir
-        .run_jj(["bookmark", "create", "local-only"])
+        .run_jj([
+            "--config=remotes.origin.auto-track-bookmarks=''",
+            "bookmark",
+            "create",
+            "local-only",
+        ])
         .success();
 
     // Mutate refs in local repository
@@ -2062,7 +2237,8 @@ fn test_bookmark_list_much_remote_divergence() {
 #[test]
 fn test_bookmark_list_tracked() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
+    test_env.add_config("remotes.upstream.auto-track-bookmarks = 'glob:*'");
 
     // Initialize remote refs
     test_env.run_jj_in(".", ["git", "init", "remote"]).success();
@@ -2136,7 +2312,13 @@ fn test_bookmark_list_tracked() {
         .run_jj(["new", "root()", "-m", "local-only"])
         .success();
     local_dir
-        .run_jj(["bookmark", "create", "local-only"])
+        .run_jj([
+            "--config=remotes.origin.auto-track-bookmarks=''",
+            "--config=remotes.upstream.auto-track-bookmarks=''",
+            "bookmark",
+            "create",
+            "local-only",
+        ])
         .success();
 
     // Mutate refs in local repository
@@ -2252,15 +2434,14 @@ fn test_bookmark_list_conflicted() {
     work_dir.run_jj(["new", "root()", "-m", "b"]).success();
     work_dir.run_jj(["bookmark", "create", "bar"]).success();
     work_dir
-        .run_jj(["bookmark", "create", "foo", "-r", "description(a)"])
+        .run_jj(["bookmark", "create", "foo", "-rsubject(glob:a)"])
         .success();
     work_dir
         .run_jj([
             "bookmark",
             "create",
             "foo",
-            "-r",
-            "description(b)",
+            "-rsubject(glob:b)",
             "--at-op=@-",
         ])
         .success();
@@ -2277,6 +2458,9 @@ fn test_bookmark_list_conflicted() {
       + rlvkpnrz 4e1b2d80 (empty) a
       + kkmpptxz a82129fb (empty) b
     [EOF]
+    ------- stderr -------
+    Hint: Some bookmarks have conflicts. Use `jj bookmark set <name> -r <rev>` to resolve.
+    [EOF]
     ");
 }
 
@@ -2392,6 +2576,82 @@ fn test_bookmark_list_sort_overriding_config() {
     ");
 }
 
+#[test]
+fn test_create_and_set_auto_track_bookmarks() {
+    let test_env = TestEnvironment::default();
+    let root_dir = test_env.work_dir("");
+    root_dir
+        .run_jj(["git", "init", "--colocate", "origin"])
+        .success();
+    test_env.add_config(
+        "
+        [remotes.origin]
+        auto-track-bookmarks = 'glob:mine/*'
+        [remotes.fork]
+        auto-track-bookmarks = 'glob:*'
+        ",
+    );
+
+    root_dir.run_jj(["git", "init", "repo"]).success();
+    let repo_dir = test_env.work_dir("repo");
+    repo_dir
+        .run_jj(["git", "remote", "add", "origin", "../origin/.git"])
+        .success();
+    repo_dir
+        .run_jj(["git", "remote", "add", "fork", "dummy"])
+        .success();
+
+    // jj bookmark create obeys remotes.<name>.auto-track-bookmarks
+    repo_dir
+        .run_jj(["bookmark", "create", "mine/create", "not-mine/create"])
+        .success();
+    let output = repo_dir.run_jj([
+        "bookmark",
+        "list",
+        "--all",
+        "mine/create",
+        "not-mine/create",
+    ]);
+    insta::assert_snapshot!(output, @r"
+    mine/create: rlvkpnrz 7eb1c95e (empty) (no description set)
+      @fork (not created yet)
+      @origin (not created yet)
+    not-mine/create: rlvkpnrz 7eb1c95e (empty) (no description set)
+      @fork (not created yet)
+    [EOF]
+    ");
+    repo_dir.run_jj(["commit", "--message", "create"]).success();
+
+    // jj bookmark set obeys remotes.<name>.auto-track-bookmarks
+    repo_dir
+        .run_jj(["bookmark", "set", "mine/set", "not-mine/set"])
+        .success();
+    let output = repo_dir.run_jj(["bookmark", "list", "--all", "mine/set", "not-mine/set"]);
+    insta::assert_snapshot!(output, @r"
+    mine/set: yqosqzyt 5fbe2b20 (empty) (no description set)
+      @fork (not created yet)
+      @origin (not created yet)
+    not-mine/set: yqosqzyt 5fbe2b20 (empty) (no description set)
+      @fork (not created yet)
+    [EOF]
+    ");
+    repo_dir.run_jj(["commit", "--message", "set"]).success();
+
+    // jj bookmark create warns when auto-tracking existing bookmark
+    repo_dir.run_jj(["git", "push"]).success();
+    repo_dir
+        .run_jj(["bookmark", "forget", "mine/create"])
+        .success();
+    let output = repo_dir.run_jj(["bookmark", "create", "mine/create"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Warning: Target revision is empty.
+    Warning: Auto-tracking bookmark that exists on the remote: mine/create@origin
+    Created 1 bookmarks pointing to znkkpsqq 2e899fb8 mine/create* | (empty) (no description set)
+    [EOF]
+    ");
+}
+
 #[must_use]
 fn get_log_output(work_dir: &TestWorkDir) -> CommandOutput {
     let template = r#"bookmarks ++ " " ++ commit_id.short()"#;
