@@ -1069,8 +1069,8 @@ fn test_workspaces() {
 
     let output = main_dir.complete_fish(["workspace", "forget", "def"]);
     insta::assert_snapshot!(output, @r"
-    def-second	(no description set)	
-    default	initial	
+    def-second	(no description set)
+    default	initial
     [EOF]
     ");
 }
@@ -1357,7 +1357,7 @@ fn test_files() {
     │  M f_modified
     │  M f_not_yet_copied
     │  R {f_not_yet_renamed => f_renamed}
-    │ ×  royxmykx test.user@example.com 2001-02-03 08:05:14 conflicted cf10549a conflict
+    │ ×  royxmykx test.user@example.com 2001-02-03 08:05:14 conflicted a24d7569 conflict
     ├─╯  conflicted
     │    A f_added_2
     │    A f_dir/dir_file_1
@@ -1388,7 +1388,7 @@ fn test_files() {
     let work_dir = test_env.work_dir("repo");
 
     let output = work_dir.complete_fish(["file", "show", "f_"]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    insta::assert_snapshot!(output, @r"
     f_added
     f_added_2
     f_another_renamed_2
@@ -1401,8 +1401,91 @@ fn test_files() {
     [EOF]
     ");
 
+    let output = work_dir.complete_fish(["file", "show", "./f_"]);
+    insta::assert_snapshot!(output, @r"
+    ./f_added
+    ./f_added_2
+    ./f_another_renamed_2
+    ./f_copied
+    ./f_dir/
+    ./f_modified
+    ./f_not_yet_copied
+    ./f_renamed
+    ./f_unchanged
+    [EOF]
+    ");
+
+    let output = work_dir.complete_fish(["file", "show", "f_dir"]);
+    insta::assert_snapshot!(output, @r"
+    f_dir/
+    [EOF]
+    ");
+
+    let output = work_dir.complete_fish(["file", "show", "f_dir/"]);
+    insta::assert_snapshot!(output, @r"
+    f_dir/dir_file_1
+    f_dir/dir_file_2
+    f_dir/dir_file_3
+    f_dir/f_renamed_3
+    [EOF]
+    ");
+
+    let output = work_dir.complete_fish(["file", "show", "f_dir/../"]);
+    insta::assert_snapshot!(output, @r"
+    f_dir/../f_added
+    f_dir/../f_added_2
+    f_dir/../f_another_renamed_2
+    f_dir/../f_copied
+    f_dir/../f_dir/
+    f_dir/../f_modified
+    f_dir/../f_not_yet_copied
+    f_dir/../f_renamed
+    f_dir/../f_unchanged
+    [EOF]
+    ");
+
+    let output = work_dir.complete_fish(["file", "show", "f_dir/../f_dir/"]);
+    insta::assert_snapshot!(output, @r"
+    f_dir/../f_dir/dir_file_1
+    f_dir/../f_dir/dir_file_2
+    f_dir/../f_dir/dir_file_3
+    f_dir/../f_dir/f_renamed_3
+    [EOF]
+    ");
+
+    let subdir = work_dir.dir("f_dir");
+    let output = subdir.complete_fish(["file", "show", "dir_"]);
+    insta::assert_snapshot!(output, @r"
+    dir_file_1
+    dir_file_2
+    dir_file_3
+    [EOF]
+    ");
+
+    let output = subdir.complete_fish(["file", "show", "./"]);
+    insta::assert_snapshot!(output, @r"
+    ./dir_file_1
+    ./dir_file_2
+    ./dir_file_3
+    ./f_renamed_3
+    [EOF]
+    ");
+
+    let output = subdir.complete_fish(["file", "show", "../"]);
+    insta::assert_snapshot!(output, @r"
+    ../f_added
+    ../f_added_2
+    ../f_another_renamed_2
+    ../f_copied
+    ../f_modified
+    ../f_not_yet_copied
+    ../f_renamed
+    ../f_unchanged
+    [EOF]
+    ");
+
     let output = work_dir.complete_fish(["file", "annotate", "-r@-", "f_"]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    insta::assert_snapshot!(output, @r"
     f_added
     f_another_renamed_2
     f_copied
@@ -1415,7 +1498,7 @@ fn test_files() {
     ");
 
     let output = work_dir.complete_fish(["diff", "-r", "@-", "f_"]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    insta::assert_snapshot!(output, @r"
     f_added	Added
     f_another_renamed_2	Renamed
     f_copied	Copied
@@ -1430,22 +1513,45 @@ fn test_files() {
     [EOF]
     ");
 
+    let output = work_dir.complete_fish(["diff", "-r", "@-", "f_dir/../"]);
+    insta::assert_snapshot!(output, @r"
+    f_dir/../f_added	Added
+    f_dir/../f_another_renamed_2	Renamed
+    f_dir/../f_copied	Copied
+    f_dir/../f_deleted	Deleted
+    f_dir/../f_dir/
+    f_dir/../f_modified	Modified
+    f_dir/../f_not_yet_copied	Modified
+    f_dir/../f_not_yet_renamed	Renamed
+    f_dir/../f_not_yet_renamed_2	Renamed
+    f_dir/../f_not_yet_renamed_3	Renamed
+    f_dir/../f_renamed	Renamed
+    [EOF]
+    ");
+
+    // Given that the path prefix uses the main separator (e.g. `\` on Windows),
+    // check that the completion continues to use the same separator.
+    // The assertion maps the main separator to some arbitrary fictitious separator
+    // (`→`) which is not used by real OSes (yet) to check that the main separator
+    // is preserved on platforms where it differs from `/`.
     let output = work_dir.complete_fish([
         "diff",
         "-r",
         "@-",
         &format!("f_dir{}", std::path::MAIN_SEPARATOR),
     ]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
-    f_dir/dir_file_1	Added
-    f_dir/dir_file_2	Added
-    f_dir/dir_file_3	Added
-    f_dir/f_renamed_3	Renamed
-    [EOF]
-    ");
+    insta::assert_snapshot!(
+        output.normalize_stdout_with(|s| s.replace(std::path::MAIN_SEPARATOR, "→")),
+        @r"
+        f_dir→dir_file_1	Added
+        f_dir→dir_file_2	Added
+        f_dir→dir_file_3	Added
+        f_dir→f_renamed_3	Renamed
+        [EOF]
+        ");
 
     let output = work_dir.complete_fish(["diff", "--from", "root()", "--to", "@-", "f_"]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    insta::assert_snapshot!(output, @r"
     f_added	Added
     f_another_renamed_2	Added
     f_copied	Added
@@ -1493,7 +1599,7 @@ fn test_files() {
         "--from=interdiff_from",
         "f_",
     ]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    insta::assert_snapshot!(output, @r"
     f_interdiff_only_from	Added
     f_interdiff_same	Added
     f_interdiff_only_to	Added
@@ -1503,7 +1609,7 @@ fn test_files() {
 
     // squash has a different behavior with --from and --to flags
     let output = work_dir.complete_fish(["squash", "-f=first", "f_"]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    insta::assert_snapshot!(output, @r"
     f_deleted	Added
     f_modified	Added
     f_not_yet_copied	Added
@@ -1515,14 +1621,14 @@ fn test_files() {
     ");
 
     let output = work_dir.complete_fish(["resolve", "-r=conflicted", "f_"]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    insta::assert_snapshot!(output, @r"
     f_dir/
     f_modified
     [EOF]
     ");
 
     let output = work_dir.complete_fish(["file", "list", "-r=first", "f_"]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    insta::assert_snapshot!(output, @r"
     f_deleted
     f_modified
     f_not_yet_copied
@@ -1534,7 +1640,7 @@ fn test_files() {
     ");
 
     let output = work_dir.complete_fish(["log", "f_"]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    insta::assert_snapshot!(output, @r"
     f_added
     f_added_2
     f_another_renamed_2
@@ -1548,7 +1654,7 @@ fn test_files() {
     ");
 
     let output = work_dir.complete_fish(["log", "-r=first", "--revisions", "conflicted", "f_"]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    insta::assert_snapshot!(output, @r"
     f_added_2
     f_deleted
     f_dir/
@@ -1566,14 +1672,14 @@ fn test_files() {
     insta::assert_snapshot!(output, @"");
 
     let output = work_dir.complete_fish(["absorb", "f_"]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    insta::assert_snapshot!(output, @r"
     f_added_2	Added
     f_modified	Modified
     [EOF]
     ");
 
     let output = work_dir.complete_fish(["absorb", "-f=conflicted", "f_"]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    insta::assert_snapshot!(output, @r"
     f_added_2	Added
     f_dir/
     f_modified	Modified
