@@ -227,6 +227,136 @@ fn test_concurrent_snapshot_wc_reloadable() {
     ");
 }
 
+#[test]
+fn test_git_head_race_condition() {
+    // Test for race condition where concurrent jj processes create divergent
+    // operations when importing/exporting Git HEAD. This test spawns two
+    // processes in parallel: one running `jj debug snapshot` repeatedly
+    // (which imports Git HEAD) and another running `jj next/prev` repeatedly
+    // (which exports Git HEAD). Without the fix, this would create divergent
+    // operations.
+
+    let test_env = TestEnvironment::default();
+    test_env
+        .run_jj_in(".", ["git", "init", "--colocate", "repo"])
+        .success();
+    let work_dir = test_env.work_dir("repo");
+
+    // Create a large initial working copy to make snapshotting slower
+    for j in 0..200 {
+        let filename = format!("file_{j}");
+        let content = format!("content for file {j}\n").repeat(100);
+        work_dir.write_file(&filename, &content);
+    }
+    work_dir.run_jj(["commit", "-m", "initial"]).success();
+
+    // Remember the initial commit for later verification
+    let initial_commit = work_dir
+        .run_jj(["log", "--no-graph", "-T=commit_id", "-r=@-"])
+        .success()
+        .stdout
+        .into_raw();
+    let initial_commit = initial_commit.trim().to_owned();
+
+    // Create additional commits to iterate through with jj next/prev
+    for i in 0..10 {
+        for j in 0..50 {
+            let filename = format!("file_{i}_{j}");
+            let content = format!("content for commit {i} file {j}\n").repeat(100);
+            work_dir.write_file(&filename, &content);
+        }
+        work_dir
+            .run_jj(["commit", "-m", &format!("commit {i}")])
+            .success();
+    }
+
+    // Extract environment from TestEnvironment to use in spawned processes
+    let base_cmd = test_env.new_jj_cmd();
+    let jj_bin = base_cmd.get_program().to_owned();
+    let base_env: Vec<_> = base_cmd
+        .get_envs()
+        .filter_map(|(k, v)| v.map(|v| (k.to_owned(), v.to_owned())))
+        // Filter out timestamp and randomness seed so each command gets different values
+        .filter(|(k, _)| k != "JJ_TIMESTAMP" && k != "JJ_OP_TIMESTAMP" && k != "JJ_RANDOMNESS_SEED")
+        .collect();
+
+    let repo_path = work_dir.root();
+    let duration = std::time::Duration::from_secs(5);
+    let start = std::time::Instant::now();
+
+    std::thread::scope(|s| {
+        s.spawn(|| {
+            while start.elapsed() < duration {
+                // Snapshot repeatedly without delay
+                let mut cmd = std::process::Command::new(&jj_bin);
+                cmd.current_dir(repo_path);
+                cmd.args(["debug", "snapshot"]);
+                for (key, value) in &base_env {
+                    cmd.env(key, value);
+                }
+                let output = cmd.output().expect("Failed to spawn jj");
+                assert!(output.status.success(), "jj debug snapshot failed");
+            }
+        });
+
+        s.spawn(|| {
+            let mut count = 0;
+            while start.elapsed() < duration {
+                // Go through commit history back and forth
+                let direction = if count % 20 < 10 { "prev" } else { "next" };
+                let mut cmd = std::process::Command::new(&jj_bin);
+                cmd.current_dir(repo_path);
+                cmd.arg(direction);
+                for (key, value) in &base_env {
+                    cmd.env(key, value);
+                }
+                let output = cmd.output().expect("Failed to spawn jj");
+                assert!(output.status.success(), "jj {direction} failed");
+                count += 1;
+            }
+        });
+    });
+
+    const IMPORT_GIT_HEAD: &str = "import git head";
+
+    // Check for concurrent operations
+    let output = work_dir.run_jj(["op", "log", "-T", "description", "--ignore-working-copy"]);
+    let concurrent_count = output
+        .stdout
+        .raw()
+        .lines()
+        .filter(|line| line.contains(IMPORT_GIT_HEAD))
+        .count();
+
+    if concurrent_count > 0 {
+        eprintln!("Found {concurrent_count} concurrent operations:");
+        eprintln!("{}", output.stdout.raw());
+        panic!("Race condition detected: {concurrent_count} concurrent operations found");
+    }
+
+    // Verify the operation description for importing Git HEAD hasn't changed
+    // First ensure we're not already on the initial commit (prev/next loop may have
+    // ended there)
+    work_dir.run_jj(["new"]).success();
+    // Use git to checkout the initial commit, then trigger import
+    std::process::Command::new("git")
+        .current_dir(work_dir.root())
+        .args(["checkout", "-q", &initial_commit])
+        .status()
+        .unwrap();
+
+    let last_op = work_dir
+        .run_jj(["op", "log", "-T", "description", "--limit=1"])
+        .success()
+        .stdout
+        .into_raw();
+
+    assert!(
+        last_op.contains(IMPORT_GIT_HEAD),
+        "Expected last operation to contain '{IMPORT_GIT_HEAD}', got: {last_op:?}"
+    );
+}
+
 #[must_use]
 fn get_log_output(work_dir: &TestWorkDir) -> CommandOutput {
     work_dir.run_jj(["log", "-T", "description"])
