@@ -154,6 +154,7 @@ fn test_debug_index() {
     let work_dir = test_env.work_dir("repo");
     let output = work_dir.run_jj(["debug", "index"]);
     assert_snapshot!(filter_index_stats(output), @r"
+    === Commits ===
     Number of commits: 2
     Number of merges: 0
     Max generation number: 1
@@ -163,6 +164,39 @@ fn test_debug_index() {
       Level 0:
         Number of commits: 2
         Name: [hash]
+    === Changed paths ===
+    Indexed commits: none
+    Stats per level:
+    [EOF]
+    ");
+
+    // Enable changed-path index, index one commit
+    let output = work_dir.run_jj(["debug", "index-changed-paths", "-n1"]);
+    assert_snapshot!(output, @r"
+    ------- stderr -------
+    Finished indexing 1..2 commits.
+    [EOF]
+    ");
+    let output = work_dir.run_jj(["debug", "index"]);
+    assert_snapshot!(filter_index_stats(output), @r"
+    === Commits ===
+    Number of commits: 2
+    Number of merges: 0
+    Max generation number: 1
+    Number of heads: 1
+    Number of changes: 2
+    Stats per level:
+      Level 0:
+        Number of commits: 2
+        Name: [hash]
+    === Changed paths ===
+    Indexed commits: 1..2
+    Stats per level:
+      Level 0:
+        Number of commits: 1
+        Number of changed paths: 0
+        Number of paths: 0
+        Name: [hash]
     [EOF]
     ");
 }
@@ -176,6 +210,7 @@ fn test_debug_reindex() {
     work_dir.run_jj(["new"]).success();
     let output = work_dir.run_jj(["debug", "index"]);
     assert_snapshot!(filter_index_stats(output), @r"
+    === Commits ===
     Number of commits: 4
     Number of merges: 0
     Max generation number: 3
@@ -188,6 +223,9 @@ fn test_debug_reindex() {
       Level 1:
         Number of commits: 1
         Name: [hash]
+    === Changed paths ===
+    Indexed commits: none
+    Stats per level:
     [EOF]
     ");
     let output = work_dir.run_jj(["debug", "reindex"]);
@@ -198,6 +236,7 @@ fn test_debug_reindex() {
     ");
     let output = work_dir.run_jj(["debug", "index"]);
     assert_snapshot!(filter_index_stats(output), @r"
+    === Commits ===
     Number of commits: 4
     Number of merges: 0
     Max generation number: 3
@@ -207,6 +246,9 @@ fn test_debug_reindex() {
       Level 0:
         Number of commits: 4
         Name: [hash]
+    === Changed paths ===
+    Indexed commits: none
+    Stats per level:
     [EOF]
     ");
 }
@@ -288,18 +330,6 @@ fn test_debug_tree() {
     );
 }
 
-#[test]
-fn test_debug_operation_id() {
-    let test_env = TestEnvironment::default();
-    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let work_dir = test_env.work_dir("repo");
-    let output = work_dir.run_jj(["debug", "operation", "--display", "id"]);
-    assert_snapshot!(filter_index_stats(output), @r"
-    8f47435a3990362feaf967ca6de2eb0a31c8b883dfcb66fba5c22200d12bbe61e3dc8bc855f1f6879285fcafaf85ac792f9a43bcc36e57d28737d18347d5e752
-    [EOF]
-    ");
-}
-
 fn filter_index_stats(output: CommandOutput) -> CommandOutput {
     let regex = Regex::new(r"    Name: [0-9a-z]+").unwrap();
     output.normalize_stdout_with(|text| regex.replace_all(&text, "    Name: [hash]").into_owned())
