@@ -295,7 +295,7 @@ fn test_duplicate_destination() {
     ");
 
     // Duplicate a single commit onto a single destination.
-    let output = work_dir.run_jj(["duplicate", "a1", "-d", "c"]);
+    let output = work_dir.run_jj(["duplicate", "a1", "-o", "c"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Duplicated 5d93a4b8f4bd as kxryzmor 08f0f980 a1
@@ -318,7 +318,7 @@ fn test_duplicate_destination() {
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
 
     // Duplicate a single commit onto multiple destinations.
-    let output = work_dir.run_jj(["duplicate", "a1", "-d", "c", "-d", "d"]);
+    let output = work_dir.run_jj(["duplicate", "a1", "-o", "c", "-o", "d"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Duplicated 5d93a4b8f4bd as xznxytkn 0bad88fb a1
@@ -342,7 +342,7 @@ fn test_duplicate_destination() {
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
 
     // Duplicate a single commit onto its descendant.
-    let output = work_dir.run_jj(["duplicate", "a1", "-d", "a3"]);
+    let output = work_dir.run_jj(["duplicate", "a1", "-o", "a3"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Warning: Duplicating commit 5d93a4b8f4bd as a descendant of itself
@@ -367,7 +367,7 @@ fn test_duplicate_destination() {
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
     // Duplicate multiple commits without a direct ancestry relationship onto a
     // single destination.
-    let output = work_dir.run_jj(["duplicate", "-r=a1", "-r=b", "-d", "c"]);
+    let output = work_dir.run_jj(["duplicate", "-r=a1", "-r=b", "-o", "c"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Duplicated 5d93a4b8f4bd as pzsxstzt 663f101c a1
@@ -394,7 +394,7 @@ fn test_duplicate_destination() {
 
     // Duplicate multiple commits without a direct ancestry relationship onto
     // multiple destinations.
-    let output = work_dir.run_jj(["duplicate", "-r=a1", "b", "-d", "c", "-d", "d"]);
+    let output = work_dir.run_jj(["duplicate", "-r=a1", "b", "-o", "c", "-o", "d"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Duplicated 5d93a4b8f4bd as qmkrwlvp b0f531b0 a1
@@ -422,7 +422,7 @@ fn test_duplicate_destination() {
 
     // Duplicate multiple commits with an ancestry relationship onto a
     // single destination.
-    let output = work_dir.run_jj(["duplicate", "a1", "a3", "-d", "c"]);
+    let output = work_dir.run_jj(["duplicate", "a1", "a3", "-o", "c"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Duplicated 5d93a4b8f4bd as qwyusntz 5f1c245f a1
@@ -448,7 +448,7 @@ fn test_duplicate_destination() {
 
     // Duplicate multiple commits with an ancestry relationship onto
     // multiple destinations.
-    let output = work_dir.run_jj(["duplicate", "a1", "a3", "-d", "c", "-d", "d"]);
+    let output = work_dir.run_jj(["duplicate", "a1", "a3", "-o", "c", "-o", "d"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Duplicated 5d93a4b8f4bd as soqnvnyz bc6678fd a1
@@ -2400,7 +2400,7 @@ fn test_rebase_duplicates() {
     [EOF]
     ");
 
-    let output = work_dir.run_jj(["rebase", "-s", "b", "-d", "root()"]);
+    let output = work_dir.run_jj(["rebase", "-s", "b", "-o", "root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 4 commits to destination
@@ -2467,7 +2467,7 @@ fn test_duplicate_description_template() {
 
     // Test empty template
     test_env.add_config("templates.duplicate_description = ''");
-    let output = work_dir.run_jj(["duplicate", "b", "-d", "root()"]);
+    let output = work_dir.run_jj(["duplicate", "b", "-o", "root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Duplicated 6960bbcfd5b1 as kpqxywon 33044659 (no description set)
@@ -2479,7 +2479,7 @@ fn test_duplicate_description_template() {
     let output = work_dir.run_jj([
         "duplicate",
         "c",
-        "-d",
+        "-o",
         "root()",
         // Use an argument here so we can actually see the log in the last test
         // (We don't have a way to remove a config in TestEnvironment)
