@@ -26,7 +26,7 @@ fn test_evolog_with_or_without_diff() {
     work_dir.write_file("file1", "foo\nbar\n");
     work_dir.write_file("file2", "foo\n");
     work_dir
-        .run_jj(["rebase", "-r", "@", "-d", "root()"])
+        .run_jj(["rebase", "-r", "@", "-o", "root()"])
         .success();
     work_dir.write_file("file1", "resolved\n");
 
@@ -34,13 +34,13 @@ fn test_evolog_with_or_without_diff() {
     insta::assert_snapshot!(output, @r"
     @  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 33c10ace
     │  my description
-    │  -- operation 3499115d3831 snapshot working copy
-    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 7f56b2a0 conflict
+    │  -- operation ae9fa2e2ea1c snapshot working copy
+    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 1d598134 conflict
     │  my description
-    │  -- operation eb87ec366530 rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    │  -- operation b5bfb716eb6f rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
     ○  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 51e08f95
     │  my description
-    │  -- operation 18a971ce330a snapshot working copy
+    │  -- operation 826347115e2d snapshot working copy
     ○  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 b955b72e
        (empty) my description
        -- operation e0f8e58b3800 new empty commit
@@ -52,13 +52,13 @@ fn test_evolog_with_or_without_diff() {
     insta::assert_snapshot!(output, @r"
     [1m[38;5;2m@[0m  [1m[38;5;13mr[38;5;8mlvkpnrz[39m [38;5;3mtest.user@example.com[39m [38;5;14m2001-02-03 08:05:10[39m [38;5;12m3[38;5;8m3c10ace[39m[0m
     │  [1mmy description[0m
-    │  [38;5;8m--[39m operation [38;5;4m3499115d3831[39m snapshot working copy
-    [1m[38;5;1m×[0m  [1m[39mr[0m[38;5;8mlvkpnrz[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:09[39m [1m[38;5;4m7[0m[38;5;8mf56b2a0[39m [38;5;1mconflict[39m
+    │  [38;5;8m--[39m operation [38;5;4mae9fa2e2ea1c[39m snapshot working copy
+    [1m[38;5;1m×[0m  [1m[39mr[0m[38;5;8mlvkpnrz[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:09[39m [1m[38;5;4m1[0m[38;5;8md598134[39m [38;5;1mconflict[39m
     │  my description
-    │  [38;5;8m--[39m operation [38;5;4meb87ec366530[39m rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    │  [38;5;8m--[39m operation [38;5;4mb5bfb716eb6f[39m rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
     ○  [1m[39mr[0m[38;5;8mlvkpnrz[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:09[39m [1m[38;5;4m5[0m[38;5;8m1e08f95[39m
     │  my description
-    │  [38;5;8m--[39m operation [38;5;4m18a971ce330a[39m snapshot working copy
+    │  [38;5;8m--[39m operation [38;5;4m826347115e2d[39m snapshot working copy
     ○  [1m[39mr[0m[38;5;8mlvkpnrz[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:08[39m [1m[38;5;4mb[0m[38;5;8m955b72e[39m
        [38;5;2m(empty)[39m my description
        [38;5;8m--[39m operation [38;5;4me0f8e58b3800[39m new empty commit
@@ -71,7 +71,7 @@ fn test_evolog_with_or_without_diff() {
     insta::assert_snapshot!(output, @r"
     @  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 33c10ace
     │  my description
-    │  -- operation 3499115d3831 snapshot working copy
+    │  -- operation ae9fa2e2ea1c snapshot working copy
     │  Resolved conflict in file1:
     │     1     : <<<<<<< Conflict 1 of 1
     │     2     : %%%%%%% Changes from base to side #1
@@ -80,12 +80,12 @@ fn test_evolog_with_or_without_diff() {
     │     5     : foo
     │     6     : bar
     │     7    1: >>>>>>> Conflict 1 of 1 endsresolved
-    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 7f56b2a0 conflict
+    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 1d598134 conflict
     │  my description
-    │  -- operation eb87ec366530 rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    │  -- operation b5bfb716eb6f rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
     ○  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 51e08f95
     │  my description
-    │  -- operation 18a971ce330a snapshot working copy
+    │  -- operation 826347115e2d snapshot working copy
     │  Modified regular file file1:
     │     1    1: foo
     │          2: bar
@@ -104,13 +104,13 @@ fn test_evolog_with_or_without_diff() {
     insta::assert_snapshot!(output, @r"
     @  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 33c10ace
     │  my description
-    │  -- operation 3499115d3831 snapshot working copy
-    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 7f56b2a0 conflict
+    │  -- operation ae9fa2e2ea1c snapshot working copy
+    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 1d598134 conflict
     │  my description
-    │  -- operation eb87ec366530 rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    │  -- operation b5bfb716eb6f rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
     ○  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 51e08f95
     │  my description
-    │  -- operation 18a971ce330a snapshot working copy
+    │  -- operation 826347115e2d snapshot working copy
     ○  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 b955b72e
        (empty) my description
        -- operation e0f8e58b3800 new empty commit
@@ -128,10 +128,10 @@ fn test_evolog_with_or_without_diff() {
     insta::assert_snapshot!(output, @r"
     @  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 33c10ace
     │  my description
-    │  -- operation 3499115d3831 snapshot working copy
-    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 7f56b2a0 conflict
+    │  -- operation ae9fa2e2ea1c snapshot working copy
+    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 1d598134 conflict
     │  my description
-    │  -- operation eb87ec366530 rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    │  -- operation b5bfb716eb6f rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
     [EOF]
     ");
 
@@ -140,13 +140,13 @@ fn test_evolog_with_or_without_diff() {
     insta::assert_snapshot!(output, @r"
     rlvkpnrz test.user@example.com 2001-02-03 08:05:10 33c10ace
     my description
-    -- operation 3499115d3831 snapshot working copy
-    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 7f56b2a0 conflict
+    -- operation ae9fa2e2ea1c snapshot working copy
+    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 1d598134 conflict
     my description
-    -- operation eb87ec366530 rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    -- operation b5bfb716eb6f rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
     rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 51e08f95
     my description
-    -- operation 18a971ce330a snapshot working copy
+    -- operation 826347115e2d snapshot working copy
     rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 b955b72e
     (empty) my description
     -- operation e0f8e58b3800 new empty commit
@@ -158,7 +158,7 @@ fn test_evolog_with_or_without_diff() {
     insta::assert_snapshot!(output, @r"
     rlvkpnrz test.user@example.com 2001-02-03 08:05:10 33c10ace
     my description
-    -- operation 3499115d3831 snapshot working copy
+    -- operation ae9fa2e2ea1c snapshot working copy
     diff --git a/file1 b/file1
     index 0000000000..2ab19ae607 100644
     --- a/file1
@@ -172,12 +172,12 @@ fn test_evolog_with_or_without_diff() {
     -bar
     ->>>>>>> Conflict 1 of 1 ends
     +resolved
-    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 7f56b2a0 conflict
+    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 1d598134 conflict
     my description
-    -- operation eb87ec366530 rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    -- operation b5bfb716eb6f rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
     rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 51e08f95
     my description
-    -- operation 18a971ce330a snapshot working copy
+    -- operation 826347115e2d snapshot working copy
     diff --git a/file1 b/file1
     index 257cc5642c..3bd1f0e297 100644
     --- a/file1
@@ -283,7 +283,7 @@ fn test_evolog_with_custom_symbols() {
     work_dir.write_file("file1", "foo\nbar\n");
     work_dir.write_file("file2", "foo\n");
     work_dir
-        .run_jj(["rebase", "-r", "@", "-d", "root()"])
+        .run_jj(["rebase", "-r", "@", "-o", "root()"])
         .success();
     work_dir.write_file("file1", "resolved\n");
 
@@ -293,13 +293,13 @@ fn test_evolog_with_custom_symbols() {
     insta::assert_snapshot!(output, @r"
     $  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 33c10ace
     │  my description
-    │  -- operation 3622beb20303 snapshot working copy
-    ┝  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 7f56b2a0 conflict
+    │  -- operation a51c9c093002 snapshot working copy
+    ┝  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 1d598134 conflict
     │  my description
-    │  -- operation eb87ec366530 rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    │  -- operation b5bfb716eb6f rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
     ┝  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 51e08f95
     │  my description
-    │  -- operation 18a971ce330a snapshot working copy
+    │  -- operation 826347115e2d snapshot working copy
     ┝  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 b955b72e
        (empty) my description
        -- operation e0f8e58b3800 new empty commit
@@ -407,16 +407,16 @@ fn test_evolog_squash() {
         .run_jj([
             "squash",
             "-msquashed 3",
-            "--from=description('fourth')|description('fifth')",
-            "--into=description('squash')",
+            "--from=subject(glob:fourth)|subject(glob:fifth)",
+            "--into=subject(glob:squash*)",
         ])
         .success();
 
-    let output = work_dir.run_jj(["evolog", "-p", "-r", "description('squash')"]);
+    let output = work_dir.run_jj(["evolog", "-p", "-rsubject(glob:squash*)"]);
     insta::assert_snapshot!(output, @r"
     ○      qpvuntsm test.user@example.com 2001-02-03 08:05:15 5f3281c6
     ├─┬─╮  squashed 3
-    │ │ │  -- operation 838e6d867fda squash commits into 5ec0619af5cb4f7707a556a71a6f96af0bc294d2
+    │ │ │  -- operation 3049d8383eb2 squash commits into 5ec0619af5cb4f7707a556a71a6f96af0bc294d2
     │ │ │  Modified commit description:
     │ │ │     1     : <<<<<<< Conflict 1 of 1
     │ │ │     2     : +++++++ Contents of side #1
@@ -428,7 +428,7 @@ fn test_evolog_squash() {
     │ │ │     8     : >>>>>>> Conflict 1 of 1 ends
     │ │ ○  vruxwmqv hidden test.user@example.com 2001-02-03 08:05:15 770795d0
     │ │ │  fifth
-    │ │ │  -- operation 1d38c000b52d snapshot working copy
+    │ │ │  -- operation 904e204f4bb0 snapshot working copy
     │ │ │  Added regular file file5:
     │ │ │          1: foo5
     │ │ ○  vruxwmqv hidden test.user@example.com 2001-02-03 08:05:14 2e0123d1
@@ -658,23 +658,21 @@ fn test_evolog_reverse_with_graph() {
     work_dir.run_jj(["describe", "-m", "b"]).success();
     work_dir.run_jj(["describe", "-m", "c"]).success();
     work_dir
-        .run_jj(["new", "-r", "description(c)", "-m", "d"])
+        .run_jj(["new", "-r", "subject(glob:c)", "-m", "d"])
         .success();
     work_dir
-        .run_jj(["new", "-r", "description(c)", "-m", "e"])
+        .run_jj(["new", "-r", "subject(glob:c)", "-m", "e"])
         .success();
     work_dir
         .run_jj([
             "squash",
-            "--from",
-            "description(d)|description(e)",
-            "--to",
-            "description(c)",
+            "--from=subject(glob:d)|subject(glob:e)",
+            "--to=subject(glob:c)",
             "-m",
             "c+d+e",
         ])
         .success();
-    let output = work_dir.run_jj(["evolog", "-r", "description(c+d+e)", "--reversed"]);
+    let output = work_dir.run_jj(["evolog", "-r", "subject(glob:c+d+e)", "--reversed"]);
     insta::assert_snapshot!(output, @r"
     ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 e8849ae1
     │  (empty) (no description set)
@@ -690,27 +688,27 @@ fn test_evolog_reverse_with_graph() {
     │  -- operation 5f4c7b5cb177 describe commit 9f43967b1cdbce4ab322cb7b4636fc0362c38373
     │ ○  mzvwutvl hidden test.user@example.com 2001-02-03 08:05:11 6a4ff8aa
     ├─╯  (empty) d
-    │    -- operation 774accf68695 new empty commit
+    │    -- operation d7ad62552658 new empty commit
     │ ○  royxmykx hidden test.user@example.com 2001-02-03 08:05:12 7dea2d1d
     ├─╯  (empty) e
-    │    -- operation 4c2c3012e2c3 new empty commit
+    │    -- operation e5c6f290120f new empty commit
     ○  qpvuntsm test.user@example.com 2001-02-03 08:05:13 78fdd026
        (empty) c+d+e
-       -- operation 2c736b66cd16 squash commits into b28cda4b118fc50495ca34a24f030abc078d032e
+       -- operation b391c05c22de squash commits into b28cda4b118fc50495ca34a24f030abc078d032e
     [EOF]
     ");
 
-    let output = work_dir.run_jj(["evolog", "-rdescription(c+d+e)", "--limit=3", "--reversed"]);
+    let output = work_dir.run_jj(["evolog", "-rsubject(glob:c+d+e)", "--limit=3", "--reversed"]);
     insta::assert_snapshot!(output, @r"
     ○  mzvwutvl hidden test.user@example.com 2001-02-03 08:05:11 6a4ff8aa
     │  (empty) d
-    │  -- operation 774accf68695 new empty commit
+    │  -- operation d7ad62552658 new empty commit
     │ ○  royxmykx hidden test.user@example.com 2001-02-03 08:05:12 7dea2d1d
     ├─╯  (empty) e
-    │    -- operation 4c2c3012e2c3 new empty commit
+    │    -- operation e5c6f290120f new empty commit
     ○  qpvuntsm test.user@example.com 2001-02-03 08:05:13 78fdd026
        (empty) c+d+e
-       -- operation 2c736b66cd16 squash commits into b28cda4b118fc50495ca34a24f030abc078d032e
+       -- operation b391c05c22de squash commits into b28cda4b118fc50495ca34a24f030abc078d032e
     [EOF]
     ");
 }
