@@ -23,7 +23,7 @@ use jj_lib::file_util::try_symlink;
 use crate::common::TestEnvironment;
 use crate::common::to_toml_value;
 
-fn set_up_fake_formatter(test_env: &TestEnvironment, args: &[&str]) {
+fn set_up_fake_formatter(test_env: &mut TestEnvironment, args: &[&str]) {
     let formatter_path = assert_cmd::cargo::cargo_bin!("fake-formatter");
     assert!(formatter_path.is_file());
     test_env.add_config(formatdoc! {"
@@ -38,6 +38,7 @@ fn set_up_fake_formatter(test_env: &TestEnvironment, args: &[&str]) {
                 .copied()
         )
     });
+    test_env.add_paths_to_normalize(formatter_path, "$FAKE_FORMATTER_PATH");
 }
 
 #[test]
@@ -51,7 +52,7 @@ fn test_config_no_tools() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Config error: No `fix.tools` are configured
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
     [EOF]
     [exit status: 1]
     ");
@@ -63,6 +64,31 @@ fn test_config_no_tools() {
     ");
 }
 
+#[test]
+fn test_config_nonexistent_tool() {
+    let test_env = TestEnvironment::default();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+    test_env.add_config(
+        r###"
+        [fix.tools.my-tool]
+        command = ["nonexistent-fix-tool-binary"]
+        patterns = ["glob:**"]
+        "###,
+    );
+
+    work_dir.write_file("file", "content\n");
+    let output = work_dir.run_jj(["fix"]);
+    // We inform the user about the non-existent tool
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Warning: Failed to start `nonexistent-fix-tool-binary`
+    Fixed 0 commits of 1 checked.
+    Nothing changed.
+    [EOF]
+    ");
+}
+
 #[test]
 fn test_config_multiple_tools() {
     let test_env = TestEnvironment::default();
@@ -143,7 +169,7 @@ fn test_config_multiple_tools_with_same_name() {
     duplicate key
 
     Hint: Check the config file: $TEST_ENV/config/config0002.toml
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
     [EOF]
     [exit status: 1]
     ");
@@ -235,7 +261,7 @@ fn test_config_disabled_tools_warning_when_all_tools_are_disabled() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Config error: At least one entry of `fix.tools` must be enabled.
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
     [EOF]
     [exit status: 1]
     ");
@@ -276,8 +302,7 @@ fn test_config_tables_overlapping_patterns() {
     let output = work_dir.run_jj(["file", "show", "bar", "-r", "@"]);
     insta::assert_snapshot!(output, @r"
     bar
-    tool-1
-    tool-2[EOF]
+    tool-1tool-2[EOF]
     ");
     let output = work_dir.run_jj(["file", "show", "baz", "-r", "@"]);
     insta::assert_snapshot!(output, @r"
@@ -310,7 +335,7 @@ fn test_config_tables_all_commands_missing() {
     Caused by: missing field `command`
 
     Hint: Check the config file: $TEST_ENV/config/config0002.toml
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
     [EOF]
     [exit status: 1]
     ");
@@ -350,7 +375,7 @@ fn test_config_tables_some_commands_missing() {
     Caused by: missing field `command`
 
     Hint: Check the config file: $TEST_ENV/config/config0002.toml
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
     [EOF]
     [exit status: 1]
     ");
@@ -431,15 +456,9 @@ fn test_config_filesets() {
     [EOF]
     ");
     let output = work_dir.run_jj(["file", "show", "b1", "-r", "@"]);
-    insta::assert_snapshot!(output, @r"
-    1b
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"1b[EOF]");
     let output = work_dir.run_jj(["file", "show", "b2", "-r", "@"]);
-    insta::assert_snapshot!(output, @r"
-    2b
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"2b[EOF]");
 }
 
 #[test]
@@ -510,6 +529,16 @@ fn test_relative_paths() {
     unfixed
     [EOF]
     ");
+
+    // The output filtered to a non-existent file should display a warning.
+    let output = work_dir.run_jj(["fix", "nonexistent"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Warning: No matching entries for paths: nonexistent
+    Fixed 0 commits of 1 checked.
+    Nothing changed.
+    [EOF]
+    ");
 }
 
 #[test]
@@ -592,10 +621,10 @@ fn test_relative_tool_path_from_subdirectory() {
 
 #[test]
 fn test_fix_empty_commit() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     let output = work_dir.run_jj(["fix", "-s", "@"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
@@ -607,10 +636,10 @@ fn test_fix_empty_commit() {
 
 #[test]
 fn test_fix_leaf_commit() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     work_dir.write_file("file", "unaffected");
     work_dir.run_jj(["new"]).success();
     work_dir.write_file("file", "affected");
@@ -619,7 +648,7 @@ fn test_fix_leaf_commit() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Fixed 1 commits of 1 checked.
-    Working copy  (@) now at: rlvkpnrz 45a3ace1 (no description set)
+    Working copy  (@) now at: rlvkpnrz f5c11961 (no description set)
     Parent commit (@-)      : qpvuntsm b37955c0 (no description set)
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
@@ -627,18 +656,15 @@ fn test_fix_leaf_commit() {
     let output = work_dir.run_jj(["file", "show", "file", "-r", "@-"]);
     insta::assert_snapshot!(output, @"unaffected[EOF]");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "@"]);
-    insta::assert_snapshot!(output, @r"
-    AFFECTED
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"AFFECTED[EOF]");
 }
 
 #[test]
 fn test_fix_parent_commit() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     // Using one file name for all commits adds coverage of some possible bugs.
     work_dir.write_file("file", "parent");
     work_dir
@@ -659,34 +685,25 @@ fn test_fix_parent_commit() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Fixed 3 commits of 3 checked.
-    Working copy  (@) now at: mzvwutvl 6842c630 child2 | (no description set)
-    Parent commit (@-)      : qpvuntsm e586ba6c parent | (no description set)
+    Working copy  (@) now at: mzvwutvl e7ba6d31 child2 | (no description set)
+    Parent commit (@-)      : qpvuntsm 49f1ddd5 parent | (no description set)
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "parent"]);
-    insta::assert_snapshot!(output, @r"
-    PARENT
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"PARENT[EOF]");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "child1"]);
-    insta::assert_snapshot!(output, @r"
-    CHILD1
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"CHILD1[EOF]");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "child2"]);
-    insta::assert_snapshot!(output, @r"
-    CHILD2
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"CHILD2[EOF]");
 }
 
 #[test]
 fn test_fix_sibling_commit() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     work_dir.write_file("file", "parent");
     work_dir
         .run_jj(["bookmark", "create", "-r@", "parent"])
@@ -711,20 +728,56 @@ fn test_fix_sibling_commit() {
     let output = work_dir.run_jj(["file", "show", "file", "-r", "parent"]);
     insta::assert_snapshot!(output, @"parent[EOF]");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "child1"]);
-    insta::assert_snapshot!(output, @r"
-    CHILD1
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"CHILD1[EOF]");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "child2"]);
     insta::assert_snapshot!(output, @"child2[EOF]");
 }
 
 #[test]
+fn test_fix_descendant_commits() {
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+    work_dir.write_file("parent", "parent");
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "parent"])
+        .success();
+    work_dir.run_jj(["new"]).success();
+    work_dir.write_file("child1", "child1");
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "child1"])
+        .success();
+    work_dir.run_jj(["new", "-r", "parent"]).success();
+    work_dir.write_file("child2", "child2");
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "child2"])
+        .success();
+
+    let output = work_dir.run_jj(["fix", "-s", "parent", "child1", "child2", "nonexistent"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Warning: No matching entries for paths: nonexistent
+    Fixed 2 commits of 3 checked.
+    Working copy  (@) now at: mzvwutvl afe0ade0 child2 | (no description set)
+    Parent commit (@-)      : qpvuntsm c9cb6288 parent | (no description set)
+    Added 0 files, modified 1 files, removed 0 files
+    [EOF]
+    ");
+    let output = work_dir.run_jj(["file", "show", "parent", "-r", "parent"]);
+    insta::assert_snapshot!(output, @"parent[EOF]");
+    let output = work_dir.run_jj(["file", "show", "child1", "-r", "child1"]);
+    insta::assert_snapshot!(output, @"CHILD1[EOF]");
+    let output = work_dir.run_jj(["file", "show", "child2", "-r", "child2"]);
+    insta::assert_snapshot!(output, @"CHILD2[EOF]");
+}
+
+#[test]
 fn test_default_revset() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     work_dir.write_file("file", "trunk1");
     work_dir
         .run_jj(["bookmark", "create", "-r@", "trunk1"])
@@ -764,8 +817,8 @@ fn test_default_revset() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Fixed 3 commits of 3 checked.
-    Working copy  (@) now at: yostqsxw 41903dfd bar2 | (no description set)
-    Parent commit (@-)      : yqosqzyt c2e6d322 bar1 | (no description set)
+    Working copy  (@) now at: yostqsxw 932b950d bar2 | (no description set)
+    Parent commit (@-)      : yqosqzyt 8a37ed67 bar1 | (no description set)
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
@@ -776,28 +829,19 @@ fn test_default_revset() {
     let output = work_dir.run_jj(["file", "show", "file", "-r", "foo"]);
     insta::assert_snapshot!(output, @"foo[EOF]");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "bar1"]);
-    insta::assert_snapshot!(output, @r"
-    BAR1
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"BAR1[EOF]");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "bar2"]);
-    insta::assert_snapshot!(output, @r"
-    BAR2
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"BAR2[EOF]");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "bar3"]);
-    insta::assert_snapshot!(output, @r"
-    BAR3
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"BAR3[EOF]");
 }
 
 #[test]
 fn test_custom_default_revset() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
 
     work_dir.write_file("file", "foo");
     work_dir
@@ -823,18 +867,15 @@ fn test_custom_default_revset() {
     let output = work_dir.run_jj(["file", "show", "file", "-r", "foo"]);
     insta::assert_snapshot!(output, @"foo[EOF]");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "bar"]);
-    insta::assert_snapshot!(output, @r"
-    BAR
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"BAR[EOF]");
 }
 
 #[test]
 fn test_fix_immutable_commit() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     work_dir.write_file("file", "immutable");
     work_dir
         .run_jj(["bookmark", "create", "-r@", "immutable"])
@@ -853,7 +894,7 @@ fn test_fix_immutable_commit() {
     Hint: Could not modify commit: qpvuntsm a86b2ecc immutable | (no description set)
     Hint: Immutable commits are used to protect shared history.
     Hint: For more information, see:
-          - https://jj-vcs.github.io/jj/latest/config/#set-of-immutable-commits
+          - https://docs.jj-vcs.dev/latest/config/#set-of-immutable-commits
           - `jj help -k config`, "Set of immutable commits"
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
@@ -867,10 +908,10 @@ fn test_fix_immutable_commit() {
 
 #[test]
 fn test_fix_empty_file() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     work_dir.write_file("file", "");
 
     let output = work_dir.run_jj(["fix", "-s", "@"]);
@@ -885,11 +926,50 @@ fn test_fix_empty_file() {
 }
 
 #[test]
+fn test_fix_large_file() {
+    let mut test_env = TestEnvironment::default();
+    // Set to byte mode, so that fake-formatter doesn't read from the stdin all at
+    // once.
+    set_up_fake_formatter(&mut test_env, &["--lowercase", "--byte-mode"]);
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+    // 512KB should be larger than most default page size, so that the stdin pipe
+    // will be blocked if fake-formatter doesn't read, and the stdout pipe will
+    // be blocked if jj doesn't read.
+    let mut large_contents = b"A".repeat(0x800_000);
+    large_contents.push(b'\n');
+    work_dir.write_file("file", &large_contents);
+
+    work_dir
+        .run_jj([
+            "config",
+            "set",
+            "--repo",
+            "snapshot.max-new-file-size",
+            &large_contents.len().to_string(),
+        ])
+        .success();
+    let output = work_dir.run_jj(["fix", "-s", "@"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Fixed 1 commits of 1 checked.
+    Working copy  (@) now at: qpvuntsm d500c021 (no description set)
+    Parent commit (@-)      : zzzzzzzz 00000000 (empty) (no description set)
+    Added 0 files, modified 1 files, removed 0 files
+    [EOF]
+    ");
+    let output = work_dir.run_jj(["file", "show", "file", "-r", "@"]);
+    let stdout_bytes = output.stdout.raw().as_bytes();
+    assert_eq!(stdout_bytes.len(), large_contents.len());
+    assert!(stdout_bytes == large_contents.to_ascii_lowercase());
+}
+
+#[test]
 fn test_fix_some_paths() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     work_dir.write_file("file1", "foo");
     work_dir.write_file("file2", "bar");
 
@@ -897,67 +977,58 @@ fn test_fix_some_paths() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Fixed 1 commits of 1 checked.
-    Working copy  (@) now at: qpvuntsm b3e6840d (no description set)
+    Working copy  (@) now at: qpvuntsm 0279baba (no description set)
     Parent commit (@-)      : zzzzzzzz 00000000 (empty) (no description set)
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
     let output = work_dir.run_jj(["file", "show", "file1"]);
-    insta::assert_snapshot!(output, @r"
-    FOO
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"FOO[EOF]");
     let output = work_dir.run_jj(["file", "show", "file2"]);
     insta::assert_snapshot!(output, @"bar[EOF]");
 }
 
 #[test]
 fn test_fix_cyclic() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--reverse"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--reverse"]);
     work_dir.write_file("file", "content\n");
 
     let output = work_dir.run_jj(["fix"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Fixed 1 commits of 1 checked.
-    Working copy  (@) now at: qpvuntsm 087e5fb0 (no description set)
+    Working copy  (@) now at: qpvuntsm ce361156 (no description set)
     Parent commit (@-)      : zzzzzzzz 00000000 (empty) (no description set)
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "@"]);
-    insta::assert_snapshot!(output, @r"
-    tnetnoc
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"tnetnoc[EOF]");
 
     let output = work_dir.run_jj(["fix"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Fixed 1 commits of 1 checked.
-    Working copy  (@) now at: qpvuntsm 832f43c5 (no description set)
+    Working copy  (@) now at: qpvuntsm 547f589b (no description set)
     Parent commit (@-)      : zzzzzzzz 00000000 (empty) (no description set)
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "@"]);
-    insta::assert_snapshot!(output, @r"
-    content
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"content[EOF]");
 }
 
 #[test]
 fn test_deduplication() {
     // Append all fixed content to a log file. Note that fix tools are always run
     // from the workspace root, so this will always write to $root/$path-fixlog.
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase", "--tee", "$path-fixlog"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase", "--tee", "$path-fixlog"]);
 
     // There are at least two interesting cases: the content is repeated immediately
     // in the child commit, or later in another descendant.
@@ -1034,10 +1105,10 @@ fn sorted_lines(path: PathBuf) -> String {
 fn test_executed_but_nothing_changed() {
     // Show that the tool ran by causing a side effect with --tee, and test that we
     // do the right thing when the tool's output is exactly equal to its input.
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--tee", "$path-copy"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--tee", "$path-copy"]);
     work_dir.write_file("file", "content\n");
 
     let output = work_dir.run_jj(["fix", "-s", "@"]);
@@ -1076,15 +1147,16 @@ fn test_executed_but_nothing_changed() {
 
 #[test]
 fn test_failure() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--fail"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--fail"]);
     work_dir.write_file("file", "content");
 
     let output = work_dir.run_jj(["fix", "-s", "@"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Fix tool `$FAKE_FORMATTER_PATH` exited with non-zero exit code for `file`
     Fixed 0 commits of 1 checked.
     Nothing changed.
     [EOF]
@@ -1095,10 +1167,13 @@ fn test_failure() {
 
 #[test]
 fn test_stderr_success() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(
+        &mut test_env,
+        &["--stderr", "error", "--stdout", "new content"],
+    );
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--stderr", "error", "--stdout", "new content"]);
     work_dir.write_file("file", "old content");
 
     // TODO: Associate the stderr lines with the relevant tool/file/commit instead
@@ -1120,13 +1195,13 @@ fn test_stderr_success() {
 
 #[test]
 fn test_stderr_failure() {
-    let test_env = TestEnvironment::default();
-    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let work_dir = test_env.work_dir("repo");
+    let mut test_env = TestEnvironment::default();
     set_up_fake_formatter(
-        &test_env,
+        &mut test_env,
         &["--stderr", "error", "--stdout", "new content", "--fail"],
     );
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
     work_dir.write_file("file", "old content");
 
     let output = work_dir.run_jj(["fix", "-s", "@"]);
@@ -1134,6 +1209,7 @@ fn test_stderr_failure() {
     ------- stderr -------
     file:
     error
+    Warning: Fix tool `$FAKE_FORMATTER_PATH` exited with non-zero exit code for `file`
     Fixed 0 commits of 1 checked.
     Nothing changed.
     [EOF]
@@ -1166,10 +1242,10 @@ fn test_missing_command() {
 
 #[test]
 fn test_fix_file_types() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     work_dir.write_file("file", "content");
     work_dir.create_dir("dir");
     try_symlink("file", work_dir.root().join("link")).unwrap();
@@ -1178,25 +1254,22 @@ fn test_fix_file_types() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Fixed 1 commits of 1 checked.
-    Working copy  (@) now at: qpvuntsm a600eba5 (no description set)
+    Working copy  (@) now at: qpvuntsm 0184b215 (no description set)
     Parent commit (@-)      : zzzzzzzz 00000000 (empty) (no description set)
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "@"]);
-    insta::assert_snapshot!(output, @r"
-    CONTENT
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"CONTENT[EOF]");
 }
 
 #[cfg(unix)]
 #[test]
 fn test_fix_executable() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     let path = work_dir.root().join("file");
     work_dir.write_file("file", "content");
     let mut permissions = std::fs::metadata(&path).unwrap().permissions();
@@ -1207,16 +1280,13 @@ fn test_fix_executable() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Fixed 1 commits of 1 checked.
-    Working copy  (@) now at: qpvuntsm ed402983 (no description set)
+    Working copy  (@) now at: qpvuntsm 5293bf26 (no description set)
     Parent commit (@-)      : zzzzzzzz 00000000 (empty) (no description set)
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
     let output = work_dir.run_jj(["file", "show", "file", "-r", "@"]);
-    insta::assert_snapshot!(output, @r"
-    CONTENT
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @r"CONTENT[EOF]");
     let executable = std::fs::metadata(&path).unwrap().permissions().mode() & 0o111;
     assert_eq!(executable, 0o111);
 }
@@ -1225,10 +1295,10 @@ fn test_fix_executable() {
 fn test_fix_trivial_merge_commit() {
     // All the changes are attributable to a parent, so none are fixed (in the same
     // way that none would be shown in `jj diff -r @`).
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     work_dir.write_file("file_a", "content a");
     work_dir.write_file("file_c", "content c");
     work_dir
@@ -1261,10 +1331,10 @@ fn test_fix_trivial_merge_commit() {
 fn test_fix_adding_merge_commit() {
     // None of the changes are attributable to a parent, so they are all fixed (in
     // the same way that they would be shown in `jj diff -r @`).
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     work_dir.write_file("file_a", "content a");
     work_dir.write_file("file_c", "content c");
     work_dir
@@ -1286,40 +1356,28 @@ fn test_fix_adding_merge_commit() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Fixed 1 commits of 1 checked.
-    Working copy  (@) now at: mzvwutvl c8d50db3 (no description set)
+    Working copy  (@) now at: mzvwutvl 9f580aac (no description set)
     Parent commit (@-)      : qpvuntsm 93f04460 a | (no description set)
     Parent commit (@-)      : kkmpptxz ad4fc36c b | (no description set)
     Added 0 files, modified 4 files, removed 0 files
     [EOF]
     ");
     let output = work_dir.run_jj(["file", "show", "file_a", "-r", "@"]);
-    insta::assert_snapshot!(output, @r"
-    CHANGE A
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"CHANGE A[EOF]");
     let output = work_dir.run_jj(["file", "show", "file_b", "-r", "@"]);
-    insta::assert_snapshot!(output, @r"
-    CHANGE B
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"CHANGE B[EOF]");
     let output = work_dir.run_jj(["file", "show", "file_c", "-r", "@"]);
-    insta::assert_snapshot!(output, @r"
-    CHANGE C
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"CHANGE C[EOF]");
     let output = work_dir.run_jj(["file", "show", "file_d", "-r", "@"]);
-    insta::assert_snapshot!(output, @r"
-    CHANGE D
-    [EOF]
-    ");
+    insta::assert_snapshot!(output, @"CHANGE D[EOF]");
 }
 
 #[test]
 fn test_fix_both_sides_of_conflict() {
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     work_dir.write_file("file", "content a\n");
     work_dir
         .run_jj(["bookmark", "create", "-r@", "a"])
@@ -1337,7 +1395,7 @@ fn test_fix_both_sides_of_conflict() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Fixed 3 commits of 3 checked.
-    Working copy  (@) now at: mzvwutvl d4d02bf0 (conflict) (empty) (no description set)
+    Working copy  (@) now at: mzvwutvl e1acdc6e (conflict) (empty) (no description set)
     Parent commit (@-)      : qpvuntsm 0eae0dae a | (no description set)
     Parent commit (@-)      : kkmpptxz eb61ba8d b | (no description set)
     Added 0 files, modified 1 files, removed 0 files
@@ -1371,10 +1429,10 @@ fn test_fix_both_sides_of_conflict() {
 fn test_fix_resolve_conflict() {
     // If both sides of the conflict look the same after being fixed, the conflict
     // will be resolved.
-    let test_env = TestEnvironment::default();
+    let mut test_env = TestEnvironment::default();
+    set_up_fake_formatter(&mut test_env, &["--uppercase"]);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
-    set_up_fake_formatter(&test_env, &["--uppercase"]);
     work_dir.write_file("file", "Content\n");
     work_dir
         .run_jj(["bookmark", "create", "-r@", "a"])
@@ -1455,6 +1513,7 @@ fn test_all_files() {
     ]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: No matching entries for paths: does_not.exist
     Fixed 2 commits of 2 checked.
     Working copy  (@) now at: rlvkpnrz d8503fee child
     Parent commit (@-)      : qpvuntsm 62c6ee98 parent
@@ -1509,8 +1568,8 @@ fn test_all_files() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Fixed 2 commits of 2 checked.
-    Working copy  (@) now at: rlvkpnrz 3675eae3 child
-    Parent commit (@-)      : qpvuntsm 6a0a8a6a parent
+    Working copy  (@) now at: rlvkpnrz 16aeb14c child
+    Parent commit (@-)      : qpvuntsm 5257b8ec parent
     Added 0 files, modified 2 files, removed 0 files
     [EOF]
     ");
@@ -1523,8 +1582,7 @@ fn test_all_files() {
     let output = work_dir.run_jj(["file", "show", "b/b", "-r", "@-"]);
     insta::assert_snapshot!(output, @r"
     parent bbb
-    fixed
-    fixed[EOF]
+    fixedfixed[EOF]
     ");
     let output = work_dir.run_jj(["file", "show", "c/c", "-r", "@-"]);
     insta::assert_snapshot!(output, @r"
@@ -1545,8 +1603,7 @@ fn test_all_files() {
     let output = work_dir.run_jj(["file", "show", "b/b", "-r", "@"]);
     insta::assert_snapshot!(output, @r"
     parent bbb
-    fixed
-    fixed[EOF]
+    fixedfixed[EOF]
     ");
     let output = work_dir.run_jj(["file", "show", "c/c", "-r", "@"]);
     insta::assert_snapshot!(output, @r"
