@@ -43,7 +43,7 @@ fn set_up_git_repo_with_file(git_repo: &gix::Repository, filename: &str) {
 fn test_git_clone() {
     let test_env = TestEnvironment::default();
     let root_dir = test_env.work_dir("");
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let git_repo_path = test_env.env_root().join("source");
     let git_repo = git::init(git_repo_path);
 
@@ -172,12 +172,12 @@ fn test_git_clone_bad_source() {
     [exit status: 2]
     "#);
 
-    // Invalid port number
-    let output = root_dir.run_jj(["git", "clone", "https://example.net:bad-port/bar", "dest"]);
+    // Invalid URL unparsable by gitoxide
+    let output = root_dir.run_jj(["git", "clone", "https://", "dest"]);
     insta::assert_snapshot!(output, @r#"
     ------- stderr -------
-    Error: URL "https://example.net:bad-port/bar" can not be parsed as valid URL
-    Caused by: invalid port number
+    Error: URL "https://" can not be parsed as valid URL
+    Caused by: Scheme requires host
     [EOF]
     [exit status: 2]
     "#);
@@ -187,7 +187,7 @@ fn test_git_clone_bad_source() {
 fn test_git_clone_colocate() {
     let test_env = TestEnvironment::default();
     let root_dir = test_env.work_dir("");
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let git_repo_path = test_env.env_root().join("source");
     let git_repo = git::init(git_repo_path);
 
@@ -561,8 +561,8 @@ fn test_git_clone_remote_default_bookmark() {
         )
         .unwrap();
 
-    // All fetched bookmarks will be imported if auto-local-bookmark is on
-    test_env.add_config("git.auto-local-bookmark = true");
+    // All fetched bookmarks will be imported if auto-track-bookmarks = 'glob:*'
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let output = root_dir.run_jj(["git", "clone", "source", "clone1"]);
     insta::assert_snapshot!(output, @r#"
     ------- stderr -------
@@ -591,8 +591,8 @@ fn test_git_clone_remote_default_bookmark() {
     [EOF]
     "#);
 
-    // Only the default bookmark will be imported if auto-local-bookmark is off
-    test_env.add_config("git.auto-local-bookmark = false");
+    // Only the default bookmark will be imported if auto-track-bookmarks = ''
+    test_env.add_config("remotes.origin.auto-track-bookmarks = ''");
     let output = root_dir.run_jj(["git", "clone", "source", "clone2"]);
     insta::assert_snapshot!(output, @r#"
     ------- stderr -------
@@ -642,7 +642,7 @@ fn test_git_clone_remote_default_bookmark() {
     [EOF]
     "#);
 
-    // No bookmarks should be imported if both auto-local-bookmark and
+    // No bookmarks should be imported if both auto-track-bookmarks and
     // track-default-bookmark-on-clone are turned off
     let output = root_dir.run_jj([
         "git",
@@ -680,6 +680,9 @@ fn test_git_clone_remote_default_bookmark() {
     ]);
     insta::assert_snapshot!(output, @r#"
     ------- stderr -------
+    Warning: Deprecated CLI-provided config: `git.auto-local-bookmark` is deprecated; use `remotes.<name>.auto-track-bookmarks` instead.
+    Example: jj config set --user remotes.origin.auto-track-bookmarks 'glob:*'
+    For details, see: https://docs.jj-vcs.dev/latest/config/#automatic-tracking-of-bookmarks
     Fetching into new repo in "$TEST_ENV/clone5"
     bookmark: feature1@origin [new] tracked
     bookmark: main@origin     [new] tracked
@@ -775,7 +778,7 @@ fn test_git_clone_ignore_working_copy() {
     ------- stderr -------
     Error: The working copy is stale (not updated since operation 8f47435a3990).
     Hint: Run `jj workspace update-stale` to update it.
-    See https://jj-vcs.github.io/jj/latest/working-copy/#stale-working-copy for more information.
+    See https://docs.jj-vcs.dev/latest/working-copy/#stale-working-copy for more information.
     [EOF]
     [exit status: 1]
     ");
@@ -802,7 +805,7 @@ fn test_git_clone_at_operation() {
 fn test_git_clone_with_remote_name() {
     let test_env = TestEnvironment::default();
     let root_dir = test_env.work_dir("");
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let git_repo_path = test_env.env_root().join("source");
     let git_repo = git::init(git_repo_path);
     set_up_non_empty_git_repo(&git_repo);
@@ -988,7 +991,7 @@ fn test_git_clone_conditional_config() {
 fn test_git_clone_with_depth() {
     let test_env = TestEnvironment::default();
     let root_dir = test_env.work_dir("");
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let clone_dir = test_env.work_dir("clone");
     let git_repo_path = test_env.env_root().join("source");
     let git_repo = git::init(git_repo_path);
@@ -1036,7 +1039,7 @@ fn test_git_clone_invalid_immutable_heads() {
     ------- stderr -------
     Config error: Invalid `revset-aliases.immutable_heads()`
     Caused by: Revision `unknown` doesn't exist
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
     [EOF]
     [exit status: 1]
     ");
@@ -1071,7 +1074,7 @@ fn test_git_clone_malformed() {
     ------- stderr -------
     Error: The working copy is stale (not updated since operation 353078ed182b).
     Hint: Run `jj workspace update-stale` to update it.
-    See https://jj-vcs.github.io/jj/latest/working-copy/#stale-working-copy for more information.
+    See https://docs.jj-vcs.dev/latest/working-copy/#stale-working-copy for more information.
     [EOF]
     [exit status: 1]
     ");
@@ -1173,7 +1176,7 @@ fn test_git_clone_no_git_executable_with_path() {
 fn test_git_clone_branch() {
     let test_env = TestEnvironment::default();
     let root_dir = test_env.work_dir("");
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let git_repo_path = test_env.env_root().join("source");
     let git_repo = git::init(&git_repo_path);
     set_up_non_empty_git_repo(&git_repo);
@@ -1182,7 +1185,7 @@ fn test_git_clone_branch() {
         "refs/heads/feature1",
         "file",
         b"content",
-        "message",
+        "feature1 message",
         &[],
     );
 
@@ -1205,15 +1208,14 @@ fn test_git_clone_branch() {
         "clone",
         "source",
         "clone_no_such_branch",
-        "--branch",
-        "nonexistent_branch",
+        "--branch=(nonexistent1 | glob:nonexistent*) & ~nonexistent2",
     ]);
     insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Fetching into new repo in "$TEST_ENV/clone_no_such_branch"
-    Error: No such branch matching pattern: nonexistent_branch
+    Warning: No matching branches found on remote: nonexistent1
+    Nothing changed.
     [EOF]
-    [exit status: 1]
     "#);
 
     // Clone a specific, non-default branch
@@ -1229,8 +1231,8 @@ fn test_git_clone_branch() {
     ------- stderr -------
     Fetching into new repo in "$TEST_ENV/clone_non_default"
     bookmark: feature1@origin [new] tracked
-    Working copy  (@) now at: pmmvwywv ea9c2659 (empty) (no description set)
-    Parent commit (@-)      : qomsplrm ebeb70d8 feature1 | message
+    Working copy  (@) now at: pmmvwywv d16a31e5 (empty) (no description set)
+    Parent commit (@-)      : yxwyzxtq 14835edf feature1 | feature1 message
     Added 1 files, modified 0 files, removed 0 files
     [EOF]
     "#);
@@ -1243,6 +1245,93 @@ fn test_git_clone_branch() {
     Nothing changed.
     [EOF]
     ");
+
+    // Clone all branches explicitly, the default branch should be checked out
+    let output = root_dir.run_jj(["git", "clone", "source", "clone_all", "--branch=glob:*"]);
+    insta::assert_snapshot!(output, @r#"
+    ------- stderr -------
+    Fetching into new repo in "$TEST_ENV/clone_all"
+    bookmark: feature1@origin [new] tracked
+    bookmark: main@origin     [new] tracked
+    Setting the revset alias `trunk()` to `main@origin`
+    Working copy  (@) now at: zxsnswpr 56139679 (empty) (no description set)
+    Parent commit (@-)      : qomsplrm ebeb70d8 main | message
+    Added 1 files, modified 0 files, removed 0 files
+    [EOF]
+    "#);
+
+    // Clone all but feature1
+    let output = root_dir.run_jj([
+        "git",
+        "clone",
+        "source",
+        "clone_all_but_feature1",
+        "--branch=~feature1",
+    ]);
+    insta::assert_snapshot!(output, @r#"
+    ------- stderr -------
+    Fetching into new repo in "$TEST_ENV/clone_all_but_feature1"
+    bookmark: main@origin [new] tracked
+    Setting the revset alias `trunk()` to `main@origin`
+    Working copy  (@) now at: nppvrztz b16020e9 (empty) (no description set)
+    Parent commit (@-)      : qomsplrm ebeb70d8 main | message
+    Added 1 files, modified 0 files, removed 0 files
+    [EOF]
+    "#);
+
+    // Perform a fetch in that same repo
+    let repo_dir = test_env.work_dir("clone_all_but_feature1");
+    let output = repo_dir.run_jj(["git", "fetch"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Nothing changed.
+    [EOF]
+    ");
+}
+
+#[test]
+fn test_git_clone_auto_track_bookmarks() {
+    let test_env = TestEnvironment::default();
+    let root_dir = test_env.work_dir("");
+    test_env.add_config(
+        "
+        [remotes.origin]
+        auto-track-bookmarks = 'glob:mine/*'
+        ",
+    );
+
+    root_dir
+        .run_jj(["git", "init", "--colocate", "origin"])
+        .success();
+    let origin_dir = test_env.work_dir("origin");
+    origin_dir.run_jj(["b", "c", "mine/foo"]).success();
+    origin_dir.run_jj(["b", "c", "not-mine/foo"]).success();
+    origin_dir.run_jj(["commit", "-mfoo"]).success();
+    let output = origin_dir.run_jj(["show", "@-"]);
+    insta::assert_snapshot!(output, @r"
+    Commit ID: d7828da83253475bf10c2ae6bd3f0f84bf4604c1
+    Change ID: qpvuntsmwlqtpsluzzsnyyzlmlwvmlnu
+    Bookmarks: mine/foo not-mine/foo mine/foo@git not-mine/foo@git
+    Author   : Test User <test.user@example.com> (2001-02-03 08:05:10)
+    Committer: Test User <test.user@example.com> (2001-02-03 08:05:10)
+
+        foo
+
+    [EOF]
+    ");
+
+    root_dir
+        .run_jj(["git", "clone", "origin/.git", "repo"])
+        .success();
+    let repo_dir = test_env.work_dir("repo");
+
+    let output = get_bookmark_output(&repo_dir);
+    insta::assert_snapshot!(output, @r"
+    mine/foo: qpvuntsm d7828da8 (empty) foo
+      @origin: qpvuntsm d7828da8 (empty) foo
+    not-mine/foo@origin: qpvuntsm d7828da8 (empty) foo
+    [EOF]
+    ");
 }
 
 #[must_use]
