@@ -138,9 +138,7 @@ fn test_git_colocated_intent_to_add() {
     // If we remove the added file, it's removed from the index
     work_dir.remove_file("file2.txt");
     work_dir.run_jj(["status"]).success();
-    insta::assert_snapshot!(get_index_state(work_dir.root()), @r"
-    Unconflicted Mode(FILE) 0839b2e9412b ctime=0:0 mtime=0:0 size=0 flags=0 file1.txt
-    ");
+    insta::assert_snapshot!(get_index_state(work_dir.root()), @"Unconflicted Mode(FILE) 0839b2e9412b ctime=0:0 mtime=0:0 size=0 flags=0 file1.txt");
 
     // If we untrack the file, it's removed from the index
     work_dir
@@ -449,7 +447,7 @@ fn test_git_colocated_bookmarks() {
 
     // Update the bookmark in Git
     let target_id = work_dir
-        .run_jj(["log", "--no-graph", "-T=commit_id", "-r=description(foo)"])
+        .run_jj(["log", "--no-graph", "-T=commit_id", "-r=subject(glob:foo)"])
         .success()
         .stdout
         .into_raw();
@@ -641,7 +639,7 @@ fn test_git_colocated_checkout_non_empty_working_copy() {
 #[test]
 fn test_git_colocated_fetch_deleted_or_moved_bookmark() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let origin_dir = test_env.work_dir("origin");
     git::init(origin_dir.root());
     origin_dir.run_jj(["git", "init", "--git-repo=."]).success();
@@ -802,7 +800,7 @@ fn test_git_colocated_external_checkout() {
     ");
 
     // Edit non-head commit
-    work_dir.run_jj(["new", "description(B)"]).success();
+    work_dir.run_jj(["new", "subject(glob:B)"]).success();
     work_dir.run_jj(["new", "-m=C", "--no-edit"]).success();
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
     ○  823204bc895aad19d46b895bc510fb3e9d0c97c7 C
@@ -1138,7 +1136,7 @@ fn test_git_colocated_update_index_merge_conflict() {
     work_dir.run_jj(["new", "left", "right"]).success();
 
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
-    @    8b05232ad2cda6f6d06b290486e07251f53c0958
+    @    985fe3b46a6caecb44b6a12d22fc2b1fc33c219d
     ├─╮
     │ ○  620e15db9fcd05fff912c52d2cafd36c9e01523c right
     ○ │  d0f55ffafa1e0e72980202c349af23d093f825be left git_head()
@@ -1162,8 +1160,8 @@ fn test_git_colocated_update_index_merge_conflict() {
     work_dir.run_jj(["new"]).success();
 
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
-    @  7c98aa1e17acd7829c9ccb9eaae705df9b255bd1
-    ×    8b05232ad2cda6f6d06b290486e07251f53c0958 git_head()
+    @  4e86bd16fa83ac6276701bfa361c683e258a653b
+    ×    985fe3b46a6caecb44b6a12d22fc2b1fc33c219d git_head()
     ├─╮
     │ ○  620e15db9fcd05fff912c52d2cafd36c9e01523c right
     ○ │  d0f55ffafa1e0e72980202c349af23d093f825be left
@@ -1241,11 +1239,11 @@ fn test_git_colocated_update_index_rebase_conflict() {
 
     // Create rebase conflict
     work_dir
-        .run_jj(["rebase", "-r", "left", "-d", "right"])
+        .run_jj(["rebase", "-r", "left", "-o", "right"])
         .success();
 
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
-    @  535388c5aab1b3a33fdc04a4bf8033de0d1b86ec left
+    @  b641af6d56002585b3152e84d4bd92f8181d7909 left
     ○  620e15db9fcd05fff912c52d2cafd36c9e01523c right git_head()
     ○  1861378a9167e6561bf8ce4a6fef2d7c0897dd87 base
     ◆  0000000000000000000000000000000000000000
@@ -1264,8 +1262,8 @@ fn test_git_colocated_update_index_rebase_conflict() {
     work_dir.run_jj(["new"]).success();
 
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
-    @  04ebd7523ac6107ccdd5bc34600a073b94e43299
-    ×  535388c5aab1b3a33fdc04a4bf8033de0d1b86ec left git_head()
+    @  3118c1f8fb0a6279d411eb484906e7274ab5c8f7
+    ×  b641af6d56002585b3152e84d4bd92f8181d7909 left git_head()
     ○  620e15db9fcd05fff912c52d2cafd36c9e01523c right
     ○  1861378a9167e6561bf8ce4a6fef2d7c0897dd87 base
     ◆  0000000000000000000000000000000000000000
@@ -1341,7 +1339,7 @@ fn test_git_colocated_update_index_3_sided_conflict() {
         .success();
 
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
-    @      3105daa0d68e3cdc22b2533d7d1b231cd41c76ec
+    @      3b8792248f224ce8e3f6652681e518a4f3cb3a0f
     ├─┬─╮
     │ │ ○  5008c8807feaa955d02e96cb1b0dcf51536fefb8 side-3
     │ ○ │  da6e0a03f8b72f6868a9ea33836123fe965c0cb4 side-2
@@ -1367,8 +1365,8 @@ fn test_git_colocated_update_index_3_sided_conflict() {
     work_dir.run_jj(["new"]).success();
 
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
-    @  5b4266a02e8fe9febc6294c7d0a02fc8463221e8
-    ×      3105daa0d68e3cdc22b2533d7d1b231cd41c76ec git_head()
+    @  b16ae318909e9bf70fc312217988f2ca0abccb62
+    ×      3b8792248f224ce8e3f6652681e518a4f3cb3a0f git_head()
     ├─┬─╮
     │ │ ○  5008c8807feaa955d02e96cb1b0dcf51536fefb8 side-3
     │ ○ │  da6e0a03f8b72f6868a9ea33836123fe965c0cb4 side-2
@@ -1606,9 +1604,7 @@ fn test_git_colocated_operation_cleanup() {
         .output()
         .unwrap();
     assert!(output.status.success());
-    insta::assert_snapshot!(String::from_utf8(output.stdout).unwrap(), @r#"
-    UU file
-    "#);
+    insta::assert_snapshot!(String::from_utf8(output.stdout).unwrap(), @"UU file");
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  588c505e689d116180684778b29c540fe7180268
     ○  cf3bb116ded416d9b202e71303f260e504c2eeb9 main git_head() 2
