@@ -130,7 +130,7 @@ fn test_git_fetch_with_default_config() {
 #[test]
 fn test_git_fetch_default_remote() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "origin");
@@ -146,7 +146,7 @@ fn test_git_fetch_default_remote() {
 #[test]
 fn test_git_fetch_single_remote() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.rem1.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "rem1");
@@ -168,7 +168,7 @@ fn test_git_fetch_single_remote() {
 #[test]
 fn test_git_fetch_single_remote_all_remotes_flag() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.rem1.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "rem1");
@@ -184,7 +184,7 @@ fn test_git_fetch_single_remote_all_remotes_flag() {
 #[test]
 fn test_git_fetch_single_remote_from_arg() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.rem1.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "rem1");
@@ -202,7 +202,7 @@ fn test_git_fetch_single_remote_from_arg() {
 #[test]
 fn test_git_fetch_single_remote_from_config() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.rem1.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "rem1");
@@ -219,7 +219,8 @@ fn test_git_fetch_single_remote_from_config() {
 #[test]
 fn test_git_fetch_multiple_remotes() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.rem1.auto-track-bookmarks = 'glob:*'");
+    test_env.add_config("remotes.rem2.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "rem1");
@@ -240,48 +241,88 @@ fn test_git_fetch_multiple_remotes() {
 #[test]
 fn test_git_fetch_with_ignored_refspecs() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
-    test_env
-        .run_jj_in(".", ["git", "init", "--colocate", "repo"])
-        .success();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let source_repo = init_git_remote(&test_env, "origin");
 
-    for branch in ["main", "foo", "foobar", "foobaz", "bar"] {
-        add_commit_to_branch(&source_repo, &format!("refs/heads/{branch}"), branch);
+    for branch in [
+        "main", "foo", "foobar", "foobaz", "bar", "sub/yes", "sub/no",
+    ] {
+        add_commit_to_branch(&source_repo, branch, branch);
     }
 
     let work_dir = test_env.work_dir("repo");
     std::fs::OpenOptions::new()
         .append(true)
-        .open(work_dir.root().join("./.git/config"))
+        .open(work_dir.root().join(".jj/repo/store/git/config"))
         .expect("failed to open config file")
         .write_all(
             br#"
             [remote "origin"]
             url = ../origin/.git
             fetch = +refs/heads/main:refs/remotes/origin/main
+            fetch = +refs/heads/sub/*:refs/remotes/origin/sub/*
             fetch = +refs/heads/foo*:refs/remotes/origin/baz*
             fetch = +refs/heads/bar*:refs/tags/bar*
             fetch = refs/heads/bar
+            fetch = ^refs/heads/sub/no
             "#,
         )
         .expect("failed to update config file");
 
-    let output = work_dir.run_jj(["git", "fetch"]).success();
-
-    insta::assert_snapshot!(output.stdout, @r"");
-    insta::assert_snapshot!(output.stderr, @r"
-    Done importing changes from the underlying Git repo.
+    // Should fetch "main" and "sub/yes" by default
+    let output = work_dir.run_jj(["git", "fetch"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
     Warning: Ignored refspec `refs/heads/bar` from `origin`: fetch-only refspecs are not supported
     Warning: Ignored refspec `+refs/heads/bar*:refs/tags/bar*` from `origin`: only refs/remotes/ is supported for fetch destinations
     Warning: Ignored refspec `+refs/heads/foo*:refs/remotes/origin/baz*` from `origin`: renaming is not supported
-    bookmark: main@origin [new] tracked
+    bookmark: main@origin    [new] untracked
+    bookmark: sub/yes@origin [new] untracked
     [EOF]
     ");
     insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
-    main: kpukqsrq f803461e main
-      @git: kpukqsrq f803461e main
-      @origin: kpukqsrq f803461e main
+    main@origin: wlltxvop a437242b main
+    sub/yes@origin: xwxtqxvy 6b64b005 sub/yes
+    [EOF]
+    ");
+
+    // Can fetch ignored "sub/no" explicitly
+    let output = work_dir.run_jj(["git", "fetch", "--branch=sub/no"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    bookmark: sub/no@origin [new] untracked
+    [EOF]
+    ");
+    insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
+    main@origin: wlltxvop a437242b main
+    sub/no@origin: tknwmolt f7d8b914 sub/no
+    sub/yes@origin: xwxtqxvy 6b64b005 sub/yes
+    [EOF]
+    ");
+
+    // Forget "sub/no" without exporting the change to Git
+    work_dir
+        .run_jj(["bookmark", "forget", "--include-remotes", "sub/no"])
+        .success();
+    insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
+    main@origin: wlltxvop a437242b main
+    sub/yes@origin: xwxtqxvy 6b64b005 sub/yes
+    [EOF]
+    ");
+
+    // Should not import "sub/no" because it is ignored by default
+    let output = work_dir.run_jj(["git", "fetch"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Warning: Ignored refspec `refs/heads/bar` from `origin`: fetch-only refspecs are not supported
+    Warning: Ignored refspec `+refs/heads/bar*:refs/tags/bar*` from `origin`: only refs/remotes/ is supported for fetch destinations
+    Warning: Ignored refspec `+refs/heads/foo*:refs/remotes/origin/baz*` from `origin`: renaming is not supported
+    Nothing changed.
+    [EOF]
+    ");
+    insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
+    main@origin: wlltxvop a437242b main
+    sub/yes@origin: xwxtqxvy 6b64b005 sub/yes
     [EOF]
     ");
 }
@@ -353,7 +394,6 @@ fn test_git_fetch_with_glob_with_no_matching_remotes() {
     let output = work_dir.run_jj(["git", "fetch", "--remote=glob:rem*"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Warning: No git remotes matching 'rem*'
     Error: No git remotes to fetch from
     [EOF]
     [exit status: 1]
@@ -365,7 +405,8 @@ fn test_git_fetch_with_glob_with_no_matching_remotes() {
 #[test]
 fn test_git_fetch_all_remotes() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.rem1.auto-track-bookmarks = 'glob:*'");
+    test_env.add_config("remotes.rem2.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "rem1");
@@ -392,7 +433,8 @@ fn test_git_fetch_all_remotes() {
 #[test]
 fn test_git_fetch_multiple_remotes_from_config() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.rem1.auto-track-bookmarks = 'glob:*'");
+    test_env.add_config("remotes.rem2.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "rem1");
@@ -418,11 +460,21 @@ fn test_git_fetch_no_matching_remote() {
     let output = work_dir.run_jj(["git", "fetch", "--remote", "rem1"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Warning: No git remotes matching 'rem1'
+    Warning: No matching remotes for names: rem1
     Error: No git remotes to fetch from
     [EOF]
     [exit status: 1]
     ");
+
+    let output = work_dir.run_jj(["git", "fetch", "--remote=rem1", "--remote=rem2"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Warning: No matching remotes for names: rem1, rem2
+    Error: No git remotes to fetch from
+    [EOF]
+    [exit status: 1]
+    ");
+
     insta::assert_snapshot!(get_bookmark_output(&work_dir), @"");
 }
 
@@ -436,7 +488,7 @@ fn test_git_fetch_nonexistent_remote() {
     let output = work_dir.run_jj(["git", "fetch", "--remote", "rem1", "--remote", "rem2"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Warning: No git remotes matching 'rem2'
+    Warning: No matching remotes for names: rem2
     bookmark: rem1@rem1 [new] untracked
     [EOF]
     ");
@@ -457,7 +509,7 @@ fn test_git_fetch_nonexistent_remote_from_config() {
     let output = work_dir.run_jj(["git", "fetch"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Warning: No git remotes matching 'rem2'
+    Warning: No matching remotes for names: rem2
     bookmark: rem1@rem1 [new] untracked
     [EOF]
     ");
@@ -470,7 +522,7 @@ fn test_git_fetch_nonexistent_remote_from_config() {
 #[test]
 fn test_git_fetch_from_remote_named_git() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.bar.auto-track-bookmarks = 'glob:*'");
     let work_dir = test_env.work_dir("repo");
     init_git_remote(&test_env, "git");
 
@@ -540,7 +592,7 @@ fn test_git_fetch_from_remote_named_git() {
 #[test]
 fn test_git_fetch_from_remote_with_slashes() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let work_dir = test_env.work_dir("repo");
     init_git_remote(&test_env, "source");
 
@@ -564,7 +616,7 @@ fn test_git_fetch_from_remote_with_slashes() {
 #[test]
 fn test_git_fetch_prune_before_updating_tips() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     let git_repo = add_git_remote(&test_env, &work_dir, "origin");
@@ -599,7 +651,7 @@ fn test_git_fetch_prune_before_updating_tips() {
 #[test]
 fn test_git_fetch_conflicting_bookmarks() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.rem1.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "rem1");
@@ -611,6 +663,7 @@ fn test_git_fetch_conflicting_bookmarks() {
         .success();
     insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
     rem1: kkmpptxz 2b17ac71 (empty) (no description set)
+      @rem1 (not created yet)
     [EOF]
     ");
 
@@ -630,7 +683,7 @@ fn test_git_fetch_conflicting_bookmarks() {
 #[test]
 fn test_git_fetch_conflicting_bookmarks_colocated() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.rem1.auto-track-bookmarks = 'glob:*'");
     let work_dir = test_env.work_dir("repo");
     git::init(work_dir.root());
     // create_colocated_repo_and_bookmarks_from_trunk1(&test_env, &repo_path);
@@ -648,6 +701,7 @@ fn test_git_fetch_conflicting_bookmarks_colocated() {
     insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
     rem1: zsuskuln c2934cfb (empty) (no description set)
       @git: zsuskuln c2934cfb (empty) (no description set)
+      @rem1 (not created yet)
     [EOF]
     ");
 
@@ -687,7 +741,7 @@ fn create_trunk2_and_rebase_bookmarks(work_dir: &TestWorkDir) -> String {
     create_commit(work_dir, "trunk2", &["trunk1"]);
     for br in ["a1", "a2", "b"] {
         work_dir
-            .run_jj(["rebase", "-b", br, "-d", "trunk2"])
+            .run_jj(["rebase", "-b", br, "-o", "trunk2"])
             .success();
     }
     format!(
@@ -699,7 +753,7 @@ fn create_trunk2_and_rebase_bookmarks(work_dir: &TestWorkDir) -> String {
 #[test]
 fn test_git_fetch_all() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
     let source_dir = test_env.work_dir("source");
     git::init(source_dir.root());
@@ -857,7 +911,7 @@ fn test_git_fetch_all() {
 #[test]
 fn test_git_fetch_some_of_many_bookmarks() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
     let source_dir = test_env.work_dir("source");
     git::init(source_dir.root());
@@ -886,7 +940,7 @@ fn test_git_fetch_some_of_many_bookmarks() {
     "#);
 
     // Test an error message
-    let output = target_dir.run_jj(["git", "fetch", "--branch", "glob:^:a*"]);
+    let output = target_dir.run_jj(["git", "fetch", "--branch", "glob:'^:a*'"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Invalid branch pattern provided. When fetching, branch names and globs may not contain the characters `:`, `^`, `?`, `[`, `]`
@@ -1002,7 +1056,7 @@ fn test_git_fetch_some_of_many_bookmarks() {
     ◆  000000000000 ""
     [EOF]
     "#);
-    let output = target_dir.run_jj(["git", "fetch", "--branch", "b", "--branch", "a1"]);
+    let output = target_dir.run_jj(["git", "fetch", "--branch=~(a2 | glob:trunk*)"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     bookmark: a1@origin [updated] tracked
@@ -1080,7 +1134,10 @@ fn test_git_fetch_some_of_many_bookmarks() {
 #[test]
 fn test_git_fetch_bookmarks_some_missing() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
+    test_env.add_config("remotes.rem1.auto-track-bookmarks = 'glob:*'");
+    test_env.add_config("remotes.rem2.auto-track-bookmarks = 'glob:*'");
+    test_env.add_config("remotes.rem3.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "origin");
@@ -1092,7 +1149,7 @@ fn test_git_fetch_bookmarks_some_missing() {
     let output = work_dir.run_jj(["git", "fetch", "--branch", "noexist"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Warning: No branch matching `noexist` found on any specified/configured remote
+    Warning: No matching branches found on any specified/configured remote: noexist
     Nothing changed.
     [EOF]
     ");
@@ -1104,7 +1161,7 @@ fn test_git_fetch_bookmarks_some_missing() {
     ]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Warning: No branch matching `noexist1`, `noexist2` found on any specified/configured remote
+    Warning: No matching branches found on any specified/configured remote: noexist1, noexist2
     Nothing changed.
     [EOF]
     ");
@@ -1154,7 +1211,7 @@ fn test_git_fetch_bookmarks_some_missing() {
     ]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Warning: No branch matching `notexist` found on any specified/configured remote
+    Warning: No matching branches found on any specified/configured remote: notexist
     Nothing changed.
     [EOF]
     ");
@@ -1192,18 +1249,44 @@ fn test_git_fetch_bookmarks_missing_with_subprocess_localized_message() {
     });
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Warning: No branch matching `unknown` found on any specified/configured remote
+    Warning: No matching branches found on any specified/configured remote: unknown
     Nothing changed.
     [EOF]
     ");
 }
 
+#[test]
+fn test_git_fetch_unsupported_branch_patterns() {
+    let test_env = TestEnvironment::default();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+    add_git_remote(&test_env, &work_dir, "origin");
+
+    let output = work_dir.run_jj(["git", "fetch", "--branch=x&y|z"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Error: Cannot use `&` in sub expression
+    Hint: Specify patterns in `(positive | ...) & ~(negative | ...)` form.
+    [EOF]
+    [exit status: 1]
+    ");
+
+    // Unsupported glob pattern in negative refspecs
+    let output = work_dir.run_jj(["git", "fetch", "--branch=~glob:'[xy]'"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Error: Invalid branch pattern provided. When fetching, branch names and globs may not contain the characters `:`, `^`, `?`, `[`, `]`
+    [EOF]
+    [exit status: 1]
+    ");
+}
+
 // See `test_undo_restore_commands.rs` for fetch-undo-push and fetch-undo-fetch
 // of the same bookmarks for various kinds of undo.
 #[test]
 fn test_git_fetch_undo() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let source_dir = test_env.work_dir("source");
     git::init(source_dir.root());
 
@@ -1286,7 +1369,7 @@ fn test_git_fetch_undo() {
 #[test]
 fn test_fetch_undo_what() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let source_dir = test_env.work_dir("source");
     git::init(source_dir.root());
 
@@ -1362,6 +1445,7 @@ fn test_fetch_undo_what() {
     b (deleted)
       @origin: yostqsxw hidden bc83465a b
     newbookmark: qpvuntsm e8849ae1 (empty) (no description set)
+      @origin (not created yet)
     [EOF]
     ");
     // Restoring just the remote-tracking state will not affect `newbookmark`, but
@@ -1387,7 +1471,7 @@ fn test_fetch_undo_what() {
 #[test]
 fn test_git_fetch_remove_fetch() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "origin");
@@ -1397,6 +1481,7 @@ fn test_git_fetch_remove_fetch() {
         .success();
     insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
     origin: qpvuntsm e8849ae1 (empty) (no description set)
+      @origin (not created yet)
     [EOF]
     ");
 
@@ -1442,7 +1527,7 @@ fn test_git_fetch_remove_fetch() {
 #[test]
 fn test_git_fetch_rename_fetch() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "origin");
@@ -1452,6 +1537,7 @@ fn test_git_fetch_rename_fetch() {
         .success();
     insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
     origin: qpvuntsm e8849ae1 (empty) (no description set)
+      @origin (not created yet)
     [EOF]
     ");
 
@@ -1487,7 +1573,7 @@ fn test_git_fetch_rename_fetch() {
 #[test]
 fn test_git_fetch_removed_bookmark() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let source_dir = test_env.work_dir("source");
     git::init(source_dir.root());
 
@@ -1585,7 +1671,7 @@ fn test_git_fetch_removed_bookmark() {
 #[test]
 fn test_git_fetch_removed_parent_bookmark() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let source_dir = test_env.work_dir("source");
     git::init(source_dir.root());
 
@@ -1651,7 +1737,7 @@ fn test_git_fetch_removed_parent_bookmark() {
     bookmark: a1@origin     [deleted] untracked
     bookmark: trunk1@origin [deleted] untracked
     Abandoned 1 commits that are no longer reachable.
-    Warning: No branch matching `master` found on any specified/configured remote
+    Warning: No matching branches found on any specified/configured remote: master
     [EOF]
     ");
     insta::assert_snapshot!(get_log_output(&target_dir), @r#"
@@ -1689,8 +1775,8 @@ fn test_git_fetch_remote_only_bookmark() {
         &[],
     );
 
-    // Fetch using git.auto_local_bookmark = true
-    test_env.add_config("git.auto-local-bookmark = true");
+    // Fetch using remotes.origin.auto-track-bookmarks = 'glob:*'
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     work_dir
         .run_jj(["git", "fetch", "--remote=origin"])
         .success();
@@ -1708,8 +1794,8 @@ fn test_git_fetch_remote_only_bookmark() {
         &[],
     );
 
-    // Fetch using git.auto_local_bookmark = false
-    test_env.add_config("git.auto-local-bookmark = false");
+    // Fetch using remotes.origin.auto-track-bookmarks = ''
+    test_env.add_config("remotes.origin.auto-track-bookmarks = ''");
     work_dir
         .run_jj(["git", "fetch", "--remote=origin"])
         .success();
@@ -1731,7 +1817,8 @@ fn test_git_fetch_remote_only_bookmark() {
 #[test]
 fn test_git_fetch_preserve_commits_across_repos() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.upstream.auto-track-bookmarks = 'glob:*'");
+    test_env.add_config("remotes.fork.auto-track-bookmarks = 'glob:*'");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
 
@@ -1829,7 +1916,7 @@ fn test_git_fetch_preserve_commits_across_repos() {
 #[test]
 fn test_git_fetch_tracked() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
 
     // Set up a remote with multiple bookmarks
     let remote_path = test_env.env_root().join("remote");
@@ -1915,7 +2002,7 @@ fn test_git_fetch_tracked() {
 #[test]
 fn test_git_fetch_tracked_no_tracked_bookmarks() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
 
     // Set up a remote with bookmarks
     let remote_path = test_env.env_root().join("remote");
@@ -1952,7 +2039,8 @@ fn test_git_fetch_tracked_no_tracked_bookmarks() {
 #[test]
 fn test_git_fetch_tracked_multiple_remotes() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
+    test_env.add_config("remotes.upstream.auto-track-bookmarks = 'glob:*'");
 
     // Set up two remotes with different branches
     let origin_path = test_env.env_root().join("origin");
@@ -2047,3 +2135,49 @@ fn test_git_fetch_tracked_multiple_remotes() {
     [EOF]
     ");
 }
+
+#[test]
+fn test_git_fetch_auto_track_bookmarks() {
+    let test_env = TestEnvironment::default();
+    let root_dir = test_env.work_dir("");
+    test_env.add_config(
+        "
+        [remotes.origin]
+        auto-track-bookmarks = 'glob:mine/*'
+        ",
+    );
+
+    root_dir
+        .run_jj(["git", "init", "--colocate", "origin"])
+        .success();
+    let origin_dir = test_env.work_dir("origin");
+    origin_dir.run_jj(["b", "c", "mine/foo"]).success();
+    origin_dir.run_jj(["b", "c", "not-mine/foo"]).success();
+    origin_dir.run_jj(["commit", "-mfoo"]).success();
+    let output = origin_dir.run_jj(["show", "@-"]);
+    insta::assert_snapshot!(output, @r"
+    Commit ID: d7828da83253475bf10c2ae6bd3f0f84bf4604c1
+    Change ID: qpvuntsmwlqtpsluzzsnyyzlmlwvmlnu
+    Bookmarks: mine/foo not-mine/foo mine/foo@git not-mine/foo@git
+    Author   : Test User <test.user@example.com> (2001-02-03 08:05:10)
+    Committer: Test User <test.user@example.com> (2001-02-03 08:05:10)
+
+        foo
+
+    [EOF]
+    ");
+
+    root_dir.run_jj(["git", "init", "repo"]).success();
+    let repo_dir = test_env.work_dir("repo");
+    repo_dir
+        .run_jj(["git", "remote", "add", "origin", "../origin/.git"])
+        .success();
+
+    let output = repo_dir.run_jj(["git", "fetch"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    bookmark: mine/foo@origin     [new] tracked
+    bookmark: not-mine/foo@origin [new] untracked
+    [EOF]
+    ");
+}
