@@ -603,7 +603,7 @@ fn test_git_init_colocated_via_git_repo_path_symlink_gitlink() {
 #[test]
 fn test_git_init_colocated_via_git_repo_path_imported_refs() {
     let test_env = TestEnvironment::default();
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
 
     // Set up remote refs
     test_env.run_jj_in(".", ["git", "init", "remote"]).success();
@@ -632,7 +632,7 @@ fn test_git_init_colocated_via_git_repo_path_imported_refs() {
             .unwrap();
     };
 
-    // With git.auto-local-bookmark = true
+    // With remotes.origin.auto-track-bookmarks = 'glob:*'
     let local_dir = test_env.work_dir("local1");
     set_up_local_repo(local_dir.root());
     let output = local_dir.run_jj(["git", "init", "--git-repo=."]);
@@ -652,8 +652,8 @@ fn test_git_init_colocated_via_git_repo_path_imported_refs() {
     [EOF]
     ");
 
-    // With git.auto-local-bookmark = false
-    test_env.add_config("git.auto-local-bookmark = false");
+    // With remotes.origin.auto-track-bookmarks = ''
+    test_env.add_config("remotes.origin.auto-track-bookmarks = ''");
     let local_dir = test_env.work_dir("local2");
     set_up_local_repo(local_dir.root());
     let output = local_dir.run_jj(["git", "init", "--git-repo=."]);
