@@ -53,7 +53,7 @@ fn set_up(test_env: &TestEnvironment) {
             [
                 "git",
                 "clone",
-                "--config=git.auto-local-bookmark=true",
+                "--config=remotes.origin.auto-track-bookmarks='glob:*'",
                 origin_git_repo_path.to_str().unwrap(),
                 "local",
             ],
@@ -87,6 +87,7 @@ fn test_git_push_nothing() {
 fn test_git_push_current_bookmark() {
     let test_env = TestEnvironment::default();
     set_up(&test_env);
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let work_dir = test_env.work_dir("local");
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
     // Update some bookmarks. `bookmark1` is not a current bookmark, but
@@ -109,10 +110,11 @@ fn test_git_push_current_bookmark() {
     bookmark2: yostqsxw 88ca14a7 (empty) foo
       @origin (behind by 1 commits): zsuskuln 38a20473 (empty) description 2
     my-bookmark: yostqsxw 88ca14a7 (empty) foo
+      @origin (not created yet)
     [EOF]
     ");
     // First dry-run. `bookmark1` should not get pushed.
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "--dry-run"]);
+    let output = work_dir.run_jj(["git", "push", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Changes to push to origin:
@@ -121,7 +123,7 @@ fn test_git_push_current_bookmark() {
     Dry-run requested, not pushing.
     [EOF]
     ");
-    let output = work_dir.run_jj(["git", "push", "--allow-new"]);
+    let output = work_dir.run_jj(["git", "push"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Changes to push to origin:
@@ -273,7 +275,10 @@ fn test_git_push_other_remote_has_bookmark() {
     // as it is on the remote. This would also work for a descendant.
     //
     // TODO: Saner test?
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "--remote=other"]);
+    work_dir
+        .run_jj(["bookmark", "track", "bookmark1@other"])
+        .success();
+    let output = work_dir.run_jj(["git", "push", "--remote=other"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Changes to push to other:
@@ -532,6 +537,7 @@ fn test_git_push_unexpectedly_deleted() {
 fn test_git_push_creation_unexpectedly_already_exists() {
     let test_env = TestEnvironment::default();
     set_up(&test_env);
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let work_dir = test_env.work_dir("local");
 
     // Forget bookmark1 locally
@@ -539,7 +545,7 @@ fn test_git_push_creation_unexpectedly_already_exists() {
         .run_jj(["bookmark", "forget", "--include-remotes", "bookmark1"])
         .success();
 
-    // Create a new branh1
+    // Create a new bookmark1
     work_dir
         .run_jj(["new", "root()", "-m=new bookmark1"])
         .success();
@@ -549,12 +555,13 @@ fn test_git_push_creation_unexpectedly_already_exists() {
         .success();
     insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
     bookmark1: yostqsxw a43cb801 new bookmark1
+      @origin (not created yet)
     bookmark2: zsuskuln 38a20473 (empty) description 2
       @origin: zsuskuln 38a20473 (empty) description 2
     [EOF]
     ");
 
-    let output = work_dir.run_jj(["git", "push", "--allow-new"]);
+    let output = work_dir.run_jj(["git", "push"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Changes to push to origin:
@@ -574,7 +581,7 @@ fn test_git_push_locally_created_and_rewritten() {
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
     // Ensure that remote bookmarks aren't tracked automatically
-    test_env.add_config("git.auto-local-bookmark = false");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = ''");
 
     // Push locally-created bookmark
     work_dir.run_jj(["new", "root()", "-mlocal 1"]).success();
@@ -585,7 +592,7 @@ fn test_git_push_locally_created_and_rewritten() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Warning: Refusing to create new remote bookmark my@origin
-    Hint: Use --allow-new to push new bookmark. Use --remote to specify the remote to push to.
+    Hint: Run `jj bookmark track my@origin` and try again.
     Nothing changed.
     [EOF]
     ");
@@ -593,6 +600,7 @@ fn test_git_push_locally_created_and_rewritten() {
     let output = work_dir.run_jj(["git", "push", "--allow-new", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: --allow-new is deprecated, track bookmarks manually or configure remotes.<name>.auto-track-bookmarks instead.
     Changes to push to origin:
       Add bookmark my to e0cba5e497ee
     Dry-run requested, not pushing.
@@ -606,6 +614,9 @@ fn test_git_push_locally_created_and_rewritten() {
     ]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: Deprecated CLI-provided config: `git.push-new-bookmarks` is deprecated; use `remotes.<name>.auto-track-bookmarks` instead.
+    Example: jj config set --user remotes.origin.auto-track-bookmarks 'glob:*'
+    For details, see: https://docs.jj-vcs.dev/latest/config/#automatic-tracking-of-bookmarks
     Changes to push to origin:
       Add bookmark my to e0cba5e497ee
     Dry-run requested, not pushing.
@@ -648,6 +659,7 @@ fn test_git_push_locally_created_and_rewritten() {
 fn test_git_push_multiple() {
     let test_env = TestEnvironment::default();
     set_up(&test_env);
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let work_dir = test_env.work_dir("local");
     work_dir
         .run_jj(["bookmark", "delete", "bookmark1"])
@@ -666,6 +678,7 @@ fn test_git_push_multiple() {
     bookmark2: yqosqzyt 352fa187 (empty) foo
       @origin (ahead by 1 commits, behind by 1 commits): zsuskuln 38a20473 (empty) description 2
     my-bookmark: yqosqzyt 352fa187 (empty) foo
+      @origin (not created yet)
     [EOF]
     ");
     // First dry-run
@@ -680,14 +693,7 @@ fn test_git_push_multiple() {
     [EOF]
     ");
     // Dry run requesting two specific bookmarks
-    let output = work_dir.run_jj([
-        "git",
-        "push",
-        "--allow-new",
-        "-b=bookmark1",
-        "-b=my-bookmark",
-        "--dry-run",
-    ]);
+    let output = work_dir.run_jj(["git", "push", "-b=bookmark1", "-b=my-bookmark", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Changes to push to origin:
@@ -700,7 +706,6 @@ fn test_git_push_multiple() {
     let output = work_dir.run_jj([
         "git",
         "push",
-        "--allow-new",
         "-b=bookmark1",
         "-b=my-bookmark",
         "-b=bookmark1",
@@ -716,7 +721,7 @@ fn test_git_push_multiple() {
     [EOF]
     ");
     // Dry run with glob pattern
-    let output = work_dir.run_jj(["git", "push", "-b=glob:bookmark?", "--dry-run"]);
+    let output = work_dir.run_jj(["git", "push", "-b=glob:'bookmark?'", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Changes to push to origin:
@@ -726,20 +731,20 @@ fn test_git_push_multiple() {
     [EOF]
     ");
 
-    // Unmatched bookmark name is error
+    // Unmatched bookmark name
     let output = work_dir.run_jj(["git", "push", "-b=foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Error: No such bookmark: foo
+    Warning: No matching bookmarks for names: foo
+    Nothing changed.
     [EOF]
-    [exit status: 1]
     ");
-    let output = work_dir.run_jj(["git", "push", "-b=foo", "-b=glob:?bookmark"]);
+    let output = work_dir.run_jj(["git", "push", "-b=foo", "-b=glob:'?bookmark'"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Error: No matching bookmarks for patterns: foo, ?bookmark
+    Warning: No matching bookmarks for names: foo
+    Nothing changed.
     [EOF]
-    [exit status: 1]
     ");
 
     // --deleted is required to push deleted bookmarks even with --all
@@ -866,7 +871,7 @@ fn test_git_push_changes() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Bookmark already exists: push-yostqsxwqrlt
-    Hint: Use 'jj bookmark move' to move it, and 'jj git push -b push-yostqsxwqrlt [--allow-new]' to push it
+    Hint: Use 'jj bookmark move' to move it, and 'jj git push -b push-yostqsxwqrlt' to push it
     [EOF]
     [exit status: 1]
     ");
@@ -1013,7 +1018,7 @@ fn test_git_push_changes_with_name() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Bookmark already exists: b1
-    Hint: Use 'jj bookmark move' to move it, and 'jj git push -b b1 [--allow-new]' to push it
+    Hint: Use 'jj bookmark move' to move it, and 'jj git push -b b1' to push it
     [EOF]
     [exit status: 1]
     ");
@@ -1213,7 +1218,11 @@ fn test_git_push_changes_with_name_untracked_or_forgotten() {
     let output = work_dir
         .run_jj(["bookmark", "list", "--all", "b1"])
         .success();
-    insta::assert_snapshot!(output, @"");
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Warning: No matching bookmarks for names: b1
+    [EOF]
+    ");
 
     // Make sure push still errors if we try to push a bookmark with the same name
     // to a different location.
@@ -1232,7 +1241,11 @@ fn test_git_push_changes_with_name_untracked_or_forgotten() {
 
     // The bookmark is still forgotten
     let output = work_dir.run_jj(["bookmark", "list", "--all", "b1"]);
-    insta::assert_snapshot!(output, @"");
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Warning: No matching bookmarks for names: b1
+    [EOF]
+    ");
     let output = work_dir.run_jj(["git", "push", "--named", "b1=@+"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
@@ -1261,6 +1274,7 @@ fn test_git_push_changes_with_name_untracked_or_forgotten() {
 fn test_git_push_revisions() {
     let test_env = TestEnvironment::default();
     set_up(&test_env);
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let work_dir = test_env.work_dir("local");
     work_dir.run_jj(["describe", "-m", "foo"]).success();
     work_dir.write_file("file", "contents");
@@ -1279,7 +1293,7 @@ fn test_git_push_revisions() {
     work_dir.write_file("file", "modified again");
 
     // Push an empty set
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "-r=none()"]);
+    let output = work_dir.run_jj(["git", "push", "-r=none()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Warning: No bookmarks point to the specified revisions: none()
@@ -1287,7 +1301,7 @@ fn test_git_push_revisions() {
     [EOF]
     ");
     // Push a revision with no bookmarks
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "-r=@--"]);
+    let output = work_dir.run_jj(["git", "push", "-r=@--"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Warning: No bookmarks point to the specified revisions: @--
@@ -1295,7 +1309,7 @@ fn test_git_push_revisions() {
     [EOF]
     ");
     // Push a revision with a single bookmark
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "-r=@-", "--dry-run"]);
+    let output = work_dir.run_jj(["git", "push", "-r=@-", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Changes to push to origin:
@@ -1304,7 +1318,7 @@ fn test_git_push_revisions() {
     [EOF]
     ");
     // Push multiple revisions of which some have bookmarks
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "-r=@--", "-r=@-", "--dry-run"]);
+    let output = work_dir.run_jj(["git", "push", "-r=@--", "-r=@-", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Warning: No bookmarks point to the specified revisions: @--
@@ -1314,7 +1328,7 @@ fn test_git_push_revisions() {
     [EOF]
     ");
     // Push a revision with a multiple bookmarks
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "-r=@", "--dry-run"]);
+    let output = work_dir.run_jj(["git", "push", "-r=@", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Changes to push to origin:
@@ -1324,7 +1338,7 @@ fn test_git_push_revisions() {
     [EOF]
     ");
     // Repeating a commit doesn't result in repeated messages about the bookmark
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "-r=@-", "-r=@-", "--dry-run"]);
+    let output = work_dir.run_jj(["git", "push", "-r=@-", "-r=@-", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Changes to push to origin:
@@ -1367,7 +1381,7 @@ fn test_git_push_mixed() {
     ------- stderr -------
     Creating bookmark push-yqosqzytrlsw for revision yqosqzytrlsw
     Error: Refusing to create new remote bookmark bookmark-1@origin
-    Hint: Use --allow-new to push new bookmark. Use --remote to specify the remote to push to.
+    Hint: Run `jj bookmark track bookmark-1@origin` and try again.
     [EOF]
     [exit status: 1]
     ");
@@ -1382,6 +1396,7 @@ fn test_git_push_mixed() {
     ]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: --allow-new is deprecated, track bookmarks manually or configure remotes.<name>.auto-track-bookmarks instead.
     Creating bookmark push-yqosqzytrlsw for revision yqosqzytrlsw
     Changes to push to origin:
       Add bookmark push-yqosqzytrlsw to 0f8164cd580b
@@ -1415,7 +1430,7 @@ fn test_git_push_conflict() {
     work_dir.run_jj(["commit", "-m", "second"]).success();
     work_dir.write_file("file", "third");
     work_dir
-        .run_jj(["rebase", "-r", "@", "-d", "@--"])
+        .run_jj(["rebase", "-r", "@", "-o", "@--"])
         .success();
     work_dir
         .run_jj(["bookmark", "create", "-r@", "my-bookmark"])
@@ -1424,8 +1439,8 @@ fn test_git_push_conflict() {
     let output = work_dir.run_jj(["git", "push", "--all"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Error: Won't push commit 654e715becca since it has conflicts
-    Hint: Rejected commit: yostqsxw 654e715b my-bookmark | (conflict) third
+    Error: Won't push commit d1b152bb0916 since it has conflicts
+    Hint: Rejected commit: yostqsxw d1b152bb my-bookmark | (conflict) third
     [EOF]
     [exit status: 1]
     ");
@@ -1435,16 +1450,17 @@ fn test_git_push_conflict() {
 fn test_git_push_no_description() {
     let test_env = TestEnvironment::default();
     set_up(&test_env);
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let work_dir = test_env.work_dir("local");
     work_dir
         .run_jj(["bookmark", "create", "-r@", "my-bookmark"])
         .success();
     work_dir.run_jj(["describe", "-m="]).success();
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "--bookmark", "my-bookmark"]);
+    let output = work_dir.run_jj(["git", "push", "--bookmark", "my-bookmark"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Won't push commit 8d23abddc924 since it has no description
-    Hint: Rejected commit: yqosqzyt 8d23abdd my-bookmark | (empty) (no description set)
+    Hint: Rejected commit: yqosqzyt 8d23abdd my-bookmark* | (empty) (no description set)
     [EOF]
     [exit status: 1]
     ");
@@ -1452,7 +1468,6 @@ fn test_git_push_no_description() {
         .run_jj([
             "git",
             "push",
-            "--allow-new",
             "--bookmark",
             "my-bookmark",
             "--allow-empty-description",
@@ -1464,6 +1479,7 @@ fn test_git_push_no_description() {
 fn test_git_push_no_description_in_immutable() {
     let test_env = TestEnvironment::default();
     set_up(&test_env);
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let work_dir = test_env.work_dir("local");
     work_dir
         .run_jj(["bookmark", "create", "-r@", "imm"])
@@ -1475,29 +1491,17 @@ fn test_git_push_no_description_in_immutable() {
         .run_jj(["bookmark", "create", "-r@", "my-bookmark"])
         .success();
 
-    let output = work_dir.run_jj([
-        "git",
-        "push",
-        "--allow-new",
-        "--bookmark=my-bookmark",
-        "--dry-run",
-    ]);
+    let output = work_dir.run_jj(["git", "push", "--bookmark=my-bookmark", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Won't push commit 8d23abddc924 since it has no description
-    Hint: Rejected commit: yqosqzyt 8d23abdd imm | (empty) (no description set)
+    Hint: Rejected commit: yqosqzyt 8d23abdd imm* | (empty) (no description set)
     [EOF]
     [exit status: 1]
     ");
 
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "imm""#);
-    let output = work_dir.run_jj([
-        "git",
-        "push",
-        "--allow-new",
-        "--bookmark=my-bookmark",
-        "--dry-run",
-    ]);
+    let output = work_dir.run_jj(["git", "push", "--bookmark=my-bookmark", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Changes to push to origin:
@@ -1511,6 +1515,7 @@ fn test_git_push_no_description_in_immutable() {
 fn test_git_push_missing_author() {
     let test_env = TestEnvironment::default();
     set_up(&test_env);
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let work_dir = test_env.work_dir("local");
     let run_without_var = |var: &str, args: &[&str]| {
         work_dir
@@ -1519,21 +1524,21 @@ fn test_git_push_missing_author() {
     };
     run_without_var("JJ_USER", &["new", "root()", "-m=initial"]);
     run_without_var("JJ_USER", &["bookmark", "create", "-r@", "missing-name"]);
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "--bookmark", "missing-name"]);
+    let output = work_dir.run_jj(["git", "push", "--bookmark", "missing-name"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Won't push commit 613adaba9d49 since it has no author and/or committer set
-    Hint: Rejected commit: vruxwmqv 613adaba missing-name | (empty) initial
+    Hint: Rejected commit: vruxwmqv 613adaba missing-name* | (empty) initial
     [EOF]
     [exit status: 1]
     ");
     run_without_var("JJ_EMAIL", &["new", "root()", "-m=initial"]);
     run_without_var("JJ_EMAIL", &["bookmark", "create", "-r@", "missing-email"]);
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "--bookmark=missing-email"]);
+    let output = work_dir.run_jj(["git", "push", "--bookmark=missing-email"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Won't push commit bb4ea60fc9ba since it has no author and/or committer set
-    Hint: Rejected commit: kpqxywon bb4ea60f missing-email | (empty) initial
+    Hint: Rejected commit: kpqxywon bb4ea60f missing-email* | (empty) initial
     [EOF]
     [exit status: 1]
     ");
@@ -1543,6 +1548,7 @@ fn test_git_push_missing_author() {
 fn test_git_push_missing_author_in_immutable() {
     let test_env = TestEnvironment::default();
     set_up(&test_env);
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let work_dir = test_env.work_dir("local");
     let run_without_var = |var: &str, args: &[&str]| {
         work_dir
@@ -1560,29 +1566,17 @@ fn test_git_push_missing_author_in_immutable() {
         .run_jj(["bookmark", "create", "-r@", "my-bookmark"])
         .success();
 
-    let output = work_dir.run_jj([
-        "git",
-        "push",
-        "--allow-new",
-        "--bookmark=my-bookmark",
-        "--dry-run",
-    ]);
+    let output = work_dir.run_jj(["git", "push", "--bookmark=my-bookmark", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Won't push commit 5c3cc711907f since it has no author and/or committer set
-    Hint: Rejected commit: yostqsxw 5c3cc711 imm | (empty) no author email
+    Hint: Rejected commit: yostqsxw 5c3cc711 imm* | (empty) no author email
     [EOF]
     [exit status: 1]
     ");
 
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "imm""#);
-    let output = work_dir.run_jj([
-        "git",
-        "push",
-        "--allow-new",
-        "--bookmark=my-bookmark",
-        "--dry-run",
-    ]);
+    let output = work_dir.run_jj(["git", "push", "--bookmark=my-bookmark", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Changes to push to origin:
@@ -1596,6 +1590,7 @@ fn test_git_push_missing_author_in_immutable() {
 fn test_git_push_missing_committer() {
     let test_env = TestEnvironment::default();
     set_up(&test_env);
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let work_dir = test_env.work_dir("local");
     let run_without_var = |var: &str, args: &[&str]| {
         work_dir
@@ -1606,11 +1601,11 @@ fn test_git_push_missing_committer() {
         .run_jj(["bookmark", "create", "-r@", "missing-name"])
         .success();
     run_without_var("JJ_USER", &["describe", "-m=no committer name"]);
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "--bookmark=missing-name"]);
+    let output = work_dir.run_jj(["git", "push", "--bookmark=missing-name"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Won't push commit e8a77cb24da9 since it has no author and/or committer set
-    Hint: Rejected commit: yqosqzyt e8a77cb2 missing-name | (empty) no committer name
+    Hint: Rejected commit: yqosqzyt e8a77cb2 missing-name* | (empty) no committer name
     [EOF]
     [exit status: 1]
     ");
@@ -1619,11 +1614,11 @@ fn test_git_push_missing_committer() {
         .run_jj(["bookmark", "create", "-r@", "missing-email"])
         .success();
     run_without_var("JJ_EMAIL", &["describe", "-m=no committer email"]);
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "--bookmark=missing-email"]);
+    let output = work_dir.run_jj(["git", "push", "--bookmark=missing-email"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Won't push commit 971c50fd8d1d since it has no author and/or committer set
-    Hint: Rejected commit: kpqxywon 971c50fd missing-email | (empty) no committer email
+    Hint: Rejected commit: kpqxywon 971c50fd missing-email* | (empty) no committer email
     [EOF]
     [exit status: 1]
     ");
@@ -1631,11 +1626,11 @@ fn test_git_push_missing_committer() {
     // Test message when there are multiple reasons (missing committer and
     // description)
     run_without_var("JJ_EMAIL", &["describe", "-m=", "missing-email"]);
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "--bookmark=missing-email"]);
+    let output = work_dir.run_jj(["git", "push", "--bookmark=missing-email"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Won't push commit 4bd3b55c7759 since it has no description and it has no author and/or committer set
-    Hint: Rejected commit: kpqxywon 4bd3b55c missing-email | (empty) (no description set)
+    Hint: Rejected commit: kpqxywon 4bd3b55c missing-email* | (empty) (no description set)
     [EOF]
     [exit status: 1]
     ");
@@ -1645,6 +1640,7 @@ fn test_git_push_missing_committer() {
 fn test_git_push_missing_committer_in_immutable() {
     let test_env = TestEnvironment::default();
     set_up(&test_env);
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let work_dir = test_env.work_dir("local");
     let run_without_var = |var: &str, args: &[&str]| {
         work_dir
@@ -1663,29 +1659,17 @@ fn test_git_push_missing_committer_in_immutable() {
         .run_jj(["bookmark", "create", "-r@", "my-bookmark"])
         .success();
 
-    let output = work_dir.run_jj([
-        "git",
-        "push",
-        "--allow-new",
-        "--bookmark=my-bookmark",
-        "--dry-run",
-    ]);
+    let output = work_dir.run_jj(["git", "push", "--bookmark=my-bookmark", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Won't push commit ab230f98c812 since it has no author and/or committer set
-    Hint: Rejected commit: yostqsxw ab230f98 imm | (empty) no committer email
+    Hint: Rejected commit: yostqsxw ab230f98 imm* | (empty) no committer email
     [EOF]
     [exit status: 1]
     ");
 
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "imm""#);
-    let output = work_dir.run_jj([
-        "git",
-        "push",
-        "--allow-new",
-        "--bookmark=my-bookmark",
-        "--dry-run",
-    ]);
+    let output = work_dir.run_jj(["git", "push", "--bookmark=my-bookmark", "--dry-run"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Changes to push to origin:
@@ -1735,7 +1719,7 @@ fn test_git_push_conflicting_bookmarks() {
     let test_env = TestEnvironment::default();
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    test_env.add_config("git.auto-local-bookmark = true");
+    test_env.add_config("remotes.origin.auto-track-bookmarks = 'glob:*'");
     let git_repo = {
         let mut git_repo_path = work_dir.root().to_owned();
         git_repo_path.extend([".jj", "repo", "store", "git"]);
@@ -1774,7 +1758,7 @@ fn test_git_push_conflicting_bookmarks() {
     };
 
     // Conflicting bookmark at @
-    let output = work_dir.run_jj(["git", "push", "--allow-new"]);
+    let output = work_dir.run_jj(["git", "push"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Warning: Bookmark bookmark2 is conflicted
@@ -1784,7 +1768,7 @@ fn test_git_push_conflicting_bookmarks() {
     ");
 
     // --bookmark should be blocked by conflicting bookmark
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "--bookmark", "bookmark2"]);
+    let output = work_dir.run_jj(["git", "push", "--bookmark", "bookmark2"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Bookmark bookmark2 is conflicted
@@ -1807,7 +1791,7 @@ fn test_git_push_conflicting_bookmarks() {
 
     // --revisions shouldn't be blocked by conflicting bookmark
     bump_bookmark1();
-    let output = work_dir.run_jj(["git", "push", "--allow-new", "-rall()"]);
+    let output = work_dir.run_jj(["git", "push", "-rall()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Warning: Bookmark bookmark2 is conflicted
@@ -1841,9 +1825,9 @@ fn test_git_push_deleted_untracked() {
     let output = work_dir.run_jj(["git", "push", "--bookmark=bookmark1"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Error: No such bookmark: bookmark1
+    Warning: No matching bookmarks for names: bookmark1
+    Nothing changed.
     [EOF]
-    [exit status: 1]
     ");
 }
 
@@ -1961,7 +1945,7 @@ fn test_git_push_moved_forward_untracked() {
     work_dir
         .run_jj(["bookmark", "untrack", "bookmark1@origin"])
         .success();
-    let output = work_dir.run_jj(["git", "push", "--allow-new"]);
+    let output = work_dir.run_jj(["git", "push"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Warning: Non-tracking remote bookmark bookmark1@origin exists
@@ -1986,7 +1970,7 @@ fn test_git_push_moved_sideways_untracked() {
     work_dir
         .run_jj(["bookmark", "untrack", "bookmark1@origin"])
         .success();
-    let output = work_dir.run_jj(["git", "push", "--allow-new"]);
+    let output = work_dir.run_jj(["git", "push"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Warning: Non-tracking remote bookmark bookmark1@origin exists
@@ -2151,8 +2135,7 @@ fn test_git_push_sign_on_push() {
         "bookmark",
         "create",
         "bookmark3",
-        "-r",
-        "description('commit which should not be signed 1')",
+        "-rsubject(glob:'commit which should not be signed 1')",
     ]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
@@ -2170,7 +2153,7 @@ fn test_git_push_sign_on_push() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Warning: Refusing to create new remote bookmark bookmark3@origin
-    Hint: Use --allow-new to push new bookmark. Use --remote to specify the remote to push to.
+    Hint: Run `jj bookmark track bookmark3@origin` and try again.
     Changes to push to origin:
       Move forward bookmark bookmark2 from d45e2adce0ad to 48ea83e9499c
     [EOF]
