@@ -350,7 +350,7 @@ fn test_invalid_filesets_looking_like_filepaths() {
       |     ^---
       |
       = expected `~` or <primary>
-    Hint: See https://jj-vcs.github.io/jj/latest/filesets/ or use `jj help -k filesets` for filesets syntax and how to match file paths.
+    Hint: See https://docs.jj-vcs.dev/latest/filesets/ or use `jj help -k filesets` for filesets syntax and how to match file paths.
     [EOF]
     [exit status: 1]
     ");
@@ -518,7 +518,7 @@ fn test_color_config() {
     Config error: Invalid type or value for ui.color
     Caused by: wanted string or table
 
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
     [EOF]
     [exit status: 1]
     ");
@@ -564,6 +564,7 @@ fn test_color_ui_messages() {
     let output = work_dir.run_jj(["log", "@"]);
     insta::assert_snapshot!(output, @r#"
     ------- stderr -------
+    [1m[38;5;3mWarning: [39mNo matching entries for paths: @[0m
     [1m[38;5;3mWarning: [39mThe argument "@" is being interpreted as a fileset expression. To specify a revset, pass -r "@" instead.[0m
     [EOF]
     "#);
@@ -744,7 +745,7 @@ fn test_config_args() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Config error: --config must be specified as NAME=VALUE
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
     [EOF]
     [exit status: 1]
     ");
@@ -759,7 +760,7 @@ fn test_config_args() {
         Caused by:
         1: Cannot access unknown.toml
         2: <redacted>
-        For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+        For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
         [EOF]
         [exit status: 1]
         ");
@@ -783,7 +784,7 @@ fn test_invalid_config() {
     unexpected key or value, expected newline, `#`
 
     Hint: Check the config file: $TEST_ENV/config/config0002.toml
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
     [EOF]
     [exit status: 1]
     ");
@@ -802,7 +803,7 @@ fn test_invalid_config_value() {
     Config error: Invalid type or value for snapshot.auto-track
     Caused by: invalid type: sequence, expected a string
 
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
     [EOF]
     [exit status: 1]
     ");
