@@ -126,6 +126,7 @@ fn test_interdiff_paths() {
         "right",
         "file1",
         "file2",
+        "nonexistent",
     ]);
     insta::assert_snapshot!(output, @r"
     Modified regular file file1:
@@ -133,6 +134,31 @@ fn test_interdiff_paths() {
     Modified regular file file2:
        1    1: barbaz
     [EOF]
+    ------- stderr -------
+    Warning: No matching entries for paths: nonexistent
+    [EOF]
+    ");
+
+    // Running interdiff on commits with deleted files should not show a warning.
+    work_dir.run_jj(["edit", "right"]).success();
+    work_dir.remove_file("file1");
+    work_dir.run_jj(["new"]).success();
+
+    let output = work_dir.run_jj([
+        "interdiff",
+        "--from",
+        "left",
+        "--to",
+        "right",
+        "file1",
+        "file2",
+    ]);
+    insta::assert_snapshot!(output, @r"
+    Removed regular file file1:
+       1     : bar
+    Modified regular file file2:
+       1    1: barbaz
+    [EOF]
     ");
 }
 
