@@ -75,6 +75,65 @@ fn test_log_with_no_template() {
     ");
 }
 
+#[test]
+fn test_log_with_diff_stats() {
+    let test_env = TestEnvironment::default();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    work_dir.write_file("file1", "foo\n");
+    work_dir.write_file("file2", "bar\nfoo\n");
+    work_dir.write_file("file3", "foo\n");
+    work_dir.write_file("file4", "foo\n");
+    work_dir.write_file("file5", "foobar\nbaz\n");
+    work_dir
+        .run_jj(["describe", "-m", "initial commit"])
+        .success();
+    work_dir.run_jj(["new", "-m", "multiple changes"]).success();
+    work_dir.write_file("file1", "foo\nbar\n");
+    work_dir.write_file("file2", "bar\n");
+    work_dir.write_file("file3", "baz\n");
+    work_dir.remove_file("file4");
+    work_dir.remove_file("file5");
+    work_dir.write_file("file6", "foobar\nbaz\n");
+
+    let template = r#"
+    self.diff().files().map(
+        |f| f.status() ++ " " ++ f.source().path() ++ " " ++ f.target().path()
+    ).join("\n")
+    ++ "\n" ++
+    self.diff().stat().files().map(
+        |f| f.status_char() ++ " " ++ f.status() ++ " " ++ f.lines_added() ++ " " ++ f.lines_removed() ++ " " ++ f.path()
+    ).join("\n")
+    "#;
+
+    let output = work_dir.run_jj(["log", "-T", template]);
+    insta::assert_snapshot!(output, @r"
+    @  modified file1 file1
+    │  modified file2 file2
+    │  modified file3 file3
+    │  removed file4 file4
+    │  renamed file5 file6
+    │  M modified 1 0 file1
+    │  M modified 0 1 file2
+    │  M modified 1 1 file3
+    │  D removed 0 1 file4
+    │  R renamed 0 0 file6
+    ○  added file1 file1
+    │  added file2 file2
+    │  added file3 file3
+    │  added file4 file4
+    │  added file5 file5
+    │  A added 1 0 file1
+    │  A added 2 0 file2
+    │  A added 1 0 file3
+    │  A added 1 0 file4
+    │  A added 2 0 file5
+    ◆
+    [EOF]
+    ");
+}
+
 #[test]
 fn test_log_with_or_without_diff() {
     let test_env = TestEnvironment::default();
@@ -422,7 +481,7 @@ fn test_log_shortest_accessors() {
     // Create 2^3 duplicates of the chain
     for _ in 0..3 {
         work_dir
-            .run_jj(["duplicate", "description(commit)"])
+            .run_jj(["duplicate", "subject(glob:commit*)"])
             .success();
     }
 
@@ -530,7 +589,7 @@ fn test_log_bad_short_prefixes() {
       | ^---
       |
       = expected <strict_identifier> or <expression>
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
     [EOF]
     [exit status: 1]
     ");
@@ -611,7 +670,7 @@ fn test_log_prefix_highlight_styled() {
     // Create 2^3 duplicates of the chain
     for _ in 0..3 {
         work_dir
-            .run_jj(["duplicate", "description(commit)"])
+            .run_jj(["duplicate", "subject(glob:commit*)"])
             .success();
     }
 
@@ -729,10 +788,12 @@ fn test_log_prefix_highlight_counts_hidden_commits() {
     work_dir.run_jj(["new", "root()", "-m", "extra"]).success();
     for _ in 0..7 {
         work_dir
-            .run_jj(["duplicate", "description(extra)"])
+            .run_jj(["duplicate", "subject(glob:extra)"])
             .success();
     }
-    work_dir.run_jj(["abandon", "description(extra)"]).success();
+    work_dir
+        .run_jj(["abandon", "subject(glob:extra)"])
+        .success();
 
     // The unique prefixes became longer.
     insta::assert_snapshot!(work_dir.run_jj(["log", "-T", prefix_format]), @r"
@@ -856,6 +917,24 @@ fn test_log_filtered_by_path() {
     work_dir.write_file("file1", "foo\nbar\n");
     work_dir.write_file("file2", "baz\n");
 
+    // The output filtered to a non-existent file should display a warning.
+    let output = work_dir.run_jj(["log", "-r", "@-", "-T", "description", "nonexistent"]);
+    insta::assert_snapshot!(output, @r#"
+    ------- stderr -------
+    Warning: No matching entries for paths: nonexistent
+    [EOF]
+    "#);
+
+    // The output filtered to a non-existent file should display a warning.
+    // The warning should be displayed at the beginning of the output.
+    let output = work_dir.run_jj(["log", "-T", "description", "nonexistent"]);
+    insta::assert_snapshot!(output, @r#"
+    ------- stderr -------
+    Warning: No matching entries for paths: nonexistent
+    Warning: The argument "nonexistent" is being interpreted as a fileset expression. To specify a revset, pass -r "nonexistent" instead.
+    [EOF]
+    "#);
+
     let output = work_dir.run_jj(["log", "-T", "description", "file1"]);
     insta::assert_snapshot!(output, @r"
     @  second
@@ -951,11 +1030,11 @@ fn test_log_limit() {
     work_dir.run_jj(["new", "-m", "b"]).success();
     work_dir.write_file("b", "");
     work_dir
-        .run_jj(["new", "-m", "c", "description(a)"])
+        .run_jj(["new", "-m", "c", "subject(glob:a)"])
         .success();
     work_dir.write_file("c", "");
     work_dir
-        .run_jj(["new", "-m", "d", "description(c)", "description(b)"])
+        .run_jj(["new", "-m", "d", "subject(glob:c)", "subject(glob:b)"])
         .success();
 
     let output = work_dir.run_jj(["log", "-T", "description", "--limit=3"]);
@@ -1016,6 +1095,9 @@ fn test_log_limit() {
     │
     ~
     [EOF]
+    ------- stderr -------
+    Warning: No matching entries for paths: b
+    [EOF]
     ");
 }
 
@@ -1048,15 +1130,21 @@ fn test_log_warn_path_might_be_revset() {
     [EOF]
     "#);
 
-    // ...but checking `jj log .` makes sense in a subdirectory.
+    // warn when checking `jj log .` in a subdirectory because this folder hasn't
+    // been added to the working copy, yet.
     let sub_dir = work_dir.create_dir_all("dir");
     let output = sub_dir.run_jj(["log", "."]);
-    insta::assert_snapshot!(output, @"");
+    insta::assert_snapshot!(output, @"
+    ------- stderr -------
+    Warning: No matching entries for paths: .
+    [EOF]
+    ");
 
     // Warn for `jj log @` instead of `jj log -r @`.
     let output = work_dir.run_jj(["log", "@", "-T", "description"]);
     insta::assert_snapshot!(output, @r#"
     ------- stderr -------
+    Warning: No matching entries for paths: @
     Warning: The argument "@" is being interpreted as a fileset expression. To specify a revset, pass -r "@" instead.
     [EOF]
     "#);
@@ -1065,13 +1153,18 @@ fn test_log_warn_path_might_be_revset() {
     let output = work_dir.run_jj(["log", "file2", "-T", "description"]);
     insta::assert_snapshot!(output, @r#"
     ------- stderr -------
+    Warning: No matching entries for paths: file2
     Warning: The argument "file2" is being interpreted as a fileset expression. To specify a revset, pass -r "file2" instead.
     [EOF]
     "#);
 
     // If an explicit revision is provided, then suppress the warning.
     let output = work_dir.run_jj(["log", "@", "-r", "@", "-T", "description"]);
-    insta::assert_snapshot!(output, @"");
+    insta::assert_snapshot!(output, @"
+    ------- stderr -------
+    Warning: No matching entries for paths: @
+    [EOF]
+    ");
 }
 
 #[test]
@@ -1230,13 +1323,7 @@ fn test_graph_styles() {
         .run_jj(["new", "-m", "side bookmark\nwith\nlong\ndescription"])
         .success();
     work_dir
-        .run_jj([
-            "new",
-            "-m",
-            "merge",
-            r#"description("main bookmark 1")"#,
-            "@",
-        ])
+        .run_jj(["new", "-m", "merge", "subject(glob:'main bookmark 1')", "@"])
         .success();
 
     // Default (curved) style
@@ -1337,7 +1424,7 @@ fn test_graph_styles() {
     Config error: Invalid type or value for ui.graph.style
     Caused by: unknown variant `unknown`, expected one of `ascii`, `ascii-large`, `curved`, `square`
 
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+    For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
     [EOF]
     [exit status: 1]
     ");
@@ -1506,13 +1593,7 @@ fn test_elided() {
         .success();
     work_dir.run_jj(["new", "-m", "side bookmark 2"]).success();
     work_dir
-        .run_jj([
-            "new",
-            "-m",
-            "merge",
-            r#"description("main bookmark 2")"#,
-            "@",
-        ])
+        .run_jj(["new", "-m", "merge", "subject(glob:'main bookmark 2')", "@"])
         .success();
 
     let get_log = |revs: &str| work_dir.run_jj(["log", "-T", r#"description ++ "\n""#, "-r", revs]);
@@ -1538,7 +1619,7 @@ fn test_elided() {
     // Elide some commits from each side of the merge. It's unclear that a revision
     // was skipped on the left side.
     test_env.add_config("ui.log-synthetic-elided-nodes = false");
-    insta::assert_snapshot!(get_log("@ | @- | description(initial)"), @r"
+    insta::assert_snapshot!(get_log("@ | @- | subject(glob:initial)"), @r"
     @    merge
     ├─╮
     │ ○  side bookmark 2
@@ -1566,7 +1647,7 @@ fn test_elided() {
 
     // Elide some commits from each side of the merge
     test_env.add_config("ui.log-synthetic-elided-nodes = true");
-    insta::assert_snapshot!(get_log("@ | @- | description(initial)"), @r"
+    insta::assert_snapshot!(get_log("@ | @- | subject(glob:initial)"), @r"
     @    merge
     ├─╮
     │ ○  side bookmark 2
@@ -1612,13 +1693,7 @@ fn test_log_with_custom_symbols() {
         .success();
     work_dir.run_jj(["new", "-m", "side bookmark 2"]).success();
     work_dir
-        .run_jj([
-            "new",
-            "-m",
-            "merge",
-            r#"description("main bookmark 2")"#,
-            "@",
-        ])
+        .run_jj(["new", "-m", "merge", "subject(glob:'main bookmark 2')", "@"])
         .success();
 
     let get_log = |revs: &str| work_dir.run_jj(["log", "-T", r#"description ++ "\n""#, "-r", revs]);
@@ -1630,7 +1705,7 @@ fn test_log_with_custom_symbols() {
         templates.log_node = 'if(self, if(current_working_copy, "$", if(root, "┴", "┝")), "🮀")'
         "###,
     );
-    insta::assert_snapshot!(get_log("@ | @- | description(initial) | root()"), @r"
+    insta::assert_snapshot!(get_log("@ | @- | subject(glob:initial) | root()"), @r"
     $    merge
     ├─╮
     │ ┝  side bookmark 2
@@ -1654,7 +1729,7 @@ fn test_log_with_custom_symbols() {
         templates.log_node = 'if(self, if(current_working_copy, "$", if(root, "^", "*")), ":")'
         "###,
     );
-    insta::assert_snapshot!(get_log("@ | @- | description(initial) | root()"), @r"
+    insta::assert_snapshot!(get_log("@ | @- | subject(glob:initial) | root()"), @r"
     $    merge
     |\
     | *  side bookmark 2
@@ -1679,7 +1754,7 @@ fn test_log_full_description_template() {
 
     work_dir
         .run_jj([
-            "describe",
+            "commit",
             "-m",
             "this is commit with a multiline description\n\n<full description>",
         ])
@@ -1687,12 +1762,16 @@ fn test_log_full_description_template() {
 
     let output = work_dir.run_jj(["log", "-T", "builtin_log_compact_full_description"]);
     insta::assert_snapshot!(output, @r"
-    @  qpvuntsm test.user@example.com 2001-02-03 08:05:08 37b69cda
+    @  rlvkpnrz test.user@example.com 2001-02-03 08:05:08 3a70504b
+    │  (empty) (no description set)
+    │
+    ○  qpvuntsm test.user@example.com 2001-02-03 08:05:08 37b69cda
     │  (empty) this is commit with a multiline description
     │
     │  <full description>
     │
     ◆  zzzzzzzz root() 00000000
+
     [EOF]
     ");
 }
@@ -1737,3 +1816,34 @@ fn test_log_anonymize() {
     [EOF]
     ");
 }
+
+#[test]
+fn test_log_count() {
+    let test_env = TestEnvironment::default();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    work_dir.write_file("file1", "foo\n");
+    work_dir.run_jj(["describe", "-m", "first"]).success();
+    work_dir.run_jj(["new", "-m", "second"]).success();
+    work_dir.write_file("file2", "bar\n");
+    work_dir.run_jj(["new", "-m", "third"]).success();
+
+    let output = work_dir.run_jj(["log", "--count"]);
+    insta::assert_snapshot!(output, @r"
+    4
+    [EOF]
+    ");
+
+    let output = work_dir.run_jj(["log", "--count", "-r", "all() ~ root()"]);
+    insta::assert_snapshot!(output, @r"
+    3
+    [EOF]
+    ");
+
+    let output = work_dir.run_jj(["log", "--count", "--limit", "2"]);
+    insta::assert_snapshot!(output, @r"
+    2
+    [EOF]
+    ");
+}
