@@ -180,7 +180,7 @@ fn test_new_merge_conflicts() {
     let output = work_dir.run_jj(["new", "2|3"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy  (@) now at: vruxwmqv 5234fbf2 (conflict) (empty) (no description set)
+    Working copy  (@) now at: vruxwmqv 0361ec6a (conflict) (empty) (no description set)
     Parent commit (@-)      : royxmykx 1b282e07 3 | 3
     Parent commit (@-)      : zsuskuln 7ac709e5 2 | 2
     Added 0 files, modified 1 files, removed 0 files
@@ -251,7 +251,7 @@ fn test_new_merge_same_change() {
     let output = work_dir.run_jj(["new", "2|3", "--config=merge.same-change=keep"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy  (@) now at: znkkpsqq 0d655a01 (conflict) (empty) (no description set)
+    Working copy  (@) now at: znkkpsqq 453a144b (conflict) (empty) (no description set)
     Parent commit (@-)      : royxmykx 1b9fe696 3 | 3
     Parent commit (@-)      : zsuskuln 829e1e90 2 | 2
     Added 1 files, modified 0 files, removed 0 files
@@ -745,8 +745,7 @@ fn test_new_conflicting_bookmarks() {
             "bookmark",
             "create",
             "foo",
-            "-r",
-            r#"description("one")"#,
+            "-rsubject(glob:one)",
         ])
         .success();
 
