@@ -203,11 +203,13 @@ fn test_next_exceeding_history() {
     work_dir.run_jj(["commit", "-m", "first"]).success();
     work_dir.run_jj(["commit", "-m", "second"]).success();
     work_dir.run_jj(["commit", "-m", "third"]).success();
-    work_dir.run_jj(["edit", "-r", "@--"]).success();
+    work_dir.run_jj(["new", "-r", "@--"]).success();
 
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
-    ○  kkmpptxzrspx third
-    @  rlvkpnrzqnoo second
+    @  mzvwutvlkqwt
+    │ ○  kkmpptxzrspx third
+    ├─╯
+    ○  rlvkpnrzqnoo second
     ○  qpvuntsmwlqt first
     ◆  zzzzzzzzzzzz
     [EOF]
@@ -218,7 +220,7 @@ fn test_next_exceeding_history() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: No other descendant found 3 commit(s) forward from the working copy parent(s)
-    Hint: Working copy parent: qpvuntsm 68a50538 (empty) first
+    Hint: Working copy parent: rlvkpnrz 9439bf06 (empty) second
     [EOF]
     [exit status: 1]
     ");
@@ -236,7 +238,7 @@ fn test_next_parent_has_multiple_descendants() {
     work_dir.run_jj(["new", "-m", "2"]).success();
     work_dir.run_jj(["new", "root()", "-m", "3"]).success();
     work_dir.run_jj(["new", "-m", "4"]).success();
-    work_dir.run_jj(["edit", "description(3)"]).success();
+    work_dir.run_jj(["edit", "subject(glob:3)"]).success();
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
     ○  mzvwutvlkqwt 4
     @  zsuskulnrvyr 3
@@ -274,7 +276,7 @@ fn test_next_with_merge_commit_parent() {
     work_dir.run_jj(["desc", "-m", "1"]).success();
     work_dir.run_jj(["new", "root()", "-m", "2"]).success();
     work_dir
-        .run_jj(["new", "description(1)", "description(2)", "-m", "3"])
+        .run_jj(["new", "subject(glob:1)", "subject(glob:2)", "-m", "3"])
         .success();
     work_dir.run_jj(["new", "-m", "4"]).success();
     work_dir.run_jj(["prev", "0"]).success();
@@ -320,10 +322,10 @@ fn test_next_on_merge_commit() {
     work_dir.run_jj(["desc", "-m", "1"]).success();
     work_dir.run_jj(["new", "root()", "-m", "2"]).success();
     work_dir
-        .run_jj(["new", "description(1)", "description(2)", "-m", "3"])
+        .run_jj(["new", "subject(glob:1)", "subject(glob:2)", "-m", "3"])
         .success();
     work_dir.run_jj(["new", "-m", "4"]).success();
-    work_dir.run_jj(["edit", "description(3)"]).success();
+    work_dir.run_jj(["edit", "subject(glob:3)"]).success();
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
     ○  mzvwutvlkqwt 4
     @    zsuskulnrvyr 3
@@ -474,6 +476,7 @@ fn test_prev_on_merge_commit() {
     ◆  zzzzzzzzzzzz
     [EOF]
     ");
+    let setup_opid = work_dir.current_operation_id();
 
     let output = work_dir.run_jj(["prev"]);
     insta::assert_snapshot!(output, @r"
@@ -483,7 +486,7 @@ fn test_prev_on_merge_commit() {
     [EOF]
     ");
 
-    work_dir.run_jj(["undo"]).success();
+    work_dir.run_jj(["op", "restore", &setup_opid]).success();
     let output = work_dir.run_jj_with(|cmd| {
         force_interactive(cmd)
             .args(["prev", "--edit"])
@@ -509,11 +512,11 @@ fn test_prev_on_merge_commit_with_parent_merge() {
     work_dir.run_jj(["desc", "-m", "x"]).success();
     work_dir.run_jj(["new", "root()", "-m", "y"]).success();
     work_dir
-        .run_jj(["new", "description(x)", "description(y)", "-m", "z"])
+        .run_jj(["new", "subject(glob:x)", "subject(glob:y)", "-m", "z"])
         .success();
     work_dir.run_jj(["new", "root()", "-m", "1"]).success();
     work_dir
-        .run_jj(["new", "description(z)", "description(1)", "-m", "M"])
+        .run_jj(["new", "subject(glob:z)", "subject(glob:1)", "-m", "M"])
         .success();
 
     // Check that the graph looks the way we expect.
@@ -530,6 +533,7 @@ fn test_prev_on_merge_commit_with_parent_merge() {
     ◆  zzzzzzzzzzzz
     [EOF]
     ");
+    let setup_opid = work_dir.current_operation_id();
 
     let output = work_dir.run_jj_with(|cmd| force_interactive(cmd).arg("prev").write_stdin("2\n"));
     insta::assert_snapshot!(output, @r"
@@ -544,7 +548,7 @@ fn test_prev_on_merge_commit_with_parent_merge() {
     [EOF]
     ");
 
-    work_dir.run_jj(["undo"]).success();
+    work_dir.run_jj(["op", "restore", &setup_opid]).success();
     let output = work_dir.run_jj_with(|cmd| {
         force_interactive(cmd)
             .args(["prev", "--edit"])
@@ -626,11 +630,11 @@ fn test_prev_prompts_on_multiple_parents() {
     ");
 
     work_dir.run_jj(["next"]).success();
-    work_dir.run_jj(["edit", "@-"]).success();
-
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
-    ○  vruxwmqvtpmx merge+1
-    @      yqosqzytrlsw merge
+    @  wqnwkozpkust
+    │ ○  vruxwmqvtpmx merge+1
+    ├─╯
+    ○      yqosqzytrlsw merge
     ├─┬─╮
     │ │ ○  qpvuntsmwlqt first
     │ ○ │  kkmpptxzrspx second
@@ -640,17 +644,6 @@ fn test_prev_prompts_on_multiple_parents() {
     ◆  zzzzzzzzzzzz
     [EOF]
     ");
-
-    let output = work_dir.run_jj(["next", "--no-edit"]);
-    insta::assert_snapshot!(output,@r"
-    ------- stderr -------
-    Error: No other descendant found 1 commit(s) forward from the working copy parent(s)
-    Hint: Working copy parent: mzvwutvl 5ec63817 (empty) third
-    Hint: Working copy parent: kkmpptxz e8959fbd (empty) second
-    Hint: Working copy parent: qpvuntsm 68a50538 (empty) first
-    [EOF]
-    [exit status: 1]
-    ");
 }
 
 #[test]
@@ -773,9 +766,9 @@ fn test_prev_conflict() {
     work_dir.run_jj(["commit", "-m", "second"]).success();
     work_dir.run_jj(["commit", "-m", "third"]).success();
     // Create a conflict in the first commit, where we'll jump to.
-    work_dir.run_jj(["edit", "description(first)"]).success();
+    work_dir.run_jj(["edit", "subject(glob:first)"]).success();
     work_dir.write_file("content.txt", "first+1");
-    work_dir.run_jj(["new", "description(third)"]).success();
+    work_dir.run_jj(["new", "subject(glob:third)"]).success();
     work_dir.run_jj(["commit", "-m", "fourth"]).success();
     // Test the setup
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
@@ -811,9 +804,9 @@ fn test_prev_conflict_editing() {
     work_dir.write_file("content.txt", "second");
     work_dir.run_jj(["commit", "-m", "third"]).success();
     // Create a conflict in the third commit, where we'll jump to.
-    work_dir.run_jj(["edit", "description(first)"]).success();
+    work_dir.run_jj(["edit", "subject(glob:first)"]).success();
     work_dir.write_file("content.txt", "first text");
-    work_dir.run_jj(["new", "description(third)"]).success();
+    work_dir.run_jj(["new", "subject(glob:third)"]).success();
     // Test the setup
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  royxmykxtrkr conflict
@@ -847,10 +840,10 @@ fn test_next_conflict() {
     // Create a conflict in the third commit.
     work_dir.write_file("content.txt", "third");
     work_dir.run_jj(["commit", "-m", "third"]).success();
-    work_dir.run_jj(["new", "description(first)"]).success();
+    work_dir.run_jj(["new", "subject(glob:first)"]).success();
     work_dir.write_file("content.txt", "first v2");
     work_dir
-        .run_jj(["squash", "--into", "description(third)"])
+        .run_jj(["squash", "--into", "subject(glob:third)"])
         .success();
     // Test the setup
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
@@ -885,7 +878,7 @@ fn test_next_conflict_editing() {
     work_dir.run_jj(["commit", "-m", "second"]).success();
     // Create a conflict in the third commit.
     work_dir.write_file("content.txt", "third");
-    work_dir.run_jj(["edit", "description(second)"]).success();
+    work_dir.run_jj(["edit", "subject(glob:second)"]).success();
     work_dir.write_file("content.txt", "modified second");
     work_dir.run_jj(["edit", "@-"]).success();
     // Test the setup
@@ -936,7 +929,7 @@ fn test_next_conflict_head() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: The working copy has no descendants with conflicts
-    Hint: Working copy: rlvkpnrz 5f088cac (conflict) (no description set)
+    Hint: Working copy: rlvkpnrz 09d24b1f (conflict) (no description set)
     [EOF]
     [exit status: 1]
     ");
@@ -1209,7 +1202,7 @@ fn test_movement_edit_mode_false() {
 }
 
 #[test]
-fn test_next_offset_when_wc_has_descendants() {
+fn test_next_when_wc_has_descendants() {
     let test_env = TestEnvironment::default();
     test_env.add_config(r#"ui.movement.edit = false"#);
 
@@ -1220,70 +1213,26 @@ fn test_next_offset_when_wc_has_descendants() {
     work_dir.run_jj(["commit", "-m", "right-wc"]).success();
     work_dir.run_jj(["commit", "-m", "right-1"]).success();
     work_dir.run_jj(["commit", "-m", "right-2"]).success();
+    work_dir
+        .run_jj(["edit", "subject(glob:right-wc)"])
+        .success();
 
-    work_dir.run_jj(["new", "description(base)"]).success();
-    work_dir.run_jj(["commit", "-m", "left-wc"]).success();
-    work_dir.run_jj(["commit", "-m", "left-1"]).success();
-    work_dir.run_jj(["commit", "-m", "left-2"]).success();
-
-    work_dir.run_jj(["edit", "description(right-wc)"]).success();
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
     ○  zsuskulnrvyr right-2
     ○  kkmpptxzrspx right-1
     @  rlvkpnrzqnoo right-wc
-    │ ○  vruxwmqvtpmx left-2
-    │ ○  yqosqzytrlsw left-1
-    │ ○  royxmykxtrkr left-wc
-    ├─╯
     ○  qpvuntsmwlqt base
     ◆  zzzzzzzzzzzz
     [EOF]
     ");
 
-    work_dir.run_jj(["next", "2"]).success();
-    insta::assert_snapshot!(get_log_output(&work_dir), @r"
-    @  kmkuslswpqwq
-    │ ○  vruxwmqvtpmx left-2
-    ├─╯
-    ○  yqosqzytrlsw left-1
-    ○  royxmykxtrkr left-wc
-    │ ○  zsuskulnrvyr right-2
-    │ ○  kkmpptxzrspx right-1
-    │ ○  rlvkpnrzqnoo right-wc
-    ├─╯
-    ○  qpvuntsmwlqt base
-    ◆  zzzzzzzzzzzz
-    [EOF]
-    ");
-
-    work_dir.run_jj(["edit", "description(left-wc)"]).success();
-    insta::assert_snapshot!(get_log_output(&work_dir), @r"
-    ○  vruxwmqvtpmx left-2
-    ○  yqosqzytrlsw left-1
-    @  royxmykxtrkr left-wc
-    │ ○  zsuskulnrvyr right-2
-    │ ○  kkmpptxzrspx right-1
-    │ ○  rlvkpnrzqnoo right-wc
-    ├─╯
-    ○  qpvuntsmwlqt base
-    ◆  zzzzzzzzzzzz
-    [EOF]
-    ");
-
-    work_dir.run_jj(["next"]).success();
-    insta::assert_snapshot!(get_log_output(&work_dir), @r"
-    @  nkmrtpmomlro
-    │ ○  zsuskulnrvyr right-2
-    │ ○  kkmpptxzrspx right-1
-    ├─╯
-    ○  rlvkpnrzqnoo right-wc
-    │ ○  vruxwmqvtpmx left-2
-    │ ○  yqosqzytrlsw left-1
-    │ ○  royxmykxtrkr left-wc
-    ├─╯
-    ○  qpvuntsmwlqt base
-    ◆  zzzzzzzzzzzz
+    let output = work_dir.run_jj(["next", "2"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Error: The working copy must not have any children
+    Hint: Create a new commit on top of this one or use `--edit`
     [EOF]
+    [exit status: 1]
     ");
 }
 
