@@ -32,9 +32,9 @@ fn test_rebase_invalid() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     error: the following required arguments were not provided:
-      <--destination <REVSETS>|--insert-after <REVSETS>|--insert-before <REVSETS>>
+      <--onto <REVSETS>|--insert-after <REVSETS>|--insert-before <REVSETS>>
 
-    Usage: jj rebase <--destination <REVSETS>|--insert-after <REVSETS>|--insert-before <REVSETS>>
+    Usage: jj rebase <--onto <REVSETS>|--insert-after <REVSETS>|--insert-before <REVSETS>>
 
     For more information, try '--help'.
     [EOF]
@@ -42,12 +42,12 @@ fn test_rebase_invalid() {
     ");
 
     // Both -r and -s
-    let output = work_dir.run_jj(["rebase", "-r", "a", "-s", "a", "-d", "b"]);
+    let output = work_dir.run_jj(["rebase", "-r", "a", "-s", "a", "-o", "b"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     error: the argument '--revisions <REVSETS>' cannot be used with '--source <REVSETS>'
 
-    Usage: jj rebase --revisions <REVSETS> <--destination <REVSETS>|--insert-after <REVSETS>|--insert-before <REVSETS>>
+    Usage: jj rebase --revisions <REVSETS> <--onto <REVSETS>|--insert-after <REVSETS>|--insert-before <REVSETS>>
 
     For more information, try '--help'.
     [EOF]
@@ -55,38 +55,38 @@ fn test_rebase_invalid() {
     ");
 
     // Both -b and -s
-    let output = work_dir.run_jj(["rebase", "-b", "a", "-s", "a", "-d", "b"]);
+    let output = work_dir.run_jj(["rebase", "-b", "a", "-s", "a", "-o", "b"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     error: the argument '--branch <REVSETS>' cannot be used with '--source <REVSETS>'
 
-    Usage: jj rebase --branch <REVSETS> <--destination <REVSETS>|--insert-after <REVSETS>|--insert-before <REVSETS>>
+    Usage: jj rebase --branch <REVSETS> <--onto <REVSETS>|--insert-after <REVSETS>|--insert-before <REVSETS>>
 
     For more information, try '--help'.
     [EOF]
     [exit status: 2]
     ");
 
-    // Both -d and --after
-    let output = work_dir.run_jj(["rebase", "-r", "a", "-d", "b", "--after", "b"]);
+    // Both -o and --after
+    let output = work_dir.run_jj(["rebase", "-r", "a", "-o", "b", "--after", "b"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    error: the argument '--destination <REVSETS>' cannot be used with '--insert-after <REVSETS>'
+    error: the argument '--onto <REVSETS>' cannot be used with '--insert-after <REVSETS>'
 
-    Usage: jj rebase --revisions <REVSETS> <--destination <REVSETS>|--insert-after <REVSETS>|--insert-before <REVSETS>>
+    Usage: jj rebase --revisions <REVSETS> <--onto <REVSETS>|--insert-after <REVSETS>|--insert-before <REVSETS>>
 
     For more information, try '--help'.
     [EOF]
     [exit status: 2]
     ");
 
-    // Both -d and --before
-    let output = work_dir.run_jj(["rebase", "-r", "a", "-d", "b", "--before", "b"]);
+    // Both -o and --before
+    let output = work_dir.run_jj(["rebase", "-r", "a", "-o", "b", "--before", "b"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    error: the argument '--destination <REVSETS>' cannot be used with '--insert-before <REVSETS>'
+    error: the argument '--onto <REVSETS>' cannot be used with '--insert-before <REVSETS>'
 
-    Usage: jj rebase --revisions <REVSETS> <--destination <REVSETS>|--insert-after <REVSETS>|--insert-before <REVSETS>>
+    Usage: jj rebase --revisions <REVSETS> <--onto <REVSETS>|--insert-after <REVSETS>|--insert-before <REVSETS>>
 
     For more information, try '--help'.
     [EOF]
@@ -94,7 +94,7 @@ fn test_rebase_invalid() {
     ");
 
     // Rebase onto self with -r
-    let output = work_dir.run_jj(["rebase", "-r", "a", "-d", "a"]);
+    let output = work_dir.run_jj(["rebase", "-r", "a", "-o", "a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Cannot rebase 7d980be7a1d4 onto itself
@@ -103,7 +103,7 @@ fn test_rebase_invalid() {
     ");
 
     // Rebase root with -r
-    let output = work_dir.run_jj(["rebase", "-r", "root()", "-d", "a"]);
+    let output = work_dir.run_jj(["rebase", "-r", "root()", "-o", "a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: The root commit 000000000000 is immutable
@@ -112,7 +112,7 @@ fn test_rebase_invalid() {
     ");
 
     // Rebase onto descendant with -s
-    let output = work_dir.run_jj(["rebase", "-s", "a", "-d", "b"]);
+    let output = work_dir.run_jj(["rebase", "-s", "a", "-o", "b"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Cannot rebase 7d980be7a1d4 onto descendant 123b4d91f6e5
@@ -121,7 +121,7 @@ fn test_rebase_invalid() {
     ");
 
     // Rebase onto itself with -s
-    let output = work_dir.run_jj(["rebase", "-s", "a", "-d", "a"]);
+    let output = work_dir.run_jj(["rebase", "-s", "a", "-o", "a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Cannot rebase 7d980be7a1d4 onto itself
@@ -194,7 +194,7 @@ fn test_rebase_bookmark() {
     ");
     let setup_opid = work_dir.current_operation_id();
 
-    let output = work_dir.run_jj(["rebase", "-b", "c", "-d", "e"]);
+    let output = work_dir.run_jj(["rebase", "-b", "c", "-o", "e"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 3 commits to destination
@@ -286,7 +286,7 @@ fn test_rebase_bookmark_with_merge() {
     ");
     let setup_opid = work_dir.current_operation_id();
 
-    let output = work_dir.run_jj(["rebase", "-b", "d", "-d", "b"]);
+    let output = work_dir.run_jj(["rebase", "-b", "d", "-o", "b"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 3 commits to destination
@@ -309,7 +309,7 @@ fn test_rebase_bookmark_with_merge() {
     ");
 
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
-    let output = work_dir.run_jj(["rebase", "-d", "b"]);
+    let output = work_dir.run_jj(["rebase", "-o", "b"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 3 commits to destination
@@ -359,7 +359,7 @@ fn test_rebase_single_revision() {
 
     // Descendants of the rebased commit "c" should be rebased onto parents. First
     // we test with a non-merge commit.
-    let output = work_dir.run_jj(["rebase", "-r", "c", "-d", "b"]);
+    let output = work_dir.run_jj(["rebase", "-r", "c", "-o", "b"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -385,7 +385,7 @@ fn test_rebase_single_revision() {
 
     // Now, let's try moving the merge commit. After, both parents of "d" ("b" and
     // "c") should become parents of "e".
-    let output = work_dir.run_jj(["rebase", "-r", "d", "-d", "a"]);
+    let output = work_dir.run_jj(["rebase", "-r", "d", "-o", "a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -434,7 +434,7 @@ fn test_rebase_single_revision_merge_parent() {
 
     // Descendants of the rebased commit should be rebased onto parents, and if
     // the descendant is a merge commit, it shouldn't forget its other parents.
-    let output = work_dir.run_jj(["rebase", "-r", "c", "-d", "a"]);
+    let output = work_dir.run_jj(["rebase", "-r", "c", "-o", "a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -493,7 +493,7 @@ fn test_rebase_multiple_revisions() {
     let setup_opid = work_dir.current_operation_id();
 
     // Test with two non-related non-merge commits.
-    let output = work_dir.run_jj(["rebase", "-r", "c", "-r", "e", "-d", "a"]);
+    let output = work_dir.run_jj(["rebase", "-r", "c", "-r", "e", "-o", "a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 2 commits to destination
@@ -526,7 +526,7 @@ fn test_rebase_multiple_revisions() {
     // Test with two related non-merge commits. Since "b" is a parent of "c", when
     // rebasing commits "b" and "c", their ancestry relationship should be
     // preserved.
-    let output = work_dir.run_jj(["rebase", "-r", "b", "-r", "c", "-d", "e"]);
+    let output = work_dir.run_jj(["rebase", "-r", "b", "-r", "c", "-o", "e"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 2 commits to destination
@@ -560,7 +560,7 @@ fn test_rebase_multiple_revisions() {
     // inherit its descendants which are not in the subtree ("c" and "d").
     // "f" will retain its parent "c" since "c" is outside the target set, and not
     // a descendant of any new children.
-    let output = work_dir.run_jj(["rebase", "-r", "e::g", "-d", "a"]);
+    let output = work_dir.run_jj(["rebase", "-r", "e::g", "-o", "a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 3 commits to destination
@@ -596,7 +596,7 @@ fn test_rebase_multiple_revisions() {
     // rebased commits. "d" should be a new parent of "f", and "f" should be a
     // new parent of "h". "f" will retain its parent "c" since "c" is outside the
     // target set, and not a descendant of any new children.
-    let output = work_dir.run_jj(["rebase", "-r", "d", "-r", "f", "-r", "h", "-d", "b"]);
+    let output = work_dir.run_jj(["rebase", "-r", "d", "-r", "f", "-r", "h", "-o", "b"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 3 commits to destination
@@ -628,7 +628,7 @@ fn test_rebase_multiple_revisions() {
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
 
     // Test rebasing a subgraph onto its descendants.
-    let output = work_dir.run_jj(["rebase", "-r", "d::e", "-d", "i"]);
+    let output = work_dir.run_jj(["rebase", "-r", "d::e", "-o", "i"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 2 commits to destination
@@ -680,7 +680,7 @@ fn test_rebase_revision_onto_descendant() {
     let setup_opid = work_dir.current_operation_id();
 
     // Simpler example
-    let output = work_dir.run_jj(["rebase", "-r", "base", "-d", "a"]);
+    let output = work_dir.run_jj(["rebase", "-r", "base", "-o", "a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -714,7 +714,7 @@ fn test_rebase_revision_onto_descendant() {
     Added 1 files, modified 0 files, removed 0 files
     [EOF]
     ");
-    let output = work_dir.run_jj(["rebase", "-r", "base", "-d", "merge"]);
+    let output = work_dir.run_jj(["rebase", "-r", "base", "-o", "merge"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -760,7 +760,7 @@ fn test_rebase_multiple_destinations() {
     [EOF]
     ");
 
-    let output = work_dir.run_jj(["rebase", "-r", "a", "-d", "b", "-d", "c"]);
+    let output = work_dir.run_jj(["rebase", "-r", "a", "-o", "b", "-o", "c"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -782,7 +782,7 @@ fn test_rebase_multiple_destinations() {
         "--config=ui.always-allow-large-revsets=false",
         "-r",
         "a",
-        "-d",
+        "-o",
         "b|c",
     ]);
     insta::assert_snapshot!(output, @r"
@@ -801,7 +801,7 @@ fn test_rebase_multiple_destinations() {
         "--config=ui.always-allow-large-revsets=false",
         "-r",
         "a",
-        "-d",
+        "-o",
         "all:b|c",
     ]);
     insta::assert_snapshot!(output, @r"
@@ -839,21 +839,21 @@ fn test_rebase_multiple_destinations() {
     [EOF]
     ");
 
-    let output = work_dir.run_jj(["rebase", "-r", "a", "-d", "b", "-d", "b"]);
+    let output = work_dir.run_jj(["rebase", "-r", "a", "-o", "b", "-o", "b"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
     [EOF]
     ");
 
-    let output = work_dir.run_jj(["rebase", "-r", "a", "-d", "b|c", "-d", "b"]);
+    let output = work_dir.run_jj(["rebase", "-r", "a", "-o", "b|c", "-o", "b"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
     [EOF]
     ");
 
-    let output = work_dir.run_jj(["rebase", "-r", "a", "-d", "b", "-d", "root()"]);
+    let output = work_dir.run_jj(["rebase", "-r", "a", "-o", "b", "-o", "root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: The Git backend does not support creating merge commits with the root commit as one of the parents.
@@ -885,7 +885,7 @@ fn test_rebase_with_descendants() {
     ");
     let setup_opid = work_dir.current_operation_id();
 
-    let output = work_dir.run_jj(["rebase", "-s", "b", "-d", "a"]);
+    let output = work_dir.run_jj(["rebase", "-s", "b", "-o", "a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 3 commits to destination
@@ -996,7 +996,7 @@ fn test_rebase_error_revision_does_not_exist() {
         .success();
     work_dir.run_jj(["new", "-r", "@-", "-m", "two"]).success();
 
-    let output = work_dir.run_jj(["rebase", "-b", "b-one", "-d", "this"]);
+    let output = work_dir.run_jj(["rebase", "-b", "b-one", "-o", "this"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Revision `this` doesn't exist
@@ -1004,7 +1004,7 @@ fn test_rebase_error_revision_does_not_exist() {
     [exit status: 1]
     ");
 
-    let output = work_dir.run_jj(["rebase", "-b", "this", "-d", "b-one"]);
+    let output = work_dir.run_jj(["rebase", "-b", "this", "-o", "b-one"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Revision `this` doesn't exist
@@ -1042,7 +1042,7 @@ fn test_rebase_with_child_and_descendant_bug_2600() {
 
     // ===================== rebase -s tests =================
     // This should be a no-op
-    let output = work_dir.run_jj(["rebase", "-s", "base", "-d", "notroot"]);
+    let output = work_dir.run_jj(["rebase", "-s", "base", "-o", "notroot"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Skipped rebase of 4 commits that were already in place
@@ -1063,7 +1063,7 @@ fn test_rebase_with_child_and_descendant_bug_2600() {
 
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
     // This should be a no-op
-    let output = work_dir.run_jj(["rebase", "-s", "a", "-d", "base"]);
+    let output = work_dir.run_jj(["rebase", "-s", "a", "-o", "base"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Skipped rebase of 3 commits that were already in place
@@ -1083,7 +1083,7 @@ fn test_rebase_with_child_and_descendant_bug_2600() {
     ");
 
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
-    let output = work_dir.run_jj(["rebase", "-s", "a", "-d", "root()"]);
+    let output = work_dir.run_jj(["rebase", "-s", "a", "-o", "root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 3 commits to destination
@@ -1122,7 +1122,7 @@ fn test_rebase_with_child_and_descendant_bug_2600() {
 
     // The commits in roots(base..c), i.e. commit "a" should be rebased onto "base",
     // which is a no-op
-    let output = work_dir.run_jj(["rebase", "-b", "c", "-d", "base"]);
+    let output = work_dir.run_jj(["rebase", "-b", "c", "-o", "base"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Skipped rebase of 3 commits that were already in place
@@ -1142,7 +1142,7 @@ fn test_rebase_with_child_and_descendant_bug_2600() {
     ");
 
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
-    let output = work_dir.run_jj(["rebase", "-b", "c", "-d", "a"]);
+    let output = work_dir.run_jj(["rebase", "-b", "c", "-o", "a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 2 commits to destination
@@ -1164,7 +1164,7 @@ fn test_rebase_with_child_and_descendant_bug_2600() {
 
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
     // This should be a no-op
-    let output = work_dir.run_jj(["rebase", "-b", "a", "-d", "root()"]);
+    let output = work_dir.run_jj(["rebase", "-b", "a", "-o", "root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Skipped rebase of 5 commits that were already in place
@@ -1198,7 +1198,7 @@ fn test_rebase_with_child_and_descendant_bug_2600() {
     [EOF]
     ");
 
-    let output = work_dir.run_jj(["rebase", "-r", "base", "-d", "root()"]);
+    let output = work_dir.run_jj(["rebase", "-r", "base", "-o", "root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -1225,7 +1225,7 @@ fn test_rebase_with_child_and_descendant_bug_2600() {
     // This tests the algorithm for rebasing onto descendants. The result should
     // have unsimplified ancestry.
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
-    let output = work_dir.run_jj(["rebase", "-r", "base", "-d", "b"]);
+    let output = work_dir.run_jj(["rebase", "-r", "base", "-o", "b"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -1251,7 +1251,7 @@ fn test_rebase_with_child_and_descendant_bug_2600() {
     // This tests the algorithm for rebasing onto descendants. The result should
     // have unsimplified ancestry.
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
-    let output = work_dir.run_jj(["rebase", "-r", "base", "-d", "a"]);
+    let output = work_dir.run_jj(["rebase", "-r", "base", "-o", "a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -1288,7 +1288,7 @@ fn test_rebase_with_child_and_descendant_bug_2600() {
     [EOF]
     ");
 
-    let output = work_dir.run_jj(["rebase", "-r", "a", "-d", "root()"]);
+    let output = work_dir.run_jj(["rebase", "-r", "a", "-o", "root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -1312,7 +1312,7 @@ fn test_rebase_with_child_and_descendant_bug_2600() {
     ");
 
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
-    let output = work_dir.run_jj(["rebase", "-r", "b", "-d", "root()"]);
+    let output = work_dir.run_jj(["rebase", "-r", "b", "-o", "root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -1340,7 +1340,7 @@ fn test_rebase_with_child_and_descendant_bug_2600() {
     // This tests the algorithm for rebasing onto descendants. The result should
     // have unsimplified ancestry.
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
-    let output = work_dir.run_jj(["rebase", "-r", "b", "-d", "c"]);
+    let output = work_dir.run_jj(["rebase", "-r", "b", "-o", "c"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -1366,7 +1366,7 @@ fn test_rebase_with_child_and_descendant_bug_2600() {
     // In this test, the commit with weird ancestry is not rebased (neither directly
     // nor indirectly).
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
-    let output = work_dir.run_jj(["rebase", "-r", "c", "-d", "a"]);
+    let output = work_dir.run_jj(["rebase", "-r", "c", "-o", "a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -2764,7 +2764,7 @@ fn test_rebase_skip_emptied() {
 
     let output = work_dir.run_jj([
         "rebase",
-        "-r=description('will become empty')",
+        "-r=subject(glob:'will become empty')",
         "-d=b",
         "--skip-emptied",
     ]);
@@ -2876,7 +2876,7 @@ fn test_rebase_skip_if_on_destination() {
     ");
 
     // Skip rebase with -b
-    let output = work_dir.run_jj(["rebase", "-b", "d", "-d", "a"]);
+    let output = work_dir.run_jj(["rebase", "-b", "d", "-o", "a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Skipped rebase of 6 commits that were already in place
@@ -2899,7 +2899,7 @@ fn test_rebase_skip_if_on_destination() {
     ");
 
     // Skip rebase with -s
-    let output = work_dir.run_jj(["rebase", "-s", "c", "-d", "b1", "-d", "b2"]);
+    let output = work_dir.run_jj(["rebase", "-s", "c", "-o", "b1", "-o", "b2"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Skipped rebase of 4 commits that were already in place
@@ -2922,7 +2922,7 @@ fn test_rebase_skip_if_on_destination() {
     ");
 
     // Skip rebase with -r since commit has no children
-    let output = work_dir.run_jj(["rebase", "-r", "d", "-d", "c"]);
+    let output = work_dir.run_jj(["rebase", "-r", "d", "-o", "c"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Skipped rebase of 1 commits that were already in place
@@ -2945,7 +2945,7 @@ fn test_rebase_skip_if_on_destination() {
     ");
 
     // Skip rebase of commit, but rebases children onto destination with -r
-    let output = work_dir.run_jj(["rebase", "-r", "e", "-d", "c"]);
+    let output = work_dir.run_jj(["rebase", "-r", "e", "-o", "c"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Skipped rebase of 1 commits that were already in place
@@ -2982,7 +2982,7 @@ fn test_rebase_skip_duplicate_divergent() {
     create_commit_with_files(&work_dir, "a", &[], &[("file1", "initial\n")]);
     create_commit_with_files(&work_dir, "b2", &["a"], &[("file1", "initial\nb\n")]);
     create_commit_with_files(&work_dir, "c", &["a"], &[("file2", "c\n")]);
-    work_dir.run_jj(["rebase", "-r", "b2", "-d", "c"]).success();
+    work_dir.run_jj(["rebase", "-r", "b2", "-o", "c"]).success();
     work_dir
         .run_jj(["bookmark", "create", "b1", "-r", "at_operation(@-, b2)"])
         .success();
@@ -3002,7 +3002,7 @@ fn test_rebase_skip_duplicate_divergent() {
     let setup_opid = work_dir.current_operation_id();
 
     // By default, rebase should skip the duplicate of commit B
-    insta::assert_snapshot!(work_dir.run_jj(["rebase", "-r", "c::", "-d", "d"]), @r"
+    insta::assert_snapshot!(work_dir.run_jj(["rebase", "-r", "c::", "-o", "d"]), @r"
     ------- stderr -------
     Abandoned 1 divergent commits that were already present in the destination:
       zsuskuln?? 3f194323 b2 | b2
@@ -3020,7 +3020,7 @@ fn test_rebase_skip_duplicate_divergent() {
 
     // Rebasing should work even if the root of the target set is abandoned
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
-    insta::assert_snapshot!(work_dir.run_jj(["rebase", "-s", "b1", "-d", "b2"]), @r"
+    insta::assert_snapshot!(work_dir.run_jj(["rebase", "-s", "b1", "-o", "b2"]), @r"
     ------- stderr -------
     Abandoned 1 divergent commits that were already present in the destination:
       zsuskuln?? 48bf33ab b1 | b2
@@ -3042,7 +3042,7 @@ fn test_rebase_skip_duplicate_divergent() {
 
     // Rebase with "--keep-divergent" shouldn't skip any duplicates
     work_dir.run_jj(["op", "restore", &setup_opid]).success();
-    insta::assert_snapshot!(work_dir.run_jj(["rebase", "-s", "c", "-d", "d", "--keep-divergent"]), @r"
+    insta::assert_snapshot!(work_dir.run_jj(["rebase", "-s", "c", "-o", "d", "--keep-divergent"]), @r"
     ------- stderr -------
     Rebased 2 commits to destination
     [EOF]
