@@ -27,18 +27,18 @@ fn test_report_conflicts() {
     work_dir.write_file("file", "C\n");
     work_dir.run_jj(["commit", "-m=C"]).success();
 
-    let output = work_dir.run_jj(["rebase", "-s=description(B)", "-d=root()"]);
+    let output = work_dir.run_jj(["rebase", "-s=subject(glob:B)", "-d=root()"]);
     insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Rebased 3 commits to destination
-    Working copy  (@) now at: zsuskuln dd37d4a5 (conflict) (empty) (no description set)
-    Parent commit (@-)      : kkmpptxz c7f5d6e5 (conflict) C
+    Working copy  (@) now at: zsuskuln 1f0443b9 (conflict) (empty) (no description set)
+    Parent commit (@-)      : kkmpptxz 94037e0e (conflict) C
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict including 1 deletion
     New conflicts appeared in 2 commits:
-      kkmpptxz c7f5d6e5 (conflict) C
-      rlvkpnrz 032a8668 (conflict) B
+      kkmpptxz 94037e0e (conflict) C
+      rlvkpnrz 871ac2e2 (conflict) B
     Hint: To resolve the conflicts, start by creating a commit on top of
     the first conflicted commit:
       jj new rlvkpnrz
@@ -48,7 +48,7 @@ fn test_report_conflicts() {
     [EOF]
     "###);
 
-    let output = work_dir.run_jj(["rebase", "-d=description(A)"]);
+    let output = work_dir.run_jj(["rebase", "-d=subject(glob:A)"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 3 commits to destination
@@ -60,19 +60,19 @@ fn test_report_conflicts() {
     ");
 
     // Can get hint about multiple root commits
-    let output = work_dir.run_jj(["rebase", "-r=description(B)", "-d=root()"]);
+    let output = work_dir.run_jj(["rebase", "-r=subject(glob:B)", "-d=root()"]);
     insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Rebased 1 commits to destination
     Rebased 2 descendant commits
-    Working copy  (@) now at: zsuskuln 5b511768 (conflict) (empty) (no description set)
-    Parent commit (@-)      : kkmpptxz 0252a7f0 (conflict) C
+    Working copy  (@) now at: zsuskuln f525f3b5 (conflict) (empty) (no description set)
+    Parent commit (@-)      : kkmpptxz 2aa6a481 (conflict) C
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict
     New conflicts appeared in 2 commits:
-      kkmpptxz 0252a7f0 (conflict) C
-      rlvkpnrz fcfd7304 (conflict) B
+      kkmpptxz 2aa6a481 (conflict) C
+      rlvkpnrz 50a742b3 (conflict) B
     Hint: To resolve the conflicts, start by creating a commit on top of
     one of the first conflicted commits:
       jj new kkmpptxz
@@ -87,8 +87,8 @@ fn test_report_conflicts() {
     let output = work_dir.run_jj(["new", "rlvkpnrzqnoo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy  (@) now at: vruxwmqv 55514f4e (conflict) (empty) (no description set)
-    Parent commit (@-)      : rlvkpnrz fcfd7304 (conflict) B
+    Working copy  (@) now at: vruxwmqv 36e37773 (conflict) (empty) (no description set)
+    Parent commit (@-)      : rlvkpnrz 50a742b3 (conflict) B
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict including 1 deletion
@@ -122,20 +122,20 @@ fn test_report_conflicts_with_divergent_commits() {
         .run_jj(["describe", "-m=C3", "--at-op=@-"])
         .success();
 
-    let output = work_dir.run_jj(["rebase", "-s=description(B)", "-d=root()"]);
+    let output = work_dir.run_jj(["rebase", "-s=subject(glob:B)", "-d=root()"]);
     insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Concurrent modification detected, resolving automatically.
     Rebased 3 commits to destination
-    Working copy  (@) now at: zsuskuln?? 08a31f4f (conflict) C2
-    Parent commit (@-)      : kkmpptxz 099d6624 (conflict) B
+    Working copy  (@) now at: zsuskuln?? e91a430b (conflict) C2
+    Parent commit (@-)      : kkmpptxz fcd54aca (conflict) B
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict including 1 deletion
     New conflicts appeared in 3 commits:
-      zsuskuln?? df34134a (conflict) C3
-      zsuskuln?? 08a31f4f (conflict) C2
-      kkmpptxz 099d6624 (conflict) B
+      zsuskuln?? 33d16252 (conflict) C3
+      zsuskuln?? e91a430b (conflict) C2
+      kkmpptxz fcd54aca (conflict) B
     Hint: To resolve the conflicts, start by creating a commit on top of
     the first conflicted commit:
       jj new kkmpptxz
@@ -145,7 +145,7 @@ fn test_report_conflicts_with_divergent_commits() {
     [EOF]
     "###);
 
-    let output = work_dir.run_jj(["rebase", "-d=description(A)"]);
+    let output = work_dir.run_jj(["rebase", "-d=subject(glob:A)"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 3 commits to destination
@@ -157,17 +157,17 @@ fn test_report_conflicts_with_divergent_commits() {
     ");
 
     // Same thing when rebasing the divergent commits one at a time
-    let output = work_dir.run_jj(["rebase", "-s=description(C2)", "-d=root()"]);
+    let output = work_dir.run_jj(["rebase", "-s=subject(glob:C2)", "-d=root()"]);
     insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Rebased 1 commits to destination
-    Working copy  (@) now at: zsuskuln?? dfe73891 (conflict) C2
+    Working copy  (@) now at: zsuskuln?? 151c23fc (conflict) C2
     Parent commit (@-)      : zzzzzzzz 00000000 (empty) (no description set)
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict including 1 deletion
     New conflicts appeared in 1 commits:
-      zsuskuln?? dfe73891 (conflict) C2
+      zsuskuln?? 151c23fc (conflict) C2
     Hint: To resolve the conflicts, start by creating a commit on top of
     the conflicted commit:
       jj new zsuskuln
@@ -177,12 +177,12 @@ fn test_report_conflicts_with_divergent_commits() {
     [EOF]
     "###);
 
-    let output = work_dir.run_jj(["rebase", "-s=description(C3)", "-d=root()"]);
+    let output = work_dir.run_jj(["rebase", "-s=subject(glob:C3)", "-d=root()"]);
     insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Rebased 1 commits to destination
     New conflicts appeared in 1 commits:
-      zsuskuln?? 02834578 (conflict) C3
+      zsuskuln?? d59fa233 (conflict) C3
     Hint: To resolve the conflicts, start by creating a commit on top of
     the conflicted commit:
       jj new zsuskuln
@@ -192,7 +192,7 @@ fn test_report_conflicts_with_divergent_commits() {
     [EOF]
     "###);
 
-    let output = work_dir.run_jj(["rebase", "-s=description(C2)", "-d=description(B)"]);
+    let output = work_dir.run_jj(["rebase", "-s=subject(glob:C2)", "-d=subject(glob:B)"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -203,7 +203,7 @@ fn test_report_conflicts_with_divergent_commits() {
     [EOF]
     ");
 
-    let output = work_dir.run_jj(["rebase", "-s=description(C3)", "-d=description(B)"]);
+    let output = work_dir.run_jj(["rebase", "-s=subject(glob:C3)", "-d=subject(glob:B)"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 commits to destination
@@ -227,21 +227,21 @@ fn test_report_conflicts_with_resolving_conflicts_hint_disabled() {
 
     let output = work_dir.run_jj([
         "rebase",
-        "-s=description(B)",
+        "-s=subject(glob:B)",
         "-d=root()",
         "--config=hints.resolving-conflicts=false",
     ]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 3 commits to destination
-    Working copy  (@) now at: zsuskuln dd37d4a5 (conflict) (empty) (no description set)
-    Parent commit (@-)      : kkmpptxz c7f5d6e5 (conflict) C
+    Working copy  (@) now at: zsuskuln 1f0443b9 (conflict) (empty) (no description set)
+    Parent commit (@-)      : kkmpptxz 94037e0e (conflict) C
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict including 1 deletion
     New conflicts appeared in 2 commits:
-      kkmpptxz c7f5d6e5 (conflict) C
-      rlvkpnrz 032a8668 (conflict) B
+      kkmpptxz 94037e0e (conflict) C
+      rlvkpnrz 871ac2e2 (conflict) B
     [EOF]
     ");
 }
