@@ -45,7 +45,7 @@ fn test_syntax_error() {
       |    ^---
       |
       = expected `::`, `..`, `~`, or <primary>
-    Hint: See https://jj-vcs.github.io/jj/latest/revsets/ or use `jj help -k revsets` for revsets syntax and how to quote symbols.
+    Hint: See https://docs.jj-vcs.dev/latest/revsets/ or use `jj help -k revsets` for revsets syntax and how to quote symbols.
     [EOF]
     [exit status: 1]
     ");
@@ -217,7 +217,7 @@ fn test_bad_function_call() {
       |     ^---
       |
       = expected <identifier>, <string_literal>, or <raw_string_literal>
-    Hint: See https://jj-vcs.github.io/jj/latest/filesets/ or use `jj help -k filesets` for filesets syntax and how to match file paths.
+    Hint: See https://docs.jj-vcs.dev/latest/filesets/ or use `jj help -k filesets` for filesets syntax and how to match file paths.
     [EOF]
     [exit status: 1]
     ");
@@ -240,7 +240,7 @@ fn test_bad_function_call() {
       |
       = Invalid file pattern
     3: Invalid file pattern kind `foo:`
-    Hint: See https://jj-vcs.github.io/jj/latest/filesets/#file-patterns or `jj help -k filesets` for valid prefixes.
+    Hint: See https://docs.jj-vcs.dev/latest/filesets/#file-patterns or `jj help -k filesets` for valid prefixes.
     [EOF]
     [exit status: 1]
     "#);
@@ -356,7 +356,7 @@ fn test_bad_function_call() {
       |                  ^---
       |
       = expected <strict_identifier> or <expression>
-    Hint: See https://jj-vcs.github.io/jj/latest/revsets/ or use `jj help -k revsets` for revsets syntax and how to quote symbols.
+    Hint: See https://docs.jj-vcs.dev/latest/revsets/ or use `jj help -k revsets` for revsets syntax and how to quote symbols.
     [EOF]
     [exit status: 1]
     ");
@@ -371,7 +371,7 @@ fn test_bad_function_call() {
       |                         ^---
       |
       = expected <expression>
-    Hint: See https://jj-vcs.github.io/jj/latest/revsets/ or use `jj help -k revsets` for revsets syntax and how to quote symbols.
+    Hint: See https://docs.jj-vcs.dev/latest/revsets/ or use `jj help -k revsets` for revsets syntax and how to quote symbols.
     [EOF]
     [exit status: 1]
     ");
@@ -511,6 +511,44 @@ fn test_bad_symbol_or_argument_should_not_be_optimized_out() {
     "#);
 }
 
+#[test]
+fn test_default_string_pattern() {
+    let test_env = TestEnvironment::default();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    // substring match by default as of jj 0.35
+    let output = work_dir.run_jj(["log", "-rauthor('test.user')"]);
+    insta::assert_snapshot!(output.normalize_backslash(), @r###"
+    @  qpvuntsm test.user@example.com 2001-02-03 08:05:07 e8849ae1
+    │  (empty) (no description set)
+    ~
+    [EOF]
+    ------- stderr -------
+    Warning: In revset expression
+     --> 1:8
+      |
+    1 | author('test.user')
+      |        ^---------^
+      |
+      = Default pattern syntax will be changed to `glob:` / `exact:` (depending on whether it looks like a glob) in a future release; use `substring:` prefix or set ui.revsets-use-glob-by-default=true to suppress this warning
+    [EOF]
+    "###);
+
+    // with default flipped
+    let output = work_dir.run_jj([
+        "log",
+        "-rauthor('*test.user*')",
+        "--config=ui.revsets-use-glob-by-default=true",
+    ]);
+    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    @  qpvuntsm test.user@example.com 2001-02-03 08:05:07 e8849ae1
+    │  (empty) (no description set)
+    ~
+    [EOF]
+    ");
+}
+
 #[test]
 fn test_alias() {
     let test_env = TestEnvironment::default();
@@ -559,7 +597,7 @@ fn test_alias() {
       |           ^---
       |
       = expected `::`, `..`, `~`, or <primary>
-    Hint: See https://jj-vcs.github.io/jj/latest/revsets/ or use `jj help -k revsets` for revsets syntax and how to quote symbols.
+    Hint: See https://docs.jj-vcs.dev/latest/revsets/ or use `jj help -k revsets` for revsets syntax and how to quote symbols.
     [EOF]
     [exit status: 1]
     ");
@@ -873,8 +911,7 @@ fn test_all_modifier() {
     [exit status: 1]
     ");
 
-    // immutable_heads() alias may be parsed as a top-level expression, but
-    // still, modifier shouldn't be allowed there.
+    // Modifier shouldn't be allowed in a top-level immutable_heads() expression
     let output = work_dir.run_jj([
         "new",
         "--config=revset-aliases.'immutable_heads()'='all:@'",
@@ -883,13 +920,13 @@ fn test_all_modifier() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Config error: Invalid `revset-aliases.immutable_heads()`
-    Caused by:  --> 1:1
+    Caused by:  --> 1:4
       |
     1 | all:@
-      | ^-^
+      |    ^
       |
-      = Modifier `all:` is not allowed in sub expression
-    For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k config`.
+      = `:` is not an infix operator
+    For help, see https://docs.jj-vcs.dev/latest/config/ or use `jj help -k config`.
     [EOF]
     [exit status: 1]
     ");
