@@ -158,6 +158,7 @@ fn test_split_by_paths() {
     let output = work_dir.run_jj(["split", "-r", "@-", "nonexistent"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
+    Warning: No matching entries for paths: nonexistent
     Warning: No changes have been selected, so the new revision will be empty
     Rebased 1 descendant commits
     Selected changes : qpvuntsm 49416632 (empty) (no description set)
@@ -180,6 +181,20 @@ fn test_split_by_paths() {
     A file2
     [EOF]
     ");
+
+    work_dir.run_jj(["new"]).success();
+    // Splitting a commit with deleted files should not show a warning.
+    work_dir.remove_file("file1");
+    let output = work_dir.run_jj(["split", "file1"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Warning: All changes have been selected, so the original revision will become empty
+    Selected changes : uyznsvlq 971ccc0b (no description set)
+    Remaining changes: xznxytkn a267cd96 (empty) (no description set)
+    Working copy  (@) now at: smwtzssm 6715dc2c (empty) (no description set)
+    Parent commit (@-)      : uyznsvlq 971ccc0b (no description set)
+    [EOF]
+    ");
 }
 
 #[test]
@@ -445,7 +460,7 @@ fn test_split_with_merge_child() {
     work_dir.write_file("file1", "foo\n");
     work_dir.write_file("file2", "bar\n");
     work_dir
-        .run_jj(["new", "description(1)", "description(a)", "-m=2"])
+        .run_jj(["new", "subject(glob:1)", "subject(glob:a)", "-m=2"])
         .success();
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @    zsuskulnrvyr true 2
@@ -463,7 +478,7 @@ fn test_split_with_merge_child() {
         ["write\nAdd file1", "next invocation\n", "write\nAdd file2"].join("\0"),
     )
     .unwrap();
-    let output = work_dir.run_jj(["split", "-r", "description(a)", "file1"]);
+    let output = work_dir.run_jj(["split", "-rsubject(glob:a)", "file1"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 descendant commits
@@ -701,7 +716,7 @@ fn test_split_parallel_with_merge_child() {
     work_dir.write_file("file1", "foo\n");
     work_dir.write_file("file2", "bar\n");
     work_dir
-        .run_jj(["new", "description(1)", "description(a)", "-m=2"])
+        .run_jj(["new", "subject(glob:1)", "subject(glob:a)", "-m=2"])
         .success();
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @    zsuskulnrvyr true 2
@@ -719,7 +734,7 @@ fn test_split_parallel_with_merge_child() {
         ["write\nAdd file1", "next invocation\n", "write\nAdd file2"].join("\0"),
     )
     .unwrap();
-    let output = work_dir.run_jj(["split", "-r", "description(a)", "--parallel", "file1"]);
+    let output = work_dir.run_jj(["split", "-rsubject(glob:a)", "--parallel", "file1"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 descendant commits
@@ -939,7 +954,7 @@ fn test_split_with_multiple_workspaces_same_working_copy() {
         .success();
     // Change the working copy in the second workspace.
     secondary_dir
-        .run_jj(["edit", "-r", "description(first-commit)"])
+        .run_jj(["edit", "-r", "subject(glob:first-commit)"])
         .success();
     // Check the working-copy commit in each workspace in the log output. The "@"
     // node in the graph indicates the current workspace's working-copy commit.
@@ -1291,7 +1306,7 @@ fn test_split_move_first_commit() {
         "file1",
         "-r",
         "qpvuntsmwlqt",
-        "--destination",
+        "--onto",
         "rlvkpnrzqnoo",
         "file1",
     ]);
@@ -1469,14 +1484,14 @@ fn test_split_with_bookmarks(bookmark_behavior: BookmarkBehavior) {
     main_dir.write_file("file1", "foo");
     main_dir.write_file("file2", "foo");
     main_dir
-        .run_jj(["bookmark", "set", "'*le-signet*'", "-r", "@"])
+        .run_jj(["bookmark", "set", "*le-signet*", "-r", "@"])
         .success();
     insta::allow_duplicates! {
-    insta::assert_snapshot!(get_log_output(&main_dir), @r#"
-    @  qpvuntsmwlqt false "*le-signet*" first-commit
+    insta::assert_snapshot!(get_log_output(&main_dir), @r"
+    @  qpvuntsmwlqt false *le-signet* first-commit
     ◆  zzzzzzzzzzzz true
     [EOF]
-    "#);
+    ");
     }
     let setup_opid = main_dir.current_operation_id();
 
@@ -1490,42 +1505,42 @@ fn test_split_with_bookmarks(bookmark_behavior: BookmarkBehavior) {
     match bookmark_behavior {
         BookmarkBehavior::LeaveBookmarkWithTarget => {
             insta::allow_duplicates! {
-            insta::assert_snapshot!(output, @r#"
+            insta::assert_snapshot!(output, @r"
             ------- stderr -------
-            Selected changes : qpvuntsm a481fe8a "*le-signet*" | first-commit
+            Selected changes : qpvuntsm a481fe8a *le-signet* | first-commit
             Remaining changes: mzvwutvl 5f597a6e second-commit
             Working copy  (@) now at: mzvwutvl 5f597a6e second-commit
-            Parent commit (@-)      : qpvuntsm a481fe8a "*le-signet*" | first-commit
+            Parent commit (@-)      : qpvuntsm a481fe8a *le-signet* | first-commit
             [EOF]
-            "#);
+            ");
             }
             insta::allow_duplicates! {
-            insta::assert_snapshot!(get_log_output(&main_dir), @r#"
+            insta::assert_snapshot!(get_log_output(&main_dir), @r"
             @  mzvwutvlkqwt false second-commit
-            ○  qpvuntsmwlqt false "*le-signet*" first-commit
+            ○  qpvuntsmwlqt false *le-signet* first-commit
             ◆  zzzzzzzzzzzz true
             [EOF]
-            "#);
+            ");
             }
         }
         BookmarkBehavior::Default | BookmarkBehavior::MoveBookmarkToChild => {
             insta::allow_duplicates! {
-            insta::assert_snapshot!(output, @r#"
+            insta::assert_snapshot!(output, @r"
             ------- stderr -------
             Selected changes : qpvuntsm a481fe8a first-commit
-            Remaining changes: mzvwutvl 5f597a6e "*le-signet*" | second-commit
-            Working copy  (@) now at: mzvwutvl 5f597a6e "*le-signet*" | second-commit
+            Remaining changes: mzvwutvl 5f597a6e *le-signet* | second-commit
+            Working copy  (@) now at: mzvwutvl 5f597a6e *le-signet* | second-commit
             Parent commit (@-)      : qpvuntsm a481fe8a first-commit
             [EOF]
-            "#);
+            ");
             }
             insta::allow_duplicates! {
-            insta::assert_snapshot!(get_log_output(&main_dir), @r#"
-            @  mzvwutvlkqwt false "*le-signet*" second-commit
+            insta::assert_snapshot!(get_log_output(&main_dir), @r"
+            @  mzvwutvlkqwt false *le-signet* second-commit
             ○  qpvuntsmwlqt false first-commit
             ◆  zzzzzzzzzzzz true
             [EOF]
-            "#);
+            ");
             }
         }
     }
@@ -1541,25 +1556,229 @@ fn test_split_with_bookmarks(bookmark_behavior: BookmarkBehavior) {
     match bookmark_behavior {
         BookmarkBehavior::LeaveBookmarkWithTarget => {
             insta::allow_duplicates! {
-            insta::assert_snapshot!(get_log_output(&main_dir), @r#"
+            insta::assert_snapshot!(get_log_output(&main_dir), @r"
             @  vruxwmqvtpmx false second-commit
-            │ ○  qpvuntsmwlqt false "*le-signet*" first-commit
+            │ ○  qpvuntsmwlqt false *le-signet* first-commit
             ├─╯
             ◆  zzzzzzzzzzzz true
             [EOF]
-            "#);
+            ");
             }
         }
         BookmarkBehavior::Default | BookmarkBehavior::MoveBookmarkToChild => {
             insta::allow_duplicates! {
-            insta::assert_snapshot!(get_log_output(&main_dir), @r#"
-            @  vruxwmqvtpmx false "*le-signet*" second-commit
+            insta::assert_snapshot!(get_log_output(&main_dir), @r"
+            @  vruxwmqvtpmx false *le-signet* second-commit
             │ ○  qpvuntsmwlqt false first-commit
             ├─╯
             ◆  zzzzzzzzzzzz true
             [EOF]
-            "#);
+            ");
             }
         }
     }
 }
+
+#[test]
+fn test_split_with_editor_and_message_args() {
+    let mut test_env = TestEnvironment::default();
+    let edit_script = test_env.set_up_fake_editor();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    work_dir.write_file("file1", "foo\n");
+    work_dir.write_file("file2", "bar\n");
+    work_dir
+        .run_jj(["describe", "-m", "original description"])
+        .success();
+
+    std::fs::write(
+        &edit_script,
+        [
+            "dump editor1",
+            "write\nedited message 1",
+            "next invocation\n",
+            "dump editor2",
+            "write\nedited message 2",
+        ]
+        .join("\0"),
+    )
+    .unwrap();
+    work_dir
+        .run_jj([
+            "split",
+            "-m",
+            "message from command line",
+            "--editor",
+            "file1",
+        ])
+        .success();
+
+    // Verify editor was opened for the first commit with message from command line
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r#"
+    JJ: Enter a description for the selected changes.
+    message from command line
+
+    JJ: Change ID: qpvuntsm
+    JJ: This commit contains the following changes:
+    JJ:     A file1
+    JJ:
+    JJ: Lines starting with "JJ:" (like this one) will be removed.
+    "#);
+
+    // Verify editor was opened for the second commit with the original description
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor2")).unwrap(), @r#"
+    JJ: Enter a description for the remaining changes.
+    original description
+
+    JJ: Change ID: kkmpptxz
+    JJ: This commit contains the following changes:
+    JJ:     A file2
+    JJ:
+    JJ: Lines starting with "JJ:" (like this one) will be removed.
+    "#);
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
+    @  kkmpptxzrspx false edited message 2
+    ○  qpvuntsmwlqt false edited message 1
+    ◆  zzzzzzzzzzzz true
+    [EOF]
+    ");
+}
+
+#[test]
+fn test_split_with_editor_and_empty_message() {
+    let mut test_env = TestEnvironment::default();
+    let edit_script = test_env.set_up_fake_editor();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    work_dir.write_file("file1", "foo\n");
+    work_dir.write_file("file2", "bar\n");
+    work_dir
+        .run_jj(["describe", "-m", "original description"])
+        .success();
+
+    // Use --editor with an empty message. The trailers should be added because
+    // the editor will be opened.
+    std::fs::write(
+        &edit_script,
+        [
+            "dump editor1",
+            "write\nfirst commit",
+            "next invocation\n",
+            "dump editor2",
+            "write\nsecond commit",
+        ]
+        .join("\0"),
+    )
+    .unwrap();
+    work_dir
+        .run_jj([
+            "split",
+            "-m",
+            "",
+            "--editor",
+            "--config",
+            r#"templates.commit_trailers='"Trailer: value"'"#,
+            "file1",
+        ])
+        .success();
+
+    // Verify editor was opened for the first commit with trailers added to the
+    // empty message
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r#"
+    JJ: Enter a description for the selected changes.
+
+
+    Trailer: value
+
+    JJ: Change ID: qpvuntsm
+    JJ: This commit contains the following changes:
+    JJ:     A file1
+    JJ:
+    JJ: Lines starting with "JJ:" (like this one) will be removed.
+    "#);
+    // Verify editor was opened for the second commit with the original description
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor2")).unwrap(), @r#"
+    JJ: Enter a description for the remaining changes.
+    original description
+
+    Trailer: value
+
+    JJ: Change ID: kkmpptxz
+    JJ: This commit contains the following changes:
+    JJ:     A file2
+    JJ:
+    JJ: Lines starting with "JJ:" (like this one) will be removed.
+    "#);
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
+    @  kkmpptxzrspx false second commit
+    ○  qpvuntsmwlqt false first commit
+    ◆  zzzzzzzzzzzz true
+    [EOF]
+    ");
+}
+
+#[test]
+fn test_split_with_editor_without_message() {
+    let mut test_env = TestEnvironment::default();
+    let edit_script = test_env.set_up_fake_editor();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    work_dir.write_file("file1", "foo\n");
+    work_dir.write_file("file2", "bar\n");
+    work_dir
+        .run_jj(["describe", "-m", "original description"])
+        .success();
+
+    // --editor without -m should behave the same as without --editor (normal flow)
+    std::fs::write(
+        &edit_script,
+        [
+            "dump editor0",
+            "write\nfrom editor1",
+            "next invocation\n",
+            "dump editor1",
+            "write\nfrom editor2",
+        ]
+        .join("\0"),
+    )
+    .unwrap();
+    work_dir.run_jj(["split", "--editor", "file1"]).success();
+
+    // Verify editor was opened for the first commit
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor0")).unwrap(), @r#"
+    JJ: Enter a description for the selected changes.
+    original description
+
+    JJ: Change ID: qpvuntsm
+    JJ: This commit contains the following changes:
+    JJ:     A file1
+    JJ:
+    JJ: Lines starting with "JJ:" (like this one) will be removed.
+    "#);
+    // Verify editor was opened for the second commit
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r#"
+    JJ: Enter a description for the remaining changes.
+    original description
+
+    JJ: Change ID: kkmpptxz
+    JJ: This commit contains the following changes:
+    JJ:     A file2
+    JJ:
+    JJ: Lines starting with "JJ:" (like this one) will be removed.
+    "#);
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
+    @  kkmpptxzrspx false from editor2
+    ○  qpvuntsmwlqt false from editor1
+    ◆  zzzzzzzzzzzz true
+    [EOF]
+    ");
+}
