@@ -142,7 +142,7 @@ fn test_tag_bad_name() {
       | ^---
       |
       = expected <identifier>, <string_literal>, or <raw_string_literal>
-    Hint: See https://jj-vcs.github.io/jj/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
+    Hint: See https://docs.jj-vcs.dev/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
     [EOF]
     [exit status: 2]
     ");
@@ -159,7 +159,7 @@ fn test_tag_bad_name() {
       | ^^
       |
       = Expected non-empty string
-    Hint: See https://jj-vcs.github.io/jj/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
+    Hint: See https://docs.jj-vcs.dev/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
     [EOF]
     [exit status: 2]
     ");
@@ -176,7 +176,7 @@ fn test_tag_bad_name() {
       |    ^---
       |
       = expected <EOI>
-    Hint: See https://jj-vcs.github.io/jj/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
+    Hint: See https://docs.jj-vcs.dev/latest/revsets/ or use `jj help -k revsets` for how to quote symbols.
     [EOF]
     [exit status: 2]
     ");
@@ -199,17 +199,16 @@ fn test_tag_unknown() {
     let output = work_dir.run_jj(["tag", "delete", "unknown"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Error: No such tag: unknown
+    Warning: No matching tags for names: unknown
+    No tags to delete.
     [EOF]
-    [exit status: 1]
     ");
 
     let output = work_dir.run_jj(["tag", "delete", "glob:unknown*"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Error: No matching tags for patterns: unknown*
+    No tags to delete.
     [EOF]
-    [exit status: 1]
     ");
 }
 
@@ -278,11 +277,22 @@ fn test_tag_list() {
     [EOF]
     ");
 
-    insta::assert_snapshot!(work_dir.run_jj(["tag", "list", "glob:test_tag?"]), @r"
+    insta::assert_snapshot!(work_dir.run_jj(["tag", "list", "glob:'test_tag?'"]), @r"
     test_tag2: zsuskuln 76abdd20 (empty) commit2
     [EOF]
     ");
 
+    // Unmatched exact name pattern should be warned. "test_tag2" exists, but
+    // isn't included in the match.
+    insta::assert_snapshot!(
+        work_dir.run_jj(["tag", "list", "glob:test* & ~glob:*2", "unknown ~ test_tag2"]), @r"
+    test_tag: rlvkpnrz 893e67dc (empty) commit1
+    [EOF]
+    ------- stderr -------
+    Warning: No matching tags for names: unknown
+    [EOF]
+    ");
+
     let template = r#"
     concat(
       "[" ++ name ++ "]\n",
