@@ -13,6 +13,7 @@
 // limitations under the License.
 
 use test_case::test_case;
+use testutils::git;
 
 use crate::common::CommandOutput;
 use crate::common::TestEnvironment;
@@ -47,22 +48,22 @@ fn test_workspaces_add_second_workspace() {
 
     // Can see the working-copy commit in each workspace in the log output. The "@"
     // node in the graph indicates the current workspace's working-copy commit.
-    insta::assert_snapshot!(get_log_output(&main_dir), @r"
+    insta::assert_snapshot!(get_log_output(&main_dir), @r#"
     @  504e3d8c1bcd default@
     │ ○  bcc858e1d93f second@
     ├─╯
-    ○  7b22a8cbe888
+    ○  7b22a8cbe888 "initial"
     ◆  000000000000
     [EOF]
-    ");
-    insta::assert_snapshot!(get_log_output(&secondary_dir), @r"
+    "#);
+    insta::assert_snapshot!(get_log_output(&secondary_dir), @r#"
     @  bcc858e1d93f second@
     │ ○  504e3d8c1bcd default@
     ├─╯
-    ○  7b22a8cbe888
+    ○  7b22a8cbe888 "initial"
     ◆  000000000000
     [EOF]
-    ");
+    "#);
 
     // Both workspaces show up when we list them
     let output = main_dir.run_jj(["workspace", "list"]);
@@ -150,17 +151,17 @@ fn test_workspaces_add_second_workspace_on_merge() {
         .success();
 
     // The new workspace's working-copy commit shares all parents with the old one.
-    insta::assert_snapshot!(get_log_output(&main_dir), @r"
-    @    46ed31b61ce9 default@
+    insta::assert_snapshot!(get_log_output(&main_dir), @r#"
+    @    46ed31b61ce9 default@ "merge"
     ├─╮
     │ │ ○  d23b2d4ff55c second@
     ╭─┬─╯
-    │ ○  3c52528f5893
-    ○ │  a3155ab1bf5a
+    │ ○  3c52528f5893 "left"
+    ○ │  a3155ab1bf5a "right"
     ├─╯
     ◆  000000000000
     [EOF]
-    ");
+    "#);
 }
 
 /// Test that --ignore-working-copy is respected
@@ -293,24 +294,24 @@ fn test_workspaces_add_workspace_at_revision() {
 
     // Can see the working-copy commit in each workspace in the log output. The "@"
     // node in the graph indicates the current workspace's working-copy commit.
-    insta::assert_snapshot!(get_log_output(&main_dir), @r"
+    insta::assert_snapshot!(get_log_output(&main_dir), @r#"
     @  5ac9178da8b2 default@
-    ○  a47d8a593529
+    ○  a47d8a593529 "second"
     │ ○  ea5860fbd622 second@
     ├─╯
-    ○  27473635a942
+    ○  27473635a942 "first"
     ◆  000000000000
     [EOF]
-    ");
-    insta::assert_snapshot!(get_log_output(&secondary_dir), @r"
+    "#);
+    insta::assert_snapshot!(get_log_output(&secondary_dir), @r#"
     @  ea5860fbd622 second@
     │ ○  5ac9178da8b2 default@
-    │ ○  a47d8a593529
+    │ ○  a47d8a593529 "second"
     ├─╯
-    ○  27473635a942
+    ○  27473635a942 "first"
     ◆  000000000000
     [EOF]
-    ");
+    "#);
 }
 
 /// Test multiple `-r` flags to `workspace add` to create a workspace
@@ -333,26 +334,26 @@ fn test_workspaces_add_workspace_multiple_revisions() {
     main_dir.run_jj(["commit", "-m", "third"]).success();
     main_dir.run_jj(["new", "-r", "root()"]).success();
 
-    insta::assert_snapshot!(get_log_output(&main_dir), @r"
+    insta::assert_snapshot!(get_log_output(&main_dir), @r#"
     @  8d23abddc924
-    │ ○  eba7f49e2358
+    │ ○  eba7f49e2358 "third"
     ├─╯
-    │ ○  62444a45efcf
+    │ ○  62444a45efcf "second"
     ├─╯
-    │ ○  27473635a942
+    │ ○  27473635a942 "first"
     ├─╯
     ◆  000000000000
     [EOF]
-    ");
+    "#);
 
     let output = main_dir.run_jj([
         "workspace",
         "add",
         "--name=merge",
         "../merged",
-        "-r=description(third)",
-        "-r=description(second)",
-        "-r=description(first)",
+        "-r=subject(glob:third)",
+        "-r=subject(glob:second)",
+        "-r=subject(glob:first)",
     ]);
     insta::assert_snapshot!(output.normalize_backslash(), @r#"
     ------- stderr -------
@@ -365,19 +366,19 @@ fn test_workspaces_add_workspace_multiple_revisions() {
     [EOF]
     "#);
 
-    insta::assert_snapshot!(get_log_output(&main_dir), @r"
+    insta::assert_snapshot!(get_log_output(&main_dir), @r#"
     @  8d23abddc924 default@
     │ ○      2d7c9a2d41dc merge@
     │ ├─┬─╮
-    │ │ │ ○  27473635a942
+    │ │ │ ○  27473635a942 "first"
     ├─────╯
-    │ │ ○  62444a45efcf
+    │ │ ○  62444a45efcf "second"
     ├───╯
-    │ ○  eba7f49e2358
+    │ ○  eba7f49e2358 "third"
     ├─╯
     ◆  000000000000
     [EOF]
-    ");
+    "#);
 }
 
 #[test]
@@ -440,11 +441,11 @@ fn test_workspaces_add_workspace_in_current_workspace() {
 
     // Workspace created despite warning
     let output = main_dir.run_jj(["workspace", "list"]);
-    insta::assert_snapshot!(output, @r"
-    default: rlvkpnrz af2d0cd5 (no description set)
+    insta::assert_snapshot!(output, @r###"
+    default: rlvkpnrz 504e3d8c (empty) (no description set)
     secondary: pmmvwywv 058f604d (empty) (no description set)
     [EOF]
-    ");
+    "###);
 
     // Use explicit path instead (no warning)
     let output = main_dir.run_jj(["workspace", "add", "./third"]);
@@ -459,22 +460,18 @@ fn test_workspaces_add_workspace_in_current_workspace() {
 
     // Both workspaces created
     let output = main_dir.run_jj(["workspace", "list"]);
-    insta::assert_snapshot!(output, @r"
-    default: rlvkpnrz b2288847 (no description set)
+    insta::assert_snapshot!(output, @r###"
+    default: rlvkpnrz 504e3d8c (empty) (no description set)
     secondary: pmmvwywv 058f604d (empty) (no description set)
     third: zxsnswpr 1c1effec (empty) (no description set)
     [EOF]
-    ");
+    "###);
 
-    // Can see files from the other workspaces in main workspace, since they are
-    // child directories and will therefore be snapshotted
     let output = main_dir.run_jj(["file", "list"]);
-    insta::assert_snapshot!(output.normalize_backslash(), @r"
+    insta::assert_snapshot!(output.normalize_backslash(), @r###"
     file
-    secondary/file
-    third/file
     [EOF]
-    ");
+    "###);
 }
 
 /// Test making changes to the working copy in a workspace as it gets rewritten
@@ -530,7 +527,7 @@ fn test_workspaces_conflicting_edits() {
     ------- stderr -------
     Error: The working copy is stale (not updated since operation bd4f780d0422).
     Hint: Run `jj workspace update-stale` to update it.
-    See https://jj-vcs.github.io/jj/latest/working-copy/#stale-working-copy for more information.
+    See https://docs.jj-vcs.dev/latest/working-copy/#stale-working-copy for more information.
     [EOF]
     [exit status: 1]
     ");
@@ -540,7 +537,7 @@ fn test_workspaces_conflicting_edits() {
     ------- stderr -------
     Error: The working copy is stale (not updated since operation bd4f780d0422).
     Hint: Run `jj workspace update-stale` to update it.
-    See https://jj-vcs.github.io/jj/latest/working-copy/#stale-working-copy for more information.
+    See https://docs.jj-vcs.dev/latest/working-copy/#stale-working-copy for more information.
     [EOF]
     [exit status: 1]
     ");
@@ -561,7 +558,7 @@ fn test_workspaces_conflicting_edits() {
     insta::assert_snapshot!(get_log_output(&secondary_dir),
     @r"
     @  90f3d42e0bff secondary@ (divergent)
-    │ ×  de7155dbea42 (divergent)
+    │ ×  3ef90f18334b (divergent)
     ├─╯
     │ ○  3a9b690d6e67 default@
     ├─╯
@@ -572,7 +569,7 @@ fn test_workspaces_conflicting_edits() {
     // The stale working copy should have been resolved by the previous command
     insta::assert_snapshot!(get_log_output(&secondary_dir), @r"
     @  90f3d42e0bff secondary@ (divergent)
-    │ ×  de7155dbea42 (divergent)
+    │ ×  3ef90f18334b (divergent)
     ├─╯
     │ ○  3a9b690d6e67 default@
     ├─╯
@@ -631,7 +628,7 @@ fn test_workspaces_updated_by_other() {
     ------- stderr -------
     Error: The working copy is stale (not updated since operation bd4f780d0422).
     Hint: Run `jj workspace update-stale` to update it.
-    See https://jj-vcs.github.io/jj/latest/working-copy/#stale-working-copy for more information.
+    See https://docs.jj-vcs.dev/latest/working-copy/#stale-working-copy for more information.
     [EOF]
     [exit status: 1]
     ");
@@ -831,7 +828,7 @@ fn test_workspaces_current_op_discarded_by_other(automatic: bool) {
         ------- stderr -------
         Error: Could not read working copy's operation.
         Hint: Run `jj workspace update-stale` to recover.
-        See https://jj-vcs.github.io/jj/latest/working-copy/#stale-working-copy for more information.
+        See https://docs.jj-vcs.dev/latest/working-copy/#stale-working-copy for more information.
         [EOF]
         [exit status: 1]
         ");
@@ -846,15 +843,15 @@ fn test_workspaces_current_op_discarded_by_other(automatic: bool) {
     }
 
     insta::allow_duplicates! {
-        insta::assert_snapshot!(get_log_output(&main_dir), @r"
+        insta::assert_snapshot!(get_log_output(&main_dir), @r#"
         @  320bc89effc9 default@
-        │ ○  18851b397d09 secondary@
+        │ ○  18851b397d09 secondary@ "RECOVERY COMMIT FROM `jj workspace update-stale`"
         │ ○  891f00062e10
         ├─╯
         ○  367415be5b44
         ◆  000000000000
         [EOF]
-        ");
+        "#);
     }
 
     // The sparse patterns should remain
@@ -939,6 +936,43 @@ fn test_workspaces_update_stale_noop() {
     ");
 }
 
+/// If the working copy was last updated to an unpublished operation, it should
+/// be reported, even if the latest published operation has the same tree.
+#[test]
+fn test_workspaces_unpublished_operation_same_tree() {
+    let test_env = TestEnvironment::default();
+    test_env.run_jj_in(".", ["git", "init", "main"]).success();
+    let main_dir = test_env.work_dir("main");
+
+    main_dir.run_jj(["desc", "-m=A"]).success();
+    let a_op_id = main_dir.current_operation_id();
+    main_dir.run_jj(["new", "-m=B"]).success();
+    let b_op_id = main_dir.current_operation_id();
+    // Make the repo forget about the B operation
+    main_dir.remove_file(format!(".jj/repo/op_heads/heads/{b_op_id}"));
+    main_dir.write_file(format!(".jj/repo/op_heads/heads/{a_op_id}"), "");
+    main_dir
+        .run_jj(["new", "-m=C", "--ignore-working-copy"])
+        .success();
+    // The working copy should be stale and should require a `jj workspace
+    // update-stale`
+    let output = main_dir.run_jj(["status"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Internal error: The repo was loaded at operation 502db81004ba, which seems to be a sibling of the working copy's operation 48631817a82e
+    [EOF]
+    [exit status: 255]
+    ");
+    let output = main_dir.run_jj(["workspace", "update-stale"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Working copy  (@) now at: zsuskuln 36a15ac4 (empty) C
+    Parent commit (@-)      : qpvuntsm 8777db25 (empty) A
+    Updated working copy to fresh commit 36a15ac414e8
+    [EOF]
+    ");
+}
+
 /// Test "update-stale" in a dirty, but not stale working copy.
 #[test]
 fn test_workspaces_update_stale_snapshot() {
@@ -978,6 +1012,127 @@ fn test_workspaces_update_stale_snapshot() {
     ");
 }
 
+/// Test that "workspace update-stale" works in colocated repos.
+///
+/// This is a regression test for a bug introduced in commit 7a296ca1 where
+/// the reload-to-HEAD logic (added to fix a race condition) would break
+/// "workspace update-stale" by reloading the repo to HEAD before snapshotting,
+/// even though recovery intentionally loads at an old operation.
+#[test]
+fn test_colocated_workspace_update_stale() {
+    let test_env = TestEnvironment::default();
+    test_env
+        .run_jj_in(".", ["git", "init", "--colocate", "main"])
+        .success();
+    let main_dir = test_env.work_dir("main");
+    let secondary_dir = test_env.work_dir("secondary");
+    let git_repo = git::open(main_dir.root());
+
+    main_dir.write_file("file", "contents\n");
+    main_dir.run_jj(["new"]).success();
+
+    // Create new bookmarked revision from the main workspace.
+    main_dir
+        .run_jj(["new", "--no-edit", "root()", "-mold book1"])
+        .success();
+    main_dir
+        .run_jj(["bookmark", "set", "-rsubject(glob:'old book1')", "book1"])
+        .success();
+
+    main_dir
+        .run_jj(["workspace", "add", "../secondary"])
+        .success();
+
+    // Rewrite the check-out commit from the secondary workspace.
+    // This makes the main (colocated) workspace's working copy stale.
+    secondary_dir.write_file("file", "changed in secondary\n");
+    secondary_dir.run_jj(["squash"]).success();
+
+    // Update and export the bookmark from the secondary workspace.
+    secondary_dir
+        .run_jj(["new", "--no-edit", "root()", "-mnew book1"])
+        .success();
+    secondary_dir
+        .run_jj([
+            "bookmark",
+            "set",
+            "-rsubject(glob:'new book1')",
+            "--allow-backwards",
+            "book1",
+        ])
+        .success();
+    secondary_dir.run_jj(["git", "export"]).success();
+
+    // Create new Git ref and commit which will be imported later by "jj
+    // workspace update-stale".
+    git::add_commit(&git_repo, "refs/heads/book2", "file", b"", "book2", &[]);
+
+    insta::assert_snapshot!(get_log_output(&secondary_dir), @r#"
+    @  9cb8253861b5 secondary@
+    │ ○  f562bf82f2da default@
+    ├─╯
+    ○  30ed2f28b710
+    │ ○  e97ad7861f78 book1 "new book1"
+    ├─╯
+    │ ○  f656b467890b "old book1"
+    ├─╯
+    ◆  000000000000
+    [EOF]
+    "#);
+
+    // The main workspace's working copy is now stale.
+    let output = main_dir.run_jj(["st"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Error: The working copy is stale (not updated since operation a3fbf68cb3f8).
+    Hint: Run `jj workspace update-stale` to update it.
+    See https://docs.jj-vcs.dev/latest/working-copy/#stale-working-copy for more information.
+    [EOF]
+    [exit status: 1]
+    ");
+
+    // Before the fix, this would fail with the same "working copy is stale" error
+    // because the colocated repo reload logic would reload to HEAD before
+    // snapshotting, breaking the recovery.
+    let output = main_dir.run_jj(["workspace", "update-stale"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Working copy  (@) now at: rlvkpnrz f562bf82 (empty) (no description set)
+    Parent commit (@-)      : qpvuntsm 30ed2f28 (no description set)
+    Added 0 files, modified 1 files, removed 0 files
+    Updated working copy to fresh commit f562bf82f2da
+    Done importing changes from the underlying Git repo.
+    [EOF]
+    ");
+
+    // Verify the workspace is now up-to-date. New bookmark "book2" should have
+    // been imported by the previous command.
+    let output = main_dir.run_jj(["st"]);
+    insta::assert_snapshot!(output, @r"
+    The working copy has no changes.
+    Working copy  (@) : rlvkpnrz f562bf82 (empty) (no description set)
+    Parent commit (@-): qpvuntsm 30ed2f28 (no description set)
+    [EOF]
+    ");
+
+    // The updated bookmark "book1" shouldn't be re-imported as an external
+    // change. If it were, the "old book1" revision would be abandoned.
+    insta::assert_snapshot!(get_log_output(&main_dir), @r#"
+    @  f562bf82f2da default@
+    │ ○  9cb8253861b5 secondary@
+    ├─╯
+    ○  30ed2f28b710
+    │ ○  7fe3ff3b9a60 book2 "book2"
+    ├─╯
+    │ ○  e97ad7861f78 book1 "new book1"
+    ├─╯
+    │ ○  f656b467890b "old book1"
+    ├─╯
+    ◆  000000000000
+    [EOF]
+    "#);
+}
+
 /// Test forgetting workspaces
 #[test]
 fn test_workspaces_forget() {
@@ -1385,8 +1540,10 @@ fn get_log_output(work_dir: &TestWorkDir) -> CommandOutput {
     let template = r#"
     separate(" ",
       commit_id.short(),
+      bookmarks,
       working_copies,
       if(divergent, "(divergent)"),
+      surround('"', '"', description.first_line()),
     )
     "#;
     work_dir.run_jj(["log", "-T", template, "-r", "all()"])
