@@ -37,8 +37,8 @@ where
     II: IntoIterator<Item = T>,
     NI: IntoIterator<Item = T>,
 {
-    let neighbors_fn = move |node: &T| to_ok_iter(neighbors_fn(node));
-    dfs_ok(to_ok_iter(start), id_fn, neighbors_fn).map(Result::unwrap)
+    let neighbors_fn = move |node: &T| to_infallibe_iter(neighbors_fn(node));
+    dfs_ok(to_infallibe_iter(start), id_fn, neighbors_fn).map(|Ok(node)| node)
 }
 
 /// Traverses nodes from `start` in depth-first order.
@@ -79,20 +79,21 @@ where
 /// Builds a list of nodes reachable from the `start` where neighbors come
 /// before the node itself.
 ///
-/// Panics if the graph has cycle.
-pub fn topo_order_forward<T, ID, II, NI>(
+/// If the graph has cycle, `cycle_fn()` is called with one of the nodes
+/// involved in the cycle.
+pub fn topo_order_forward<T, ID, E, II, NI>(
     start: II,
     id_fn: impl Fn(&T) -> ID,
     mut neighbors_fn: impl FnMut(&T) -> NI,
-) -> Vec<T>
+    cycle_fn: impl FnOnce(T) -> E,
+) -> Result<Vec<T>, E>
 where
     ID: Hash + Eq + Clone,
     II: IntoIterator<Item = T>,
     NI: IntoIterator<Item = T>,
 {
     let neighbors_fn = move |node: &T| to_ok_iter(neighbors_fn(node));
-    let cycle_fn = |_| panic!("graph has cycle");
-    topo_order_forward_ok(to_ok_iter(start), id_fn, neighbors_fn, cycle_fn).unwrap()
+    topo_order_forward_ok(to_ok_iter(start), id_fn, neighbors_fn, cycle_fn)
 }
 
 /// Builds a list of `Ok` nodes reachable from the `start` where neighbors come
@@ -143,20 +144,21 @@ where
 /// Builds a list of nodes reachable from the `start` where neighbors come after
 /// the node itself.
 ///
-/// Panics if the graph has cycle.
-pub fn topo_order_reverse<T, ID, II, NI>(
+/// If the graph has cycle, `cycle_fn()` is called with one of the nodes
+/// involved in the cycle.
+pub fn topo_order_reverse<T, ID, E, II, NI>(
     start: II,
     id_fn: impl Fn(&T) -> ID,
     mut neighbors_fn: impl FnMut(&T) -> NI,
-) -> Vec<T>
+    cycle_fn: impl FnOnce(T) -> E,
+) -> Result<Vec<T>, E>
 where
     ID: Hash + Eq + Clone,
     II: IntoIterator<Item = T>,
     NI: IntoIterator<Item = T>,
 {
     let neighbors_fn = move |node: &T| to_ok_iter(neighbors_fn(node));
-    let cycle_fn = |_| panic!("graph has cycle");
-    topo_order_reverse_ok(to_ok_iter(start), id_fn, neighbors_fn, cycle_fn).unwrap()
+    topo_order_reverse_ok(to_ok_iter(start), id_fn, neighbors_fn, cycle_fn)
 }
 
 /// Builds a list of `Ok` nodes reachable from the `start` where neighbors come
@@ -190,12 +192,14 @@ where
 /// Use `topo_order_reverse()` if the DAG is heavily branched. This can
 /// only process linear part lazily.
 ///
-/// Panics if the graph has cycle.
-pub fn topo_order_reverse_lazy<T, ID, II, NI>(
+/// If the graph has cycle, `cycle_fn()` is called with one of the nodes
+/// involved in the cycle.
+pub fn topo_order_reverse_lazy<T, ID, E, II, NI>(
     start: II,
     id_fn: impl Fn(&T) -> ID,
     mut neighbors_fn: impl FnMut(&T) -> NI,
-) -> impl Iterator<Item = T>
+    cycle_fn: impl FnMut(T) -> E,
+) -> impl Iterator<Item = Result<T, E>>
 where
     T: Ord,
     ID: Hash + Eq + Clone,
@@ -203,8 +207,7 @@ where
     NI: IntoIterator<Item = T>,
 {
     let neighbors_fn = move |node: &T| to_ok_iter(neighbors_fn(node));
-    let cycle_fn = |_| panic!("graph has cycle");
-    topo_order_reverse_lazy_ok(to_ok_iter(start), id_fn, neighbors_fn, cycle_fn).map(Result::unwrap)
+    topo_order_reverse_lazy_ok(to_ok_iter(start), id_fn, neighbors_fn, cycle_fn)
 }
 
 /// Like `topo_order_reverse_ok()`, but can iterate linear DAG lazily.
@@ -442,12 +445,14 @@ where
 /// Unlike `topo_order_reverse()`, nodes are sorted in reverse `T: Ord` order so
 /// long as they can respect the topological requirement.
 ///
-/// Panics if the graph has cycle.
-pub fn topo_order_reverse_ord<T, ID, II, NI>(
+/// If the graph has cycle, `cycle_fn()` is called with one of the nodes
+/// involved in the cycle.
+pub fn topo_order_reverse_ord<T, ID, E, II, NI>(
     start: II,
     id_fn: impl Fn(&T) -> ID,
     mut neighbors_fn: impl FnMut(&T) -> NI,
-) -> Vec<T>
+    cycle_fn: impl FnOnce(T) -> E,
+) -> Result<Vec<T>, E>
 where
     T: Ord,
     ID: Hash + Eq + Clone,
@@ -455,8 +460,7 @@ where
     NI: IntoIterator<Item = T>,
 {
     let neighbors_fn = move |node: &T| to_ok_iter(neighbors_fn(node));
-    let cycle_fn = |_| panic!("graph has cycle");
-    topo_order_reverse_ord_ok(to_ok_iter(start), id_fn, neighbors_fn, cycle_fn).unwrap()
+    topo_order_reverse_ord_ok(to_ok_iter(start), id_fn, neighbors_fn, cycle_fn)
 }
 
 /// Builds a list of `Ok` nodes reachable from the `start` where neighbors come
@@ -567,8 +571,9 @@ where
     II: IntoIterator<Item = T>,
     NI: IntoIterator<Item = T>,
 {
-    let neighbors_fn = move |node: &T| to_ok_iter(neighbors_fn(node));
-    heads_ok(to_ok_iter(start), id_fn, neighbors_fn).unwrap()
+    let neighbors_fn = move |node: &T| to_infallibe_iter(neighbors_fn(node));
+    let Ok(node) = heads_ok(to_infallibe_iter(start), id_fn, neighbors_fn);
+    node
 }
 
 /// Finds `Ok` nodes in the start set that are not reachable from other nodes in
@@ -626,8 +631,14 @@ where
     II2: IntoIterator<Item = T>,
     NI: IntoIterator<Item = T>,
 {
-    let neighbors_fn = move |node: &T| to_ok_iter(neighbors_fn(node));
-    closest_common_node_ok(to_ok_iter(set1), to_ok_iter(set2), id_fn, neighbors_fn).unwrap()
+    let neighbors_fn = move |node: &T| to_infallibe_iter(neighbors_fn(node));
+    let Ok(node) = closest_common_node_ok(
+        to_infallibe_iter(set1),
+        to_infallibe_iter(set2),
+        id_fn,
+        neighbors_fn,
+    );
+    node
 }
 
 /// Finds the closest common `Ok` neighbor among the `set1` and `set2`.
@@ -688,14 +699,18 @@ where
     Ok(None)
 }
 
-fn to_ok_iter<T>(iter: impl IntoIterator<Item = T>) -> impl Iterator<Item = Result<T, Infallible>> {
+fn to_ok_iter<T, E>(iter: impl IntoIterator<Item = T>) -> impl Iterator<Item = Result<T, E>> {
     iter.into_iter().map(Ok)
 }
 
+fn to_infallibe_iter<T>(
+    iter: impl IntoIterator<Item = T>,
+) -> impl Iterator<Item = Result<T, Infallible>> {
+    to_ok_iter(iter)
+}
+
 #[cfg(test)]
 mod tests {
-    use std::panic;
-
     use assert_matches::assert_matches;
     use maplit::hashmap;
     use maplit::hashset;
@@ -731,26 +746,33 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
+        let cycle_fn = |id| id;
 
-        let common = topo_order_reverse(vec!['C'], id_fn, neighbors_fn);
+        let common = topo_order_reverse(vec!['C'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['C', 'B', 'A']);
-        let common = topo_order_reverse(vec!['C', 'B'], id_fn, neighbors_fn);
+        let common = topo_order_reverse(vec!['C', 'B'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['C', 'B', 'A']);
-        let common = topo_order_reverse(vec!['B', 'C'], id_fn, neighbors_fn);
+        let common = topo_order_reverse(vec!['B', 'C'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['C', 'B', 'A']);
 
-        let common = topo_order_reverse_lazy(vec!['C'], id_fn, neighbors_fn).collect_vec();
+        let common: Vec<_> = topo_order_reverse_lazy(vec!['C'], id_fn, neighbors_fn, cycle_fn)
+            .try_collect()
+            .unwrap();
         assert_eq!(common, vec!['C', 'B', 'A']);
-        let common = topo_order_reverse_lazy(vec!['C', 'B'], id_fn, neighbors_fn).collect_vec();
+        let common: Vec<_> = topo_order_reverse_lazy(vec!['C', 'B'], id_fn, neighbors_fn, cycle_fn)
+            .try_collect()
+            .unwrap();
         assert_eq!(common, vec!['C', 'B', 'A']);
-        let common = topo_order_reverse_lazy(vec!['B', 'C'], id_fn, neighbors_fn).collect_vec();
+        let common: Vec<_> = topo_order_reverse_lazy(vec!['B', 'C'], id_fn, neighbors_fn, cycle_fn)
+            .try_collect()
+            .unwrap();
         assert_eq!(common, vec!['C', 'B', 'A']);
 
-        let common = topo_order_reverse_ord(vec!['C'], id_fn, neighbors_fn);
+        let common = topo_order_reverse_ord(vec!['C'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['C', 'B', 'A']);
-        let common = topo_order_reverse_ord(vec!['C', 'B'], id_fn, neighbors_fn);
+        let common = topo_order_reverse_ord(vec!['C', 'B'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['C', 'B', 'A']);
-        let common = topo_order_reverse_ord(vec!['B', 'C'], id_fn, neighbors_fn);
+        let common = topo_order_reverse_ord(vec!['B', 'C'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['C', 'B', 'A']);
     }
 
@@ -776,28 +798,39 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
+        let cycle_fn = |id| id;
 
-        let common = topo_order_reverse(vec!['F'], id_fn, neighbors_fn);
-        assert_eq!(common, vec!['F', 'E', 'D', 'C', 'B', 'A']);
-        let common = topo_order_reverse(vec!['F', 'E', 'C'], id_fn, neighbors_fn);
-        assert_eq!(common, vec!['F', 'D', 'E', 'C', 'B', 'A']);
-        let common = topo_order_reverse(vec!['F', 'D', 'E'], id_fn, neighbors_fn);
-        assert_eq!(common, vec!['F', 'D', 'C', 'B', 'E', 'A']);
-
-        let common = topo_order_reverse_lazy(vec!['F'], id_fn, neighbors_fn).collect_vec();
+        let common = topo_order_reverse(vec!['F'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['F', 'E', 'D', 'C', 'B', 'A']);
         let common =
-            topo_order_reverse_lazy(vec!['F', 'E', 'C'], id_fn, neighbors_fn).collect_vec();
+            topo_order_reverse(vec!['F', 'E', 'C'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['F', 'D', 'E', 'C', 'B', 'A']);
         let common =
-            topo_order_reverse_lazy(vec!['F', 'D', 'E'], id_fn, neighbors_fn).collect_vec();
+            topo_order_reverse(vec!['F', 'D', 'E'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['F', 'D', 'C', 'B', 'E', 'A']);
 
-        let common = topo_order_reverse_ord(vec!['F'], id_fn, neighbors_fn);
+        let common: Vec<_> = topo_order_reverse_lazy(vec!['F'], id_fn, neighbors_fn, cycle_fn)
+            .try_collect()
+            .unwrap();
         assert_eq!(common, vec!['F', 'E', 'D', 'C', 'B', 'A']);
-        let common = topo_order_reverse_ord(vec!['F', 'E', 'C'], id_fn, neighbors_fn);
+        let common: Vec<_> =
+            topo_order_reverse_lazy(vec!['F', 'E', 'C'], id_fn, neighbors_fn, cycle_fn)
+                .try_collect()
+                .unwrap();
+        assert_eq!(common, vec!['F', 'D', 'E', 'C', 'B', 'A']);
+        let common: Vec<_> =
+            topo_order_reverse_lazy(vec!['F', 'D', 'E'], id_fn, neighbors_fn, cycle_fn)
+                .try_collect()
+                .unwrap();
+        assert_eq!(common, vec!['F', 'D', 'C', 'B', 'E', 'A']);
+
+        let common = topo_order_reverse_ord(vec!['F'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['F', 'E', 'D', 'C', 'B', 'A']);
-        let common = topo_order_reverse_ord(vec!['F', 'D', 'E'], id_fn, neighbors_fn);
+        let common =
+            topo_order_reverse_ord(vec!['F', 'E', 'C'], id_fn, neighbors_fn, cycle_fn).unwrap();
+        assert_eq!(common, vec!['F', 'E', 'D', 'C', 'B', 'A']);
+        let common =
+            topo_order_reverse_ord(vec!['F', 'D', 'E'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['F', 'E', 'D', 'C', 'B', 'A']);
     }
 
@@ -830,14 +863,17 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
+        let cycle_fn = |id| id;
 
-        let common = topo_order_reverse(vec!['I'], id_fn, neighbors_fn);
+        let common = topo_order_reverse(vec!['I'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['I', 'D', 'B', 'H', 'F', 'G', 'E', 'C', 'A']);
 
-        let common = topo_order_reverse_lazy(vec!['I'], id_fn, neighbors_fn).collect_vec();
+        let common: Vec<_> = topo_order_reverse_lazy(vec!['I'], id_fn, neighbors_fn, cycle_fn)
+            .try_collect()
+            .unwrap();
         assert_eq!(common, vec!['I', 'D', 'B', 'H', 'F', 'G', 'E', 'C', 'A']);
 
-        let common = topo_order_reverse_ord(vec!['I'], id_fn, neighbors_fn);
+        let common = topo_order_reverse_ord(vec!['I'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['I', 'H', 'G', 'F', 'E', 'D', 'C', 'B', 'A']);
     }
 
@@ -872,14 +908,17 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
+        let cycle_fn = |id| id;
 
-        let common = topo_order_reverse(vec!['I'], id_fn, neighbors_fn);
+        let common = topo_order_reverse(vec!['I'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['I', 'h', 'G', 'e', 'C', 'f', 'D', 'b', 'A']);
 
-        let common = topo_order_reverse_lazy(vec!['I'], id_fn, neighbors_fn).collect_vec();
+        let common: Vec<_> = topo_order_reverse_lazy(vec!['I'], id_fn, neighbors_fn, cycle_fn)
+            .try_collect()
+            .unwrap();
         assert_eq!(common, vec!['I', 'h', 'G', 'e', 'C', 'f', 'D', 'b', 'A']);
 
-        let common = topo_order_reverse_ord(vec!['I'], id_fn, neighbors_fn);
+        let common = topo_order_reverse_ord(vec!['I'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['I', 'h', 'f', 'G', 'e', 'D', 'b', 'C', 'A']);
     }
 
@@ -903,16 +942,19 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
+        let cycle_fn = |id| id;
 
-        let common = topo_order_reverse(vec!['E'], id_fn, neighbors_fn);
+        let common = topo_order_reverse(vec!['E'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['E', 'D', 'C', 'B', 'a']);
 
         // The root node 'a' is visited before 'C'. If the graph were split there,
         // the branch 'C->B->a' would be orphaned.
-        let common = topo_order_reverse_lazy(vec!['E'], id_fn, neighbors_fn).collect_vec();
+        let common: Vec<_> = topo_order_reverse_lazy(vec!['E'], id_fn, neighbors_fn, cycle_fn)
+            .try_collect()
+            .unwrap();
         assert_eq!(common, vec!['E', 'D', 'C', 'B', 'a']);
 
-        let common = topo_order_reverse_ord(vec!['E'], id_fn, neighbors_fn);
+        let common = topo_order_reverse_ord(vec!['E'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['E', 'D', 'C', 'B', 'a']);
     }
 
@@ -940,19 +982,21 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
+        let cycle_fn = |id| id;
 
-        let common = topo_order_reverse(vec!['G'], id_fn, neighbors_fn);
+        let common = topo_order_reverse(vec!['G'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['G', 'E', 'F', 'D', 'C', 'B', 'A']);
 
-        let common = topo_order_reverse_lazy(vec!['G'], id_fn, neighbors_fn).collect_vec();
+        let common: Vec<_> = topo_order_reverse_lazy(vec!['G'], id_fn, neighbors_fn, cycle_fn)
+            .try_collect()
+            .unwrap();
         assert_eq!(common, vec!['G', 'E', 'F', 'D', 'C', 'B', 'A']);
 
-        let common = topo_order_reverse_ord(vec!['G'], id_fn, neighbors_fn);
+        let common = topo_order_reverse_ord(vec!['G'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['G', 'F', 'E', 'D', 'C', 'B', 'A']);
 
         // Iterator can be lazy for linear chunks.
         let neighbors_fn = |node: &char| to_ok_iter(neighbors[node].iter().copied());
-        let cycle_fn = |_| panic!("graph has cycle");
         let mut inner_iter = TopoOrderReverseLazyInner::empty();
         inner_iter.extend([Ok('G')]);
         assert_eq!(
@@ -1011,20 +1055,22 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
+        let cycle_fn = |id| id;
 
-        let common = topo_order_reverse(vec!['G'], id_fn, neighbors_fn);
+        let common = topo_order_reverse(vec!['G'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['G', 'F', 'E', 'D', 'c', 'B', 'A']);
 
-        let common = topo_order_reverse_lazy(vec!['G'], id_fn, neighbors_fn).collect_vec();
+        let common: Vec<_> = topo_order_reverse_lazy(vec!['G'], id_fn, neighbors_fn, cycle_fn)
+            .try_collect()
+            .unwrap();
         assert_eq!(common, vec!['G', 'F', 'E', 'D', 'c', 'B', 'A']);
 
-        let common = topo_order_reverse_ord(vec!['G'], id_fn, neighbors_fn);
+        let common = topo_order_reverse_ord(vec!['G'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['G', 'F', 'E', 'D', 'c', 'B', 'A']);
 
         // Iterator can be lazy for linear chunks. The node 'c' is visited before 'D',
         // but it will be processed lazily.
         let neighbors_fn = |node: &char| to_ok_iter(neighbors[node].iter().copied());
-        let cycle_fn = |_| panic!("graph has cycle");
         let mut inner_iter = TopoOrderReverseLazyInner::empty();
         inner_iter.extend([Ok('G')]);
         assert_eq!(
@@ -1083,19 +1129,21 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
+        let cycle_fn = |id| id;
 
-        let common = topo_order_reverse(vec!['G'], id_fn, neighbors_fn);
+        let common = topo_order_reverse(vec!['G'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['G', 'f', 'e', 'd', 'C', 'B', 'A']);
 
-        let common = topo_order_reverse_lazy(vec!['G'], id_fn, neighbors_fn).collect_vec();
+        let common: Vec<_> = topo_order_reverse_lazy(vec!['G'], id_fn, neighbors_fn, cycle_fn)
+            .try_collect()
+            .unwrap();
         assert_eq!(common, vec!['G', 'f', 'e', 'd', 'C', 'B', 'A']);
 
-        let common = topo_order_reverse_ord(vec!['G'], id_fn, neighbors_fn);
+        let common = topo_order_reverse_ord(vec!['G'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['G', 'f', 'e', 'd', 'C', 'B', 'A']);
 
         // Iterator can be lazy for linear chunks.
         let neighbors_fn = |node: &char| to_ok_iter(neighbors[node].iter().copied());
-        let cycle_fn = |_| panic!("graph has cycle");
         let mut inner_iter = TopoOrderReverseLazyInner::empty();
         inner_iter.extend([Ok('G')]);
         assert_eq!(
@@ -1154,14 +1202,17 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
+        let cycle_fn = |id| id;
 
-        let common = topo_order_reverse(vec!['F', 'C'], id_fn, neighbors_fn);
+        let common = topo_order_reverse(vec!['F', 'C'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['F', 'E', 'D', 'C', 'B', 'A']);
 
-        let common = topo_order_reverse_lazy(vec!['F', 'C'], id_fn, neighbors_fn).collect_vec();
+        let common: Vec<_> = topo_order_reverse_lazy(vec!['F', 'C'], id_fn, neighbors_fn, cycle_fn)
+            .try_collect()
+            .unwrap();
         assert_eq!(common, vec!['F', 'E', 'D', 'C', 'B', 'A']);
 
-        let common = topo_order_reverse_ord(vec!['F', 'C'], id_fn, neighbors_fn);
+        let common = topo_order_reverse_ord(vec!['F', 'C'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['F', 'E', 'D', 'C', 'B', 'A']);
     }
 
@@ -1182,14 +1233,17 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
+        let cycle_fn = |id| id;
 
-        let common = topo_order_reverse(vec!['D'], id_fn, neighbors_fn);
+        let common = topo_order_reverse(vec!['D'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['D', 'C', 'B', 'A']);
 
-        let common = topo_order_reverse_lazy(vec!['D'], id_fn, neighbors_fn).collect_vec();
+        let common: Vec<_> = topo_order_reverse_lazy(vec!['D'], id_fn, neighbors_fn, cycle_fn)
+            .try_collect()
+            .unwrap();
         assert_eq!(common, vec!['D', 'C', 'B', 'A']);
 
-        let common = topo_order_reverse_ord(vec!['D'], id_fn, neighbors_fn);
+        let common = topo_order_reverse_ord(vec!['D'], id_fn, neighbors_fn, cycle_fn).unwrap();
         assert_eq!(common, vec!['D', 'C', 'B', 'A']);
     }
 
@@ -1207,38 +1261,22 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
+        let cycle_fn = |id| id;
 
-        let result = panic::catch_unwind(|| topo_order_reverse(vec!['C'], id_fn, neighbors_fn));
-        assert!(result.is_err());
+        let result: Result<Vec<_>, _> =
+            topo_order_reverse(vec!['C'], id_fn, neighbors_fn, cycle_fn);
+        assert_matches!(result, Err('C' | 'B' | 'A'));
 
-        topo_order_reverse_lazy(vec!['C'], id_fn, neighbors_fn)
-            .take(3)
-            .collect_vec(); // sanity check
-        let result = panic::catch_unwind(|| {
-            topo_order_reverse_lazy(vec!['C'], id_fn, neighbors_fn)
-                .take(4)
-                .collect_vec()
-        });
-        assert!(result.is_err());
-
-        let result = panic::catch_unwind(|| topo_order_reverse_ord(vec!['C'], id_fn, neighbors_fn));
-        assert!(result.is_err());
-
-        // Try again with non-panicking cycle handler
-        let neighbors_fn = |node: &char| neighbors[node].iter().copied().map(Ok);
-        let cycle_fn = |node: char| node;
+        let result = topo_order_reverse_lazy(vec!['C'], id_fn, neighbors_fn, cycle_fn)
+            .take(4)
+            .collect_vec();
         assert_matches!(
-            topo_order_reverse_ok([Ok('C')], id_fn, neighbors_fn, cycle_fn),
-            Err('C' | 'B' | 'A')
-        );
-        assert_matches!(
-            topo_order_reverse_lazy_ok([Ok('C')], id_fn, neighbors_fn, cycle_fn).nth(3),
-            Some(Err('C' | 'B' | 'A'))
-        );
-        assert_matches!(
-            topo_order_reverse_ord_ok([Ok('C')], id_fn, neighbors_fn, cycle_fn),
-            Err('C' | 'B' | 'A')
+            result[..],
+            [Ok('C'), Ok('B'), Ok('A'), Err('C' | 'B' | 'A')]
         );
+
+        let result = topo_order_reverse_ord(vec!['C'], id_fn, neighbors_fn, cycle_fn);
+        assert_matches!(result, Err('C' | 'B' | 'A'));
     }
 
     #[test]
@@ -1259,38 +1297,22 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
+        let cycle_fn = |id| id;
 
-        let result = panic::catch_unwind(|| topo_order_reverse(vec!['D'], id_fn, neighbors_fn));
-        assert!(result.is_err());
+        let result = topo_order_reverse(vec!['D'], id_fn, neighbors_fn, cycle_fn);
+        assert_matches!(result, Err('C' | 'B' | 'A'));
 
-        topo_order_reverse_lazy(vec!['D'], id_fn, neighbors_fn)
-            .take(4)
-            .collect_vec(); // sanity check
-        let result = panic::catch_unwind(|| {
-            topo_order_reverse_lazy(vec!['D'], id_fn, neighbors_fn)
-                .take(5)
-                .collect_vec()
-        });
-        assert!(result.is_err());
-
-        let result = panic::catch_unwind(|| topo_order_reverse_ord(vec!['D'], id_fn, neighbors_fn));
-        assert!(result.is_err());
-
-        // Try again with non-panicking cycle handler
-        let neighbors_fn = |node: &char| neighbors[node].iter().copied().map(Ok);
-        let cycle_fn = |node: char| node;
+        let result = topo_order_reverse_lazy(vec!['D'], id_fn, neighbors_fn, cycle_fn)
+            .take(5)
+            .collect_vec();
         assert_matches!(
-            topo_order_reverse_ok([Ok('D')], id_fn, neighbors_fn, cycle_fn),
-            Err('C' | 'B' | 'A')
-        );
-        assert_matches!(
-            topo_order_reverse_lazy_ok([Ok('D')], id_fn, neighbors_fn, cycle_fn).nth(4),
-            Some(Err('C' | 'B' | 'A'))
-        );
-        assert_matches!(
-            topo_order_reverse_ord_ok([Ok('D')], id_fn, neighbors_fn, cycle_fn),
-            Err('C' | 'B' | 'A')
+            result[..],
+            [Ok('D'), Ok('C'), Ok('B'), Ok('A'), Err('C' | 'B' | 'A')]
         );
+
+        let result: Result<Vec<_>, _> =
+            topo_order_reverse_ord(vec!['D'], id_fn, neighbors_fn, cycle_fn);
+        assert_matches!(result, Err('C' | 'B' | 'A'));
     }
 
     #[test]
@@ -1311,9 +1333,11 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
+        let cycle_fn = |id| id;
 
-        let result =
-            panic::catch_unwind(|| topo_order_reverse_lazy(['D'], id_fn, neighbors_fn).nth(1));
+        let result = topo_order_reverse_lazy(['D'], id_fn, neighbors_fn, cycle_fn)
+            .nth(1)
+            .unwrap();
         assert!(result.is_err());
 
         // Try again with non-panicking cycle handler
@@ -1342,7 +1366,7 @@ mod tests {
         };
         let id_fn = |node: &char| *node;
         let neighbors_fn = |node: &char| neighbors[node].clone();
-        let cycle_fn = |_| panic!("graph has cycle");
+        let cycle_fn = |id| id;
 
         // Terminates at Err('X') no matter if the sorting order is forward or
         // reverse. The visiting order matters.
