@@ -569,7 +569,7 @@ pub(super) async fn collect_changed_paths(
 ) -> BackendResult<Vec<RepoPathBuf>> {
     let parents: Vec<_> = commit.parents_async().await?;
     if let [p] = parents.as_slice()
-        && commit.tree_id() == p.tree_id()
+        && commit.tree_ids() == p.tree_ids()
     {
         return Ok(vec![]);
     }
@@ -578,7 +578,7 @@ pub(super) async fn collect_changed_paths(
     tracing::trace!(?commit, parents_count = parents.len(), "calculating diffs");
     let store = commit.store();
     let from_tree = merge_commit_trees_no_resolve_without_repo(store, index, &parents).await?;
-    let to_tree = commit.tree_async().await?;
+    let to_tree = commit.tree();
     let tree_diff = from_tree.diff_stream(&to_tree, &EverythingMatcher);
     let paths = tree_diff
         .map(|entry| entry.values.map(|values| (entry.path, values)))
