@@ -41,10 +41,9 @@ use super::revset_engine;
 use crate::backend::ChangeId;
 use crate::backend::CommitId;
 use crate::hex_util;
-use crate::index::AllHeadsForGcUnsupported;
 use crate::index::ChangeIdIndex;
 use crate::index::Index;
-use crate::index::IndexError;
+use crate::index::IndexResult;
 use crate::object_id::HexPrefix;
 use crate::object_id::ObjectId as _;
 use crate::object_id::PrefixResolution;
@@ -368,7 +367,7 @@ impl CompositeCommitIndex {
         self.heads_pos(result)
     }
 
-    pub(super) fn all_heads(&self) -> impl Iterator<Item = CommitId> + use<'_> {
+    pub(super) fn all_heads(&self) -> impl Iterator<Item = CommitId> {
         self.all_heads_pos()
             .map(move |pos| self.entry_by_pos(pos).commit_id())
     }
@@ -569,44 +568,46 @@ impl AsCompositeIndex for CompositeIndex {
 
 // In revset engine, we need to convert &CompositeIndex to &dyn Index.
 impl Index for CompositeIndex {
-    fn shortest_unique_commit_id_prefix_len(&self, commit_id: &CommitId) -> usize {
-        self.commits()
-            .shortest_unique_commit_id_prefix_len(commit_id)
+    fn shortest_unique_commit_id_prefix_len(&self, commit_id: &CommitId) -> IndexResult<usize> {
+        Ok(self
+            .commits()
+            .shortest_unique_commit_id_prefix_len(commit_id))
     }
 
-    fn resolve_commit_id_prefix(&self, prefix: &HexPrefix) -> PrefixResolution<CommitId> {
-        self.commits().resolve_commit_id_prefix(prefix)
-    }
-
-    fn has_id(&self, commit_id: &CommitId) -> bool {
-        self.commits().has_id(commit_id)
-    }
-
-    fn is_ancestor(&self, ancestor_id: &CommitId, descendant_id: &CommitId) -> bool {
-        self.commits().is_ancestor(ancestor_id, descendant_id)
-    }
-
-    fn common_ancestors(&self, set1: &[CommitId], set2: &[CommitId]) -> Vec<CommitId> {
-        self.commits().common_ancestors(set1, set2)
-    }
-
-    fn all_heads_for_gc(
+    fn resolve_commit_id_prefix(
         &self,
-    ) -> Result<Box<dyn Iterator<Item = CommitId> + '_>, AllHeadsForGcUnsupported> {
+        prefix: &HexPrefix,
+    ) -> IndexResult<PrefixResolution<CommitId>> {
+        Ok(self.commits().resolve_commit_id_prefix(prefix))
+    }
+
+    fn has_id(&self, commit_id: &CommitId) -> IndexResult<bool> {
+        Ok(self.commits().has_id(commit_id))
+    }
+
+    fn is_ancestor(&self, ancestor_id: &CommitId, descendant_id: &CommitId) -> IndexResult<bool> {
+        Ok(self.commits().is_ancestor(ancestor_id, descendant_id))
+    }
+
+    fn common_ancestors(&self, set1: &[CommitId], set2: &[CommitId]) -> IndexResult<Vec<CommitId>> {
+        Ok(self.commits().common_ancestors(set1, set2))
+    }
+
+    fn all_heads_for_gc(&self) -> IndexResult<Box<dyn Iterator<Item = CommitId> + '_>> {
         Ok(Box::new(self.commits().all_heads()))
     }
 
     fn heads(
         &self,
         candidate_ids: &mut dyn Iterator<Item = &CommitId>,
-    ) -> Result<Vec<CommitId>, IndexError> {
+    ) -> IndexResult<Vec<CommitId>> {
         Ok(self.commits().heads(candidate_ids))
     }
 
     fn changed_paths_in_commit(
         &self,
         commit_id: &CommitId,
-    ) -> Result<Option<Box<dyn Iterator<Item = RepoPathBuf> + '_>>, IndexError> {
+    ) -> IndexResult<Option<Box<dyn Iterator<Item = RepoPathBuf> + '_>>> {
         let Some(paths) = self
             .commits()
             .commit_id_to_pos(commit_id)
@@ -653,9 +654,9 @@ impl<I: AsCompositeIndex + Send + Sync> ChangeIdIndex for ChangeIdIndexImpl<I> {
     // If `SingleMatch` is returned, the commits including in the set are all
     // visible. `AmbiguousMatch` may be returned even if the prefix is unique
     // within the visible entries.
-    fn resolve_prefix(&self, prefix: &HexPrefix) -> PrefixResolution<Vec<CommitId>> {
+    fn resolve_prefix(&self, prefix: &HexPrefix) -> IndexResult<PrefixResolution<Vec<CommitId>>> {
         let index = self.index.as_composite().commits();
-        match index.resolve_change_id_prefix(prefix) {
+        let prefix = match index.resolve_change_id_prefix(prefix) {
             PrefixResolution::NoMatch => PrefixResolution::NoMatch,
             PrefixResolution::SingleMatch((_change_id, positions)) => {
                 debug_assert!(positions.is_sorted_by(|a, b| a < b));
@@ -673,7 +674,8 @@ impl<I: AsCompositeIndex + Send + Sync> ChangeIdIndex for ChangeIdIndexImpl<I> {
                 }
             }
             PrefixResolution::AmbiguousMatch => PrefixResolution::AmbiguousMatch,
-        }
+        };
+        Ok(prefix)
     }
 
     // Calculates the shortest prefix length of the given `change_id` among all
@@ -682,9 +684,9 @@ impl<I: AsCompositeIndex + Send + Sync> ChangeIdIndex for ChangeIdIndexImpl<I> {
     // The returned length is usually a few digits longer than the minimum
     // length necessary to disambiguate within the visible entries since hidden
     // entries are also considered when determining the prefix length.
-    fn shortest_unique_prefix_len(&self, change_id: &ChangeId) -> usize {
+    fn shortest_unique_prefix_len(&self, change_id: &ChangeId) -> IndexResult<usize> {
         let index = self.index.as_composite().commits();
-        index.shortest_unique_change_id_prefix_len(change_id)
+        Ok(index.shortest_unique_change_id_prefix_len(change_id))
     }
 }
 
