@@ -102,6 +102,22 @@ mod tests {
         index.stats()
     }
 
+    fn common_ancestors(
+        index: &DefaultMutableIndex,
+        set1: &[CommitId],
+        set2: &[CommitId],
+    ) -> Vec<CommitId> {
+        index.common_ancestors(set1, set2).unwrap()
+    }
+
+    fn is_ancestor(
+        index: &DefaultMutableIndex,
+        ancestor_id: &CommitId,
+        descendant_id: &CommitId,
+    ) -> bool {
+        index.is_ancestor(ancestor_id, descendant_id).unwrap()
+    }
+
     #[test_case(false; "memory")]
     #[test_case(true; "file")]
     fn index_empty(on_disk: bool) {
@@ -1034,17 +1050,17 @@ mod tests {
         index.add_commit_data(id_4.clone(), new_change_id(), &[id_1.clone()]);
         index.add_commit_data(id_5.clone(), new_change_id(), &[id_4.clone(), id_2.clone()]);
 
-        assert!(index.is_ancestor(&id_0, &id_0));
-        assert!(index.is_ancestor(&id_0, &id_1));
-        assert!(index.is_ancestor(&id_2, &id_3));
-        assert!(index.is_ancestor(&id_2, &id_5));
-        assert!(index.is_ancestor(&id_1, &id_5));
-        assert!(index.is_ancestor(&id_0, &id_5));
-        assert!(!index.is_ancestor(&id_1, &id_0));
-        assert!(!index.is_ancestor(&id_5, &id_3));
-        assert!(!index.is_ancestor(&id_3, &id_5));
-        assert!(!index.is_ancestor(&id_2, &id_4));
-        assert!(!index.is_ancestor(&id_4, &id_2));
+        assert!(is_ancestor(&index, &id_0, &id_0));
+        assert!(is_ancestor(&index, &id_0, &id_1));
+        assert!(is_ancestor(&index, &id_2, &id_3));
+        assert!(is_ancestor(&index, &id_2, &id_5));
+        assert!(is_ancestor(&index, &id_1, &id_5));
+        assert!(is_ancestor(&index, &id_0, &id_5));
+        assert!(!is_ancestor(&index, &id_1, &id_0));
+        assert!(!is_ancestor(&index, &id_5, &id_3));
+        assert!(!is_ancestor(&index, &id_3, &id_5));
+        assert!(!is_ancestor(&index, &id_2, &id_4));
+        assert!(!is_ancestor(&index, &id_4, &id_2));
     }
 
     #[test]
@@ -1075,70 +1091,70 @@ mod tests {
         index.add_commit_data(id_6.clone(), new_change_id(), &[id_4.clone()]);
 
         assert_eq!(
-            index.common_ancestors(&[id_0.clone()], &[id_0.clone()]),
+            common_ancestors(&index, &[id_0.clone()], &[id_0.clone()]),
             vec![id_0.clone()]
         );
         assert_eq!(
-            index.common_ancestors(&[id_5.clone()], &[id_5.clone()]),
+            common_ancestors(&index, &[id_5.clone()], &[id_5.clone()]),
             vec![id_5.clone()]
         );
         assert_eq!(
-            index.common_ancestors(&[id_1.clone()], &[id_2.clone()]),
+            common_ancestors(&index, &[id_1.clone()], &[id_2.clone()]),
             vec![id_0.clone()]
         );
         assert_eq!(
-            index.common_ancestors(&[id_2.clone()], &[id_1.clone()]),
+            common_ancestors(&index, &[id_2.clone()], &[id_1.clone()]),
             vec![id_0.clone()]
         );
         assert_eq!(
-            index.common_ancestors(&[id_1.clone()], &[id_4.clone()]),
+            common_ancestors(&index, &[id_1.clone()], &[id_4.clone()]),
             vec![id_1.clone()]
         );
         assert_eq!(
-            index.common_ancestors(&[id_4.clone()], &[id_1.clone()]),
+            common_ancestors(&index, &[id_4.clone()], &[id_1.clone()]),
             vec![id_1.clone()]
         );
         assert_eq!(
-            index.common_ancestors(&[id_3.clone()], &[id_5.clone()]),
+            common_ancestors(&index, &[id_3.clone()], &[id_5.clone()]),
             vec![id_0.clone()]
         );
         assert_eq!(
-            index.common_ancestors(&[id_5.clone()], &[id_3.clone()]),
+            common_ancestors(&index, &[id_5.clone()], &[id_3.clone()]),
             vec![id_0.clone()]
         );
         assert_eq!(
-            index.common_ancestors(&[id_2.clone()], &[id_6.clone()]),
+            common_ancestors(&index, &[id_2.clone()], &[id_6.clone()]),
             vec![id_0.clone()]
         );
 
         // With multiple commits in an input set
         assert_eq!(
-            index.common_ancestors(&[id_0.clone(), id_1.clone()], &[id_0.clone()]),
+            common_ancestors(&index, &[id_0.clone(), id_1.clone()], &[id_0.clone()]),
             vec![id_0.clone()]
         );
         assert_eq!(
-            index.common_ancestors(&[id_0.clone(), id_1.clone()], &[id_1.clone()]),
+            common_ancestors(&index, &[id_0.clone(), id_1.clone()], &[id_1.clone()]),
             vec![id_1.clone()]
         );
         assert_eq!(
-            index.common_ancestors(&[id_1.clone(), id_2.clone()], &[id_1.clone()]),
+            common_ancestors(&index, &[id_1.clone(), id_2.clone()], &[id_1.clone()]),
             vec![id_1.clone()]
         );
         assert_eq!(
-            index.common_ancestors(&[id_1.clone(), id_2.clone()], &[id_4]),
+            common_ancestors(&index, &[id_1.clone(), id_2.clone()], &[id_4]),
             vec![id_1.clone()]
         );
         assert_eq!(
-            index.common_ancestors(&[id_5.clone(), id_6.clone()], &[id_2.clone()]),
+            common_ancestors(&index, &[id_5.clone(), id_6.clone()], &[id_2.clone()]),
             &[id_2.clone()]
         );
         // Both 1 and 2 are returned since (5) expands to (2, 4), which expands
         // to (1,2) and matches the (1,2) of the first input set.
         assert_eq!(
-            index.common_ancestors(&[id_1.clone(), id_2.clone()], &[id_5]),
+            common_ancestors(&index, &[id_1.clone(), id_2.clone()], &[id_5]),
             vec![id_2.clone(), id_1.clone()]
         );
-        assert_eq!(index.common_ancestors(&[id_1, id_2], &[id_3]), vec![id_0]);
+        assert_eq!(common_ancestors(&index, &[id_1, id_2], &[id_3]), vec![id_0]);
     }
 
     #[test]
@@ -1161,7 +1177,7 @@ mod tests {
         index.add_commit_data(id_3.clone(), new_change_id(), &[id_1.clone(), id_2.clone()]);
         index.add_commit_data(id_4.clone(), new_change_id(), &[id_1.clone(), id_2.clone()]);
 
-        let mut common_ancestors = index.common_ancestors(&[id_3], &[id_4]);
+        let mut common_ancestors = common_ancestors(&index, &[id_3], &[id_4]);
         common_ancestors.sort();
         assert_eq!(common_ancestors, vec![id_1, id_2]);
     }
@@ -1188,7 +1204,7 @@ mod tests {
         index.add_commit_data(id_4.clone(), new_change_id(), &[id_0.clone(), id_2.clone()]);
         index.add_commit_data(id_5.clone(), new_change_id(), &[id_0, id_2.clone()]);
 
-        let mut common_ancestors = index.common_ancestors(&[id_4], &[id_5]);
+        let mut common_ancestors = common_ancestors(&index, &[id_4], &[id_5]);
         common_ancestors.sort();
         assert_eq!(common_ancestors, vec![id_2]);
     }
