@@ -47,10 +47,9 @@ use super::revset_engine::RevsetImpl;
 use crate::backend::ChangeId;
 use crate::backend::CommitId;
 use crate::graph::GraphNode;
-use crate::index::AllHeadsForGcUnsupported;
 use crate::index::ChangeIdIndex;
 use crate::index::Index;
-use crate::index::IndexError;
+use crate::index::IndexResult;
 use crate::index::MutableIndex;
 use crate::index::ReadonlyIndex;
 use crate::object_id::HexPrefix;
@@ -250,7 +249,7 @@ impl CommitGraphEntry<'_> {
 //       ids
 // TODO: add a fanout table like git's commit graph has?
 pub(super) struct ReadonlyCommitIndexSegment {
-    parent_file: Option<Arc<ReadonlyCommitIndexSegment>>,
+    parent_file: Option<Arc<Self>>,
     num_parent_commits: u32,
     id: CommitIndexSegmentId,
     field_lengths: FieldLengths,
@@ -458,7 +457,7 @@ impl ReadonlyCommitIndexSegment {
     fn overflow_changes_from(
         &self,
         overflow_pos: u32,
-    ) -> impl Iterator<Item = LocalCommitPosition> + use<'_> {
+    ) -> impl Iterator<Item = LocalCommitPosition> {
         let table = &self.data[self.change_overflow_base..];
         let offset = (overflow_pos as usize) * 4;
         let (chunks, _remainder) = table[offset..].as_chunks();
@@ -629,6 +628,10 @@ impl DefaultReadonlyIndex {
         self.0.changed_paths()
     }
 
+    pub(super) fn has_id_impl(&self, commit_id: &CommitId) -> bool {
+        self.0.commits().has_id(commit_id)
+    }
+
     /// Returns the number of all indexed commits.
     pub fn num_commits(&self) -> u32 {
         self.0.commits().num_commits()
@@ -713,43 +716,41 @@ impl AsCompositeIndex for DefaultReadonlyIndex {
 }
 
 impl Index for DefaultReadonlyIndex {
-    fn shortest_unique_commit_id_prefix_len(&self, commit_id: &CommitId) -> usize {
+    fn shortest_unique_commit_id_prefix_len(&self, commit_id: &CommitId) -> IndexResult<usize> {
         self.0.shortest_unique_commit_id_prefix_len(commit_id)
     }
 
-    fn resolve_commit_id_prefix(&self, prefix: &HexPrefix) -> PrefixResolution<CommitId> {
+    fn resolve_commit_id_prefix(
+        &self,
+        prefix: &HexPrefix,
+    ) -> IndexResult<PrefixResolution<CommitId>> {
         self.0.resolve_commit_id_prefix(prefix)
     }
 
-    fn has_id(&self, commit_id: &CommitId) -> bool {
-        self.0.has_id(commit_id)
+    fn has_id(&self, commit_id: &CommitId) -> IndexResult<bool> {
+        Ok(self.has_id_impl(commit_id))
     }
 
-    fn is_ancestor(&self, ancestor_id: &CommitId, descendant_id: &CommitId) -> bool {
+    fn is_ancestor(&self, ancestor_id: &CommitId, descendant_id: &CommitId) -> IndexResult<bool> {
         self.0.is_ancestor(ancestor_id, descendant_id)
     }
 
-    fn common_ancestors(&self, set1: &[CommitId], set2: &[CommitId]) -> Vec<CommitId> {
+    fn common_ancestors(&self, set1: &[CommitId], set2: &[CommitId]) -> IndexResult<Vec<CommitId>> {
         self.0.common_ancestors(set1, set2)
     }
 
-    fn all_heads_for_gc(
-        &self,
-    ) -> Result<Box<dyn Iterator<Item = CommitId> + '_>, AllHeadsForGcUnsupported> {
+    fn all_heads_for_gc(&self) -> IndexResult<Box<dyn Iterator<Item = CommitId> + '_>> {
         self.0.all_heads_for_gc()
     }
 
-    fn heads(
-        &self,
-        candidates: &mut dyn Iterator<Item = &CommitId>,
-    ) -> Result<Vec<CommitId>, IndexError> {
+    fn heads(&self, candidates: &mut dyn Iterator<Item = &CommitId>) -> IndexResult<Vec<CommitId>> {
         self.0.heads(candidates)
     }
 
     fn changed_paths_in_commit(
         &self,
         commit_id: &CommitId,
-    ) -> Result<Option<Box<dyn Iterator<Item = RepoPathBuf> + '_>>, IndexError> {
+    ) -> IndexResult<Option<Box<dyn Iterator<Item = RepoPathBuf> + '_>>> {
         self.0.changed_paths_in_commit(commit_id)
     }
 
