@@ -25,6 +25,7 @@ use thiserror::Error;
 use crate::backend::ChangeId;
 use crate::backend::CommitId;
 use crate::hex_util;
+use crate::index::IndexResult;
 use crate::object_id::HexPrefix;
 use crate::object_id::ObjectId;
 use crate::object_id::PrefixResolution;
@@ -159,7 +160,7 @@ impl IdPrefixIndex<'_> {
         &self,
         repo: &dyn Repo,
         prefix: &HexPrefix,
-    ) -> PrefixResolution<CommitId> {
+    ) -> IndexResult<PrefixResolution<CommitId>> {
         if let Some(indexes) = self.indexes {
             let resolution = indexes
                 .commit_index
@@ -171,14 +172,14 @@ impl IdPrefixIndex<'_> {
                 PrefixResolution::SingleMatch(id) => {
                     // The disambiguation set may be loaded from a different repo,
                     // and contain a commit that doesn't exist in the current repo.
-                    if repo.index().has_id(&id) {
-                        return PrefixResolution::SingleMatch(id);
+                    if repo.index().has_id(&id)? {
+                        return Ok(PrefixResolution::SingleMatch(id));
                     } else {
-                        return PrefixResolution::NoMatch;
+                        return Ok(PrefixResolution::NoMatch);
                     }
                 }
                 PrefixResolution::AmbiguousMatch => {
-                    return PrefixResolution::AmbiguousMatch;
+                    return Ok(PrefixResolution::AmbiguousMatch);
                 }
             }
         }
@@ -187,18 +188,30 @@ impl IdPrefixIndex<'_> {
 
     /// Returns the shortest length of a prefix of `commit_id` that can still be
     /// resolved by `resolve_commit_prefix()` and [`SymbolResolver`].
-    pub fn shortest_commit_prefix_len(&self, repo: &dyn Repo, commit_id: &CommitId) -> usize {
-        let len = self.shortest_commit_prefix_len_exact(repo, commit_id);
-        disambiguate_prefix_with_refs(repo.view(), &commit_id.to_string(), len)
+    pub fn shortest_commit_prefix_len(
+        &self,
+        repo: &dyn Repo,
+        commit_id: &CommitId,
+    ) -> IndexResult<usize> {
+        let len = self.shortest_commit_prefix_len_exact(repo, commit_id)?;
+        Ok(disambiguate_prefix_with_refs(
+            repo.view(),
+            &commit_id.to_string(),
+            len,
+        ))
     }
 
-    pub fn shortest_commit_prefix_len_exact(&self, repo: &dyn Repo, commit_id: &CommitId) -> usize {
+    pub fn shortest_commit_prefix_len_exact(
+        &self,
+        repo: &dyn Repo,
+        commit_id: &CommitId,
+    ) -> IndexResult<usize> {
         if let Some(indexes) = self.indexes
             && let Some(lookup) = indexes
                 .commit_index
                 .lookup_exact(&*indexes.commit_change_ids, commit_id)
         {
-            return lookup.shortest_unique_prefix_len();
+            return Ok(lookup.shortest_unique_prefix_len());
         }
         repo.index().shortest_unique_commit_id_prefix_len(commit_id)
     }
@@ -208,7 +221,7 @@ impl IdPrefixIndex<'_> {
         &self,
         repo: &dyn Repo,
         prefix: &HexPrefix,
-    ) -> PrefixResolution<Vec<CommitId>> {
+    ) -> IndexResult<PrefixResolution<Vec<CommitId>>> {
         if let Some(indexes) = self.indexes {
             let resolution = indexes
                 .change_index
@@ -218,15 +231,15 @@ impl IdPrefixIndex<'_> {
                     // Fall back to resolving in entire repo
                 }
                 PrefixResolution::SingleMatch(change_id) => {
-                    return match repo.resolve_change_id(&change_id) {
+                    return match repo.resolve_change_id(&change_id)? {
                         // There may be more commits with this change id outside the narrower sets.
-                        Some(commit_ids) => PrefixResolution::SingleMatch(commit_ids),
+                        Some(commit_ids) => Ok(PrefixResolution::SingleMatch(commit_ids)),
                         // The disambiguation set may contain hidden commits.
-                        None => PrefixResolution::NoMatch,
+                        None => Ok(PrefixResolution::NoMatch),
                     };
                 }
                 PrefixResolution::AmbiguousMatch => {
-                    return PrefixResolution::AmbiguousMatch;
+                    return Ok(PrefixResolution::AmbiguousMatch);
                 }
             }
         }
@@ -235,18 +248,30 @@ impl IdPrefixIndex<'_> {
 
     /// Returns the shortest length of a prefix of `change_id` that can still be
     /// resolved by `resolve_change_prefix()` and [`SymbolResolver`].
-    pub fn shortest_change_prefix_len(&self, repo: &dyn Repo, change_id: &ChangeId) -> usize {
-        let len = self.shortest_change_prefix_len_exact(repo, change_id);
-        disambiguate_prefix_with_refs(repo.view(), &change_id.to_string(), len)
+    pub fn shortest_change_prefix_len(
+        &self,
+        repo: &dyn Repo,
+        change_id: &ChangeId,
+    ) -> IndexResult<usize> {
+        let len = self.shortest_change_prefix_len_exact(repo, change_id)?;
+        Ok(disambiguate_prefix_with_refs(
+            repo.view(),
+            &change_id.to_string(),
+            len,
+        ))
     }
 
-    fn shortest_change_prefix_len_exact(&self, repo: &dyn Repo, change_id: &ChangeId) -> usize {
+    fn shortest_change_prefix_len_exact(
+        &self,
+        repo: &dyn Repo,
+        change_id: &ChangeId,
+    ) -> IndexResult<usize> {
         if let Some(indexes) = self.indexes
             && let Some(lookup) = indexes
                 .change_index
                 .lookup_exact(&*indexes.commit_change_ids, change_id)
         {
-            return lookup.shortest_unique_prefix_len();
+            return Ok(lookup.shortest_unique_prefix_len());
         }
         repo.shortest_unique_change_id_prefix_len(change_id)
     }
