@@ -14,7 +14,6 @@
 
 //! Provides a backend for testing ACLs
 
-use std::any::Any;
 use std::path::Path;
 use std::pin::Pin;
 use std::time::SystemTime;
@@ -30,8 +29,6 @@ use crate::backend::BackendResult;
 use crate::backend::ChangeId;
 use crate::backend::Commit;
 use crate::backend::CommitId;
-use crate::backend::Conflict;
-use crate::backend::ConflictId;
 use crate::backend::CopyHistory;
 use crate::backend::CopyId;
 use crate::backend::CopyRecord;
@@ -89,10 +86,6 @@ impl SecretBackend {
 
 #[async_trait]
 impl Backend for SecretBackend {
-    fn as_any(&self) -> &dyn Any {
-        self
-    }
-
     fn name(&self) -> &str {
         Self::name()
     }
@@ -189,14 +182,6 @@ impl Backend for SecretBackend {
         self.inner.write_tree(path, contents).await
     }
 
-    fn read_conflict(&self, path: &RepoPath, id: &ConflictId) -> BackendResult<Conflict> {
-        self.inner.read_conflict(path, id)
-    }
-
-    fn write_conflict(&self, path: &RepoPath, contents: &Conflict) -> BackendResult<ConflictId> {
-        self.inner.write_conflict(path, contents)
-    }
-
     async fn read_commit(&self, id: &CommitId) -> BackendResult<Commit> {
         self.inner.read_commit(id).await
     }
