@@ -14,12 +14,13 @@
 
 #![expect(missing_docs)]
 
-use std::path::PathBuf;
+use std::collections::HashMap;
 use std::str::FromStr;
 use std::sync::Arc;
 use std::sync::Mutex;
 
 use chrono::DateTime;
+use itertools::Itertools as _;
 use rand::prelude::*;
 use rand_chacha::ChaCha20Rng;
 use serde::Deserialize;
@@ -35,7 +36,9 @@ use crate::config::ConfigValue;
 use crate::config::StackedConfig;
 use crate::config::ToConfigNamePath;
 use crate::fmt_util::binary_prefix;
+use crate::ref_name::RemoteNameBuf;
 use crate::signing::SignBehavior;
+use crate::str_util::StringPattern;
 
 #[derive(Debug, Clone)]
 pub struct UserSettings {
@@ -57,23 +60,34 @@ struct UserSettingsData {
 }
 
 #[derive(Debug, Clone)]
-pub struct GitSettings {
-    pub auto_local_bookmark: bool,
-    pub abandon_unreachable_commits: bool,
-    pub executable_path: PathBuf,
-    pub write_change_id_header: bool,
-    pub colocate: bool,
+pub struct RemoteSettings {
+    pub auto_track_bookmarks: StringPattern,
 }
 
-impl GitSettings {
-    pub fn from_settings(settings: &UserSettings) -> Result<Self, ConfigGetError> {
-        Ok(Self {
-            auto_local_bookmark: settings.get_bool("git.auto-local-bookmark")?,
-            abandon_unreachable_commits: settings.get_bool("git.abandon-unreachable-commits")?,
-            executable_path: settings.get("git.executable-path")?,
-            write_change_id_header: settings.get("git.write-change-id-header")?,
-            colocate: settings.get("git.colocate")?,
-        })
+impl RemoteSettings {
+    pub fn table_from_settings(
+        settings: &UserSettings,
+    ) -> Result<HashMap<RemoteNameBuf, Self>, ConfigGetError> {
+        settings
+            .table_keys("remotes")
+            .map(|name| {
+                Ok((
+                    name.into(),
+                    Self {
+                        auto_track_bookmarks: settings.get_value_with(
+                            ["remotes", name, "auto-track-bookmarks"],
+                            |value| -> Result<_, Box<dyn std::error::Error + Send + Sync>> {
+                                Ok(StringPattern::parse(
+                                    value
+                                        .as_str()
+                                        .ok_or_else(|| "expected a string".to_string())?,
+                                )?)
+                            },
+                        )?,
+                    },
+                ))
+            })
+            .try_collect()
     }
 }
 
@@ -214,8 +228,10 @@ impl UserSettings {
         &self.config
     }
 
-    pub fn git_settings(&self) -> Result<GitSettings, ConfigGetError> {
-        GitSettings::from_settings(self)
+    pub fn remote_settings(
+        &self,
+    ) -> Result<HashMap<RemoteNameBuf, RemoteSettings>, ConfigGetError> {
+        RemoteSettings::table_from_settings(self)
     }
 
     // separate from sign_settings as those two are needed in pretty different
