@@ -19,6 +19,7 @@ use std::borrow::Cow;
 use std::collections::BTreeMap;
 use std::fmt;
 use std::fmt::Debug;
+use std::iter;
 use std::ops::Deref;
 
 use bstr::ByteSlice as _;
@@ -91,6 +92,12 @@ fn parse_glob(src: &str, icase: bool) -> Result<GlobPattern, StringPatternParseE
     Ok(GlobPattern { glob, regex })
 }
 
+fn is_glob_char(c: char) -> bool {
+    // See globset::escape(). In addition to that, backslash is parsed as an
+    // escape sequence on all platforms.
+    matches!(c, '?' | '*' | '[' | ']' | '{' | '}' | '\\')
+}
+
 /// Pattern to be tested against string property like commit description or
 /// bookmark name.
 #[derive(Clone, Debug)]
@@ -155,12 +162,17 @@ impl StringPattern {
 
     /// Parses the given string as a glob pattern.
     pub fn glob(src: &str) -> Result<Self, StringPatternParseError> {
-        // TODO: if no meta character found, it can be mapped to Exact.
+        if !src.contains(is_glob_char) {
+            return Ok(Self::exact(src));
+        }
         Ok(Self::Glob(Box::new(parse_glob(src, false)?)))
     }
 
     /// Parses the given string as a case‐insensitive glob pattern.
     pub fn glob_i(src: &str) -> Result<Self, StringPatternParseError> {
+        // No special case for !src.contains(is_glob_char) because it's unclear
+        // whether we'll use unicode case comparison for "exact-i" patterns.
+        // "glob-i" should always be ASCII-based.
         Ok(Self::GlobI(Box::new(parse_glob(src, true)?)))
     }
 
@@ -415,6 +427,35 @@ impl StringExpression {
         Self::Intersection(Box::new(self), Box::new(other))
     }
 
+    fn dfs_pre(&self) -> impl Iterator<Item = &Self> {
+        let mut stack: Vec<&Self> = vec![self];
+        iter::from_fn(move || {
+            let expr = stack.pop()?;
+            match expr {
+                Self::Pattern(_) => {}
+                Self::NotIn(expr) => stack.push(expr),
+                Self::Union(expr1, expr2) | Self::Intersection(expr1, expr2) => {
+                    stack.push(expr2);
+                    stack.push(expr1);
+                }
+            }
+            Some(expr)
+        })
+    }
+
+    /// Iterates exact string patterns recursively from this expression.
+    ///
+    /// For example, `"a", "b", "c"` will be yielded in that order for
+    /// expression `"a" | glob:"?" & "b" | ~"c"`.
+    pub fn exact_strings(&self) -> impl Iterator<Item = &str> {
+        // pre/post-ordering doesn't matter so long as children are visited from
+        // left to right.
+        self.dfs_pre().filter_map(|expr| match expr {
+            Self::Pattern(pattern) => pattern.as_exact(),
+            _ => None,
+        })
+    }
+
     /// Transforms the expression tree to matcher object.
     pub fn to_matcher(&self) -> StringMatcher {
         match self {
@@ -787,11 +828,11 @@ mod tests {
     fn test_glob_pattern_to_matcher() {
         assert_matches!(
             StringPattern::glob("").unwrap().to_matcher(),
-            StringMatcher::Fn(_) // or Exact
+            StringMatcher::Exact(_)
         );
         assert_matches!(
             StringPattern::glob("x").unwrap().to_matcher(),
-            StringMatcher::Fn(_) // or Exact
+            StringMatcher::Exact(_)
         );
         assert_matches!(
             StringPattern::glob("x?").unwrap().to_matcher(),
@@ -801,6 +842,10 @@ mod tests {
             StringPattern::glob("*").unwrap().to_matcher(),
             StringMatcher::All
         );
+        assert_matches!(
+            StringPattern::glob(r"\\").unwrap().to_matcher(),
+            StringMatcher::Fn(_) // or Exact(r"\")
+        );
 
         assert_matches!(
             StringPattern::glob_i("").unwrap().to_matcher(),
@@ -907,6 +952,27 @@ mod tests {
         "#);
     }
 
+    #[test]
+    fn test_exact_strings_in_expression() {
+        assert_eq!(
+            StringExpression::all().exact_strings().collect_vec(),
+            [""; 0]
+        );
+        assert_eq!(
+            StringExpression::union_all(vec![
+                StringExpression::exact("a"),
+                StringExpression::substring("b"),
+                StringExpression::intersection(
+                    StringExpression::exact("c"),
+                    StringExpression::exact("d").negated(),
+                ),
+            ])
+            .exact_strings()
+            .collect_vec(),
+            ["a", "c", "d"]
+        );
+    }
+
     #[test]
     fn test_trivial_expression_to_matcher() {
         assert_matches!(StringExpression::all().to_matcher(), StringMatcher::All);
