@@ -319,6 +319,14 @@ impl View {
     }
 
     /// Iterates remote `(name, view)`s in lexicographical order.
+    pub fn remote_views(&self) -> impl Iterator<Item = (&RemoteName, &RemoteView)> {
+        self.data
+            .remote_views
+            .iter()
+            .map(|(name, view)| (name.as_ref(), view))
+    }
+
+    /// Iterates matching remote `(name, view)`s in lexicographical order.
     pub fn remote_views_matching(
         &self,
         matcher: &StringMatcher,
@@ -328,6 +336,11 @@ impl View {
             .map(|(name, view)| (name.as_ref(), view))
     }
 
+    /// Returns the remote view for `name`.
+    pub fn get_remote_view(&self, name: &RemoteName) -> Option<&RemoteView> {
+        self.data.remote_views.get(name)
+    }
+
     /// Adds remote view if it doesn't exist.
     pub fn ensure_remote(&mut self, remote_name: &RemoteName) {
         if self.data.remote_views.contains_key(remote_name) {
