@@ -28,12 +28,12 @@ use thiserror::Error;
 use tracing::instrument;
 
 use crate::backend::BackendError;
-use crate::backend::MergedTreeId;
 use crate::commit::Commit;
 use crate::dag_walk;
 use crate::gitignore::GitIgnoreError;
 use crate::gitignore::GitIgnoreFile;
 use crate::matchers::Matcher;
+use crate::merged_tree::MergedTree;
 use crate::op_store::OpStoreError;
 use crate::op_store::OperationId;
 use crate::operation::Operation;
@@ -61,8 +61,8 @@ pub trait WorkingCopy: Any + Send {
     /// The operation this working copy was most recently updated to.
     fn operation_id(&self) -> &OperationId;
 
-    /// The ID of the tree this working copy was most recently updated to.
-    fn tree_id(&self) -> Result<&MergedTreeId, WorkingCopyStateError>;
+    /// The tree this working copy was most recently updated to.
+    fn tree(&self) -> Result<&MergedTree, WorkingCopyStateError>;
 
     /// Patterns that decide which paths from the current tree should be checked
     /// out in the working copy. An empty list means that no paths should be
@@ -112,13 +112,13 @@ pub trait LockedWorkingCopy: Any + Send {
     fn old_operation_id(&self) -> &OperationId;
 
     /// The tree at the time the lock was taken
-    fn old_tree_id(&self) -> &MergedTreeId;
+    fn old_tree(&self) -> &MergedTree;
 
-    /// Snapshot the working copy. Returns the tree id and stats.
+    /// Snapshot the working copy. Returns the tree and stats.
     async fn snapshot(
         &mut self,
         options: &SnapshotOptions,
-    ) -> Result<(MergedTreeId, SnapshotStats), SnapshotError>;
+    ) -> Result<(MergedTree, SnapshotStats), SnapshotError>;
 
     /// Check out the specified commit in the working copy.
     async fn check_out(&mut self, commit: &Commit) -> Result<CheckoutStats, CheckoutError>;
@@ -221,6 +221,9 @@ pub struct SnapshotOptions<'a> {
     /// For new files that are not already tracked, start tracking them if they
     /// match this.
     pub start_tracking_matcher: &'a dyn Matcher,
+    /// For files that match the ignore patterns or are too large, start
+    /// tracking them anyway if they match this.
+    pub force_tracking_matcher: &'a dyn Matcher,
     /// The size of the largest file that should be allowed to become tracked
     /// (already tracked files are always snapshotted). If there are larger
     /// files in the working copy, then `LockedWorkingCopy::snapshot()` may
@@ -362,9 +365,8 @@ impl WorkingCopyFreshness {
         wc_commit: &Commit,
         repo: &ReadonlyRepo,
     ) -> Result<Self, OpStoreError> {
-        // Check if the working copy's tree matches the repo's view
-        let wc_tree_id = locked_wc.old_tree_id();
-        if wc_commit.tree_id() == wc_tree_id {
+        // Check if the working copy's operation matches the repo's operation
+        if locked_wc.old_operation_id() == repo.op_id() {
             // The working copy isn't stale, and no need to reload the repo.
             Ok(Self::Fresh)
         } else {
@@ -384,7 +386,12 @@ impl WorkingCopyFreshness {
             } else if ancestor_op.id() == wc_operation.id() {
                 // The working copy was not updated when some repo operation committed,
                 // meaning that it's stale compared to the repo view.
-                Ok(Self::WorkingCopyStale)
+                if locked_wc.old_tree().tree_ids() == wc_commit.tree_ids() {
+                    // The working copy doesn't require any changes
+                    Ok(Self::Fresh)
+                } else {
+                    Ok(Self::WorkingCopyStale)
+                }
             } else {
                 Ok(Self::SiblingOperation)
             }
@@ -430,7 +437,7 @@ pub fn create_and_check_out_recovery_commit(
         })?;
     let commit = repo.store().get_commit(commit_id)?;
     let new_commit = repo_mut
-        .new_commit(vec![commit_id.clone()], commit.tree_id().clone())
+        .new_commit(vec![commit_id.clone()], commit.tree())
         .set_description(description)
         .write()?;
     repo_mut.set_wc_commit(workspace_name, new_commit.id().clone())?;
