@@ -261,6 +261,29 @@ fn test_materialize_conflict_three_sides() {
     line 5
     "
     );
+    // Test materializing "diff-experimental" conflict markers
+    insta::assert_snapshot!(
+        &materialize_conflict_string(store, path, &conflict, ConflictMarkerStyle::DiffExperimental),
+        @r"
+    line 1
+    <<<<<<< Conflict 1 of 1
+    +++++++ Contents of side #1
+    line 2 a.1
+    line 3 a.2
+    line 4 base
+    %%%%%%% Changes from base #1 to side #2
+    -line 2 base
+    +line 2 b.1
+     line 3 base
+    -line 4 base
+    +line 4 b.2
+    %%%%%%% Changes from base #2 to side #3
+     line 2 base
+    +line 3 c.2
+    >>>>>>> Conflict 1 of 1 ends
+    line 5
+    "
+    );
     // Test materializing "snapshot" conflict markers
     insta::assert_snapshot!(
         &materialize_conflict_string(store, path, &conflict, ConflictMarkerStyle::Snapshot),
@@ -767,7 +790,6 @@ fn test_materialize_conflict_two_forward_diffs() {
     // ----
     // E
     // >>>>
-    // TODO: Maybe we should never have negative snapshots
     let path = repo_path("file");
     let a_id = testutils::write_file(store, path, "A\n");
     let b_id = testutils::write_file(store, path, "B\n");
@@ -784,6 +806,8 @@ fn test_materialize_conflict_two_forward_diffs() {
             Some(c_id.clone()),
         ],
     );
+    // The materialized conflict should still have exactly one snapshot despite our
+    // attempted temptation.
     insta::assert_snapshot!(
         &materialize_conflict_string(store, path, &conflict, ConflictMarkerStyle::Diff),
         @r"
@@ -792,12 +816,12 @@ fn test_materialize_conflict_two_forward_diffs() {
     A
     %%%%%%% Changes from base #1 to side #2
      B
-    +++++++ Contents of side #3
-    D
-    %%%%%%% Changes from base #2 to side #4
-     C
-    ------- Contents of base #3
-    E
+    %%%%%%% Changes from base #2 to side #3
+    -C
+    +D
+    %%%%%%% Changes from base #3 to side #4
+    -E
+    +C
     >>>>>>> Conflict 1 of 1 ends
     "
     );
