@@ -25,6 +25,7 @@ use jj_lib::workspace::Workspace;
 use test_case::test_case;
 use testutils::TestRepoBackend;
 use testutils::TestWorkspace;
+use testutils::assert_tree_eq;
 use testutils::git;
 use testutils::write_random_commit;
 
@@ -153,7 +154,7 @@ fn test_init_checkout(backend: TestRepoBackend) {
         .get_wc_commit_id(WorkspaceName::DEFAULT)
         .unwrap();
     let wc_commit = repo.store().get_commit(wc_commit_id).unwrap();
-    assert_eq!(*wc_commit.tree_id(), repo.store().empty_merged_tree_id());
+    assert_tree_eq!(wc_commit.tree(), repo.store().empty_merged_tree());
     assert_eq!(
         wc_commit.store_commit().parents,
         vec![repo.store().root_commit_id().clone()]
@@ -171,7 +172,6 @@ fn test_init_checkout(backend: TestRepoBackend) {
 }
 
 #[cfg(unix)]
-#[cfg_attr(target_os = "macos", ignore = "APFS/HFS+ don't like non-UTF-8 paths")]
 #[test]
 fn test_init_load_non_utf8_path() {
     use std::ffi::OsStr;
@@ -183,6 +183,15 @@ fn test_init_load_non_utf8_path() {
     let settings = testutils::user_settings();
     let test_env = TestEnvironment::init();
 
+    if testutils::check_strict_utf8_fs(test_env.root()) {
+        eprintln!(
+            "Skipping test \"test_init_load_non_utf8_path\" due to strict UTF-8 filesystem for \
+             path {:?}",
+            test_env.root()
+        );
+        return;
+    }
+
     let git_repo_path = test_env.root().join(OsStr::from_bytes(b"git\xe0"));
     assert!(git_repo_path.to_str().is_none());
     git::init(&git_repo_path);
