@@ -324,7 +324,10 @@ impl Backend for TestBackend {
                 copies.keys(),
                 |id| *id,
                 |id| copies.get(*id).unwrap().parents.iter(),
-            ) {
+                |_| panic!("graph has cycle"),
+            )
+            .unwrap()
+            {
                 histories.push(copies.get(id).unwrap().clone());
             }
             Ok(histories)
